/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iexec.c
 *
 * Parse and exec the kexec command
 ***************************************************************************/

#include <stdio.h>

#include "interp.h"
#include "kediti.h"

static char *documentation =
"kexec {-oid <node>,<name>}\n\
This command performs an exec request on the given node, wich can be\n\
provided either through its node identifier, or through its symbolic\n\
name (if the \"mkdb\" command has been run).\n" ;

/***************************************************************************
 * ParseExec: parse and exec Exec request
 *
 * RETURNS: 0
 ***************************************************************************/

static int ParseExec (argc, argv)
    int  argc ;
    char **argv ;

{
    KID      oid ;
    KeStatus status ;

/*
 * Skip command name and get oid :
 */
    argc-- ;
    argv++ ;
    if ( ParseId (argc, argv, &oid) <= 0 )
	return (1) ;
/*
 * Perform in sync mode:
 */

    if ( status = KecExec (KecConn, oid) )
    {
	/*
	 * This means that the send request failed, not that the server
	 * refused to execute the node.
	 */
	printf ("\tExec request failed.\n") ;
    }
    else if ( TraceFlag )
    {
	printf ("\tExec request succeeded.\n") ;
    }

    return (0) ;
}

/***************************************************************************
 * ExecInitialize: declare the 'kexec' command
 *
 * RETURNS: void
 ***************************************************************************/

void ExecInitialize ()

{
    DeclareCommand ("kexec"
		   , documentation
		   , ParseExec) ;
    return ;
}


