/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iget.c
 *
 * Parse the 'kget' command.
 ***************************************************************************/

#include <stdio.h>

#include "interp.h"
#include "kediti.h"

static char *documentation =
"kget {-oid <node>,<name>} [format]\n\
This command perform a get request on the given node wich can be\n\
provided either as an node identifier, or using its symbolic name\n\
if \"mkdb\" command has been run only).\n\
The format is an optional argument indicating the format in wich you\n\
want the value to be printed. Possible formats are:\n\
%s\tindicating that the value is to be printed as an ASCII string\n\
%d\tindicating that the value is to be printed as an integer.\n\
If no format is provided, an hexa dump of the value will be printed.\n" ;

/***************************************************************************
 * ParseGet: parse the kget command
 *
 * RETURNS: 0
 ***************************************************************************/

static int ParseGet (argc, argv)
    int  argc ;
    char **argv ;

{
    KID		oid ;
    KecValue	result ;
    KeStatus	status ;
    char 	*fmt = (char *) 0 ;
    int         count ;

/*
 * Skip command name and get object id :
 */
    argc-- ;
    argv++ ;
    if ( ( count = ParseId (argc, argv, &oid) ) <= 0 )
	return (1) ;

/*
 * Decode rest of command line :
 */

    argc -= count ;
    argv += count ;
    if ( argc )
	fmt = (char *) *argv ;
/*
 * Perform the Get request, in synchronous mode:
 */

    if ( status = KecGetSync (KecConn, oid, &result) )
    {
	KecError error = KecErrorOfConnection (KecConn) ;

	/*
	 * An error occured, retreive it from connection, and display 
	 * the optional error message provided by target server.
	 */

	printf ("\tget failed (status = %d).\n", status) ;
	if ( error->msg )
	    printf ("\t%s\n", error->msg) ;
	return (0) ;
    }

/*
 * Ok, got the result, check and display according to format
 */

    if ( ! result )
    {
	/*
	 * I don't think this can happen !
	 */
	printf ("\tno value returned.\n") ;
	return (0) ;
    }


    if ( fmt )
    {
	switch (*(fmt+1))
	{
	  case 's':
	    {
		int size = result->size ;
		char *p  = result->data ;
		
		while ( size-- )
		    fputc (*p++, stdout) ;
		
		fputc ('\n', stdout) ;
	    }
	    break ;
	  case 'd':
	    {
		short card16 ;
		int   card32 ;
		
		if ( result->size == sizeof (CARD16) ) {
		    card16 = *((short *) result->data) ;
		    printf (fmt, card16) ;
		    fputc ('\n', stdout) ;
		} else if (result->size == sizeof (CARD32) ) {
		    card32 = *((int *) result->data) ;
		    printf (fmt, card32) ;
		    fputc ('\n', stdout) ;
		} else {
		    DumpMemory (stdout, result->size, result->data) ;
		}
	    }
	    break ;
	  default:
	    DumpMemory (stdout, result->size, result->data) ;
	    break ;
	}
    }
    else
    {
	DumpMemory (stdout, result->size, result->data) ;
    }

    KeFree (result) ;
    return (0) ;
}

/***************************************************************************
 * GetInitialize: declare the kget command
 *
 * RETURNS: void
 ***************************************************************************/

void GetInitialize ()

{
    DeclareCommand ("kget"
		    , documentation
		    , ParseGet) ;

    return ;
}

