/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * imisc.c
 *
 * Miscleaneous commands implementation.
 ***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/times.h>

#include "interp.h"

/***************************************************************************
 * ParseLoad: parse a 'load' command
 *
 * Execute line in file
 *
 * RETURN: 0
 ***************************************************************************/

int ParseLoad (argc, argv)
    int  argc ;
    char **argv ;

{
    char buffer[4096] ;
    FILE *fd ;
    int trace = 0 ;
    char *fn ;

    /* Decode args */
    if ( (argc == 2) && (*argv[1] != '-') ) {
	fn = argv[1] ;
    } else if ( (argc == 3) && ( ! strcmp (argv[1], "-trace")) ) {
	trace = 1 ;
	fn = argv[2] ;
    } else {
	printf ("load [-trace] <filename>\n") ;
	return (1) ;
    }
    /* Do it */
    if ( (fd = fopen (fn, "r")) == NULL )
    {
	printf ("\tcouldn't open file %s\n", fn) ;
	return (0) ;
    }
    else
    {
	while ( fgets (buffer, 4096, fd) )
	{
	    /*
	     * Trace if needed and ParseAndExec :
	     */
	    if ( trace )
		printf (">>> %s", buffer) ;
	    ParseAndExec (buffer) ;
	}
	fclose (fd) ;
	return (0) ;
    }
}
	       
/***************************************************************************
 * ParseShell: parse the "sh" command
 *
 * RETURNS: status of launched command
 ***************************************************************************/

int ParseShell (argc, argv)
    int argc ;
    char **argv ;

{
    char *p ;

    if ( argc == 1 )
    {
	printf ("sh <command line>.\n") ;
	return (1) ;
    }

/*
 * As *I* now what happend behind, I can do this hack :
 */
    for ( argc -= 2 , argv++ , p = *argv ; argc ; p++ )
    {
	if ( ! *p )
	{
	    argc-- ;
	    *p = ' ' ;
	}
    }

    printf ("\treturned status: %d\n", system (*argv)) ;
    return (0) ;
}

/***************************************************************************
 * ParseExit: parse the exit command
 *
 * RETURNS: !!
 ***************************************************************************/

int ParseExit (int argc, unsigned char **argv) 

{
    printf ("\t1003.1003: User exited normally.\n") ;
    exit (1) ;
}

/***************************************************************************
 * ParseRepeat: parse the 'repeat' command
 *
 * RETURNS: repeated command error code
 ***************************************************************************/

int ParseRepeat (argc, argv)
    int  argc ;
    char **argv ;

{
    int count, error ;

    /* Get the repeat count, and reparse end of command */
    if ( argc <= 2 )
    {
	printf ("\trepeat <count> <command>.\n") ;
	return (1) ;
    }
    for ( count = atoi (argv[1]) ; count > 0 ; count-- )
    {
	printf ( "Remaining iterations (;-) %d\n", count) ;
	if ( (error = DispatchCommand (argc - 2, argv + 2)) < 0 )
	    break ;
    }

    return ( error ) ;
}

/***************************************************************************
 * ParseTime: parse the 'times' command
 *
 * RETUNS: subcommanbd error
 ***************************************************************************/

int ParseTimes (argc, argv)
    int  argc ;
    char **argv ;

{
    struct tms start, stop ;
    int        error ;

    times (&start) ;
    error = DispatchCommand (argc - 1, argv + 1) ;
    times (&stop) ;

    fprintf (stderr, "\tuser delta: %d\n", stop.tms_utime - start.tms_utime) ;
    fprintf (stderr, "\tsys  delta: %d\n", stop.tms_stime - start.tms_stime) ;
    return (error) ;
}


