/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * inotify.c
 *
 * Parse and execute the knotiofy command
 ***************************************************************************/

#include <stdio.h>

#include "interp.h"
#include "kediti.h"

static char *documentation =
"knotify {-oid <node>,<name>} <mask>\n\
This command performs a Notify request on the provided node wich can be\n\
either a node identifier or a symbolic node name (if the \"mkdb\" command\n\
has been run.\n\
The <mask> is a space separated list of one of the followings:\n\
read\tget event when someone performs a Get request\n\
gread\tget a valuated event when someone perform a Set request\n\
write\tget an event when someone performes a Set request\n\
gwrite\tget an valuated event when someone performs a Set request\n\
exec\tget an event when someone performs a Exec request\n\
create\tget an event when a new node is created\n\
delete\tget an event when a node is deleted\n\
Valuated events are events containing a node value (on read, this value\n\
is the value to the client that has performed the Get request, on write\n\
it is the new node value).\n" ;

/***************************************************************************
 * ParseNotify: parse and execute the notify command
 *
 * RETURNS: 0
 ***************************************************************************/

static int ParseNotify (argc, argv)
    int  argc ;
    char **argv ;

{
    KeStatus      status ;
    KID           oid ;
    int           count ;
    unsigned long mask = 0 ;

/*
 * Skip command name and get oid :
 */
    argc-- ;
    argv++ ;
    if ( (count = ParseId (argc, argv, &oid)) <= 0 )
	return (1) ;
/*
 * Decode rest of line :
 */
    argc -= count ;
    argv += count ;

    while ( argc )
    {
	if ( ! strcmp (*argv, "notify") ) {
	    mask |= KEDIT_NotifyMask ;
	} else if ( ! strcmp (*argv, "read") ) {
	    mask |= KEDIT_ReadMask ;
	} else if ( ! strcmp (*argv, "gread") ) {
	    mask |= KEDIT_ReadWithValueMask ;
	} else if ( ! strcmp (*argv, "write") ) {
	    mask |= KEDIT_WriteMask ;
	} else if ( ! strcmp (*argv, "gwrite") ) {
	    mask |= KEDIT_WriteWithValueMask ;
	} else if ( ! strcmp (*argv, "exec") ) {
	    mask |= KEDIT_ExecMask ;
	} else if ( ! strcmp (*argv, "create") ) {
	    mask |= KEDIT_CreateMask ;
	} else if ( ! strcmp (*argv, "delete") ) {
	    mask |= KEDIT_DeleteMask ;
	}
	else
	{
	    return (-1) ;
	}
	argv++ ;
	argc-- ;
    }
	

/*
 * Perform :
 */
	
    if ( status = KecNotify (KecConn, oid, mask) )
    {
	/*
	 * Request couldn't be sent properly (this is not the same thing as the
	 * server sending back an error).
	 */
	printf ("\tNotify request failed.\n") ;
    }
    else if ( TraceFlag )
	printf ("\tNotify request sent.\n") ;

    return (0) ;
}

/***************************************************************************
 * NotifyInitialize: declare the knotify command
 *
 * RETURNS: void
 ***************************************************************************/

void NotifyInitialize ()

{
    DeclareCommand ("knotify"
		    , documentation
		    , ParseNotify) ;
    return ;
}
