/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iset.c
 *
 * Parse and execute the Set command
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>

#include "interp.h"
#include "kediti.h"

static char *documentation = 
"kset {-oid <node>,<name>} <value>\n\
This command perform a Set request on the given node, wich can be provided\n\
either through a node identifier, or through its symbolic name (if \"mkdb\" \n\
command has been run).\n\
The <value> format can be:\n\
-bytes <xx>*: value will be consituted of the list of hexa bytes.\n\
-card32 <v> : value is a 32 bits unsigned integer\n\
-card16 <v> : value is a 16 bits unsigned integer\n\
-none       : no value at all\n\
otherwise, the value is considered to be a C ASCII string.\n" ;

/***************************************************************************
 * ParseSet: parse and exec the set command.
 *
 * RETURNS: 0
 ***************************************************************************/

static int ParseSet (argc, argv)
    int  argc ;
    char **argv ;

{
    unsigned char bytes[256] ;
    KID		  oid ;
    KeStatus 	  status ;
    int           count ;
    int           card32 ;
    short         card16 ;
    KePointer     value = (KePointer) 0 ;
    int           size  = 0 ;

/*
 * Skip the command name and get object id (mandatory)
 */

    argc-- ;
    argv++ ;
    if ( (count = ParseId (argc, argv, &oid)) <= 0 )
	return (1) ;

/*
 * Decode rest of args :
 */

    argv += count ;
    argc -= count ;
    if ( ! strcmp (*argv, "-bytes") ) {
	/* Value is given as a list of hexa bytes */
	unsigned char *p = bytes ;
	char          *v ;

	for ( --argc, ++argv ; argc ; argc--, argv++ ) {
	    *p++ = (unsigned char) atoi (*argv) ;
	}
	value = (KePointer) bytes ;
	size  = p - bytes ;
    } else if ( ! strcmp (*argv, "-card32") ) {
	/* Value is a card32 */
	card32 = atoi (*++argv) ;
	value  = (KePointer) &card32 ;
	size   = sizeof (CARD32) ;
    } else if ( ! strcmp (*argv, "-card16") ) {
	card16 = (short) atoi (*++argv) ;
	value = (KePointer) &card16 ;
	size  = sizeof (CARD16) ;
    } else if ( ! strcmp (*argv, "-none") ) {
	value = (KePointer) 0 ;
	size  = 0 ;
    } else if ( argc ) {
	/* Default to a string */
	value = (KePointer) *argv ;
	size  = strlen (*argv) + 1 ;
    }
    else
    {
	printf ("\tUnable to make a value from arg\n") ;
	return (1) ;
    }

/*
 * Perform the request, 'value' is a byte array of size 'size' wich is the
 * value we want to set the node to.
 * Request in synchronous mode.
 */

    if ( status = KecSet (KecConn, oid, size, value) )
    {
	/*
	 * An error occured occured while *sending* the request, trace
	 * this (this is different from the server sending back an 
	 * error, wich in any case will be detected after.
	 */

	printf ("\tset failed (status = %d).\n", status) ;
    }
    else if ( TraceFlag )
    {
	printf ("\tset succeeded.\n") ;
    }

    return (0) ;
}


/***************************************************************************
 * SetInitialize: initialize the command
 *
 * RETURNS: void
 ***************************************************************************/

void SetInitialize ()

{
    DeclareCommand ("kset"
		    , documentation
		    , ParseSet) ;
    return ;
}
