/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * istat.c
 *
 * Parse the stat command
 ***************************************************************************/

#include <stdio.h>

#include "interp.h"
#include "kediti.h"

static char *documentation =
"kstat {-oid <node>,<name>}\n\
This command performs a Stat request on the provided node. The node can\n\
be provided either through its identifier, or through its name (if \"mkdb\"\n\
has been run.\n" ;

/***************************************************************************
 * ParseStat: parse the stat command
 *
 * RETURNS: 0
 ***************************************************************************/

static int ParseStat (argc, argv)
    int argc ;
    char **argv ;

{
    KID            oid ;
    int            count ;
    KeNodeStat     result ;
    KeStatus       status ;

/*
 * Retreive target node identifier:
 */

    argc-- ; 
    argv++ ;
    if ( (count = ParseId (argc, argv, &oid)) <= 0 )
	return (1) ;

/*
 * Perform the Stat request in synchronous mode
 */
    if ( status = KecStatSync (KecConn, oid, &result) )
    {
	KecError error = KecErrorOfConnection (KecConn) ;

	/*
	 * An error occured, retreive it from the connection, and display
	 * the optional error message from server.
	 */

	printf ("\tkls failed (status = %d)\n", status) ;
	if ( error->msg )
	    printf ("\t%s\n", error->msg) ;
    }
    else
    {
	DBEntry entry = GetNodeEntry (result->info) ;

	/*
	 * Got it !
	 * Display all infos in human-readable form, and free the provided 
	 * result.
	 */
	 
	fprintf (stdout, "\tid  : %10d (x%08x)\n", result->id, result->id) ;
	fprintf (stdout, "\tinfo: %10d (x%08x) <%s>\n"
		 , result->info
		 , result->info
		 , entry ? entry->name : "not found in cache") ;
	fprintf (stdout, "\tname: %s\n", result->name) ;
	fprintf (stdout, "\tmode: %s\n", PrintMode (result->mode)) ;
	fprintf (stdout, "\tmask: %s\n", PrintMask (result->mask)) ;
	fprintf (stdout, "\tsize: %d\n", result->size) ;
	KeFree (result) ;
    }

    return (0) ;
}

/***************************************************************************
 * StatInitialize: declare the stat command
 *
 * RETURNS: void
 ***************************************************************************/

void StatInitialize ()

{
    DeclareCommand ("kstat"
		    , documentation
		    , ParseStat) ;
    return ;
}
