/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * kediti.c
 *
 * A simple interpretor wich can send any kedit request to any kedit 
 * server.
 * This is a quick implementation.
 ***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>

#include <unistd.h>

extern int errno ;

#include "interp.h"
#include "kediti.h"

#include <KeClient.h>

KecConnection KecConn = (KecConnection) 0 ;	/* target server connection */

int BatchMode = 0 ;				/* defaults to interactive mode */

/***************************************************************************
 * MainLoop:
 *
 * Select on kediti input and stdin, perform action accordingly.
 ***************************************************************************/

MainLoop (KecConnection kc) 

{
    fd_set read_fds ;
    int    found ;
    Bool   reply_ready ;
    
    if ( ! BatchMode )
    {
	printf ("kedit> ") ; 
	fflush (stdout) ;
    }

    while (1) 
    {
	FD_ZERO ( &read_fds ) ;
	FD_SET (0, &read_fds) ;
	FD_SET (IceConnectionNumber(kc->ice_connection), &read_fds) ;
	
	/*
	 * Select :
	 */
	if ( (found = select (getdtablesize(), &read_fds, NULL, NULL, NULL)) < 0 )
	{
	    fprintf (stderr, ">>> MainLoop: error in select call (errno = %d)\n", errno) ;
	    exit (1) ;
	}
	/*
	 * Read pending data 
	 */
	if ( FD_ISSET (0, &read_fds) )
	{
	    char buffer[4096] ;
	
	    gets (buffer) ;
	    ParseAndExec (buffer) ;
	    if ( ! BatchMode )
	    {
		printf ("kedit> ") ;
		fflush (stdout) ;
	    }
	}
	if ( FD_ISSET (IceConnectionNumber(kc->ice_connection), &read_fds) )
	{
	    IceProcessMessages (kc->ice_connection, NULL, &reply_ready) ;
	}
    }

/* NOT REACHED */

}

 
/***************************************************************************
 * KeditiNotificationHandler: name tells it
 *
 ***************************************************************************/

void KeditiNotificationHandler (KecConnection kc, KecEvent event)

{

    PrintEvent (stdout, event) ;
    fflush (stdout) ;

    KecEventFree (event) ;

    return ;
}
   
/***************************************************************************
 * Initialize: set a connection for the parse package
 *
 * RETURN: a status
 ***************************************************************************/

int Initialize (name)
    char *name ;

{
/*
 * Initialize the intrep stuff:
 */
    
    InterpInitialize () ;

    LsInitialize () ;
    StatInitialize () ;
    GetInitialize () ;
    SetInitialize () ;
    ExecInitialize () ;
    NotifyInitialize () ;
    SyncInitialize () ;
    DBInitialize () ;

/*
 * Open the connection:
 */

    if ( ! (KecConn = KecOpenConnection (name) ) )
    {
	fprintf (stderr, "Connection failed.\n") ;
	return (1) ;
    }

    return (0) ;
}

/***************************************************************************
 * main
 ***************************************************************************/

main (argc, argv)
    int argc ;
    char **argv ;

{
    char buffer[4096] ;

/*
 * Parse command line:
 */

    --argc ; ++argv ;

    while ( argc && *argv )
    {
	if ( ! strcmp (*argv, "-batch") )
	{
	    BatchMode = 1 ;
	    argv++ ; argc-- ;
	}
	else
	{
	    break ;
	}
    }
    
    if ( ! argc )
    {
	fprintf (stderr, "Usage: kediti [-batch] <application>.\n") ;
	exit (1) ;
    }
    
/*
 * Initialize:
 */

    if ( Initialize (*argv) )
	exit (1) ;

/* 
 * KecConn holds our connection ;
 */

    KecSetEventHandler (KecConn, KeditiNotificationHandler) ;
     
    MainLoop (KecConn) ;

}
	
    
