/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * kprint.c
 *
 * Various printing routines for KEDIT objects
 ***************************************************************************/

#include <stdio.h>

#include <KeClient.h>

/***************************************************************************
 * DumpMemory: dump mem in hexa
 * PrintMode : returns a human readable string for given mode
 * PrintMask : returns a human readable string for given mask
 ***************************************************************************/

void DumpMemory (fd, size, data)
    FILE  *fd ;
    int   size ;
    char  *data ;

{
    if ( ! fd )
	fd = stderr ;

    while ( size-- )
    {
	fprintf (fd, "%02x ", *data++ & 0xFF) ;
	if ( ! (size % 16) )
	    fputc ('\n', fd) ;
    }
    fprintf (fd, "\n") ;
    fflush (fd) ;
    return ;
}

char *PrintMode (mode)
    unsigned long mode ;

{
    static char retval[5] = "DRWX" ;

    retval[0] = (mode & KEDIT_DirectoryMode) ? 'D' : 'd' ;
    retval[1] = (mode & KEDIT_ReadMode)      ? 'R' : 'r' ;
    retval[2] = (mode & KEDIT_WriteMode)     ? 'W' : 'w' ;
    retval[3] = (mode & KEDIT_ExecMode)      ? 'X' : 'x' ;

    return (retval) ;
}

char *PrintMask (mask)
    unsigned long mask ;

{
    static char retval[] = "CDNRGWG" ;

    retval[0] = (mask & KEDIT_DeleteMask)         ? 'D' : 'd' ;
    retval[1] = (mask & KEDIT_CreateMask)         ? 'C' : 'c' ;
    retval[2] = (mask & KEDIT_NotifyMask)         ? 'N' : 'n' ;
    retval[3] = (mask & KEDIT_ReadMask)           ? 'R' : 'r' ;
    retval[4] = (mask & KEDIT_ReadWithValueMask)  ? 'G' : 'g' ;
    retval[5] = (mask & KEDIT_WriteMask)          ? 'W' : 'w' ;
    retval[6] = (mask & KEDIT_WriteWithValueMask) ? 'G' : 'g' ;

    return (retval) ;
}

/***************************************************************************
 * PrintEvent: print a readable event
 *
 * RETURNS: void
 ***************************************************************************/

void PrintEvent (fd, event)
    FILE     *fd ;
    KecEvent event ;

{
    
    switch ( event->any.event_type )
    {
      case KEDIT_DeleteEvent:
	{
	    KecDeleteEvent delete = &(event->delete_event) ;

	    fprintf (fd, "[%4d] deleted node %d by %d\n"
		     , delete->timestamp
		     , delete->node
		     , delete->client_tag) ;
	}
	break ;
      case KEDIT_CreateEvent:
	{
	    KecCreateEvent create = &(event->create_event) ;

	    fprintf (fd, "[%4d] created child %d of %d by %d\n"
		     , create->timestamp
		     , create->child
		     , create->node
		     , create->client_tag) ;
	}
	break ;
      case KEDIT_NotifyEvent:
	{
	    KecNotifyEvent notify = &(event->notify_event) ;

	    fprintf (fd, "[%4d] new mask %s on %d by %d\n"
		     , notify->timestamp
		     , PrintMask (notify->mask) 
		     , notify->node
		     , notify->client_tag) ;
	}
	break ;
      case KEDIT_ReadEvent:
	{
	    KecReadEvent read = &(event->read_event) ;

	    fprintf (fd, "[%4d] node %d read by %d\n"
		     , read->timestamp
		     , read->node
		     , read->client_tag) ;
	}
	break ;
      case KEDIT_ReadWithValueEvent:
	{
	    KecReadWithValueEvent read_with_value_event = &(event->read_with_value_event) ;
	    
	    fprintf (fd, "[%4d] node %d read by %d\n"
		     , read_with_value_event->timestamp
		     , read_with_value_event->node
		     , read_with_value_event->client_tag) ;

	    DumpMemory (fd
			, read_with_value_event->size
			, read_with_value_event->data) ;
	}
	break ;
      case KEDIT_WriteEvent:
	{
	    KecWriteEvent write = &(event->write_event) ;

	    fprintf (fd, "[%4d] node %d written by %d\n"
		     , write->timestamp
		     , write->node
		     , write->client_tag) ;
	}
	break ;
      case KEDIT_WriteWithValueEvent:
	{
	    KecWriteWithValueEvent write_with_value_event = &(event->write_with_value_event) ;

	    fprintf (fd, "[%4d] node %d written by %d\n"
		     , write_with_value_event->timestamp
		     , write_with_value_event->node
		     , write_with_value_event->client_tag) ;

	    DumpMemory (fd
			, write_with_value_event->size
			, write_with_value_event->data) ;
	}
	break ;
      default:
	fprintf (fd, "PrintEvent: unknown event type %d\n"
		 , event->any.event_type) ;
	break ;
    }

    return ;
}
