/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * kelsr.c
 *
 * A simple example program: list recursevely all available nodes in 
 * the arg-provided server.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <KeClient.h>

char *ServerName = (char *) 0 ;

/***************************************************************************
 * StatPrint: display the given stat struct at indent level
 ***************************************************************************/

void StatPrint (fd, stat, level)
    FILE       *fd ;
    KeNodeStat stat ;
    int        level ;

{
    char format[128] ;
    char mask[] = "CDNRGWG" ;
    char mode[] = "DRWX" ;


    mask[0] = (stat->mask & KEDIT_DeleteMask)         ? 'D' : 'd' ;
    mask[1] = (stat->mask & KEDIT_CreateMask)         ? 'C' : 'c' ;
    mask[2] = (stat->mask & KEDIT_NotifyMask)         ? 'N' : 'n' ;
    mask[3] = (stat->mask & KEDIT_ReadMask)           ? 'R' : 'r' ;
    mask[4] = (stat->mask & KEDIT_ReadWithValueMask)  ? 'G' : 'g' ;
    mask[5] = (stat->mask & KEDIT_WriteMask)          ? 'W' : 'w' ;
    mask[6] = (stat->mask & KEDIT_WriteWithValueMask) ? 'G' : 'g' ;


    mode[0] = (stat->mode & KEDIT_DirectoryMode) ? 'D' : 'd' ;
    mode[1] = (stat->mode & KEDIT_ReadMode)      ? 'R' : 'r' ;
    mode[2] = (stat->mode & KEDIT_WriteMode)     ? 'W' : 'w' ;
    mode[3] = (stat->mode & KEDIT_ExecMode)      ? 'X' : 'x' ;

    sprintf (format, "%%-%d.%ds x%%08x %%s %%s %%d\n"
	     , 32 - level, 32 - level) ;

    while ( level-- )
	fputc (' ', fd) ;

    fprintf (fd, format
	     , stat->name
	     , stat->id
	     , mode
	     , mask
	     , stat->size) ;

    return ;
}
    
/***************************************************************************
 * DepthFirstDump: dump in depth-first the remote hierarchy
 *
 * This demonstrates the use of the KecLsl () function, wich returns an
 * array of stat (one per children node of the querried node). For each
 * node found by KecLsl, print it and if it is a directory call ourselve
 * recursevily. 
 * This algo is very time consuming, because it implies one round trip
 * with the server for each directory. For a *much* more efficient dump 
 * of a server, see the file in kediti "mkdb.c".
 ***************************************************************************/

int DepthFirstDump (fd, connection, kid, level)
    FILE          *fd ;
    KecConnection connection ;
    KID           kid ;
    int           level ;

{
    KeNodeStat *stats, *pstat ;
    int        count  = KecLsl (connection, kid, &stats) ;
    
    for ( pstat = stats ; count ; pstat++, count-- )
    {
	StatPrint (fd, *pstat, level) ;
	if ( (*pstat)->mode & KEDIT_DirectoryMode )
	    DepthFirstDump (fd, connection, (*pstat)->id, level+1) ;
    }
    KecLslFree (stats) ;

    return (count) ;
}

/***************************************************************************
 * usage:
 ***************************************************************************/

void usage ()

{
    fprintf (stderr, "kelsr <server-coordinates>\n") ;
    return ;
}

/***************************************************************************
 * main: decode cmd line, and perform
 ***************************************************************************/

int main (argc, argv)
    int  argc ;
    char **argv ;

{
    KecConnection connection ;

/*
 * Get command line options:
 */
    
    argc-- ;
    argv++ ;

    if ( argc == 1 )
    {
	    ServerName = *argv ;
    } else {
	usage () ;
	exit (1) ;
    }

/*
 * Connect to server, and dump its node hierarchy:
 */

    if ( ! (connection = KecOpenConnection (ServerName)) )
    {
	fprintf (stderr, "kedir: unable to connect to <%s>\n", ServerName) ;
	exit (1) ;
    }

/*
 * Depth first walk through the nodes:
 */

    DepthFirstDump (stdout, connection, (KID) 0, 0) ;

    exit (0) ;
}

	
