/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * font.c
 *
 * Specific editor for fonts.
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <setjmp.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>	

#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>

#include <KeServer.h>
#include <KeClient.h>
#include <KeCodec.h>
#include <KesXt.h>

/*
 * Context: all usefull info for callbacks packed in a struct :
 */

typedef struct _Context {
    XtAppContext  app_con ;
    Display       *dpy ;
    KecConnection kec_conn ;
    KID		  font_node ;
    Font	  font ;
    Atom	  font_atom ;
    Widget        toplevel ;
    Widget        wtext ;
} ContextRec, *Context ;

ContextRec TheContext ;

/*
 * Fallback resources:
 */

static String fallback_resources[] = { 
    NULL ,
} ;

static void _FontSetActionProc (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget
    , XEvent *event
    , String *params
    , Cardinal *num_params
#endif
) ;


static char *TextTranslation = "<Key>Return: text-set()" ;

static XtActionsRec TextActions[] = {
    {
	"text-set",
	(XtActionProc) _FontSetActionProc
    }
} ;

static void _DoSetFont (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget    widget 
    , XtPointer ctxt_ptr 
    , XtPointer unused 
#endif
) ;



/***************************************************************************
 * Usage: print it and exit
 ***************************************************************************/

static void usage ()

{
    printf ("usage: pixel -display <> -font <> <server>\n") ;
    exit (1) ;
}

/***************************************************************************
 * KeditError: display a k-Edit error
 *
 * RETURNS: the given status.
 ***************************************************************************/

static KeStatus KeditError (msg, status)
    char     *msg ;
    KeStatus status ;

{
    fprintf (stderr, "%s: %d\n", msg, status) ;
    return (status) ;
}

static void SetUserMessage (ctxt, msg)
    Context ctxt ;
    char    *msg ;

{
    fprintf (stderr, "%s\n", msg) ;
    return ;
}

/***************************************************************************
 * GetFont: retreiev the font id, from the k-Edit server
 *
 * By the way check that types are correct.
 *
 * RETURN: KeStatus, and context updated
 ***************************************************************************/

KeStatus GetFont (ctxt)
    Context  ctxt ;

{
    KecValue   	   value ;
    KeStatus   	   status ;
    KeCodec    	   card32_codec = KeCodecGet ("CARD32") ;
    KeNodeStat 	   node = (KeNodeStat) 0 ;
    KeNodeStat 	   type = (KeNodeStat) 0 ;
    KePointer  	   to   = (KePointer) 0 ;
    char       	   *emsg ;
    KeErrorFreeFct efree ;
/*
 * According to x protocol, Colormap and Pixel are CARD32, check it:
 */

    if ( ( status = KecStatSync (ctxt->kec_conn, ctxt->font_node, &node) )
	|| ( status = KecStatSync (ctxt->kec_conn, node->info, &type) )
	|| ( strcmp (type->name, "CARD32") ) )
    {
	if ( node ) KeFree (node) ;
	if ( type ) KeFree (type) ;
	KeditError ("Checking font node: ", status) ;
	return (status) ;
    }

    KeFree (node) ; node = (KeNodeStat) 0 ;
    KeFree (type) ; type = (KeNodeStat) 0 ;

/*
 * Get values, and decode them to the right place:
 */

    value = (KecValue) 0 ;
    to    = (KePointer) &(ctxt->font) ;
    if ( ( status = KecGetSync (ctxt->kec_conn, ctxt->font_node, &value))
	|| ( status = KeCodecDecode (card32_codec
				     , value->size
				     , (KePointer) value->data
				     , KecConnectionSwapped (ctxt->kec_conn)
				     , (KePointer *) &to
				     , &emsg, &efree)) )
	
    {
	if ( value) KeFree (value) ;
	KeditError (emsg, status) ;
	if ( efree )
	    (*efree) (emsg) ;
	return (status) ;
    }

    KeFree (value) ;
    
    return (KEDIT_Success) ;
}

/***************************************************************************
 * SetFont: set font values according to provided args
 *
 * RETURNS: KeStatus
 ***************************************************************************/

KeStatus SetFont (ctxt, font)
    Context  ctxt ;
    Font     font ;

{
    KeStatus  	   status ;
    KePointer 	   pointer ;
    int       	   size ;
    KeCodec   	   card32_codec = KeCodecGet ("CARD32") ;
    char      	   *emsg ;
    KeErrorFreeFct efree ;

    if ( status = KeCodecEncode (card32_codec
				 , (KePointer) &font
				 , &size
				 , &pointer
				 , &emsg, &efree) )
    {
	KeditError (emsg, status) ;
	if ( efree )
	    (*efree) (emsg) ;
	return ;
    }
    
    if ( status = KecSet (ctxt->kec_conn
			  , ctxt->font_node
			  , size
			  , pointer) )
    {
	KeditError ("SetValue failed: ", status) ;
	return ;
    }

    KecConnectionDoSync (ctxt->kec_conn) ;
    
    return ;
}
    
/***************************************************************************
 * EditFont: a simple (very simple) color editor
 *
 * There is no point here in having fancy stuff, I'm just looking at a 
 * proof of concept
 *
 * RETURNS: never (enters Xt main loop)
 ***************************************************************************/

static void _DoExit (widget, unused1, unused2)
    Widget    widget ;
    XtPointer unused1 ;
    XtPointer unused2 ;

{
    exit (0) ;
}

static void _FontSetActionProc (widget, event, params, num_params)
    Widget widget ;
    XEvent *event ;
    String *params ;
    Cardinal *num_params ;

{
    _DoSetFont (widget, (XtPointer) &TheContext, (XtPointer) 0) ;
    return ;
}


static void _DoSetFont (widget, ctxt_ptr, unused)
    Widget    widget ;
    XtPointer ctxt_ptr ;
    XtPointer unused ;

{
    Context ctxt = (Context) ctxt_ptr ;
    char    *strvalue, *p ;
    Font    font ;

    XtVaGetValues (ctxt->wtext, XtNstring, &strvalue, NULL) ;

/*
 * Convert font name to font id, sync with X server *before* asking target
 * server to use the newly created font-id.
 */

    font = XLoadFont (ctxt->dpy, strvalue) ;
    
    XSync (ctxt->dpy, False) ;

    SetFont (ctxt, font) ;

    return ;
}

static void _DoGetFont (widget, ctxt_ptr, unused)
    Widget    widget ;
    XtPointer ctxt_ptr ;
    XtPointer unused ;

{
    static char str[256] ;
    Context     ctxt = (Context) ctxt_ptr ;
    XFontStruct *font_struct ;
    KeStatus    status ;
    int         i ;

    str[0] = '\0' ;

    if ( status = GetFont (ctxt) )
	return ;

/*
 * Convert font id to string, using the FONT atom value:
 */

    if ( ctxt->font_atom == None )
    {
	ctxt->font_atom = XInternAtom (ctxt->dpy, "FONT", False) ;
	if ( ctxt->font_atom == None )
	{
	    SetUserMessage (ctxt, "Couldn't intern FONT atom.") ;
	    return ;
	}
    }

    if ( ! ( font_struct = XQueryFont (ctxt->dpy, ctxt->font) ) )
    {
	SetUserMessage (ctxt, "Unable to querry on this font.") ;
	return ;
    }

    for ( i = 0 ; i < font_struct->n_properties ; i++ )
    {
	if ( ( font_struct->properties[i].name == ctxt->font_atom )
	    && ( font_struct->properties[i].card32 != (Atom) -1 ) )
	{
	    char *value = XGetAtomName (ctxt->dpy
					, font_struct->properties[i].card32);

	    strcpy (str, value) ;
	    break ;
	}
    }
    
    XFreeFont (ctxt->dpy, font_struct) ;

    if ( str[0] )
	XtVaSetValues (ctxt->wtext, XtNstring, str, NULL) ;
    else
	SetUserMessage (ctxt, "Unable to find font FONT atom.") ;

    return ;
}

void EditFont (ctxt)
    Context ctxt ;

{
    Widget       hpane, box, entry ;

/*
 * Initialize Xt, create widget hierarchy:
 */

    hpane = XtCreateManagedWidget ("hpane"
				   , panedWidgetClass
				   , ctxt->toplevel
				   , NULL, ZERO) ;

    box   = XtCreateManagedWidget ("box"
				   , boxWidgetClass
				   , hpane
				   , NULL, ZERO) ;

    entry = XtCreateManagedWidget ("set"
				   , commandWidgetClass
				   , box
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoSetFont, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("get"
				   , commandWidgetClass
				   , box
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoGetFont, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("exit"
				   , commandWidgetClass
				   , box
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoExit, (XtPointer) ctxt) ;
    
    ctxt->wtext = XtVaCreateManagedWidget ("text"
					   , asciiTextWidgetClass
					   , hpane
					   , XtNeditType, XawtextEdit
					   , NULL) ;

    XtOverrideTranslations (ctxt->wtext
			    , XtParseTranslationTable (TextTranslation));

    _DoGetFont (ctxt->wtext, (XtPointer) ctxt, (XtPointer) 0) ;

    XtRealizeWidget (ctxt->toplevel) ;
    
    return ;
}

/***************************************************************************
 * main: 
 ***************************************************************************/

main (argc, argv)
    int  argc ;
    char **argv ;

{
    Context       ctxt = &TheContext ;
    KID           font_node ;
    char          *server_name  = (char *) 0 ;
    char          *display_name = (char *) 0 ;
    KesServer     server ;
    jmp_buf	  ice_error_buf ;

    ctxt->kec_conn  = (KecConnection) 0 ;
    ctxt->dpy       = (Display *) 0 ;
    ctxt->font_node = (KID) -1 ;
    ctxt->font_atom = None ;

/*
 * Get command line options, and check for all parameters:
 */

    argc-- ; argv++ ;
    while ( argc )
    {
	if ( ! strcmp (*argv, "-font") ) {
	    argc-- ; argv++ ;
	    ctxt->font_node = atoi (*argv) ;
	    argc-- ; argv++ ;
	}
	else if ( ! strcmp (*argv, "-display") ) {
	    argc-- ; argv++ ;
	    display_name = *argv ;
	    argc-- ; argv++ ;
	}
	else if ( ! server_name )
	{
	    server_name = *argv ;
	    argc-- ; argv++ ;
	}
	else
	{
	    usage () ;
	}
    }

    if ( ( ctxt->font_node == (KID) -1 )
	|| ( ! server_name ) )
    {
	usage() ;
    }

/*
 * Connect:
 * We assume here that pixel is connected to the same Xserver than target
 * server. This is by no mean always true (should use dysplay_name if not)
 */

    ctxt->toplevel = XtAppInitialize (&(ctxt->app_con)
				      , "XKFont"
				      , NULL, ZERO
				      , &argc, argv
				      , fallback_resources
				      , NULL, ZERO) ;
    
    XtAppAddActions (ctxt->app_con
		     , TextActions
		     , XtNumber(TextActions)) ;

    ctxt->dpy = XtDisplay (ctxt->toplevel) ;

    if ( ! ( ctxt->kec_conn = KecOpenConnection (server_name)) )
    {
	fprintf (stderr, "Unable to connect to %s\n", server_name) ;
	exit (1) ;
    }

/*
 * Edit this pixel value:
 */

    EditFont (ctxt) ;

    if ( ! ( server = KesServerInitialize ("Pixel") ) )
    {
	fprintf (stderr, "Pixel: unable to KesServerInitialize.\n") ;
	exit (-1) ;
    }

    if ( ! setjmp (ice_error_buf) )
	KesXtServerPlug (ctxt->app_con, server, ctxt->toplevel, ice_error_buf);

    XtAppMainLoop (ctxt->app_con) ;

    exit (0) ;
}
