/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * pixel.c
 *
 * Specific editor for pixels.
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <setjmp.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>	

#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>

#include <KeServer.h>
#include <KeClient.h>
#include <KeCodec.h>
#include <KesXt.h>

/*
 * Context: all usefull info for callbacks packed in a struct :
 */

typedef struct _Context {
    XtAppContext  app_con ;
    Display       *dpy ;
    KecConnection kec_conn ;
    KID           pixel_node ;
    KID           colormap_node ;
    Colormap      colormap ;
    Pixel         pixel ;
    Widget        toplevel ;
    Widget        wtext ;
} ContextRec, *Context ;

ContextRec TheContext ;

/*
 * Fallback resources:
 */

static String fallback_resources[] = { 
    NULL ,
} ;

/*
 * Action for setting value on return:
 */

static void _PixelSetActionProc (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget
    , XEvent *event
    , String *params
    , Cardinal *num_params
#endif
) ;


static char *TextTranslation = "<Key>Return: text-set()" ;

static XtActionsRec TextActions[] = {
    {
	"text-set",
	(XtActionProc) _PixelSetActionProc
    }
} ;

static void _DoSetColor (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget    widget 
    , XtPointer ctxt_ptr 
    , XtPointer unused 
#endif
) ;


/*
 * Macro to parse text input:
 */

#define IS_HEXA(c) \
    ( (((c) >= '0') && ((c) <='9')) \
     || (((c) >= 'A') && ((c) <= 'F')) \
     || (((c) >= 'a') && ((c) <= 'f')) )

#define HEXA_VAL(c) \
  ( (((c) >= '0') && ((c) <= '9')) \
   ? ((c) - '0') \
   : 10 + (((c) <= 'F') ? ((c) - 'A') : ((c) - 'a')) )

#define CHECK_BYTE(p) \
    ( (! IS_HEXA (*(p))) || ( ! IS_HEXA (*((p)+1))) )

#define GET_BYTE(p) \
    ( (HEXA_VAL (*(p)) << 4) & (HEXA_VAL (*((p)+1))) & 0xFF )

/***************************************************************************
 * Usage: print it and exit
 ***************************************************************************/

static void usage ()

{
    printf ("usage: pixel -display <> -colormap <> -pixel <> <server>\n") ;
    exit (1) ;
}

/***************************************************************************
 * KeditError: display a k-Edit error
 *
 * RETURNS: the given status.
 ***************************************************************************/

static KeStatus KeditError (msg, status)
    char     *msg ;
    KeStatus status ;

{
    fprintf (stderr, "%s: %d\n", msg, status) ;
    return (status) ;
}

static void SetUserMessage (ctxt, msg)
    Context ctxt ;
    char    *msg ;

{
    fprintf (stderr, "%s\n", msg) ;
    return ;
}

/***************************************************************************
 * GetPixel: retreive values for colormap and pixel from target server
 *
 * By the way check that types are correct.
 *
 * RETURN: KeStatus, and X colormap and pixel
 ***************************************************************************/

KeStatus GetPixel (ctxt)
    Context  ctxt ;

{
    KecValue   	   value ;
    KeStatus   	   status ;
    KeCodec    	   card32_codec = KeCodecGet ("CARD32") ;
    KeNodeStat 	   node = (KeNodeStat) 0 ;
    KeNodeStat 	   type = (KeNodeStat) 0 ;
    KePointer  	   to   = (KePointer) 0 ;
    char       	   *emsg ;
    KeErrorFreeFct efree ;
/*
 * According to x protocol, Colormap and Pixel are CARD32, check it:
 */

    if ( ( status = KecStatSync (ctxt->kec_conn, ctxt->colormap_node, &node) )
	|| ( status = KecStatSync (ctxt->kec_conn, node->info, &type) )
	|| ( strcmp (type->name, "CARD32") ) )
    {
	if ( node ) KeFree (node) ;
	if ( type ) KeFree (type) ;
	KeditError ("Checking colormap node: ", status) ;
	return (status) ;
    }

    KeFree (node) ; node = (KeNodeStat) 0 ;
    KeFree (type) ; type = (KeNodeStat) 0 ;

    if ( ( status = KecStatSync (ctxt->kec_conn, ctxt->pixel_node, &node) )
	|| ( status = KecStatSync (ctxt->kec_conn, node->info, &type) )
	|| ( strcmp (type->name, "CARD32") ) )
    {
	if ( node ) KeFree (node) ;
	if ( type ) KeFree (type) ;
	KeditError ("Checking colormap node: ", status) ;
	return (status) ;
    }

    KeFree (node) ;
    KeFree (type) ;

/*
 * Get values, and decode them to the right place:
 */

    value = (KecValue) 0 ;
    to    = (KePointer) &(ctxt->colormap) ;
    if ( ( status = KecGetSync (ctxt->kec_conn, ctxt->colormap_node, &value))
	|| ( status = KeCodecDecode (card32_codec
				     , value->size
				     , (KePointer) value->data
				     , KecConnectionSwapped (ctxt->kec_conn)
				     , (KePointer *) &to
				     , &emsg, &efree)) )
	
    {
	if ( value) KeFree (value) ;
	KeditError (emsg, status) ;
	if ( efree )
	    (*efree) (emsg) ;
	return (status) ;
    }

    KeFree (value) ;

    value = (KecValue) 0 ;
    to    = (KePointer) &(ctxt->pixel) ;
    if ( ( status = KecGetSync (ctxt->kec_conn, ctxt->pixel_node, &value))
	|| ( status = KeCodecDecode (card32_codec
				     , value->size
				     , (KePointer) value->data
				     , KecConnectionSwapped (ctxt->kec_conn)
				     , (KePointer *) &to
				     , &emsg, &efree)) )
	
    {
	if ( value) KeFree (value) ;
	KeditError (emsg, status) ;
	if ( efree )
	    (*efree) (emsg) ;
	return (status) ;
    }

    KeFree (value) ;
    
    return (KEDIT_Success) ;
}

/***************************************************************************
 * SetPixel: set colormap/pixel values according to provided args
 *
 * RETURNS: KeStatus
 ***************************************************************************/

KeStatus SetPixel (ctxt, colormap, pixel)
    Context  ctxt ;
    Colormap colormap ;
    Pixel    pixel ;

{
    KeStatus  	   status ;
    KePointer 	   pointer ;
    int       	   size ;
    KeCodec   	   card32_codec = KeCodecGet ("CARD32") ;
    char      	   *emsg ;
    KeErrorFreeFct efree ;

    if ( status = KeCodecEncode (card32_codec
				 , (KePointer) &pixel
				 , &size
				 , &pointer
				 , &emsg, &efree) )
    {
	KeditError (emsg, status) ;
	if ( efree )
	    (*efree) (emsg) ;
	return ;
    }
    
    if ( status = KecSet (ctxt->kec_conn
			  , ctxt->pixel_node
			  , size
			  , pointer) )
    {
	KeditError ("SetValue failed: ", status) ;
	return ;
    }

    KecConnectionDoSync (ctxt->kec_conn) ;
    
    return ;
}
    
/***************************************************************************
 * EditColor: a simple (very simple) color editor
 *
 * There is no point here in having fancy stuff, I'm just looking at a 
 * proof of concept
 *
 * RETURNS: never (enters Xt main loop)
 ***************************************************************************/

static void _DoExit (widget, unused1, unused2)
    Widget    widget ;
    XtPointer unused1 ;
    XtPointer unused2 ;

{
    exit (0) ;
}

static void _PixelSetActionProc (widget, event, params, num_params)
    Widget widget ;
    XEvent *event ;
    String *params ;
    Cardinal *num_params ;

{
    _DoSetColor (widget, (XtPointer) &TheContext, (XtPointer) 0) ;
    return ;
}

static void _DoSetColor (widget, ctxt_ptr, unused)
    Widget    widget ;
    XtPointer ctxt_ptr ;
    XtPointer unused ;

{
    Context ctxt = (Context) ctxt_ptr ;
    char    *strvalue, *p ;
    XColor  color ;

/*
 * Get and parse text input (strict rules, once again it is not our pb here)
 */

    XtVaGetValues (ctxt->wtext, XtNstring, &strvalue, NULL) ;

    if ( *strvalue == '#' )
    {
	if ( ( CHECK_BYTE (strvalue + 1) )
	    || ( CHECK_BYTE (strvalue + 3) )
	    || ( CHECK_BYTE (strvalue + 5) ) )
	{
	    SetUserMessage (ctxt, "Color can be \"#rrggbb\" or color name.") ;
	    return ;
	}

	color.red   = ( GET_BYTE (strvalue + 1) << 8) & 0xFF00 ;
	color.green = ( GET_BYTE (strvalue + 3) << 8) & 0xFF00 ;
	color.blue  = ( GET_BYTE (strvalue + 5) << 8) & 0xFF00 ;

	if ( ! XAllocColor (ctxt->dpy, ctxt->colormap, &color) )
	{
	    SetUserMessage (ctxt, "Couldn't allocate color-cell.") ;
	    return ;
	}
    }
    else
    {
	XColor unused ;

	if ( ! XAllocNamedColor (ctxt->dpy
				 , ctxt->colormap
				 , strvalue
				 , &color
				 , &unused) )
	{
	    SetUserMessage (ctxt, "Invalid color name.") ;
	    return ;
	}
    }

    XSync (ctxt->dpy, False) ;

    SetPixel (ctxt, ctxt->colormap, color.pixel) ;
	
    return ;
}

static void _DoGetColor (widget, ctxt_ptr, unused)
    Widget    widget ;
    XtPointer ctxt_ptr ;
    XtPointer unused ;

{
    static char str[8] ;
    Context  ctxt = (Context) ctxt_ptr ;
    XColor   color ;
    KeStatus status ;

    if ( status = GetPixel (ctxt) )
	return ;

    color.pixel = ctxt->pixel ;
    
    XQueryColor (ctxt->dpy, ctxt->colormap, &color) ;

    sprintf (str, "#%02x%02x%02x"
	     , (color.red   >> 8) & 0xFF
	     , (color.green >> 8) & 0xFF
	     , (color.blue  >> 8) & 0xFF) ;

    XtVaSetValues (ctxt->wtext, XtNstring, str, NULL) ;

    return ;
}

void EditPixel (ctxt)
    Context ctxt ;

{
    Widget       hpane, box, entry ;

/*
 * Initialize Xt, create widget hierarchy:
 */

    hpane = XtCreateManagedWidget ("hpane"
				   , panedWidgetClass
				   , ctxt->toplevel
				   , NULL, ZERO) ;

    box   = XtCreateManagedWidget ("box"
				   , boxWidgetClass
				   , hpane
				   , NULL, ZERO) ;

    entry = XtCreateManagedWidget ("set"
				   , commandWidgetClass
				   , box
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoSetColor, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("get"
				   , commandWidgetClass
				   , box
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoGetColor, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("exit"
				   , commandWidgetClass
				   , box
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoExit, (XtPointer) ctxt) ;
    
    ctxt->wtext = XtVaCreateManagedWidget ("text"
					   , asciiTextWidgetClass
					   , hpane
					   , XtNeditType, XawtextEdit
					   , NULL) ;

    XtOverrideTranslations (ctxt->wtext
			    , XtParseTranslationTable (TextTranslation));

    _DoGetColor (ctxt->wtext, (XtPointer) ctxt, (XtPointer) 0) ;

    XtRealizeWidget (ctxt->toplevel) ;

    return ;
}

/***************************************************************************
 * main: 
 ***************************************************************************/

main (argc, argv)
    int  argc ;
    char **argv ;

{
    Context       ctxt = &TheContext ;
    KID           colormap_node = (KID) -1 ;
    KID           pixel_node    = (KID) -1 ;
    char          *server_name  = (char *) 0 ;
    char          *display_name = (char *) 0 ;
    Colormap      colormap ;
    Pixel         pixel ;
    KesServer     server ;
    jmp_buf       ice_error_buf ;

    ctxt->kec_conn      = (KecConnection) 0 ;
    ctxt->dpy           = (Display *) 0 ;
    ctxt->pixel_node    = (KID) -1 ;
    ctxt->colormap_node = (KID) -1 ;

/*
 * Get command line options, and check for all parameters:
 */

    argc-- ; argv++ ;
    while ( argc )
    {
	if ( ! strcmp (*argv, "-colormap") ) {
	    argc-- ; argv++ ;
	    ctxt->colormap_node = atoi (*argv) ;
	    argc-- ; argv++ ;
	} else if ( ! strcmp (*argv, "-pixel") ) {
	    argc-- ; argv++ ;
	    ctxt->pixel_node = atoi (*argv) ;
	    argc-- ; argv++ ;
	}
	else if ( ! strcmp (*argv, "-display") ) {
	    argc-- ; argv++ ;
	    display_name = *argv ;
	    argc-- ; argv++ ;
	}
	else if ( ! server_name )
	{
	    server_name = *argv ;
	    argc-- ; argv++ ;
	}
	else
	{
	    usage () ;
	}
    }

    if ( ( ctxt->colormap_node == (KID) -1 )
	|| ( ctxt->pixel_node == (KID) -1 )
	|| ( ! server_name ) )
    {
	usage() ;
    }

/*
 * Connect:
 * We assume here that pixel is connected to the same Xserver than target
 * server. This is by no mean always true (should use dysplay_name if not)
 */

    ctxt->toplevel = XtAppInitialize (&(ctxt->app_con)
				      , "XKPixel"
				      , NULL, ZERO
				      , &argc, argv
				      , fallback_resources
				      , NULL, ZERO) ;

    XtAppAddActions (ctxt->app_con
		     , TextActions
		     , XtNumber(TextActions)) ;

    ctxt->dpy = XtDisplay (ctxt->toplevel) ;

    if ( ! ( ctxt->kec_conn = KecOpenConnection (server_name)) )
    {
	fprintf (stderr, "Unable to connect to %s\n", server_name) ;
	exit (1) ;
    }

/*
 * Edit this pixel value:
 */

    EditPixel (ctxt) ;

    if ( ! ( server = KesServerInitialize ("Pixel") ) )
    {
	fprintf (stderr, "Pixel: unable to KesServerInitialize.\n") ;
	exit (-1) ;
    }

    if ( ! setjmp (ice_error_buf) )
	KesXtServerPlug (ctxt->app_con, server, ctxt->toplevel, ice_error_buf);

    XtAppMainLoop (ctxt->app_con) ;

    exit (0) ;
}
