/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * kedit-codecs.c
 ***************************************************************************/

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>	

#include "xkedit.h"


/*
 * Local typedef : from node identifier to codec
 */

typedef struct _CodecNode {
    KID		node ;
    KeCodec	codec ;
} CodecNodeRec, *CodecNode ;

/***************************************************************************
 * _codecnode_hash_fct: 
 * _codecnode_cmp_fct :
 *
 * KeHashtable interface
 ***************************************************************************/

static unsigned long _codecnode_hash_fct (codecnode)
    CodecNode codecnode ;

{
    return ((unsigned long) codecnode->node) ;
}

static int _codecnode_cmp_fct (codecnode1, codecnode2)
    CodecNode codecnode1, codecnode2 ;

{
    return ( codecnode1->node != codecnode2->node ) ;
}

/***************************************************************************
 * _UpdateNodeToCode: update KeHashtable by querying server
 ***************************************************************************/

static void _NodeToCodecAdd (hashtable, key)
    KeHashtable hashtable ;
    CodecNode key ;

{
    CodecNode codecnode ;

/*
 * If already defined, update it (should not change, in fact):
 */

    if ( codecnode = (CodecNode) KeHashtableFind (hashtable, key) )
    {
	codecnode->node  = key->node ;
	codecnode->codec = key->codec ;
	return ;
    }

/*
 * Else add it:
 */

    codecnode = (CodecNode) malloc (sizeof (CodecNodeRec)) ;
    
    codecnode->node  = key->node ;
    codecnode->codec = key->codec ;

    KeHashtableAdd (hashtable, codecnode) ;

    return ;
}

static void _UpdateNodeToCodec (ctxt)
    Context ctxt ;

{
    KID		 codec_dir = (KID) -1 ;
    int		 i, count ;
    KeNodeStat	 *stats ;
    CodecNodeRec key ;
/*
 * List root nodes, to get the Codec directory:
 */

    if ( (count = KecLsl (ctxt->kec_conn
			  , KecRootNodeOfConnection (ctxt->kec_conn)
			  , &stats)) < 0 )
    {
	SetUserMessage (ctxt, "KecLsl: failed on root node.") ;
	return ;
    }

    for ( i = 0 ; i < count ; i++ )
    {
	if ( ! strcmp (stats[i]->name, "Codecs") )
	{
	    codec_dir = stats[i]->id ;
	    break ;
	}
    }

    if ( codec_dir == (KID) -1 )
    {
	SetUserMessage (ctxt, "No codec directory on target server.") ;
	return ;
    }

    KecLslFree (stats) ;

/*
 * List the codec directory:
 */

    if ( (count = KecLsl (ctxt->kec_conn, codec_dir, &stats)) <= 0 )
    {
	SetUserMessage (ctxt, "No codec on target server.") ;
	return ;
    }

    if ( ! ctxt->node2codec )
    {
	ctxt->node2codec = KeHashtableMake (NODE2CODEC_HASHTABLE_SIZE
					  , _codecnode_hash_fct
					  , _codecnode_cmp_fct) ;
    }

    for ( i = 0 ; i < count ; i++ )
    {
	key.node  = stats[i]->id ;
	key.codec = KeCodecGet (stats[i]->name) ;

	_NodeToCodecAdd (ctxt->node2codec, &key) ;
    }

    KecLslFree (stats) ;

    return ;
}

/***************************************************************************
 * GetCodec: retreive codec associated to node 
 ***************************************************************************/

KeCodec GetCodec (ctxt, node)
    Context     ctxt ;
    KID		node ;

{ 
    CodecNode    codecnode ;
    CodecNodeRec key ;

/*
 * Initialize if not done:
 */

    if ( ! ctxt->node2codec )
	_UpdateNodeToCodec (ctxt) ;

/*
 * Return querried codec:
 */

    key.node = node ;

    if ( codecnode = (CodecNode) KeHashtableFind (ctxt->node2codec, &key) )
	return (codecnode->codec) ;

    return ((KeCodec) 0) ;
}
