/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * kedit-editors.c
 *
 * Launch an editor for a given viewer
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>	

#include "xkedit.h"

/*
 * This should be taken from application resources:
 */

static char **Editors ;

#define IS_SEP(c) \
    (((c)==' ') || ((c)=='\t') || ((c)=='\n') || ((c)=='/') || ((c)=='\0'))

/***************************************************************************
 * _FindCommandLine: find command line for given node name
 *
 * RETURNS: command skeleton
 ***************************************************************************/

static char *_FindCommandLine (name)
    char *name ;

{
    char **peditors ;

    for ( peditors = Editors ; *peditors ; peditors += 2 )
    {
	if ( ! strcmp (name, *peditors) )
	    return (*(peditors+1)) ;
    }

    return ((char *) 0) ;
}

/***************************************************************************
 * _DoReplacements: replace all "variables" in skel by values
 ***************************************************************************/

static char *_GetWord (w, plength)
    char *w ;
    int  *plength ;

{
    static char word[128] ;
    char        *p = word ;

    while ( ! IS_SEP (*w) )
	*p++ = *w++ ;
    *p++ = '\0' ;

    *plength = p - word - 1 ;
    return (word) ;
}

static char *_ExclReplacement (ctxt, viewer, key)
    Context ctxt ;
    Viewer  viewer ;
    char    *key ;

{
    static char word[128] ;

    if ( ! strcmp (key, "server") ) {
	strcpy (word, KecNameOfConnection (ctxt->kec_conn)) ;
	return (word) ;
    } else if ( ! strcmp (key, "node") ) {
	sprintf (word, "%d", viewer->stat->id) ;
	return (word) ;
    } else if ( ! strcmp (key, "bindir") ) {
	strcpy (word, Defaults.bindir) ;
	return (word) ;
    }

    return ((char *) 0) ;
}

static char *_PercReplacement (ctxt, viewer, key)
    Context ctxt ;
    Viewer  viewer ;
    char    *key ;

{
    static char word[128] ;
    KeNodeStat  *stats ;
    int         i, count ;

/*
 * Stat on node's parent, to find the replacament:
 */

    if ( viewer->parent_node == (KID) -1 )
    {
	count = KecLsl (ctxt->kec_conn
			, KecRootNodeOfConnection (ctxt->kec_conn)
			, &stats) ;
    }
    else
    {
	count = KecLsl (ctxt->kec_conn
			, viewer->parent_node
			, &stats) ;
    }

/*
 * Look for it:
 */

    for ( i = 0 ; i < count ; i++ )
    {
	if ( ! strcmp (key, stats[i]->name) )
	{
	    sprintf (word, "%d", stats[i]->id) ;
	    KecLslFree (stats) ;
	    return (word) ;
	}
    }

    KecLslFree (stats) ;
    return ((char *) 0) ;
}

static char *_DoReplacements (ctxt, viewer, skel)
    Context ctxt ;
    Viewer  viewer ;
    char    *skel ;

{
    static char command[4096] ;
    char        *p = command ;
    char        *word, *subst ;
    int         length ;

/*
 * Copy, replacing on the fly:
 */

    while ( *skel )
    {
	switch (*skel)
	{
	  case '\\':
	    *p++ = *skel++ ;
	    if ( *skel )
		*p++ = *skel++ ;
	    break ;
	  case '!':
	    word = _GetWord (++skel, &length) ;
	    skel += length ;
	    if ( ! ( subst = _ExclReplacement (ctxt, viewer, word) ) )
		return ((char *) 0) ;
	    while ( *subst )
		*p++ = *subst++ ;
	    break ;
	  case '%':
	    word = _GetWord (++skel, &length) ;
	    skel += length ;
	    if ( ! ( subst = _PercReplacement (ctxt, viewer, word) ) )
		return ((char *) 0) ;
	    while ( *subst )
		*p++ = *subst++ ;
	    break ;
	  default:
	    *p++ = *skel++ ;
	    break ;
	}
    }

    *p++ = ' ' ;
    *p++ = '&' ;
    *p++ = '\n' ;
    *p++ = '\0' ;
    
    return (command) ;
}

/***************************************************************************
 * _ParseEditors: parse the resource editor list into a suitable array
 ***************************************************************************/

static void _ParseEditors ()

{
    char *p, *w, **pp ;
    int  entry ;

/*
 * Count number of lines:
 */
    for ( entry = 2, p = Defaults.editors ; *p ; p++ )
    {
	if ( *p == '\n' )
	    entry += 2 ;
    }
/*
 * Allocate pointer array:
 */
    if ( ! (Editors = (char **) malloc (sizeof (char *) * (entry + 1))) )
    {
	fprintf (stderr, "xkedit: malloc failed.\n") ;
	exit (-1) ;
    }
/*
 * Parse:
 */
    for (pp = Editors, w = p = Defaults.editors ; *p ;  )
    {
	/* Skip leading separators */
	while ( *p && IS_SEP (*p) ) 
	    p++ ;
	w = p ;
	/* Get resource name */
	while ( *p && ( ! IS_SEP (*p)) )
	    p++ ;
	if ( *p == '\n' )
	{
	    p++ ;
	    fprintf (stderr, "_ParseEditors: invalid editors resource,\n") ;
	}
	else
	{
	    *pp++ = w ;
	    *p++  = '\0' ;
	    /* Skip seprators */
	    while ( IS_SEP (*p) )
		p++ ;
	    /* Get command line */
	    for ( w = p ; *p && ( *p != '\n' ) ; p++ )
		;
	    *pp++ = w ;
	    if ( *p )
		*p++  = '\0' ;
	}
    }

    *pp++ = (char *) 0 ;
    return ;
}

/***************************************************************************
 * RunEditor: run an editor for given viewer
 ***************************************************************************/

void RunEditor (viewer)
    Viewer viewer ;

{
    static int parsed = 0 ;
    Context    ctxt = viewer->inspector->context ;
    char       *skeleton ;
    char       *command ;
    char       **peditors ;

    if ( ! parsed )
    {
	parsed = 1 ;
	_ParseEditors () ;
    }

    if ( ! ( skeleton = _FindCommandLine (viewer->stat->name) ) )
    {
	SetUserMessage (ctxt, "No command matching the node name.") ;
	return ;
    }

    if ( ! ( command = _DoReplacements (ctxt, viewer, skeleton)) )
    {
	SetUserMessage (ctxt, "Invalid command syntax.") ;
	return ;
    }

    if ( system (command) == 127 )
    {
	SetUserMessage (ctxt
			, "Couldn't launch command (check editorsDirectory).");
    }

    return ;
}

