/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * xaw-main.c
 *
 * Make the main window, with buttons, etc
 ***************************************************************************/

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>	

#include <X11/Xaw/Box.h>	
#include <X11/Xaw/MenuButton.h>	
#include <X11/Xaw/Paned.h>	
#include <X11/Xaw/SmeBSB.h>	
#include <X11/Xaw/SimpleMenu.h>	

#include "xkedit.h"

/***************************************************************************
 * SetUserMessage   : display message in user message zone
 * DisplayKeditError: specfic k-Edit error message
 ***************************************************************************/

void SetUserMessage (ctxt, message)
    Context ctxt ;
    char    *message ;

{
    XtVaSetValues (ctxt->wmessage, XtNlabel, message, NULL) ;
    XFlush (XtDisplay (ctxt->wmessage)) ;

    return ;
}

void DisplayKeditError (ctxt, msg, status)
    Context  ctxt ;
    char     *msg ;
    KeStatus status ;

{
    static char *oldmsg, *newmsg ;
    char buffer[128] ;

    sprintf (buffer, "%s: %d error", msg, status) ;
    
    newmsg = (char *) strdup (buffer) ;
    SetUserMessage (ctxt, newmsg) ;
    free (oldmsg) ;
    oldmsg = newmsg ;

    SetUserMessage (ctxt, newmsg) ;

    return ;
}

/***************************************************************************
 * CreateMainWindow: create xkedit main window
 *
 * _CreateTopArea   : create the top area (menu box + user message)
 *   _CreateFileMenu: create the file menu (quit, server, etc...)
 *   _CreateNodeMenu: create the node menu (inspect, info, ...)
 *
 * RETURNS: main window widget
 ***************************************************************************/

static void _DoAbout (widget, ctxt_ptr, unused)
    Widget    widget ;
    XtPointer ctxt_ptr ;
    XtPointer unused ;

{
    DisplayFile ((Context) ctxt_ptr, ABOUT_FILE) ;
    return ;
}

static void _DoMainHelp (widget, ctxt_ptr, unused)
    Widget    widget ;
    XtPointer ctxt_ptr ;
    XtPointer unused ;

{
    DisplayFile ((Context) ctxt_ptr, MAINWINDOW_FILE) ;
    return ;
}

static void _DoQuit(widget, unused1, unused2)
    Widget    widget ;
    XtPointer unused1 ;
    XtPointer unused2 ;

{
    exit (0) ;
}

static Widget _CreateFileMenu (ctxt, parent)
    Context ctxt ;
    Widget  parent ;

{
    Widget button, entry, menu ;

    button = XtCreateManagedWidget ("file-menu"
				    , menuButtonWidgetClass
				    , parent
				    , NULL, ZERO) ;

    menu   = XtCreatePopupShell ("menu"
				 , simpleMenuWidgetClass
				 , button
				 , NULL, ZERO) ;

    entry = XtCreateManagedWidget ("help"
				   , smeBSBObjectClass
				   , menu
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoMainHelp, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("about"
				   , smeBSBObjectClass
				   , menu
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoAbout, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("quit"
				   , smeBSBObjectClass
				   , menu
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, _DoQuit, (XtPointer) 0) ;

    return (parent) ;
}

static void DoMakeInspector (widget, ctxt, unused)
    Widget widget ;
    Context ctxt ;
    XtPointer unused ;

{
    XTreeNode *selected ;
    int       i, count ;
    Inspector inspector ;


    if ( (count  = XTreeGetSelection (ctxt->xtree, &selected)) > 0 )
    {
	inspector = MakeInspector (ctxt) ;

	for ( i = 0 ; i < count ; i++ )
	    MakeViewer (ctxt, selected[i]) ;

	XTreeUnselectAll (ctxt->xtree) ;
	
	InspectorResize (inspector) ;
    }
    else
    {
	SetUserMessage (ctxt, "No selected nodes.") ;
    }

    return ;
}

static void DoCreateNotify (widget, ctxt, unused)
    Widget widget ;
    Context ctxt ;
    XtPointer unused ;

{
    XTreeNode *selected ;
    int       j, i, count ;
    void      _KeditCreateHandler (), _KeditDeleteHandler() ;
    NodeInfo  info ;

    if ( (count  = XTreeGetSelection (ctxt->xtree, &selected)) > 0 )
    {
	for ( i = 0 ; i < count ; i++ )
	{
	    info = (NodeInfo) XTreeNodeData (selected[i]) ;

	    if ( ! info->ncreate )
		info->ncreate = KecNotifyAdd (ctxt->kec_conn
					      , info->node
					      , KEDIT_CreateMask
					      , _KeditCreateHandler
					      , (void *) info) ;
	    for (j = 0 ; j < info->xnode->children_count ; j++ ) {
		NodeInfo ci = XTreeNodeData (info->xnode->children[j]) ;
		if ( ! ci->ndelete )
		    ci->ndelete = KecNotifyAdd (ctxt->kec_conn
						, ci->node
						, KEDIT_DeleteMask
						, _KeditDeleteHandler
						, (void *) ci) ;
	    }
	    info->flags |= NI_DELETE_NOTIFY ;
	}
	KecConnectionDoSync (ctxt->kec_conn) ;
    }

    XTreeUnselectAll (ctxt->xtree) ;

    return ;
}
    
static Widget _CreateNodeMenu (ctxt, parent)
    Context ctxt ;
    Widget  parent ;

{
    Widget button, entry, menu ;

    button = XtCreateManagedWidget ("node-menu"
				    , menuButtonWidgetClass
				    , parent
				    , NULL, ZERO) ;

    menu   = XtCreatePopupShell ("menu"
				 , simpleMenuWidgetClass
				 , button
				 , NULL, ZERO) ;

    entry = XtCreateManagedWidget ("inspect-entry"
				   , smeBSBObjectClass
				   , menu
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, DoMakeInspector, (XtPointer) ctxt) ;

    entry = XtCreateManagedWidget ("notify"
				   , smeBSBObjectClass
				   , menu
				   , NULL, ZERO) ;

    XtAddCallback (entry, XtNcallback, DoCreateNotify, (XtPointer) ctxt) ;

    return (parent) ;
}

static Widget _CreateTopArea (ctxt, parent)
    Context ctxt ;
    Widget  parent ;

{
    Widget box, file_menu, node_menu ;

    box = XtCreateManagedWidget ("box"
				 , boxWidgetClass
				 , parent
				 , NULL, ZERO) ;

    file_menu = _CreateFileMenu (ctxt, box) ; 
    node_menu = _CreateNodeMenu (ctxt, box) ; 

    ctxt->wmessage   = XtCreateManagedWidget ("message"
					      , labelWidgetClass
					      , parent
					      , NULL, ZERO) ;

    return (parent) ;
}

Widget CreateMainWindow (ctxt, toplevel)
    Context ctxt ;
    Widget  toplevel ;

{
    Widget paned ;
    XTree  xtree ;

    paned = XtCreateManagedWidget ("paned"
				   , panedWidgetClass
				   , toplevel
				   , NULL, ZERO) ;
    
    _CreateTopArea (ctxt, paned) ; 

    return (paned) ;
}
    
