/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * xaw-tree.c
 *
 * This file provides a simple tree canvas. Although, it uses the
 * Xaw viewport and simple widget classes, it should be fearly easy to
 * make it run on top of Motif (just changing the widget class should be
 * enough).
 * The tree is *fully independent* of the application, so it can be 
 * reused by itself, using the xaw-tree.h and xaw-tree.c files.
 *
 * Now, this file is really messy, I am sorry for this, although some
 * more optimizations could have been done, I didn't made them (no time).
 * The API is described in xaw-tree.h
 ***************************************************************************/

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Cardinals.h>	
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Toggle.h>

#ifdef HAVE_XPM
#include <X11/xpm.h>
#endif

#include "xkedit.h"

#define XTREED(x) 

/*
 * Internal prototypes.
 * All these functions are used internally, serach for them if you want
 * more infos on want they do.
 */

static int _XTreeLayout (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree xtree
    , XTreeNode node
    , Dimension x, Dimension y
    , Dimension w, Dimension h
#endif
) ;

static void _XTreeSelectNodeAction (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget
    , XEvent *event
    , String *params
    , Cardinal *num_params
#endif
) ;

static void _XTreeFoldNodeAction (
#ifdef KEDIT_NeedsFunctionPrototype				  
      Widget widget
    , XEvent *event 
    , String *params
    , Cardinal *num_params
#endif
) ;

static void _XTreeNodeActivateAction (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget
    , XEvent *event 
    , String *params
    , Cardinal *num_params
#endif
) ;

static XTreeNode _XTreeFindNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTreeNode node
    , Dimension x
    , Dimension y
#endif
) ;

static void _XTreeDrawOneNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree xtree
    , XTreeNode node
#endif
) ;

static void _XTreeFreeNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
#endif
) ;

static void _XTreeDeleteAllChildren (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
#endif
) ;

static XTree _XTreeOfWidget (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget
#endif
) ;

static int _XTreeSelectedNodeIndex (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
#endif 
) ;

static void _XTreeAddSelectedNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
#endif
) ;

static void _XTreeRemoveSelectedNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
#endif
) ;

/*
 * Statics:
 */

static char *XTreeTranslations = 
"<Btn1Down>: xtree-selector()\n\
 <Btn2Down>: xtree-folder()\n\
 <Btn3Down>: xtree-activate()\n" ;

static XtActionsRec XTreeActions[] = {
    {
	"xtree-selector",
	(XtActionProc) _XTreeSelectNodeAction
    }
    , {
	"xtree-folder",
	(XtActionProc) _XTreeFoldNodeAction
    }
    , {
	"xtree-activate",
	(XtActionProc) _XTreeNodeActivateAction
    }
} ;

#ifndef HAVE_XPM 

static char dir_closed[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0x88, 0x00, 0x04, 0x3f,
   0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20,
   0xfc, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

static char dir_open[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0x88, 0x00, 0x04, 0x3f,
   0x04, 0x20, 0xf4, 0xff, 0x14, 0x80, 0x0c, 0x40, 0x0c, 0x40, 0x04, 0x20,
   0xfc, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

static char file_plain[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x03, 0x08, 0x06, 0x08, 0x0a,
   0x08, 0x1e, 0x08, 0x10, 0x08, 0x10, 0x08, 0x10, 0x08, 0x10, 0x08, 0x10,
   0x08, 0x10, 0xf8, 0x1f, 0x00, 0x00, 0x00, 0x00};

#else

#define	BACKGROUND	". c grey90   m white g4 grey90"

static char *dir_closed[] = {
	"16 16 3 1",
	BACKGROUND,
	"b c black  m black g4 black",
	"y c yellow m white g4 grey80",
	"................",
	"................",
	"................",
	"..bbbb..........",
	".byyyyb.........",
	"bbbbbbbbbbbbbb..",
	"byyyyyyyyyyyyb..",
	"byyyyyyyyyyyyb..",
	"byyyyyyyyyyyyb..",
	"byyyyyyyyyyyyb..",
	"byyyyyyyyyyyyb..",
	"byyyyyyyyyyyyb..",
	"bbbbbbbbbbbbbb..",
	"................",
	"................",
	"................",
};	

static char *dir_open[] = {
	"16 16 4 1",
	BACKGROUND,
	"b c black  m black g4 black",
	"y c yellow m white g4 grey80",
	"g c grey70 m white g4 grey70",
	"................",
	"................",
	"................",
	"..bbbb..........",
	".bggggb.........",
	"bggggggbbbbbbb..",
	"bggggggggggggb..",
	"bgbbbbbbbbbbbbbb",
	"bgbyyyyyyyyyyybb",
	"bbyyyyyyyyyyyyb.",
	"bbyyyyyyyyyyybb.",
	"byyyyyyyyyyyyb..",
	"bbbbbbbbbbbbbb..",
	"................",
	"................",
	"................",
};	


static char *file_plain[] = {
	"16 16 3 1",
	BACKGROUND,
	"b c black  m black g4 black",
	"g c white m white g4 white",
	"................",
	"................",
	"..bbbbbbbbbb....",
	"..bgggggggbgb...",
	"..bgggggggbggb..",
	"..bgggggggbbbbb..",
	"..bgggggggggggb.",
	"..bgggggggggggb.",
	"..bgggggggggggb.",
	"..bgggggggggggb.",
	"..bgggggggggggb.",
	"..bgggggggggggb.",
	"..bbbbbbbbbbbbb..",
	"................",
	"................",
	"................",
};

#endif /* HAVE_XPM */

/*
 * Macros:
 */

#define INRANGE(_x,_l,_r) ( (_l <= _x) && (_x <= _r) )

#define INBOX(_x,_y,_bx,_by,_bw,_bh) \
    (INRANGE (_x, _bx, _bx+_bw) && INRANGE(_y, _by, _by+_bh))

#define INTERSECTSEG(_l,_r,_sl,_sr) \
    ( INRANGE(_l,_sl,_sr) || INRANGE(_r,_sl,_sr) \
     || ( ( _l <= _sl ) && ( _sr <= _r ) ) )

#define INTERSECTBOX(_x1,_y1,_bw1,_bh1,_x2,_y2,_bw2,_bh2) \
    ( INTERSECTSEG(_x1,_x1+_bw1,_x2,_x2+_bw2) \
     && INTERSECTSEG (_y1,_y1+_bh1,_y2,_y2+_bh2) )

#ifndef MIN
#define MIN(x,y) ((x > y) ? y : x)
#endif

#ifndef MAX
#define MAX(x,y) ((x > y) ? x : y)
#endif

/***************************************************************************
 * _XTreeInitializeGC: initialize gc's for given xtree
 *
 * - normal_fill_gc: is used to fill rectangle under node label
 * - normal_text_gc: for the label itself
 * - selected...   : same, but if the node is selected.
 *
 * RETURNS: void (exit on error !)
 ***************************************************************************/

void _XTreeInitializeGC (xtree)
    XTree xtree ;

{
    XtGCMask  mask;
    XGCValues values;
    Widget    widget = xtree->wsimple ;
    Display   *dpy = XtDisplay (xtree->wsimple) ;
    
    if ( ! xtree->normal_text_gc )
    {
	mask = GCFont | GCForeground ;
	values.foreground = XBlackPixel (dpy, XDefaultScreen (dpy)) ;
	values.font       = Defaults.glyph_font->fid ;
    
	xtree->normal_text_gc = XtGetGC (widget, mask, &values) ;
    }

    if ( ! xtree->normal_fill_gc )
    {
	mask = GCForeground ;
	values.foreground = Defaults.normal_node_fill_foreground ;
	xtree->normal_fill_gc = XtGetGC (widget, mask, &values) ;
    }

    if ( ! xtree->selected_fill_gc )
    {
	mask = GCForeground ;
	values.foreground = Defaults.selected_node_fill_foreground ;
	xtree->selected_fill_gc = XtGetGC (widget, mask, &values) ;
    }


    if ( ! xtree->line_gc )
    {
	mask = GCForeground | GCGraphicsExposures ;
	values.foreground = XBlackPixel (dpy, XDefaultScreen (dpy)) ;
	values.graphics_exposures = False ;
	xtree->line_gc = XtGetGC (widget, mask, &values) ;
    }

    return ;
}

/***************************************************************************
 * _XTreePixmapInitialize: load needed pixmaps.
 *
 * This function just initializes the default pixmaps. A node can be
 * assigned another pixmap by using XTreeNodePixmap(). To be sure we
 * have no problems here, pixmaps are included in data format in top
 * of file. 
 * There are three needed pixmap:
 * - close_dir: for a closed directory (folded)
 * - open_dir : for an opened directory (unfolded)
 * - file     : for a normal file.
 *
 * If HAVE_PIXMAP is defined, then we use the Xpm library (wich you should
 * really have if you don't), otherwise, we trun to black and white bitmaps
 * wich are quite ugly.
 *
 * RETURNS: void (exit on error !)
 ***************************************************************************/

static void _XTreePixmapInitialize (xtree)
    XTree xtree ;

{
    Widget  w      = xtree->wsimple ;
    Display *dpy   = XtDisplay (w) ;
    Pixel   black  = XBlackPixel (dpy, XDefaultScreen (dpy)) ;
    Pixel   white  = XWhitePixel (dpy, XDefaultScreen (dpy)) ;
    int     planes = XDisplayPlanes (dpy, XDefaultScreen (dpy)) ;
    
    if ( xtree->open_dir && xtree->close_dir && xtree->file )
	return ;

#ifdef HAVE_XPM
    {
	Pixmap        unused ;
	XpmAttributes attr ;

	XpmCreatePixmapFromData (dpy
				 , XtWindow (w)
				 , dir_closed
				 , &(xtree->close_dir)
				 , &unused
				 , (XpmAttributes *) 0) ;

	XpmCreatePixmapFromData (dpy
				 , XtWindow (w)
				 , dir_open
				 , &(xtree->open_dir)
				 , &unused
				 , (XpmAttributes *) 0) ;

	XpmCreatePixmapFromData (dpy
				 , XtWindow (w)
				 , file_plain
				 , &(xtree->file)
				 , &unused
				 , (XpmAttributes *) 0) ;
    }

#else
    
    xtree->open_dir = XCreatePixmapFromBitmapData (XtDisplay (w)
						   , XtWindow(w)
						   , dir_closed
						   , 32, 32
						   , black, white
						   , planes) ;
    xtree->close_dir = XCreatePixmapFromBitmapData (XtDisplay (w)
						    , XtWindow (w)
						    , dir_open
						    , 32, 32
						    , black, white
						    , planes) ;
    xtree->file      = XCreatePixmapFromBitmapData (XtDisplay (w)
						    , XtWindow (w)
						    , file_plain
						    , 32, 32
						    , black, white
						    , planes) ;

#endif /* HAVE_XPM */

    if ( xtree->open_dir && xtree->close_dir && xtree->file )
	return ;

    fprintf (stderr, "_XTreePixmapInitialize: unable to load bitmaps.\n");
    exit (1) ;
}

/***************************************************************************
 * _XTreeNodeIndex: find index of given children in parent
 * 
 * This function should never fail, otherwise, it is an implementation 
 * error, that's why we exit() on error.
 *
 * RETURNS: index (exit on error)
 ***************************************************************************/

static int _XTreeNodeIndex (parent, node)
    XTreeNode parent ;
    XTreeNode node ;

{
    register int i ;
    
    for ( i = 0 ; i < parent->children_count ; i++ )
    {
	if ( parent->children[i] == node )
	    return (i) ;
    }

    fprintf (stderr, "_XTreeNodeIndex: couldn't find child (fatal).\n") ;
    exit (1) ;
}

/***************************************************************************
 * _XTreeOfWidget: retreive xtree associated to widget 
 * 
 * We use XContext's to assocaite the drawing area simpleWidgetClass) with
 * the xtree it is displaying. This is taken from editres.
 * As this function should only be called from simpleWidget callbacks, it
 * should never fail, otherwise it is an implementation error.
 *
 * RETURNS: XTree (exit on error)
 ***************************************************************************/

XTree _XTreeOfWidget (widget)
    Widget widget ;

{
    XPointer data_return ;

    if ( ! XFindContext (XtDisplay (widget)
			 , (Window) widget
			 , XTREE_CONTEXT
			 , &data_return) )
    {
	return ((XTree) data_return) ;
    }

    fprintf (stderr, "_XTreeOfWidget: unable to FindContext !\n") ;
    exit (1) ;
}

/***************************************************************************
 * _XTreeSelectNodeAction:   select node under pointer
 * _XTreeFoldNodeAction:     fold/unfold under pointer node
 * _XtreeNodeActivateAction: triggers the tree action on node
 *
 * these are the default provided actions. They allow to:
 * - select a given node
 * - fold/unfold a directory
 *
 * RETURNS: void according to XtActionProc.
 ***************************************************************************/

static void _XTreeSelectNodeAction (wsimple, event, args, nargs)
    Widget   wsimple ;
    XEvent   *event ;
    String   *args ;
    Cardinal *nargs ;

{
    XButtonEvent *button = (XButtonEvent *) event ;
    XTree        xtree = _XTreeOfWidget (wsimple) ;
    XTreeNode    node ;

    if ( node = _XTreeFindNode (xtree->root, button->x, button->y) )
    {
	if ( XTREE_SELECTED (node) )
	{
	    node->flags &= ~XTREE_NODE_SELECTED ;
	    _XTreeRemoveSelectedNode (xtree, node) ;
	}
	else
	{
	    node->flags |= XTREE_NODE_SELECTED ;
	    _XTreeAddSelectedNode (xtree, node) ;
	}

	_XTreeDrawOneNode (xtree, node) ;
    }
    else
    {
	XBell (XtDisplay(wsimple), 100) ;
    }

    return ;
}

static void _XTreeFoldNodeAction (wsimple, event, args, nargs)
    Widget   wsimple ;
    XEvent   *event ;
    String   *args ;
    Cardinal *nargs ;

{
    XButtonEvent *button = (XButtonEvent *) event ;
    XTreeNode    node ;
    XTree        xtree = _XTreeOfWidget (wsimple) ;

    if ( node = _XTreeFindNode (xtree->root, button->x, button->y) )
    {
	XTreeFoldUnfoldNode (xtree, node) ;
	XTreeRefresh (xtree, node) ;
    }
    else
    {
	XBell (XtDisplay(wsimple), 100) ;
    }

    return ;
}

static void _XTreeNodeActivateAction (wsimple, event, args, nargs)
    Widget   wsimple ;
    XEvent   *event ;
    String   *args ;
    Cardinal *nargs ;

{
    XButtonEvent *button = (XButtonEvent *) event ;
    XTreeNode    node ;
    XTree        xtree = _XTreeOfWidget (wsimple) ;

    if ( node = _XTreeFindNode (xtree->root, button->x, button->y) )
    {
	if ( xtree->action )
	    (xtree->action)(xtree, node, xtree->action_closure) ;
    }
    else
    {
	XBell (XtDisplay(wsimple), 100) ;
    }

    return ;
}

/***************************************************************************
 * _XTreeFindNode: find node given its coordinates
 *
 * RETURNS: node or NULL (if none was found)
 ***************************************************************************/

#ifdef KEDIT_NeedsFunctionPrototype
static XTreeNode _XTreeFindNode (XTreeNode node, Dimension x, Dimension y)
#else
static XTreeNode _XTreeFindNode (node, x, y)
    XTreeNode node ;
    Dimension x, y ;
#endif /* KEDIT_NeedsFunctionPrototype */

{
    XTreeNode found ;
    int       i ;

    if ( ! INBOX (x, y
		  , node->x, node->y - Defaults.node_height
		  , node->w, node->h + Defaults.node_height) )
    {
	return ((XTreeNode) 0) ;
    }

    if ( INBOX (x, y, node->x, node->y, node->width, Defaults.node_height) )
    {
	return (node) ;
    }
    
	     
    for ( i = 0 ; i < node->children_count ; i++ )
    {
	if ( found = _XTreeFindNode (node->children[i], x, y) )
	    return (found) ;
    }

    return ((XTreeNode) 0) ;
}


/***************************************************************************
 * XTreeDrawNode: draw given node if needed for bounding box
 *
 * Uses _XTreeDrawOneNode to really draw the node. The XTreeDrawNode 
 * function filters according to bounding box.
 ***************************************************************************/

void _XTreeDrawOneNode (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{
    Widget      widget = xtree->wsimple ;
    XTreeNode   parent ;
    GC          fill_gc, text_gc ;
    XFontStruct *font  = Defaults.glyph_font ;
    Dimension   nh     = Defaults.node_height ;
    Dimension   nhdiv2 = (nh / 2) ;

    XTREED ( fprintf (stderr, "_XTreeDrawOneNode: %s\n", node->name) ; ) ;

    if ( XTREE_SELECTED (node) )
    {
	fill_gc = xtree->selected_fill_gc ;
	text_gc = xtree->normal_text_gc ;
    }
    else
    {
	fill_gc = xtree->normal_fill_gc ;
	text_gc = xtree->normal_text_gc ;
    }

    XCopyArea (XtDisplay (widget)
	       , ( XTREE_DIRECTORY (node)
		  ? ( XTREE_FOLDED (node) ? xtree->close_dir : xtree->open_dir)
		  : xtree->file)
	       , XtWindow (widget)
	       , xtree->line_gc
	       , 0, 0, XTREE_PIXMAP_WIDTH, XTREE_PIXMAP_WIDTH
	       , node->x 
	       , node->y + ((nh - XTREE_PIXMAP_WIDTH) / 2)) ;

    XFillRectangle (XtDisplay (widget), XtWindow (widget)
		    , fill_gc
		    , node->x + XTREE_PIXMAP_WIDTH 
		    , node->y 
		    , node->width - XTREE_PIXMAP_WIDTH 
		    , nh) ;

    XDrawRectangle (XtDisplay (widget), XtWindow (widget)
		    , xtree->line_gc
		    , node->x, node->y 
		    , node->width, nh) ;

    XDrawString (XtDisplay (widget), XtWindow (widget)
		 , xtree->normal_text_gc
		 , node->x + XTREE_PIXMAP_WIDTH + XTREE_TEXT_XSHIFT
		 , node->y + nh - ((nh - font->ascent) / 2)
		 , node->name
		 , strlen (node->name)) ;

    return ;
	
}

void XTreeDrawNode (xtree, node, x, y, w, h)
    XTree     xtree ;
    XTreeNode node ;
    Dimension x, y ;
    Dimension w, h ;

{
    Widget widget = xtree->wsimple ;
    int    i ;
    int    nh     = Defaults.node_height ;
    int    nhdiv2 = Defaults.node_height / 2 ;
    int    nt     = node->width ;
    int    nl     = XLINE_LENGTH ;

    XTREED ( fprintf (stderr, "node %s: %d %d %d %d\n"
		      , node->name 
		      , node->x , node->y
		      , node->w, node->h) ; ) ;
       
    XTREED ( fprintf (stderr, "box : %d %d %d %d\n", x, y, w, h) ; ) ;

/*
 * Does this node needs some recomputation:
 */

    _XTreeLayout (xtree, node, x, y, w, h) ;

/*
 * Make sure that pixmap are ready:
 */

    _XTreePixmapInitialize (xtree) ;

/*
 * Draw parts of node:
 * - Core (pixmap + label) 
 * - vertical line
 * - horizontal line
 */

    if ( INTERSECTBOX (node->x - nl, node->y
		       , node->width + nl, nh
		       , x, y, w, h) )
    {
	_XTreeDrawOneNode (xtree, node) ;
    }

    if ( ( node->children_count > 0 )
	&& INTERSECTBOX (node->x + XLINE_SHIFT - 1
			 , node->y + XLINE_YSHIFT + nh
			 , 2
			 , node->children[node->children_count-1]->y + nhdiv2
			 , x, y, w, h) )
    {
	XDrawLine (XtDisplay (widget), XtWindow (widget)
		   , xtree->line_gc
		   , node->x + XLINE_SHIFT
		   , node->y + XLINE_YSHIFT + nh
		   , node->x + XLINE_SHIFT
		   , node->children[node->children_count-1]->y + nhdiv2) ;
    }

    if ( node->parent 
	&& INTERSECTBOX (node->x - XLINE_LENGTH, node->y + nhdiv2 - 1
			 , XLINE_LENGTH, 2
			 , x, y, w, h) )
    {
	XDrawLine (XtDisplay (widget), XtWindow (widget)
		   , xtree->line_gc
		   , node->x - XLINE_LENGTH
		   , node->y + nhdiv2
		   , node->x
		   , node->y + nhdiv2 ) ;
    }

/*
 * Draw children:
 */

    if ( XTREE_UNFOLDED (node) )
    {
	for ( i = 0 ; i < node->children_count ; i++ )
	{
	    if ( ( node->children[i]->x - nl <= x + w )
		&& ( node->children[i]->y <= y + h ) )
	    {
		XTreeDrawNode (xtree, node->children[i], x, y, w, h) ;
	    }
	}
    }

    return ;
}

void XTreeRefresh (xtree, node)
    XTree     xtree ;
    XTreeNode node ;
{
    Dimension x, y, w, h ;


    if ( node )
    {
	_XTreeLayout (xtree, node
		      , xtree->x, xtree->y
		      , xtree->w, xtree->h) ;

	XClearArea (XtDisplay(xtree->wsimple), XtWindow(xtree->wsimple)
		    , xtree->x, node->y
		    , xtree->w, xtree->h 
		    , True) ;
    }
    else
    {
	_XTreeLayout (xtree, xtree->root
		      , xtree->x, xtree->y
		      , xtree->w, xtree->h) ;

	XClearArea (XtDisplay(xtree->wsimple), XtWindow(xtree->wsimple)
		    , xtree->x, xtree->y
		    , xtree->w, xtree->h
		    , True) ;
    }

    return ;
}

/***************************************************************************
 * Various callabcks:
 ***************************************************************************/

static void ViewportCallback (viewport, xtree_ptr, report_ptr)
    Widget    viewport ;
    XtPointer xtree_ptr ;
    XtPointer report_ptr ;

{
    XTree           xtree = (XTree) xtree_ptr ;
    XawPannerReport *report = (XawPannerReport *) report_ptr ;

    
    xtree->x = report->slider_x ;
    xtree->y = report->slider_y ;
    xtree->w = report->canvas_width ;
    xtree->h = report->canvas_height ;

    XTREED ( fprintf (stderr, "slider: x %d y %d width %d height %d\n"
		      , report->slider_x, report->slider_y
		      , report->slider_width, report->slider_height) ; ) ;
    XTREED ( fprintf (stderr, "canvas: width %d height %d\n"
		      , report->canvas_width, report->canvas_height) ; ) ;

    return ;
}

static void SimpleExposure (simple, xtree_ptr, event, continue_ret)
    Widget    simple ;
    XtPointer xtree_ptr ;
    XEvent    *event ;
    Bool      *continue_ret ;

{
    XExposeEvent *expose = (XExposeEvent *) event ;
    XTree        xtree   = (XTree) xtree_ptr ;

    XTreeDrawNode (xtree
		   , xtree->root
		   , expose->x, expose->y
		   , expose->width, expose->height) ;
    return ;
}

/***************************************************************************
 * XTreeLayout: re-layout given tree
 ***************************************************************************/

static void _XTreeDirtUp (node, index)
    XTreeNode node ;
    int       index ;

{
    XTreeNode parent ;
    int       i ;

/*
 * Dirt this one:
 */
    if ( ( node->dirt_index < 0 ) || ( node->dirt_index > index ) )
    {
	node->dirt_index = index ;
    }

/*
 * And parent:
 */

    while ( parent = node->parent )
    {
	i = _XTreeNodeIndex (parent, node) ;
	if ( (parent->dirt_index < 0 ) || ( parent->dirt_index > i ) )
	{
	    parent->dirt_index = i ;
	}
	node = parent ;
    }

    return ;
}

#ifdef KEDIT_NeedsFunctionPrototype
static int _XTreeLayout (XTree xtree
			 , XTreeNode node
			 , Dimension x, Dimension y
			 , Dimension w, Dimension h)
#else
static int _XTreeLayout (xtree, node, x, y, w, h)
    XTree     xtree ;
    XTreeNode node ;
    Dimension x, y ;
    Dimension w, h ;
#endif /* KEDIT_NeedsFunctionPrototype */

{
    XTreeNode dirty, last ;
    int       i ;

/*
 * Skip unusable cases:
 */

    if ( node->dirt_index < 0 ) 
	return (0) ;

/*
 * Relayout folded nodes:
 */

    if ( XTREE_FOLDED (node) )
    {
	int       idx ;
	XTreeNode prev, parent = node->parent ;

	node->dirt_index = -1 ;

	if ( ! parent )
	    return (0) ;

	if ( ! (idx  = _XTreeNodeIndex (node->parent, node)) )
	{
	    node->x = parent->x + XLINE_LENGTH + XLINE_SHIFT ;
	    node->y = parent->y + Defaults.node_height + XLINE_YSHIFT ;
	    return (0) ;
	}
	else
	{
	    prev = parent->children[idx-1] ;
	    node->x = prev->x ;
	    node->y = prev->y + prev->h + XLINE_YSHIFT ;
	}
	return (1) ;
    }

/*
 * Do the job:
 * - First child node is specific 
 * - Other child are computed relative to first one
 */

    if ( node->dirt_index == 0 )
    {
	dirty = *(node->children) ;
	dirty->x = node->x + XLINE_LENGTH + XLINE_SHIFT ;
	dirty->y = node->y + Defaults.node_height + XLINE_YSHIFT ;

	if ( dirty->children_count )
	    dirty->dirt_index = 0 ;
	_XTreeLayout (xtree, dirty, x, y, w, h) ;

	node->w = dirty->w + XLINE_LENGTH + XLINE_SHIFT ;
	node->h = XLINE_YSHIFT + dirty->h + (3 * Defaults.node_height / 2) ;
	node->dirt_index++ ;
    }
    else
    {
	node->w = node->width ;
	node->h = Defaults.node_height ;
	for ( i = 0 ; i < node->dirt_index ; i++ )
	{
	    if ( node->children[i]->w > node->w )
		node->w = node->children[i]->w ;
	    node->h += (XLINE_YSHIFT + node->children[i]->h) ;
	}
    }

/*
 * Re-layout children:
 */

    for ( i = node->dirt_index ; i < node->children_count ; i++ )
    {
	dirty = node->children[i] ;
	last  = node->children[i - 1] ;

	dirty->x = last->x ;
	dirty->y = last->y + last->h + XLINE_YSHIFT ;
	if ( dirty->children_count )
	    dirty->dirt_index = 0 ;
	_XTreeLayout (xtree, dirty, x, y, w, h) ;

	if ( dirty->w + (dirty->x - node->x) > node->w )
	    node->w = dirty->w + (dirty->x - node->x) ;
	node->h += (XLINE_YSHIFT + dirty->h) ;
    }
	
/*
 * If this was the root node, resize the simple widget:
 */

    if ( ! node->parent )
    {
	XtVaSetValues (xtree->wsimple
		       , XtNwidth, xtree->root->w + 40
		       , XtNheight, xtree->root->h + 40
		       , NULL) ;
    }

    node->dirt_index = -1 ;

    return (1) ;
}

void XTreeFoldUnfoldNode (xtree, node)
    XTree xtree ;
    XTreeNode node ;

{
/*
 * Skip debile case:
 */

    if ( ! XTREE_DIRECTORY (node) )
	return ;

/*
 * Easy ! no ?
 */

    if ( XTREE_UNFOLDED (node) )
    {
	node->flags |= XTREE_NODE_FOLDED ;
	node->w = node->width ;
	node->h = Defaults.node_height ;

	if ( XTREE_DIRECTORY_LAZY (node) )
	    _XTreeDeleteAllChildren (xtree, node) ;

	if ( node->parent )
	    _XTreeDirtUp (node->parent, _XTreeNodeIndex (node->parent, node));

	
    }
    else
    {
	node->flags &= ~XTREE_NODE_FOLDED ;

	if ( XTREE_DIRECTORY_LAZY (node) )
	    (xtree->filler) (xtree, node, xtree->filler_closure) ;
	
	node->dirt_index = node->children_count ? 0 : -1 ;

	if ( node->parent )
	    _XTreeDirtUp (node->parent, _XTreeNodeIndex (node->parent, node));
    }

    XTreeRefresh (xtree, node) ;
    return ;
}

/***************************************************************************
 * Selected node set management functions:
 * _XTreeAddSelectedNode: add this node in the selected node set
 * _XTreeRemoveSelectedNode: remove this node from selected set
 *
 * XTreeGetSelectedNodes: returns the array of selected nodes
 * XTreeUnselectAll     : unselect all nodes
 *
 * THIS IS THE THIRD TIME I WRITE A SET MANAGEMENT FUNCTION *IN THE SAME
 * PROGRAM*: THIS IS BAD !!!!!
 ***************************************************************************/

static int _XTreeSelectedNodeIndex (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{
    int i ;

    for ( i = 0 ; i < xtree->selected_count ; i++ )
    {
	if ( xtree->selected[i] == node )
	    return (i) ;
    }

    fprintf (stderr, "_XTreeSelectedNodeIndex: implementation bug !\n") ;
    return ;
}

static void _XTreeAddSelectedNode (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{
    if ( ! xtree->selected_size )
    {
	int size = DEFAULT_SELECTED_SIZE ;

	xtree->selected = (XTreeNode *) malloc (sizeof (XTreeNode) * size) ;
	xtree->selected_size = size ;
    }
    else if ( xtree->selected_count + 1 >= xtree->selected_size )
    {
	int size = xtree->selected_size + DEFAULT_SELECTED_INC ;

	xtree->selected = (XTreeNode *) realloc (xtree->selected
						 , sizeof (XTreeNode) * size) ;
	xtree->selected_size = size ;
    }

    xtree->selected[xtree->selected_count++] = node ;

    return ;
}

static void _XTreeRemoveSelectedNode (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{
    int idx = _XTreeSelectedNodeIndex (xtree, node) ;

/*
 * Remove node from set :
 */

    if ( idx < xtree->selected_count - 1)
    {
	bcopy (xtree->selected + idx + 1
	       , xtree->selected + idx
	       , sizeof (XTreeNode) * (xtree->selected_count - idx)) ;
    }

    xtree->selected_count-- ;

    return ;
}

int XTreeGetSelection (xtree, nodes_ret)
    XTree     xtree ;
    XTreeNode **nodes_ret ;

{
    if ( xtree->selected_count )
	*nodes_ret = xtree->selected ;
    return (xtree->selected_count) ;
}

void XTreeUnselectAll (xtree)
    XTree xtree ;

{
    int i ;

    for ( i = 0 ; i < xtree->selected_count ; i++ )
    {
	xtree->selected[i]->flags &= ~XTREE_NODE_SELECTED ;
	_XTreeDrawOneNode (xtree, xtree->selected[i]) ;
    }

    xtree->selected_count = 0 ;

    return ;
}

/***************************************************************************
 * XTreeCreateNode: create a new node rooted at parent
 *
 * This function uses _XTreeAddNode to add the node to its parent list. It
 * does not recompute layout, as this is done incrementally (when needed
 * for a refresh). dirt_index is set to advertise beginning of structure
 * that needs a re-layout.
 * The _XTreeAddNode gauaranties that the node is added in *last* position
 * in the array.
 ***************************************************************************/

static void _XTreeFreeNode (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{

    XTREED (fprintf (stderr, "_XTreeFreeNode: %s\n", node->name) ;) ;

    if ( node->children_size )
	_XTreeDeleteAllChildren (xtree, node) ;

    if ( xtree->killer )
	(xtree->killer) (xtree, node, xtree->killer_closure) ;

    free (node->name) ;
    free (node) ;

    return ;
}

static void _XTreeDeleteAllChildren (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{
    int i ;

    XTREED (fprintf (stderr, "_XTreeDeleteAllChildren: %s\n", node->name) ;) ;

    for ( i = 0 ; i < node->children_count ; i++ )
	_XTreeFreeNode (xtree, node->children[i]) ;

    if ( node->children )
	free (node->children) ;

    node->children_size  = 0 ;
    node->children_count = 0 ;
    node->children       = (XTreeNode *) 0 ;

    return ;
}

static int _XTreeAddNode (parent, child)
    XTreeNode parent ;
    XTreeNode child ;

{
    int index ;

/*
 * Reallocate or allocate parent children array:
 */

    if ( ! parent->children_size )
    {
	int size = XTREE_CHILDREN_SIZE ;

	parent->children = (XTreeNode *) malloc (sizeof (XTreeNode) * size) ;
	parent->children_size  = size ;
    }
    else if ( parent->children_count + 1 >= parent->children_size )
    {
	int size = parent->children_size + XTREE_CHILDREN_SIZE_INC ;

	parent->children_size = size ;
	parent->children = (XTreeNode *) realloc (parent->children
						  , sizeof (XTreeNode) * size);
    }

/*
 * Add it :
 */
    
    index                   = parent->children_count++ ;
    parent->children[index] = child ;

    if ( ( parent->dirt_index < 0 ) || ( index < parent->dirt_index ) )
	parent->dirt_index = index ;

    return (index) ;
}


XTreeNode XTreeCreateNode (parent, name, flags, user_data)
    XTreeNode     parent ;
    char          *name ;
    unsigned long flags ;
    void	  *user_data ;

{
    XTreeNode node ;
    int	      index ;

/*
 * Allocate memory, and set default values:
 */

    if ( ! ( node = (XTreeNode) malloc (sizeof (XTreeNodeRec)) ) )
	return (node) ;

    node->name   = (char *) strdup (name) ;
    node->x      = 0 ;
    node->y      = 0 ;
    node->w      = (  XTREE_PIXMAP_WIDTH
		    + XTREE_TEXT_XSHIFT 
		    + XTextWidth (Defaults.glyph_font, name, strlen (name))) ;
    node->width     = node->w ;
    node->h         = Defaults.node_height ;
    node->parent    = parent ;
    node->flags     = flags ;
    node->user_data = user_data ;

    node->dirt_index     = -1 ;
    node->children_count = 0 ;
    node->children_size  = 0 ;
    node->children       = (XTreeNode *) 0 ;

/*
 * If no parent, root node, we are done, else add it, and propagate dirt up
 */

    if ( ! parent )
    {
	node->x = Defaults.node_height ;
	node->y = Defaults.node_height ;
	return (node) ;
    }

    index = _XTreeAddNode (parent, node) ;

    XTREED ( fprintf (stderr, "Added node %s (x%08x) @ %d to %s\n"
		      , node->name
		      , node
		      , index
		      , parent->name) ; ) ;

    _XTreeDirtUp (parent, index) ;

    return (node) ;
}

/***************************************************************************
 * XTreeDeleteNode: remove this node
 ***************************************************************************/

void XTreeDeleteNode (xtree, node)
    XTree     xtree ;
    XTreeNode node ;

{
    XTreeNode parent = node->parent ;
    int       idx ;

    if ( ! parent )
	return ;
    idx = _XTreeNodeIndex (parent, node) ;
    _XTreeFreeNode (xtree, node) ;
    if ( idx < parent->children_count -1 )
	bcopy (parent->children + idx + 1
	       , parent->children + idx
	       , sizeof (XTreeNode) * (parent->children_count - idx)) ;
    parent->children_count-- ;
    _XTreeDirtUp (parent, idx) ;
    if ( idx == parent->children_count )
	XTreeRefresh (xtree, parent->children[idx-1]) ;
    else
	XTreeRefresh (xtree, parent->children[idx]);
    return ;
}

/***************************************************************************
 * XTreeCreate: create the tree displayer in given widget
 ***************************************************************************/

void XTreeDelete (xtree)
    XTree xtree ;

{
    fprintf (stderr, "XTreeDelete: nothing done.\n") ;
    return ;
}

XTree XTreeCreate (parent, root
		   , filler, filler_ptr
		   , killer, killer_ptr
		   , action, action_ptr)
    Widget      parent ;
    XTreeNode   root ;
    XTreeFiller filler ;
    void        *filler_ptr ;
    XTreeKiller killer ;
    void        *killer_ptr ;
    XTreeAction action ;
    void        *action_ptr ;

{
    XTree        xtree ;
    Widget       simple ;
    
/*
 * Allocate and initialize the descriptor:
 */

    if ( xtree = (XTree) malloc (sizeof (XTreeRec)) )
    {
	/*
	 * Initialize XTree data members:
	 */

	xtree->normal_text_gc   = (GC) 0 ;
	xtree->selected_text_gc = (GC) 0 ;
	xtree->normal_fill_gc   = (GC) 0 ;
	xtree->selected_fill_gc = (GC) 0 ;
	xtree->line_gc          = (GC) 0 ;

	xtree->open_dir  = (Pixmap) 0 ;
	xtree->close_dir = (Pixmap) 0 ;
	xtree->file      = (Pixmap) 0 ;

	xtree->filler         = filler ;
	xtree->filler_closure = filler_ptr ;

	xtree->killer         = killer ;
	xtree->killer_closure = killer_ptr ;

	xtree->action	      = action ;
	xtree->action_closure = action_ptr ;

	xtree->selected       = (XTreeNode *) 0 ;
	xtree->selected_size  = 0 ;
	xtree->selected_count = 0 ;

	/*
	 * Create widget hierarchy:
	 */

	xtree->wviewport = XtCreateManagedWidget ("viewport"
						  , viewportWidgetClass
						  , parent
						  , NULL, ZERO) ;
	XtAddCallback (xtree->wviewport
		       , XtNreportCallback
		       , ViewportCallback
		       , (XtPointer) xtree) ;

	xtree->wsimple   = XtCreateManagedWidget ("simple"
						  , simpleWidgetClass
						  , xtree->wviewport
						  , NULL, ZERO) ;
    
	/*
	 * Save context, and setup actions, translations, and exposures:
	 */

	XSaveContext (XtDisplay (xtree->wsimple)
		      , (Window) xtree->wsimple
		      , XTREE_CONTEXT
		      , (XtPointer) xtree) ;

	XtAddEventHandler (xtree->wsimple
			   , ExposureMask
			   , True
			   , SimpleExposure
			   , (XtPointer) xtree) ;

	XtAppAddActions (XtWidgetToApplicationContext (parent)
			 , XTreeActions
			 , XtNumber (XTreeActions)) ;

	XtOverrideTranslations (xtree->wsimple
				, XtParseTranslationTable (XTreeTranslations));
	
	xtree->root = root ;

	_XTreeInitializeGC (xtree) ;
    }


    return (xtree) ;
}

    
