/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * xaw-tree.h
 *
 * Structure to maintain the drawing of the tree
 ***************************************************************************/

#ifndef _XAW_TREE_H_
#define _XAW_TREE_H_

/*
 * - XLINE_SHIFT
 *    from node-> to vertical line
 * - XLINE_LENGTH
 *    from node->x + XLINE_SHIFT to end of subdir line
 * - XLINE_YSTEP
 *    minimal space between two xline
 * - XTREE_PIXMAP_WIDTH
 *    pixmaps should be of this size (well, they will be clipped to this
 *    size, wich is also the height).
 * - XTREE_TEXT_XSHIFT
 *    number of pixels between the pixmap and the node label.
 */

#define XLINE_SHIFT		10
#define XLINE_LENGTH    	20


#define XLINE_YSHIFT		5

#define XTREE_PIXMAP_WIDTH	16
#define XTREE_TEXT_XSHIFT	10

/*
 * Context are used to save the xtree associated to a widget.
 * The following context number has been taken from Editres
 */

#define XTREE_CONTEXT	((XContext) 42)

/*
 * Allocation custom:
 * - XTREE_CHILDREN_SIZE    : starting size for children array
 * - XTREE_CHILDREN_SIZE_INC: increment for resiz
 */

#define XTREE_CHILDREN_SIZE	1
#define XTREE_CHILDREN_SIZE_INC 10

typedef struct _XTreeNode {
    char	      *name ;		/* node name */
    Dimension         x, y, w, h ;	/* bounding box for subtree */
    Dimension	      width ;		/* width of active region */
    unsigned long     flags ;		/* node flags (see below) */
    int		      dirt_index ;	/* first dirty child, or -1 */
    int		      children_count ;	/* # of children for this node */
    int		      children_size ;	/* size of following array */
    struct _XTreeNode **children ;	/* children of this node */
    struct _XTreeNode *parent ;		/* parent of this node */
    void	      *user_data ;	/* user data for node */
} XTreeNodeRec, *XTreeNode ;

#define XTREE_NODE_FOLDED		(1 << 0)
#define XTREE_NODE_SELECTED		(1 << 1)
#define XTREE_NODE_DIRECTORY		(1 << 2)
#define XTREE_NODE_DIRECTORY_LAZY	(1 << 3)

#define XTREE_SELECTED(n)	((n)->flags & XTREE_NODE_SELECTED)
#define XTREE_FOLDED(n)		((n)->flags & XTREE_NODE_FOLDED)
#define XTREE_UNFOLDED(n)	(! XTREE_FOLDED(n))
#define XTREE_DIRECTORY(n)	((n)->flags & XTREE_NODE_DIRECTORY)
#define XTREE_DIRECTORY_LAZY(n) ((n)->flags & XTREE_NODE_DIRECTORY_LAZY) 

#define XTreeNodeData(n)	((n)->user_data)
#define XTreeSetNodeData(n,c)	((n)->user_data = (c))

/*
 * The Xtree:
 */

#define DEFAULT_SELECTED_SIZE	8
#define DEFAULT_SELECTED_INC    8

typedef struct _XTree *XTree ;

typedef void (*XTreeFiller) (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
    , void      *closure 
#endif
) ;		

typedef void (*XTreeKiller) (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node	    
    , void      *closure
#endif
) ;

typedef void (*XTreeAction) (
#ifdef KEDIT_NeedsFunctionProtoype
      XTree	tree
    , XTreeNode node
    , void      *closure 
#endif
) ;

typedef struct _XTree {
    Widget	    wviewport ;
    Widget	    wsimple ;
    Dimension       x, y, w, h ;
    XTreeNode       root ;
    XTreeNode	    *selected ;
    int             selected_size ;
    int             selected_count ;
    XTreeFiller     filler ;
    void	    *filler_closure ;
    XTreeKiller     killer ;
    void            *killer_closure ;
    XTreeAction	    action ;
    void            *action_closure ;
    GC              normal_text_gc ;
    GC		    selected_text_gc ;
    GC  	    normal_fill_gc ;
    GC              selected_fill_gc ;
    GC		    line_gc ;
    Pixmap	    open_dir, close_dir, file ;
} XTreeRec ;

XTreeNode XTreeCreateNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTreeNode parent
    , char *name
    , unsigned long flags
    , void *user_data
#endif
) ;

void XTreeDeleteNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node
#endif
);

XTree XTreeCreate (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget      parent
    , XTreeNode   root
    , XTreeFiller filler
    , void        *filler_closure
    , XTreeKiller killer
    , void        *killer_closure
    , XTreeAction action
    , void        *action_ptr 
#endif
) ;

void XTreeDelete (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree xtree 
#endif
) ;

void XTreeFoldUnfoldNode (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree xtree
    , XTreeNode node
#endif
) ;

void XTreeRefresh (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree     xtree
    , XTreeNode node 
#endif
) ;

int XTreeGetSelection (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree xtree
    , XTreeNode **node_ret
#endif
) ;

void XTreeUnselectAll (
#ifdef KEDIT_NeedsFunctionPrototype
      XTree xtree 
#endif
) ;

#endif /* _XAW_TREE_H_ */
