/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * xkedit.c
 ***************************************************************************/

#include <stdio.h>
#include <setjmp.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>	

#include "xkedit.h"

#include <KeServer.h>
#include <KesXt.h>

#define DEFAULT_GLYPH_FONT          "9x13"
#define DEFAULT_SELECTED_BACKGROUND "cyan"
#define DEFAULT_NORMAL_BACKGROUND   "blue"
#define DEFAULT_VIEWER_FLASH_COLOR  "blue"
#define DEFAULT_VIEWER_FLASH_ON	    50
#define DEFAULT_VIEWER_FLASH_COUNT  10
#define DEFAULT_HELP_DIRECTORY	    "./Help"
#define DEFAULT_BINDIR		    "../Editors"

/*
 * Default resources description: 
 */

static XtResource resources[] = {
    { 
	"glyphFont",
	"GlyphFont",
	XtRFontStruct,
	sizeof (XFontStruct *),
	XtOffsetOf (XkeditDefaultsRec, glyph_font),
	XtRString,
	DEFAULT_GLYPH_FONT
    } 
    , {
	"selectedNodeFillForeground",
	"SelectedNodeFillForeground",
	XtRPixel,
	sizeof (Pixel),
	XtOffsetOf (XkeditDefaultsRec, selected_node_fill_foreground),
	XtRString,
	DEFAULT_SELECTED_BACKGROUND
    }
    , {
	"normalNodeFillForeground",
	"NormalNodeFillForeground",
	XtRPixel,
	sizeof (Pixel),
	XtOffsetOf (XkeditDefaultsRec, normal_node_fill_foreground),
	XtRString,
	DEFAULT_NORMAL_BACKGROUND
    }
    , {
	"nodeHeight",
	"NodeHeight",
	XtRDimension,
	sizeof (Dimension),
	XtOffsetOf (XkeditDefaults, node_height),
	XtRImmediate,
	(XtPointer) 25
    }
    , {
	"viewerFlashColor",
	"ViewerFlashColor",
	XtRPixel,
	sizeof (Pixel),
	XtOffsetOf (XkeditDefaultsRec, viewer_flash_color),
	XtRString,
	(XtPointer) DEFAULT_VIEWER_FLASH_COLOR
    }
    , { 
	"viewerFlashOn" ,
	"ViewerFlashOn" ,
	XtRInt,
	sizeof (int),
	XtOffsetOf (XkeditDefaultsRec, viewer_flash_on), 
	XtRImmediate,
	(XtPointer) DEFAULT_VIEWER_FLASH_ON
    }
    , { 
	"viewerFlashCount" ,
	"ViewerFlashCount" ,
	XtRInt,
	sizeof (int),
	XtOffsetOf (XkeditDefaultsRec, viewer_flash_count), 
	XtRImmediate,
	(XtPointer) DEFAULT_VIEWER_FLASH_COUNT
    }
    , {
	"helpDirectory",
	"HelpDirectory",
	XtRString,
	sizeof (int),
	XtOffsetOf (XkeditDefaultsRec, help_directory),
	XtRString,
	DEFAULT_HELP_DIRECTORY
    }
    , {
	"editorsDirectory",
	"EditorsDirectory",
	XtRString,
	sizeof (int),
	XtOffsetOf (XkeditDefaultsRec, bindir),
	XtRString,
	DEFAULT_BINDIR
    }
    , {
	"editors",
	"Editors",
	XtRString,
	sizeof (int),
	XtOffsetOf (XkeditDefaultsRec, editors),
	XtRString,
	DEFAULT_BINDIR
    }
} ;

/*
 * Fallback resources:
 */

static String fallback_resources[] = { 
    NULL ,
} ;

/*
 * Globals:
 */

XkeditDefaultsRec Defaults ;

/***************************************************************************
 * main: drives
 ***************************************************************************/

main (argc, argv)
    int  argc ;
    char **argv ;

{
    ContextRec    context ;
    XTreeNode     root ;
    unsigned long flags ;
    KesServer     server ;
    jmp_buf       ice_error ;

/*
 * Initioalize:
 * - Toolkit
 * - Get appplication resources
 * - Create main window
 * - plug k-edit (we act as a k-Edit server also)
 */

    memset (&context, 0, sizeof (ContextRec)) ;

    context.wtop = XtAppInitialize (&(context.app_con)
				    , "XKedit"
				    , NULL, ZERO
				    , &argc, argv
				    , fallback_resources
				    , NULL, ZERO) ;
    
    XtVaGetApplicationResources (context.wtop
				 , &Defaults
				 , resources
				 , XtNumber (resources)
				 , NULL) ;

    context.wpaned =  CreateMainWindow (&context, context.wtop) ;

    if ( ! ( server = KesServerInitialize ("xkedit") ) )
    {
	fprintf (stderr, "Unable to initialize as k-Edit server.\n") ;
	exit (1) ;
    }

/*
 * Connect and set root:
 */

    if ( argc != 2 )
    {
	fprintf (stderr, "xkedit <server-name>\n") ;
	exit (1) ;
    }

    ConnectToServer (&context, *++argv) ;

/*
 * Realize, enter main loop:
 */

    XtRealizeWidget (context.wtop) ;

    SetUserMessage (&context, "Hello world !") ;

    if ( ! setjmp (ice_error) )
	KesXtServerPlug (context.app_con, server, context.wtop, ice_error) ;

    XtAppMainLoop (context.app_con) ;

}
