/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * xkedit.h
 ***************************************************************************/

#ifndef _XKEDIT_H_
#define _XKEDIT_H_

#include "xaw-tree.h"

#include <KeClient.h>
#include <KeCodec.h>

/*
 * Help files:
 */

#define ABOUT_FILE		"about.txt"
#define MAINWINDOW_FILE		"main-window.txt"
#define INSPECTOR_FILE		"inspector.txt"
#define VIEWER_FILE		"viewer.txt"

/*
 * Default resources structure:
 */

typedef struct {
    XFontStruct *glyph_font ;
    Pixel       selected_node_fill_foreground ;
    Pixel       normal_node_fill_foreground ;
    Dimension   node_height ;
    Pixel       viewer_flash_color ;
    int         viewer_flash_count ;
    int         viewer_flash_on ;
    char	*help_directory ;
    char	*bindir ;
    char	*editors ;
} XkeditDefaultsRec, XkeditDefaults ;

/*
 * Kedit context for one target server:
 */

#define NODE2CODEC_HASHTABLE_SIZE	16

typedef struct _Context {
    XtAppContext  app_con ;
    XtInputId     xt_input ;
    XTree	  xtree ;		/* current tree area */
    KecConnection kec_conn ;		/* current connection */
    KeHashtable	  node2codec ;
    Widget	  wtop ;		/* toplevel of app */
    Widget	  wmessage ;		/* message area */
    Widget        wpaned ;		/* parent of tree drawinf area */
} ContextRec, *Context ;

/*
 * Node info: infos that get attached to a node in the XTree :
 */

typedef struct _NodeInfo {
    unsigned long flags ;
    XTreeNode	  xnode ;		/* back pointer */
    KID		  parent ;		/* usefull for cmd replacmenets */
    KID		  node ;		/* k-Edit node identifier */
    KecNotifyId   ncreate ;		/* structure notification */
    KecNotifyId   ndelete ;
} NodeInfoRec, *NodeInfo ;

#define NI_DELETE_NOTIFY	1

/*
 * Inspector and viewers:
 */

#define DEFAULT_VIEWERS_SIZE	8
#define DEFAULT_VIEWERS_INC	8

typedef struct _Inspector *Inspector ;

typedef struct _Viewer {
    Inspector    inspector ;		/* parent */
    KeCodec	 codec ;		/* codec for this node */
    void	 (*tostring)() ;
    void 	 *(*fromstring)() ;
    KID		 parent_node ;		/* usefull for replacements */
    KeNodeStat	 stat ;			/* of viewed node */
    Widget	 wtop ;			/* of this viewer */
    Widget	 wtext ;
    Pixel	 background ;		/* saved when flashing */
    XtIntervalId timeout ;
    int          flash_count ;
    KecNotifyId  read ;
    KecNotifyId  read_with_value ;
    KecNotifyId  write ;
    KecNotifyId  write_with_value ;
    KecNotifyId  exec ;
    KecNotifyId  notify ;
} ViewerRec, *Viewer ;

typedef struct _Inspector {
    Inspector   next ;
    Context     context ;
    Widget	wtop ;
    Widget	wbox ;
    int         viewers_count ;
    int		viewers_size ;
    Viewer      *viewers ;
} InspectorRec ;

/*
 * Externs:
 */

extern XkeditDefaultsRec Defaults ;

/*
 * Prototypes:
 */

Widget CreateMainWindow (Context ctxt, Widget toplevel) ;

void   SetUserMessage (
#ifdef KEDIT_NeedsFunctionPrototype
      Context ctxt
    , char    *msg
#endif
) ;

void DisplayKeditError (
#ifdef KEDIT_NeedsFunctionPrototype
      Context  ctxt
    , char     *msg
    , KeStatus status 
#endif
) ;

Inspector MakeInspector (
#ifdef KEDIT_NeedsFunctionPrototype
      Context ctxt
#endif
) ;

void DeleteInspector (
#ifdef KEDIT_NeedsFunctionPrototype
      Inspector inspector 
#endif
) ;

Viewer MakeViewer (
#ifdef KEDIT_NeedsFunctionPrototype
      Context   ctxt
    , XTreeNode node 
#endif
) ;

void DeleteViewer (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer
    , int    resize		/* resize inspector ? */
    , int    delete		/* delete inspector if no more viewers */
#endif
) ;

XTreeNode MakeNode (
#ifdef KEDIT_NeedsFunctionPrototype
      KecConnection kec_conn
    , XTreeNode     parent
    , KeNodeStat    stat
#endif
) ;

KeCodec GetCodec (
#ifdef KEDIT_NeedsFunctionPrototype
      Context ctxt 
    , KID     node
#endif
) ;

void ViewerGetValue (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerSetValue (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerExec (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerWriteWithValueMonitor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerReadWithValueMonitor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerReadMonitor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;


void ViewerWriteMonitor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;


void ViewerExecMonitor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerNotifyMonitor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void ViewerFlash (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer 
#endif
) ;

void RunEditor (
#ifdef KEDIT_NeedsFunctionPrototype
      Viewer viewer
#endif
) ;


#endif
