/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * xmsample.c
 *
 * A Motif example of a k-Editized application.
 ***************************************************************************/

#include <stdio.h>
#include <setjmp.h>

#include <Xm/RowColumn.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>

#include <KeServer.h>
#include <KeCodec.h>
#include <KesXt.h>
#include <KesXtP.h>

/***************************************************************************
 * button_cb: the one invoked for all three buttons 
 *
 * As long as this function doesn't use the 'unused' arg, the callback
 * can safely be triggered by a k-Edit client.
 ***************************************************************************/

static void exit_cb (widget, unused1, unused2)
    Widget    widget ;
    XtPointer unused1 ;
    XtPointer unused2 ;

{
    fprintf (stderr, "Bye !\n") ;
    exit (0) ;
}

static void button_cb (widget, text_ptr, unused)
    Widget    widget ;
    XtPointer text_ptr ;
    XtPointer unused ;

{
    fprintf (stderr, (char *) text_ptr) ;
    return ;
}

/***************************************************************************
 * xmstring_get
 * xmstring_set
 *
 * A sample example, which demonstrates how resource info is provided
 * to k-Edit. We export here the XmRXmString resource type for all widget
 * classes, and all resources of this type.
 *
 * This makes the button labels's value available to k-Edit client.
 * Hope I didn't got anything wrong: I'm not a Motif expert !
 ***************************************************************************/

static KeStatus xmstring_get (widget, info, value_ret, emsg, efree)
    Widget            widget ;
    KesXtResourceInfo info ;
    KePointer         *value_ret ;
    char	      **emsg ;
    KeErrorFreeFct    *efree ;

{
    XmString string ;

    XtVaGetValues (widget, info->name, &string, NULL) ;
    if ( XmStringGetLtoR (string
			  , XmSTRING_DEFAULT_CHARSET
			  , (char **) value_ret) )
    {
	return (KEDIT_Success) ;
    }
    else
    {
	*emsg  = "XmStringLToR failed." ;
	*efree = (KeErrorFreeFct) 0 ;	/* static error msg */
	return (KEDIT_Failure) ;
    }

    /* NOT REACHED */
}

static KeStatus xmstring_set (widget, info, value, emsg, efree)
    Widget            widget ;
    KesXtResourceInfo info ;
    KePointer         value ;
    char	      **emsg ;
    KeErrorFreeFct    *efree ;

{
    XmString string ;

/*
 * Motif copies string before using them, so we can free this afterward (?)
 */

    if ( value )
	string = XmStringCreateSimple (value) ;
    else
	string = XmStringCreateSimple ("") ;

    XtVaSetValues (widget, info->name, string, NULL) ;
    KeCodecDecoderFree (info->codec, value) ;

    XmStringFree (string) ;

    return (KEDIT_Success) ;
}

static void ExportXmStringResources () 

{
    KesXtResourceInfo xmstring_info ;

    xmstring_info = (KesXtResourceInfo) malloc (sizeof (KesXtResourceInfoRec));

    if ( ! xmstring_info )
    {
	fprintf (stderr, "xmsample: malloc error !!\n") ;
	exit (-1) ;
    }

    xmstring_info->name  = (char *) 0 ;
    xmstring_info->codec = KeCodecSTRINGGet () ;
    xmstring_info->mode  = KEDIT_ReadMode | KEDIT_WriteMode ;
    xmstring_info->mask  = (unsigned long) 0 ;
    xmstring_info->get   = xmstring_get ;
    xmstring_info->set   = xmstring_set ;
    xmstring_info->exec  = KesXtNullExec ;

    KesXtRealResourceDefine ((XrmQuark) 0
			     , (XrmQuark) XrmStringToQuark (XmRXmString)
			     , xmstring_info) ;
    
    return ;
}

/***************************************************************************
 * main:
 ***************************************************************************/

main (argc, argv)
    int  argc ;
    char **argv ;

{
    XtAppContext      app_con ;
    Widget            toplevel, rowcolumn, button, text ;
    KesServer         server ;
    jmp_buf           ice_error_buf ;

/*
 * Create any widget hierarchy: 
 */

    toplevel = XtVaAppInitialize (&app_con
				  , "XmSample"
				  , NULL, 0
				  , &argc, argv
				  , NULL, NULL) ;

    rowcolumn = XtVaCreateWidget ("rowcolumn"
				  , xmRowColumnWidgetClass
				  , toplevel
				  , NULL) ;

    button = XtVaCreateManagedWidget ("quit"
				      , xmPushButtonWidgetClass
				      , rowcolumn
				      , NULL) ;

    XtAddCallback (button
		   , XmNactivateCallback
		   , exit_cb
		   , (XtPointer) 0) ;

    button = XtVaCreateManagedWidget ("button-1"
				      , xmPushButtonWidgetClass
				      , rowcolumn
				      , NULL) ;

    XtAddCallback (button
		   , XmNactivateCallback
		   , button_cb
		   , (XtPointer) "Button 1 Activated !\n") ; 

    button = XtVaCreateManagedWidget ("button-2"
				      , xmPushButtonWidgetClass
				      , rowcolumn
				      , NULL) ;

    XtAddCallback (button
		   , XmNactivateCallback
		   , button_cb
		   , (XtPointer) "Button 2 Activated !\n") ; 

    text = XtVaCreateManagedWidget ("text"
				    , xmTextWidgetClass
				    , rowcolumn
				    , NULL) ;

    XtManageChild (rowcolumn) ;
    
    XtRealizeWidget (toplevel) ;


    ExportXmStringResources () ;

    if ( ! ( server = KesServerInitialize ("XmSample") ) )
    {
	fprintf (stderr, "xmsample: unable to KesServerInitialize.\n") ;
	exit (-1) ;
    }

    if ( ! setjmp ( ice_error_buf ) )
	KesXtServerPlug (app_con, server, toplevel, ice_error_buf) ;
				  
    XtAppMainLoop (app_con) ;
}
