/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSpacket.h
 *
 * This file describe the Ks lib view of the network packet. Because client
 * and server doesn't have the same API, they are two descriptions of 
 * request/reply. One for the client lib and one for the Server
 * lib (this one).
 * Each request contain a "id_info" field usefull for the upper layer to
 * gives any information to the final user.
 ***************************************************************************/

#ifndef _KS_PACKET_H_
#define _KS_PACKET_H_

/*
 * Common request/reply header :
 */

#define KES_COMMON_HEADER  \
    int			type ;     \
    int			subtype ;  \
    struct _KesServer	*server ;  \
    struct _KesClient	*client ;  \
    KePointer		closure ; \
    KesHandlerSet       handler_set ; \
    unsigned int	serial

typedef struct _KesHeader {
    KES_COMMON_HEADER ;
} KesHeaderRec, *KesHeader ;


/***************************************************************************
 * Request formats, for the C-lib binding
 * With the C-API, these structures defines the structure that will be
 * used to discuss between Kedit library, and Kedit interface implentors.
 *
 * List of request :
 *
 * KesOpenConnectionReq		KesOpenConnectionReply
 * KesLsReq			KesLsReply
 * KesStatReq			KesStatReply
 * KesGetReq			KesGetReply
 * KesSetReq			[KesError]
 * KesExecReq			[KesError]
 * KesNotifyReq			[KesError]
 * 
 * If you changes this:
 * 1. Contact authorized persons.
 * 2. Respect structure layouts (no inconstistencies with generic types)
 *
 * The following typedefs exhibit a few repetitions. This is done on
 * *purpose*. Do not try to reduce the negligible effect of duplication
 * this would lost the flexibility.
 * In last stage, this can be rewritten properly. It is not time, as
 * we don't jnow in wich way the protocol will evolve, to try to
 * do it better.
 ***************************************************************************/

/***************************************************************************
 * OpenConnection
 ***************************************************************************/


typedef struct _KesOpenConnectionReq {
    KES_COMMON_HEADER ;				/* KEDIT_OpenConnection */
} KesOpenConnectionReqRec, *KesOpenConnectionReq ;

typedef struct _KesOpenConnectionReply {
    KES_COMMON_HEADER ;				/* KEDIT_OpenConnection */
    unsigned long       client_tag ;		/* client tag */
    KID                 root ;			/* this server's root node */
} KesOpenConnectionReplyRec, *KesOpenConnectionReply ;

/***************************************************************************
 * Ls
 ***************************************************************************/

typedef struct _KesLsReq {
    KES_COMMON_HEADER ;				/* KEDIT_Ls */
    KID			node ;	
} KesLsReqRec, *KesLsReq ;

typedef struct _KesLsReply {
    KES_COMMON_HEADER ;				/* KEDIT_Ls */
    int                 num_children ;
    KID			*children ;
} KesLsReplyRec, *KesLsReply ;

#define KesLsReplyFill(r,n,c,f) \
    ( (r)->ru.ls.type = KEDIT_Ls \
     , (r)->ru.ls.num_children = (n) \
     , (r)->ru.ls.children     = (c) \
     , (r)->free_fct           = (f) \
     , (r) )

#define KesLsReplyFillStatic(r,n,c) KesLsReplyFill(r,n,c,(KesReplyFreeFct) 0)

/***************************************************************************
 * Stat
 ***************************************************************************/

typedef struct _KesStatReq {
    KES_COMMON_HEADER ;
    KID			node ;
} KesStatReqRec, *KesStatReq ;

typedef struct _KesStatReply {
    KES_COMMON_HEADER ;
    KeNodeStatRec	stat ;
} KesStatReplyRec, *KesStatReply ;

#define KesStatReplyFill(_r,_n,_m,_i,_t,_s,_f) \
    ( (_r)->ru.stat.type = KEDIT_Stat \
     , (_r)->ru.stat.stat.name = (_n) \
     , (_r)->ru.stat.stat.mode = (_m) \
     , (_r)->ru.stat.stat.mask = (_i) \
     , (_r)->ru.stat.stat.info = (_t) \
     , (_r)->ru.stat.stat.size = (_s) \
     , (_r)->free_fct = (_f) \
     , (_r) )

#define KesStatReplyFillStatic(r,n,m,i,t,s) \
     KesStatReplyFill(r,n,m,i,t,s,(KesReplyFreeFct) 0)

/***************************************************************************
 * Get
 ***************************************************************************/

typedef struct _KesGetReq {
    KES_COMMON_HEADER ;				/* KEDIT_Get */
    KID			node ;		
} KesGetReqRec, *KesGetReq ;

typedef struct _KesGetReply {
    KES_COMMON_HEADER ;				/* KEDIT_Get */
    int			size ;			/* value size */
    KePointer		value ;			/* value */
} KesGetReplyRec, *KesGetReply ;

#define KesGetReplyFill(r,s,v,f) \
    ( (r)->ru.get.type   = KEDIT_Get \
     , (r)->ru.get.size  = (s) \
     , (r)->ru.get.value = (v) \
     , (r)->free_fct     = (f) \
     , (r) )

#define KesGetReplyFillStatic(r,s,v) KesGetReplyFill (r,s,v,(KesReplyFreeFct) 0)

/***************************************************************************
 * Noop
 ***************************************************************************/

typedef struct _KesNoopReq {
    KES_COMMON_HEADER ;				/* KEDIT_Noop */
} KesNoopReqRec, *KesNoopReq ;

typedef struct _KesNoopReply {
    KES_COMMON_HEADER ;				/* KEDIT_Noop */
} KesNoopReplyRec, *KesNoopReply ;

/***************************************************************************
 * Set
 ***************************************************************************/

typedef struct _KesSetReq {
    KES_COMMON_HEADER ;				/* KEDIT_Set */
    KID			node ;		
    int			size ;			/* size of value */
    char		value[1] ;		/* (new) value */
} KesSetReqRec, *KesSetReq ;

#define KesSizeOfSetRequest(r)	((r)->size)
#define KesDataOfSetRequest(r)	((r)->value)

/***************************************************************************
 * Exec
 ***************************************************************************/

typedef struct _KesExecReq {
    KES_COMMON_HEADER ;				/* KEDIT_Exec */
    KID			node ;			/* node to be executed */
} KesExecReqRec, *KesExecReq ;

/***************************************************************************
 * Notify
 ***************************************************************************/

typedef struct _KesNotifyReq {
    KES_COMMON_HEADER ;				/* KEDIT_Exec */
    KID			node ;
    unsigned long	mask ;
} KesNotifyReqRec, *KesNotifyReq ;

#define KesMaskOfNotifyRequest(r)	((r)->mask)

/***************************************************************************
 * Error
 ***************************************************************************/

typedef struct _KesError {
    KES_COMMON_HEADER ;				/* KEDIT_Error */
    KID			node ;
    KeStatus		status ;
    char		*msg ;
} KesErrorRec, *KesError ;

#define KesErrorReplyFill(r,s,m,f) \
    ( (r)->ru.h.type        = KEDIT_Error \
     , (r)->ru.error.status = (s) \
     , (r)->ru.error.msg    = (m) \
     , (r)->free_fct        = (f) \
     , (r) )

#define KesErrorReplyFillStatic(r,s,m) KesErrorReplyFill(r,s,m,(KesReplyFreeFct)0)

/***************************************************************************
 * Events
 ***************************************************************************/

typedef struct _KesGenericEvent {
    KES_COMMON_HEADER ;				/* KEDIT_Event */
    int			event_type ;
    KID			node ;
    unsigned long	timestamp ;
    unsigned long	client_tag ;
} KesDeleteEventRec, *KesDeleteEvent
, KesExecEventRec, *KesExecEvent
, KesReadEventRec, *KesReadEvent 
, KesWriteEventRec, *KesWriteEvent 
, KesAnyEventRec, *KesAnyEvent ;

typedef struct _KesCreateEvent {
    KES_COMMON_HEADER ;				/* KEDIT_Event */
    int			event_type ;
    KID			node ;
    unsigned long	timestamp ;
    unsigned long	client_tag ;
    KID			child ;
} KesCreateEventRec, *KesCreateEvent ;

typedef struct _KesNotifyEvent {
    KES_COMMON_HEADER ;				/* KEDIT_Event */
    int			event_type ;
    KID			node ;
    unsigned long	timestamp ;
    unsigned long	client_tag ;
    unsigned long	mask ;
} KesNotifyEventRec, *KesNotifyEvent ;

typedef struct _KesWithValueEvent {
    KES_COMMON_HEADER ;				/* KEDIT_Event */
    int			event_type ;
    KID			node ;
    unsigned long	timestamp ;
    unsigned long	client_tag ;
    unsigned long	size ;
    KePointer		value ;
} KesReadWithValueEventRec, *KesReadWithValueEvent 
, KesWriteWithValueEventRec, *KesWriteWithValueEvent ;

typedef union _KesEvent {
    KesAnyEventRec		any ;
    KesDeleteEventRec		delete_event ;
    KesCreateEventRec 		create_event ;
    KesNotifyEventRec      	notify_event ;
    KesReadEventRec		read_event ;
    KesWriteEventRec		write_event ;
    KesReadWithValueEventRec 	read_with_value_event ;
    KesWriteWithValueEventRec	write_with_value_event ;
} KesEventRec, *KesEvent ;

/***************************************************************************
 * Now, the generic reply, and the generic request
 *
 * Wherever you have a reply, you should be able to cast it from this one
 * Wherever you have a request, ...
 *
 * Those are followed by a set of accessors macros.
 ***************************************************************************/


typedef void (*KesReplyFreeFct)(
#ifdef KEDIT_NeedsFunctionPrototype
      void *reply
#endif
) ;

typedef struct _KesReply {
    /* This should be in first position */
    union _KesInternalReplyForSize {
	struct _KesHeader	h ;
	KesEventRec		event ;
	KesErrorRec		error ;
	KesLsReplyRec		ls ;
	KesStatReplyRec		stat ;
	KesGetReplyRec		get ;
	KesNoopReplyRec		noop ;
    } ru ;
    KesReplyFreeFct		free_fct ;
} KesReplyRec, *KesReply ;

#define KesTypeOfReply(r)		( (r)->ru.h.type )
#define KesSubtypeOfReply(r)		( (r)->ru.h.subtype )
#define KesServerOfReply(r)		( (r)->ru.h.server )
#define KesClientOfReply(r)		( (r)->ru.h.client )
#define KesSerialOfReply(r)		( (r)->ru.h.serial )
#define KesFreeFctOfReply(r)		( (r)->free_fct )

typedef struct _KesRequest {
    union _KesInternalRequestForSize {
	struct _KesHeader 	      h ;
	struct _KesOpenConnectionReq open_connection ;
	struct _KesLsReq	     ls ;
	struct _KesGetReq	     get ;
	struct _KesSetReq	     set ;
	struct _KesExecReq	     exec ;
	struct _KesNotifyReq	     notify ;
    } ru ;
} KesRequestRec, *KesRequest ;

#define KesTypeOfRequest(r)	 	( ((KesRequest) (r))->ru.h.type )
#define KesSubtypeOfRequest(r)	 	( ((KesRequest) (r))->ru.h.subtype )
#define KesServerOfRequest(r)    	( ((KesRequest) (r))->ru.h.server )
#define KesClientOfRequest(r)    	( ((KesRequest) (r))->ru.h.client )
#define KesSerialOfRequest(r)	 	( ((KesRequest) (r))->ru.h.serial )
#define KesHandlerSetOfRequest(r)	( ((KesRequest) (r))->ru.h.handler_set)
#define KesClosureOfRequest(r)		( ((KesRequest) (r))->ru.h.closure)

#define KesNodeOfRequest(r)		(((KesLsReq) r)->node)

#undef KES_COMMON_HEADER

#endif /* _KS_PACKET_H_ */
