/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KeCodecs.h
 *
 * Codecs management function, public API
 ***************************************************************************/

#ifndef KE_CODECS_H_
#define KE_CODECS_H_

#define KE_CODEC_HASHTABLE_SIZE	256

/*
 * The type data type !
 */

typedef KeStatus (*KeCodecEncoder) (
#ifdef KEDIT_NeedsFunctionPrototype
      KePointer pointer			/* value to be encoded */
    , int       *size_ret		/* size of encoded ret value */
    , KePointer *value_ret		/* encoded ret value */
    , char      **msg_ret 
    , KeErrorFreeFct *free_ret 
#endif
) ;

typedef KeStatus (*KeCodecSizer) (
#ifdef KEDIT_NeedsFunctionPrototype
      KePointer pointer			/* value to be sized */
    , int       *size_ret		/* size of encoded value */
#endif
) ;

typedef KeStatus (*KeCodecDecoder) (
#ifdef KEDIT_NeedsFunctionPrototype
      int       size			/* size of value to be decoded */
    , KePointer value			/* value to be decoded */
    , int       swap			/* is value to be swapped ? */
    , KePointer *to			/* where do I put decoded value ? */
    , char	**msg_ret 
    , KeErrorFreeFct *free_ret 				    
#endif
) ;

typedef void (*KeCodecFreeer) (
#ifdef KEDIT_NeedsFunctionPrototype
      KePointer pointer			/* value to be freed */
#endif
) ;

/*
 * The type data type:
 */

typedef struct _KeCodec {
    char                 *name ;	/* name of encoder, as exported */
    KeCodecSizer         sizer ;	/* get value encoded size */
    KeCodecEncoder       encoder ;	/* encode a value */
    KeCodecFreeer	 encoder_free;	/* free encoded value */
    KeCodecDecoder       decoder ;	/* decode a value */
    KeCodecFreeer	 decoder_free;  /* free decoder ret value */
} KeCodecRec, *KeCodec ;
     
#define KeCodecEncoderFreeer(c)	((c)->encoder_free)
#define KeCodecDecoderFreeer(c)	((c)->decoder_free)

/*
 * Prototypes:
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

KeCodec KeCodecUNDEFGet() ;
KeCodec KeCodecCARD32Get() ;
KeCodec KeCodecSTRINGGet() ;
KeCodec KeCodecBYTEGet() ;
KeCodec KeCodecCARD16Get() ;

KeCodec KeCodecDefine (
#ifdef KEDIT_NeedsFunctionPrototype
      char           *name		/* name of codec */
    , KeCodecSizer   sizer		
    , KeCodecEncoder encoder
    , KeCodecFreeer  encoder_free
    , KeCodecDecoder decoder
    , KeCodecFreeer  decoder_free
#endif
) ;

KeCodec KeCodecGet (
#ifdef KEDIT_NeedsFunctionPrototype
      char *name 
#endif
) ;

KeStatus KeCodecDecode (
#ifdef KEDIT_NeedsFunctionPrototype
      KeCodec        codec
    , int            size
    , KePointer      value
    , int            swapped
    , KePointer      *to
    , char           **emsg 
    , KeErrorFreeFct *efree 
#endif
) ;

KeStatus KeCodecEncode (
#ifdef KEDIT_NeedsFunctionPrototype
      KeCodec        codec
    , KePointer      pointer
    , int            *size_ret
    , KePointer      *value_ret 
    , char           **emsg 
    , KeErrorFreeFct *efree 
#endif
) ;
		
KeStatus KeCodecSize (
#ifdef KEDIT_NeedsFunctionPrototype
      KeCodec   codec
    , KePointer pointer
    , int       *size_ret 
#endif
) ;
    
void KeCodecEncoderFree (
#ifdef KEDIT_NeedsFunctionPrototype
      KeCodec   codec
    , KePointer pointer
#endif
) ;

void KeCodecDecoderFree (
#ifdef KEDIT_NeedsFunctionPrototype
      KeCodec   codec
    , KePointer pointer
#endif
) ;

#ifdef __cplusplus
} /* End of c++ verbosities */
#endif /* __cplusplus */


#endif /* _KE_CODECS_H_ */
