/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KeLib.h
 * 
 * This file contains the definitions shared by the server and the client.
 * All data internal to kedit should have the prefix 'KeRaw'
 ***************************************************************************/

#ifndef _KLIB_H_
#define _KLIB_H_

#include <config.h>
#include <Kproto.h>
#include <khash.h>

/* Verbose macro, expanse if debug, always while coding ... */

/* #ifdef DEBUG */
#define KV(s) 
/* #else */
/* #define KV(s) */

/*
 * Ze k-edit pointer type :
 */

typedef unsigned char *KePointer ;

/*
 * Return value for lots of the functions 
 */

typedef int KeStatus  ;

typedef CARD32	KID ;		/* any kEdit ID */

/*
 * Error free function, used by function returning error messages:
 */

typedef void (*KeErrorFreeFct)(
#ifdef KEDIT_NeedsFunctionPrototype
    char *error_msg 			       
#endif
) ;

/*
 * Booleans : 
 */

#ifndef False
#    define False 0
#endif

#ifndef True
#    define True  1
#endif

/*
 * Variable size struct (saves a few malloc) ;
 */

typedef struct _KeIdArray {
    int			count ;
    KID		        nodes[1] ;
} KeIdArrayRec, *KeIdArray ;

#define KeCountOfIdArray(a)	((a)->count)
#define KeNodesOfIdArray(a)	((a)->nodes)

/*
 * The NodeStat struct provides a full description of an id
 */

typedef struct _KeNodeStat {
    char	  *name ;
    unsigned long mode ;
    unsigned long mask ;
    unsigned long size ;
    KID           id ;
    KID           info ;
} KeNodeStatRec, *KeNodeStat ;

/*
 * Some prototypes :
 */

/*
 * From Lib/Misc/alloc.c
 */

typedef void (*KeMallocPurgeHook)() ;
typedef void (*KeMallocFailHook)() ;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

void KeFree (
#ifdef KEDIT_NeedsFunctionPrototype
      void *ptr 
#endif
) ;

void *KeMalloc (
#ifdef KEDIT_NeedsFunctionPrototype
      KeMallocPurgeHook purge_hook
    , KeMallocFailHook  fail_hook
    , int size
#endif
) ;

void *KeRealloc (
#ifdef KEDIT_NeedsFunctionPrototype
      KeMallocPurgeHook purge_hook
    , KeMallocFailHook  fail_hook
    , void              *buffer
    , int               size
#endif
) ;

/*
 * From Lib/Misc/compat.c
 */

#ifdef NEEDS_STRSTR
char *strstr (
#ifdef KEDIT_NeedsFunctionPrototye
      char *string
    , char *substring
#endif
) ;
#endif /* NEEDS_STRSTR */

#ifdef NEEDS_STRDUP
char *strdup (
#ifdef KEDIT_NeedsFunctionPrototype
      char *p
#endif
) ;
#endif /* NEEDS_STRDUP */

#ifdef __cplusplus
} /* End of c++ verbosities */
#endif /* __cplusplus */

#endif /* _KLIB_H_ */
