/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KeServer.h
 *
 * This file should be included by the server part of an appliaction. It 
 * defines all the datatype used by the Ks library
 ***************************************************************************/

#ifndef _KS_H_
#define _KS_H_

#include <KeLib.h>

/*
 * Default hashtable size for notification database (can be changed)
 */

#define KES_NOTIFICATIONS_HASHTABLE_SIZE 128

/***************************************************************************
 * Data structure are heavily imbricated, take care !
 ***************************************************************************/


struct _KesServer ;
typedef struct _KesServer *KesServer ;
struct _KesClient ;
typedef struct _KesClient *KesClient ;
struct _KesIdInfo ;

/*
 * Opaque data types:
 */

typedef struct _KesHandlerSet *KesHandlerSet ;
typedef struct _KesNodeInfo   *KesNodeInfo ;
typedef struct _KesDirectory  *KesDirectory ;

#include <KSpacket.h>

/***************************************************************************
 * The server's view of a client
 * 
 * From client programmers point of view, this should be a read-only struct
 ***************************************************************************/


typedef struct _KesClient {
    IceConn			 ice_connection ;
    unsigned int		 serial ;
    unsigned long                client_tag ; 	/* this client's tag */
    struct _KesServer		 *server ;	/* server for this client */
    KePointer			 user_data ;	/* some user data per client */
    unsigned int		 notification ;	/* # of notify */
} KesClientRec ;

#define KesServerOfClient(c)	    ((c)->server) 
#define KesSerialOfClient(c)	    ((c)->serial)
#define KesTagOfClient(c)	    ((c)->client_tag)
#define KesGetUserDataOfClient(c)   ((c)->user_data)
#define KesSetUserDataOfClient(c,v) ((c)->user_data = (KePointer) (v))
#define KesClientSwapped(c)	    (IceSwapping ((c)->ice_connection))

/*
 * Identifier management functions:
 * - KesNodeRangeIndex should be used to reteive the index inside a node range
 */

#define KesNodeRangeIndex(n,b) (((unsigned long) (n)) - ((unsigned long) (b)))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

KeStatus KesNodeToInfo (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server 	/* server the node belongs to */
    , KID               node		/* node to retreive info from */
    , KePointer         *closure_ret	/* ret: closure of node */
    , KesHandlerSet     *handler_set_ret 
#endif
) ;

KeStatus KesInfoToNode (
#ifdef KEDIT_NeedsFUnctionPrototype
      KesServer         server		/* server the node belongs to */
    , KesHandlerSet     handler_set	/* handlers for this node */
    , KePointer         closure		/* associated closure */
    , KID               *node_ret
#endif
) ;

KeStatus KesInfoToNodeRange (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server
    , KesHandlerSet     handler_set
    , KePointer         closure
    , int               count
    , KID               *base_ret
#endif
) ;

KeStatus KesNodeMake (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server		/* target server */
    , KesHandlerSet     handler_set     /* hanlders for the new node */
    , KePointer         closure		/* closure of node */
    , KID               *node_ret	/* returned node identifier */
#endif
) ;

KeStatus KesNodeRangeMake (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server		/* target server */
    , KesHandlerSet     handler_set	/* handlers for node of the range */
    , KePointer         closure		/* closure of node range */
    , int               count		/* how many nodes in range */
    , KID               *base_ret	/* base node identifier */
#endif
) ;

KeStatus KesNodeFree (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server		/* server the node belongs to */
    , KID               node		/* node to delete */
#endif
) ;

KeStatus KesNodeRangeFree (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server		/* tagert server */
    , KID               base		/* base of node range to be freed */
#endif
) ;

#ifdef __cplusplus
} /* End of c++ verbosities */
#endif /* __cplusplus */

/***************************************************************************
 * Full server state
 * This of course, should be read only fro server programmers
 ***************************************************************************/

typedef KeStatus (*KesConnectionHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesClient client			/* new connected client */
#endif
) ;

typedef KeStatus (*KesErrorHook) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesRequest request		/* failed request */
    , KesReply   hook_reply		/* reply that will be sent back */
    , KesError   error			/* default error */
#endif
) ;

typedef int (*KesLockHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer         server		/* server to lock/unlock */
#endif
) ;

typedef void (*KesKillHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesClient client			/* client to kill */
#endif 
) ;


typedef struct _KesServer {
    char 	  	  *name ;		/* name of server */
    int			  connections ;		/* client counter */

    KesLockHandler        lock, unlock ;	/* thread safety */

    unsigned long	  timestamp ;		/* server time */

    KesKillHandler	  kill_handler ;	/* client kill handler */
    KePointer		  kill_closure ;	/* and its closure */
    KesConnectionHandler  connection_handler ;	/* client connection handler */
    KePointer		  connection_closure ;	/* and its closure */
    KesErrorHook	  error_hook ;		/* error correction hook */

    KeMallocPurgeHook     purge_hook ;
    KeMallocFailHook      fail_hook ;

    KePointer		  user_data ;		/* associated user data */

    KesHandlerSet	  root_handler_set ; 	/* root hacked handlers */
    KePointer		  root_closure ; 	/* root hacked closure */

    unsigned long	  node_key ;		/* to create uniq IDs */
    int                   nodes_count ;		/* current defined nodes */
    int			  nodes_size ;		/* following array size */
    KesNodeInfo		  *nodes ;		/* this server nodes */
    KeHashtable		  notifications ;	/* Hashtable of notify */
    
    KesDirectory          codec_directory ; 	/* experiments */

} KesServerRec ;

#define KSERVER_LOCK(s)	  (((s)->lock) ? (*((s)->lock))(s) : 0) 
#define KSERVER_UNLOCK(s) (((s)->unlock) ? (*((s)->unlock))(s) : 0) 

#define KesWriteToClient(r) _KesWriteReplyToClient(r)

#define KesNameOfServer(s)		  ((s)->name)
#define KesGetUserDataOfServer(s)	  ((s)->user_data)
#define KesSetUserDataOfServer(s,v)	  ((s)->user_data = (v))

#define KesGetRootHandlerSetOfServer	  ((s)->root_handler_set)
#define KesSetRootHandlerSetOfServer(s,c) ((s)->root_handler_set = (c))

#define KesRootDirectoryOfServer(s)	  ((KesDirectory) (s)->root_closure)
#define KesGetRootClosureOfServer(s)	  ((s)->root_closure)
#define KesSetRootClosureOfServer(s,v)	  ((s)->root_closure = ((KePointer) v))

#define KesMalloc(s,n) \
    KeMalloc ((s)->purge_hook, (s)->fail_hook, n)

#define KesRealloc(s,p,n) \
    KeRealloc ((s)->purge_hook, (s)->fail_hook, p, n)

/***************************************************************************
 * Function prototypes
 ***************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * From KSTMloop.c
 */

void IceMainLoop () ;

/* 
 * From KSServer.c 
 */

KesServer KesServerInitialize (
#ifdef KEDIT_NeedsFunctionPrototype
      char *name			/* name as exported */
#endif
) ;

void KesServerKill (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer server			/* server to be closed */
#endif 
) ;

KesKillHandler KesSetKillHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer server			/* server to set */
    , KesKillHandler handler		/* new kill handler */
#endif
) ;

KesConnectionHandler KesSetConnectionHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer            server	/* server to set */
    , KesConnectionHandler handler	/* new connection handler */
#endif
) ;

KesErrorHook KesSetErrorHook (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer server			/* server to set */
    , KesErrorHook hook			/* new error hook */
#endif
) ;

void KesClientKill (
#ifdef KEDIT_NeedsFunctionPrototype
      KesClient client			/* the client to kill */
#endif
) ;

/* 
 * From KSTDisp.c 
 */

void KesRequestDispatch (
#ifdef KEDIT_NeedsFunctionPrototype
      KesRequest request		/* request to dispatch */
#endif
) ;

/* 
 * From KSIce.c
 */

void KesRequestFree (
#ifdef KEDIT_NeedsFunctionPrototype
      KesRequest request		/* request to be freed */
#endif
) ;

/*
 *  From KSpevent.c 
 */

void KesRequestPrint (
#ifdef KEDIT_NeedsFunctionPrototype
      FILE       *fd			
    , KesRequest request		
#endif
) ;

#ifdef __cplusplus 
} /* End of c++ verbosities */
#endif /* __cplusplus */

/***************************************************************************
 * Handler set management functions
 ***************************************************************************/

/*
 * Handler set access functions:
 *
 * As long as KesHandlerSet is an opaque type, these are to be functions.
 * This is not a problem since handler set access should be very limited
 * and performed only at initialization time.
 */

typedef KeStatus (*KesLsHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesLsReq	request				/* request as input param */
    , KesReply  reply_ret			/* reply to be filled */
#endif
) ;

typedef KeStatus (*KesStatHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStatReq request			/* request as input parameter */
    , KesReply   reply_ret			/* reply to be filled */
#endif 
) ;

typedef KeStatus (*KesSetHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesSetReq request				/* request as input parameter */
    , KesReply  reply_ret			/* reply to be filled */
#endif
) ;

typedef KeStatus (*KesGetHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesGetReq request				/* request as input parameter */
    , KesReply  reply_ret			/* reply to be filled */
#endif
) ;

typedef KeStatus (*KesExecHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesExecReq request			/* request as input parameter */
    , KesReply   reply_ret			/* reply to be filled */
#endif
) ;

typedef KeStatus (*KesNotifyHandler) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesNotifyReq request			/* request as input parameter */
    , KesReply     reply_ret			/* reply to be filled */
#endif
) ;

#define KesNullLsHandler     ((KesLsHandler) 0)
#define KesNullStatHandler   ((KesStatHandler) 0)
#define KesNullSetHandler    ((KesSetHandler) 0)
#define KesNullGetHandler    ((KesGetHandler) 0)
#define KesNullExecHandler   ((KesExecHandler) 0)
#define KesNullNotifyHandler ((KesNotifyHandler) 0)

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

KesHandlerSet KesHandlerSetMake (
#ifdef KEDIT_NeedsFunctionPrototype
      void
#endif
) ;

KesLsHandler KesHandlerSetLsHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesHandlerSet handler_set
    , KesLsHandler  handler
#endif
) ;

KesStatHandler KesHandlerSetStatHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesHandlerSet  handler_set
    , KesStatHandler handler
#endif
) ;

KesGetHandler KesHandlerSetGetHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesHandlerSet handler_set
    , KesGetHandler handler
#endif
) ;

KesSetHandler KesHandlerSetSetHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesHandlerSet handler_set
    , KesSetHandler handler
#endif
) ;

KesExecHandler KesHandlerSetExecHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesHandlerSet  handler_set
    , KesExecHandler handler
#endif
) ;

KesNotifyHandler KesHandlerSetNotifyHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesHandlerSet    handler_set
    , KesNotifyHandler handler
#endif
) ;

#ifdef __cplusplus 
}
#endif /* __cplusplus */

/***************************************************************************
 * Directories management functions
 ***************************************************************************/

typedef int (*KesDirectoryFilter)(
#ifdef KEDIT_NeedsFunctionPrototype
      KesDirectory directory
    , KesLsReq request				  
    , KID      id
#endif
) ;

typedef struct _KesDirectory {
    KesServer          server ;		/* server for this directory */
    KePointer          closure ;	/* associated to the directory */
    KesDirectoryFilter filter ;		/* filter for ls command */
    KID                node ;		/* associated node identifier */
    char               *name ;		/* name of directory */
    int	 	       used ;		/* current number of subdirs */
    int                size ;		/* allocated size of subdirs array */
    KID		       *subdirs ; 	/* list of sub-directories */
} KesDirectoryRec ;

#define KesClosureOfDirectory(d)	((d)->closure)
#define KesClosureOfDirectorySet(d,p)	((d)->closure = (p))
#define KesNodeOfDirectory(d)		((d)->node)

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

KesDirectory KesDirectoryMake (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer server			/* target server */
    , char      *name			/* name of directory to be created */
#endif
) ;

void KesDirectoryDelete (
#ifdef KEDIT_NeedsFunctionPrototype
      KesDirectory directory		/* directory to delete */
#endif
) ;

KesDirectory KesDirectoryAddChild (
#ifdef KEDIT_NeedsFunctionPrototype
      KesDirectory directory		/* directory to add a child to */
    , KID          subdir_node		/* subdir to add */
#endif
) ;

KesDirectory KesDirectoryRemoveChild (
#ifdef KEDIT_NeedsFunctionPrototype
      KesDirectory directory		/* directory to remove a child from */
    , KID          subdir_node		/* subdir to remove */
#endif
) ;

KesDirectoryFilter KesDirectoryFilterSet (
#ifdef KEDIT_NeedsFunctionPrototype
      KesDirectory       directory
    , KesDirectoryFilter filter
#endif
) ;

KesHandlerSet KesDirectoryHandlerSetGet () ;
      
#ifdef __cplusplus 
}
#endif /* __cplusplus */


#define KesDomainMake(s,i) \
    KesDirectoryAddChild((KesDirectory) (s)->fake_root_info.closure,(i))
#define KesDomainDelete(s,i) \
    KesDirectoryRemoveChild((KesDirectory) (s)->fake_root_info.closure, (i))

/***************************************************************************
 * Notifcation management functions.
 ***************************************************************************/

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

void KesNodeNotifyClientKillHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesClient client			/* client to be killed */
#endif
) ;

KeStatus KesNodeNotifySet (
#ifdef KEDIT_NeedsFunctionPrototype
      KesClient     client		/* client to add to notify DB */
    , KID           node		/* node */
    , unsigned long mask		/* new mask */
#endif
) ;

KeStatus KesNodeNotifyRemoveById (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer server			/* server the id belongs to */
    , KID       node			/* node to be unnotified */
#endif
) ;

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#define KesNodeNotifyAccept(r) \
    KesNodeNotifySet ( KesClientOfRequest (r) \
		      , (r)->node_id \
		      , (r)->mask)
    
#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

int KesDeleteNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that triggered event */
    , KID           node		/* deleted node id */
#endif
) ;

int KesCreateNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that trigerred action */
    , KID           father		/* father of created node */
    , KID           node		/* created node */
#endif
) ;

int KesNotifyNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that triggered event */
    , KID           node		/* node that has beeen selected */
    , unsigned long mask		/* new notification mask */
#endif
) ;

int KesExecNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that triggered event */
    , KID           node		/* node that has beeen execed */
#endif
) ;

int KesReadNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that trigered event */
    , KID           node		/* node that has been read */
#endif
) ;

int KesWriteNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that triggered event */
    , KID           node		/* node that has been written */
#endif
) ;

int KesReadWithValueNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node belongs to */
    , unsigned long client_tag		/* client that triggered the read */
    , KID           node		/* node that has been read */
    , unsigned int  size		/* size of read value */
    , KePointer     value		/* bytes of read value */
#endif
) ;

int KesWriteWithValueNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer     server		/* server the node bel;ongs to */
    , unsigned long client_tag		/* client that triggered write */
    , KID           node		/* node that has been written */
    , unsigned int  size		/* size of written value */
    , KePointer     value		/* bytes of written value */
#endif
) ;

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* _KS_H_ */
