/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KesProto.h
 *
 * Prototypes typedefs. A rptotype is a container of string values. No
 * other types are handled in a prototype for the sake of simplicity (use
 * Xtif you want fancy customizable objects, see Prunet Rmo's).
 ***************************************************************************/


#ifndef _KES_PROTO_H_
#define _KES_PROTO_H_

struct _KesPrototypeInstance ;

/*
 * A prototype slot description ;
 */

typedef struct _KesPrototypeSlot {
    unsigned char 		*name ;		/* slot name */
    unsigned long		u_mode ;	/* user access mode */
    unsigned long		o_mode ;	/* other access mode */
} KesPrototypeSlotRec, *KesPrototypeSlot ;

/*
 * What is a prototype class:
 * It just contains all the necessary informations to create an instance
 */

typedef KeStatus (*KesPrototypeHook) (struct _KesPrototypeInstance *instance
				      , unsigned char *name) ;

typedef struct _KesPrototypeClass {
    KesServer 			 server ;	/* this class server ctxt */
    KeNodeStat			 stat ;		/* export node */
    unsigned char		 *name ;	/* class name */
    int				 num_slot ;	/* # of slots for the class */
    KesPrototypeSlot		 *slots ;	/* slot descriptions */
    struct _KesPrototypeInstance *instances ;	/* list of instances */
    KesPrototypeHook		 get_hook ;	/* get hook */
    KesPrototypeHook		 set_hook ;	/* set hook */
    KesPrototypeHook		 exec_hook ;	/* exec hook */
    KesPrototypeHook		 notify_hook ;	/* notify hook */
} KesPrototypeClassRec, *KesPrototypeClass ;

typedef struct _KesPrototypeSlotValue {
    KID				 id ; 		/* id as exported node */
    unsigned int 		 size ;		/* value size */
    unsigned char		 *value ;	/* this slot value */
    struct _KesPrototypeInstance *instance ;	/* this may seem funny ! */
} KesPrototypeSlotValueRec, *KesPrototypeSlotValue ;

typedef struct _KesPrototypeInstance {
    struct _KesPrototypeInstance *next ;	/* class list of instances */
    KesPrototypeClass		 class ;	/* prototype class */
    KesClient			 owner ;	/* owner of this instance */
    unsigned char		 *name ;	/* instance name */
    KesPrototypeSlotValue	 values ;	/* array of ValueRec */
} KesPrototypeInstanceRec, *KesPrototypeInstance ;

/*
 * Macros :
 */

#define KesPrototypeClassSetExecHook(c,h)	((c)->exec_hook = (KesPrototypeHook) h)

/*
 * Prototypes :
 */

KesPrototypeClass KesPrototypeClassMake (KesServer server
					 , unsigned char *name
					 , int num_slot
					 , KesPrototypeSlotRec slots[] ) ;

KesPrototypeInstance KesPrototypeInstanceMake (unsigned char *name
					       , KesPrototypeClass class
					       , KesClient owner) ;
					       
void KesPrototypeInstanceFree (KesPrototypeInstance proto) ;

KeStatus KesPrototypeInstanceSetValue (KesPrototypeInstance prototype
				       , unsigned char *slot
				       , unsigned int size
				       , unsigned char *value) ;
KeStatus KesPrototypeInstanceGetValue (KesPrototypeInstance prototype
				       , unsigned char *slot
				       , unsigned int *psize 
				       , unsigned char **pval) ;
				       
#endif /* _KES_PROTO_H_ */
