/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KesServerP.h
 *
 * This is a private include for the server library. It includes all 
 * the unexported prototypes and typedefs. All these definitions, are
 * preeceded by _.
 ***************************************************************************/

#ifndef _KES_SERVER_P_H
#define _KES_SERVER_P_H

/*
 * K-Edit, Ice related static variables (HATE)
 */

extern int       _IceKeditPaProtocolNumber ;
extern KesClient _KesClientIndex[] ;

/*
 * Private macros:
 */

#define _IceConnToClient(i)     (_KesClientIndex[IceConnectionNumber(i)])
#define _DescriptorToClient(fd) (_KesClientIndex[(fd)])

/*
 * Identifier management function and opaque types. 
 * An identifier is a 32 bits word, having the 16 high ones used as a hashkey in
 * the server 'nodes' hashtable, and the 16 low ones used as a "uniquifier", so
 * that a same identifier is never assigned to different objects, even if nodes
 * are created/deleted.
 *
 * - KES_INDEX_BITS       : part reserved for the uniquifier in node id.
 * - KES_KEY_BITS         : part reserved for hash value in node id
 * - KES_KEY_RANGE        :  size of key part
 * - KES_INDEX_RANGE      : size of index part
 * - KES_NODESET_STARTSIZE: creationsize for the nodes array
 */

#define KES_INDEX_BITS		0x0000FFFF
#define KES_KEY_BITS		(~KES_INDEX_BITS)

#define KES_KEY_RANGE		0x0000FFFF
#define KES_INDEX_RANGE		0x0000FFFF

#define KES_NODESET_STARTSIZE	256

#define KesNodeIndex(i)     ((i)  & KES_INDEX_BITS)
#define KesNodeKey(i)       (((i) & KES_KEY_BITS) >> 16)
#define KesNodeCompose(i,k) (((i) & KES_INDEX_BITS) | (((k) & KES_KEY_RANGE) << 16))

#define KesClosureHash(c)		(((unsigned long) c) >> 2)


typedef struct _KesNodeInfo {
    struct _KesNodeInfo 	*next ;		  /* collision list */
    KesHandlerSet		handler_set ;	  /* supporting handlers */
    KePointer			closure ;	  /* associated closure */
    int				count ;		  /* # of ids (if range) */
    KID				node ;		  /* base node id */
} KesNodeInfoRec ;

typedef struct _KesHandlerSet {
    KesLsHandler		ls_handler ;	  /* ls handler */
    KesStatHandler		stat_handler ;	  /* stat node handler */
    KesGetHandler		get_handler ;	  /* get handler */
    KesSetHandler		set_handler ;	  /* set handler */
    KesExecHandler		exec_handler ;	  /* exec handler */
    KesNotifyHandler    	notify_handler ;  /* notify handler */
} KesHandlerSetRec ;

/*
 * Internal prototypes:
 */

void _KesIOErrorHandler (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer server				  /* server of error */
    , KesClient client				  /* client of error if any */
    , int       error				  /* errno */
#endif
) ;

/*
 * From KSIce.c
 */

void _KesWriteReplyToClient (
#ifdef KEDIT_NeedsFunctionPrototype
    KesReply reply			/* reply to write (contains dest) */
#endif
) ;


#endif /* _KES_SERVER_P_H_ */
