/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * Kproto.h
 *
 * Kedit wire protocol event description, a try.
 ***************************************************************************/

#ifndef _KPROTO_H_
#define _KPROTO_H_

#include <X11/Xos.h>
#include <X11/Xfuncs.h>
#include <X11/Xmd.h>

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

/*
 * Protocol description:
 *
 * - define _NEEDS_KEDIT_PO_DESCRIPTION_ if you want ICE k-Edit PO description
 * - define _NEEDS_KEDIT_PA_DESCRIPTION_ if you want ICE k-Edit PA description
 */


#define KEDIT_MajorNumber	1
#define KEDIT_MinorNumber	0
#define KEDIT_ProtocolName	"K-Edit"
#define KEDIT_Vendor		"None"
#define KEDIT_Release		"K-Edit 1.0"

#ifdef _NEEDS_KEDIT_PO_DESCRIPTION_

void _IceKeditPOProcessMessage_1_0 (
#ifdef KEDIT_NeedsFunctionPrototype
      IceConn ice_conn
    , IcePointer client_data
    , int opcode
    , unsigned long length
    , Bool swap
    , IceReplyWaitInfo *reply_wait
    , Bool *reply_ready
#endif
) ;

static IcePoVersionRec IceKeditPOVersions[] = 
    { { KEDIT_MajorNumber,
	KEDIT_MinorNumber,
        _IceKeditPOProcessMessage_1_0 } } ;

#define KEDIT_PO_NUM_VERSION ((sizeof (IceKeditPOVersions))/sizeof (IcePoVersionRec))

#endif /* _NEEDS_KEDIT_DESCRIPTION_ */

#ifdef _NEEDS_KEDIT_PA_DESCRIPTION_

extern void _IceKeditPAProcessMessage_1_0 () ;

static IcePaVersionRec IceKeditPAVersions[] = 
    { { KEDIT_MajorNumber,
        KEDIT_MinorNumber,
        _IceKeditPAProcessMessage_1_0 } } ;

#define KEDIT_PA_NUM_VERSION ((sizeof (IceKeditPAVersions))/sizeof (IcePaVersionRec))

#endif /* _NEEDS_KEDIT_PA_DESCRIPTION_ */

/*
 * KEDIT Minor opcodes:
 */

#define KEDIT_Error			0
#define KEDIT_Reply			1
#define KEDIT_OpenConnection		2
#define KEDIT_Ls			3
#define KEDIT_Stat			4
#define KEDIT_Get			5
#define KEDIT_Set			6
#define KEDIT_Exec			7
#define KEDIT_Notify			8
#define KEDIT_Event			9
#define KEDIT_Noop			10

#define KEDIT_LastMinor			11

/*
 * Mode for nodes:
 */

#define KEDIT_DirectoryMode 		(1 << 0)
#define KEDIT_ReadMode   		(1 << 1)
#define KEDIT_WriteMode			(1 << 2)
#define KEDIT_ExecMode			(1 << 3)

/*
 * Known statuses :
 */

#define KEDIT_Success		  ((KeStatus) 0)
#define KEDIT_Failure		  ((KeStatus) 1)
#define KEDIT_InvalidId		  ((KeStatus) 2)
#define KEDIT_InvalidOp		  ((KeStatus) 3)
#define KEDIT_InvalidData	  ((KeStatus) 4)
#define KEDIT_InvalidHandlerSet   ((KeStatus) 5)
#define KEDIT_InvalidRangeSize	  ((KeStatus) 6)
#define KEDIT_InvalidHandlerIndex ((KeStatus) 7)
#define KEDIT_BadMalloc		  ((KeStatus) 8)

/*
 * Known event subtypes:
 */

#define KEDIT_DeleteEvent		1	/* node deleted event */
#define KEDIT_CreateEvent 		2	/* node created event */
#define KEDIT_NotifyEvent 		3	/* node select-input mask changed */
#define KEDIT_ExecEvent			4 	/* exec event */
#define KEDIT_ReadEvent 		5	/* node read event */
#define KEDIT_ReadWithValueEvent	6       /* read (with value) */
#define KEDIT_WriteEvent		7	/* node write event */
#define KEDIT_WriteWithValueEvent	8	/* node write (with value) */

/*
 * Event masks:
 */

#define KEDIT_DeleteMask		(1 << 0)
#define KEDIT_CreateMask		(1 << 1)
#define KEDIT_NotifyMask		(1 << 2)
#define KEDIT_ExecMask			(1 << 3)
#define KEDIT_ReadMask			(1 << 4)
#define KEDIT_ReadWithValueMask		(1 << 5)
#define KEDIT_WriteMask			(1 << 6)
#define KEDIT_WriteWithValueMask	(1 << 7)


/***************************************************************************
 * Common request/reply headers
 *
 * Each request begins with the eight following bytes (following Ice):
 *    CARD8	majorOpcode 	; request type
 *    CARD8	minorOpcode  	; request subtype
 *    CARD8	data[2] 	; any data
 *    CARD32	length     	; full length of request in words
 *
 * Each reply begins with the following 8 bytes:
 *    CARD8	majorOpcode 	; should always be K_Reply
 *    CARD8	minorOpcode 	; K_Reply
 *    CARD8	data[2]		; any data
 *    CARD32	length    	; full length of reply in words
 *    CARD32	serial    	; the serial of the request we are replying to.
 *    CARD32    <any stuff>
 *
 ***************************************************************************/


typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	data[2] ;
    CARD32	length B32 ;
} keditRequest ;

/*
 * WARNING:
 * This cannot be a real packet, as it is NOT padded to eight bytes
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	serial B32 ;		/* always needed, in fact */
} keditReply ;
    
typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* always o - ICE */
    CARD16	errorClass ;
    CARD32	length B32 ;
    CARD8	offendingMinorOpcode ;
    CARD8	severity ;
    CARD16	unused ;
    CARD32	offendingSequenceNum B32 ;
    CARD32	status ;		/* K-edit status code */
    CARD32	node ;			/* offending node identifier */
    /* XPCS for error message */
} keditError ;

/***************************************************************************
 * Events encoding.
 * 
 * Every event type begin with:
 *    CARD8	majorOpcode 	; K-edit protocol number
 *    CARD8	minorOpcode	; KEDIT_Event
 *    CARD8	subtype		: event subtype (see below)
 *    CARD32	length 		; full length of request in words
 *    CARD32	node_id   	; the node that triggered the event
 *    CARD32	timestamp 	; server's timestamp of event
 *    CARD32    client_tag	: client that triggered the event (0)
 *    CARD32	<according to event type>
 ***************************************************************************/

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	event_type ;		/* event subtype */
    CARD8	data ;			/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node identifier */
    CARD32	timestamp B32 ;		/* event timestamp */
    CARD32	client_tag B32 ;	/* client that triggered (0 if server) */
    CARD32	unused ;		/* *USED* for create events */
} keditEvent
, keditCreateEvent, keditDeleteEvent
, keditExecEvent
, keditReadEvent, keditWriteEvent ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	subtype ;		/* event subtype */
    CARD8	data ;			/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node identifier */
    CARD32	timestamp B32 ;		/* event timestamp */
    CARD32	client_tag B32 ;        /* client */
    CARD16	mask ;			/* new input mask */
    CARD16	unused ;
} keditNotifyEvent ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	subtype ;		/* event subtype */
    CARD8	data ;			/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node identifier */
    CARD32	timestamp B32 ;		/* event timestamp */
    CARD32	client_tag B32 ;        /* client */
    CARD32	size ;			/* size of following data */
    /* <size> data bytes */
} keditReadWithValueEvent, keditWriteWithValueEvent ;

/***************************************************************************
 * Request/Reply encoding
 *
 * Following is the list of request/reply pairs:
 *     . keditOpenConnectionReq 	keditOpenConnectionReply
 *     . keditLsReq			keditLsReply 
 *     . keditGetReq			keditGetReply
 *     . keditSetReq			-- asynchronous --
 *     . keditExecReq			-- asynchronous --
 *     . keditNotifyReq			-- asynchronous --
 ***************************************************************************/

/*
 * KEDIT_OpenConnection request:
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* KEDIT_OpenConnection */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
} keditOpenConnectionRequest ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32  ;		/* in 8 bytes */
    CARD32	client B32  ;		/* server's client identifier */
    CARD32	node B32 ;		/* root identifier for this server */
} keditOpenConnectionReply ;

/*
 * KEDIT_LsRequest:
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* KEDIT_Ls */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32  ;		/* in 8 bytes */
    CARD32	node B32  ;		/* node identifier to List */
    CARD32	unused B32 ;		/* unused (pad) to 8 bytes */
} keditLsRequest ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	serial B32 ; 
    CARD32	num_node B32 ;		/* # of children node */
    /* followed by a ListOfCARD32 and optional pad */
} keditLsReply ;

/*
 * KEDIT_Stat :
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Stat */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32  ;		/* in 8 bytes */
    CARD32	node B32  ;		/* node identifier to Stat */
    CARD32	unused B32 ;		/* unused (pad to eight bytes) */
} keditStatRequest ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	serial B32 ;
    CARD16	mode ;			/* the node mode */
    CARD16	mask ;			/* mask of supported select-input */
    CARD32	node ;			/* that has been stat, for convenience */
    CARD32	size ;
    CARD32      info ;			/* linked node, for additional info */
    CARD32      unused32 ;
    /* XPCS string for node name */
} keditStatReply ;

/*
 * KEDIT_Get request :
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Get */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node identifier to get */
    CARD32	unused B32 ;
} keditGetRequest ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	serial B32 ;
    CARD32	size B32 ;		/* exact # of following bytes value */
    /* size bytes of data */
} keditGetReply ;

/*
 * KEDIT_Set request :
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node identifier */
    CARD32	size B32 ;		/* data size */
    /* size data bytes for value */
} keditSetRequest ;

/*
 * Exec request :
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node to be executed */
    CARD32	unused B32 ;		/* unused data */
} keditExecRequest ;

/*
 * Notify request :
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	node B32 ;		/* node identifier */
    CARD16	mask ;			/* select-input mask */
    CARD16	unused ;		/* pad */
} keditNotifyRequest ;

/*
 * Noop request (usefull fro synchronization) :
 */

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
} keditNoopRequest ;

typedef struct {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;		/* should always be KEDIT_Reply */
    CARD8	data[2] ;		/* unused */
    CARD32	length B32 ;		/* in 8 bytes */
    CARD32	serial B32 ;
    CARD32	unused B32 ;
} keditNoopReply ;
    
#endif /* _KPROTO_H_ */
