/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * khash.h
 *
 * Simple hash table management functions.
 ***************************************************************************/


#ifndef _HASHTABLE_H_
#define _HASHTABLE_H_

typedef struct _Hashslot {
    struct _Hashslot 	*next ;		/* collision handling */
    void		*item ;		/* item at this entry */
} HashslotRec, *Hashslot ;

typedef struct _KeHashtable {
    unsigned long	(*hash_fct)() ;	/* hash function on those items */
    int			(*cmp_fct)() ;	/* compare function on those items */
    unsigned int	size ;		/* # of entries in table */
    unsigned int	item_count ;	/* # of items actually stored */
    unsigned int	mode ;		/* mode of the hashtable */
    Hashslot		*slots ;	/* table of slots */
    Hashslot		current_slot ;	/* current lookup value (FindNext) */
} KeHashtableRec, *KeHashtable ;

/*
 * Various defines for the hashtable 'mode' field :
 */

#define HASH_OVERRIDE		1

/*
 * Macros to access slots :
 */

#define KeHashtableSize(h)	 ((h)->size)
#define KeHashtableCount(h)	 ((h)->item_count)

#define KeHashtableOverrideMode(h) ((h)->mode |= HASH_OVERRIDE)

/*
 * Prototypes :
 */

KeHashtable KeHashtableMake ( unsigned int size
			 , unsigned long (*hash_fct)()
			 , int (*cmp_fct)()) ;

void *KeHashtableAdd  (KeHashtable hashtable, void *item) ;
void *KeHashtableRemove (KeHashtable hashtable, void *item) ;
void *KeHashtableFind (KeHashtable hashtable, void *item) ;
void *KeHashtableFindNext (KeHashtable hashtable) ;
int  KeHashtableMap (KeHashtable hashtable, void (*func)(), void *closure) ;

int  KeHashtableKeyedMap (
#ifdef KEDIT_NeedsFunctionPrototype
      KeHashtable hashtable
    , void      *key
    , void      (*func)()
    , void      *closure
#endif
) ;

#endif /* _HASHTABLE_H_ */
