/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSTStruct.c
 *
 * This file is the struct interface on top of the raw K-edit primitives.
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <KeServer.h>
#include <KesCodec.h>
#include <KeStruct.h>

static KesHandlerSet KesStructHandlerSet = (KesHandlerSet) 0;
static KesHandlerSet KesSlotHandlerSet   = (KesHandlerSet) 0 ;

static char *ErrBadOp    = "Slot doesn't support this operation." ;
static char *ErrSlotInfo = "Bad slot ID." ;
static char *ErrDecoder  = "Decoder failed." ;
static char *ErrEncoder  = "Encoder failed." ;

static KesRequest theRequest = (KesRequest) 0 ;

#define SET_CURREQ(r)	{ theRequest = (KesRequest) (r) ; }
#define UNSET_CURREQ()  { theRequest = (KesRequest) 0 ; } 

/***************************************************************************
 * KesStructHandlerSet defines the generic structure callback set:
 * KesStructLsHandler    : handle ls request
 * KesStructStatHandler  : handle stat request
 * KesStructNotifyHandler: handle notification requests
 *
 * These are the only operations supported by struct.
 ***************************************************************************/

static KeStatus KesStructLsHandler (request, reply)
    KesLsReq request ;
    KesReply reply ;

{
    KesStructClass    kclass ;
    KesStruct         object ;
    int               i ;
    KID		      *slots ;

/*
 * Get the informations from request:
 */

    object = (KesStruct) request->closure ;
    kclass = object->kclass ;

/*
 * Perform :
 */
    
    slots = (KID *) KesMalloc (request->server
			       , sizeof (KID) * kclass->accessors_size) ;

    for ( i = 0 ; i < kclass->accessors_size ; i++ )
	slots[i] = object->slot + i ;

    KesLsReplyFill (reply, kclass->accessors_size, slots, KeFree) ;

    return (KEDIT_Success) ;
}

static KeStatus KesStructStatHandler (request, reply)
    KesStatReq request ;
    KesReply   reply ;
    
{
    KesStructClass    kclass ;
    KesStruct         object ;

/*
 * Get infos from request:
 */
    
    object = (KesStruct) KesClosureOfRequest (request) ;
    kclass = object->kclass ;

/*
 * Perform:
 */

    KesStatReplyFillStatic (reply
			    , object->name
			    , KEDIT_DirectoryMode
			    , object->mask
			    , (KID) 0
			    , 0) ;

    return (KEDIT_Success) ;
}

static KeStatus KesStructNotifyHandler (request, reply)
    KesNotifyReq request ;
    KesReply     reply ;

{
    KesStruct object = (KesStruct) KesClosureOfRequest (request) ;

    if ( object->mask & request->mask == request->mask )
	KesNodeNotifySet (request->client, request->node, request->mask) ;
    else
	KesErrorReplyFillStatic (reply
				 , KEDIT_InvalidOp
				 , "Invalid notification mask.") ;
    return (KEDIT_Success) ;
}

KesHandlerSet KesStructHandlerSetGet ()

{
    if ( ! KesStructHandlerSet )
    {
	KesStructHandlerSet = KesHandlerSetMake () ;
	
	KesHandlerSetLsHandler   (KesStructHandlerSet
				  , KesStructLsHandler) ;
	KesHandlerSetStatHandler (KesStructHandlerSet
				  , KesStructStatHandler) ;
	KesHandlerSetNotifyHandler (KesStructHandlerSet
				    , KesStructNotifyHandler) ;
    }

    return (KesStructHandlerSet) ;
}

/***************************************************************************
 * KesSlotHandlerSet: struct slot manipulation functions
 *
 ***************************************************************************/

KeStatus KesStructSlotNotify (object, client_tag, accessor, mask)
    KesStruct         object ;
    unsigned long     client_tag ;
    KesStructAccessor accessor ;
    unsigned long     mask ;

{
    KID      	   node = object->slot + (accessor-object->kclass->accessors) ;
    KeStatus 	   status ;
    char     	   *emsg ;
    KeErrorFreeFct efree ;

    if ( accessor->mask & KEDIT_WriteMask & mask )
	KesWriteNotify (object->server, client_tag, node) ;

    if ( accessor->mask & KEDIT_WriteWithValueMask & mask )
    {
	KePointer value, coded_value ;
	int       size ;
	char *location = ((char *) object->pointer) + accessor->offset ;

	if ( ! accessor->get )
	{
	    value = (KePointer) location ;
	}
	else
	{
	    char           *emsg ;
	    KeErrorFreeFct efree ;

	    if (status = (accessor->get)(object,accessor,&value,&emsg,&efree)){
		if ( efree && emsg ) 
		    (*efree) (emsg) ;
		return (status) ;
	    }
	}

	if ( status = KeCodecEncode (accessor->codec
				     , value
				     , &size
				     , &coded_value
				     , &emsg, &efree) ) {
	    if ( efree && emsg ) 
		(*efree) (emsg) ;
	    return (status) ;
	}

	KesWriteWithValueNotify (object->server
				 , client_tag
				 , node
				 , size
				 , coded_value) ;
	    
	KeCodecEncoderFree (accessor->codec, coded_value) ;
    }

    if ( accessor->mask & KEDIT_ExecMask & mask ) {
        KesExecNotify (object->server, client_tag, node) ;
    }

    return (KEDIT_Success) ;
}

static KeStatus KesSlotInfoFromRequest (request, paccessor)
    KesLsReq          request ;
    KesStructAccessor *paccessor ;

{
    KesStruct object = (KesStruct) request->closure ;
    int       position ;

    object   = (KesStruct) KesClosureOfRequest (request) ;
    
    position = KesNodeRangeIndex (request->node, object->slot) ;

    if ( position >= object->kclass->accessors_size )
	return (KEDIT_InvalidId) ;

    *paccessor = object->kclass->accessors + position ;

    return (KEDIT_Success) ;
}

static KeStatus KesSlotStatHandler (request, reply)
    KesStatReq request ;
    KesReply   reply ;

{
    KesStruct         object = (KesStruct) request->closure ;
    KesStructAccessor accessor ;
    KID               codec_node ;
    int               size ;
    KeStatus	      status ;

    SET_CURREQ (request) ;
    
    if ( status = KesSlotInfoFromRequest (request, &accessor) )
	return (status) ;

/*
 * Now, access the field to eventually compute its size:
 */

    size = -1 ;
    if ( accessor->get ) {
	KePointer      value ;
	char           *emsg ;
	KeErrorFreeFct efree ;

	if ( ! (accessor->get) (object, accessor, &value, &emsg, &efree) ) {
	    KeCodecSize (accessor->codec, value, &size) ;
	} else if ( efree && emsg ) {
	    (*efree) (emsg) ;
	}
    } else {
	char *location = ((char *) object->pointer) + accessor->offset ;
	
	KeCodecSize (accessor->codec, (KePointer) location, &size) ;
    }
	
    if ( status = KesCodecRegister (request->server
				    , accessor->codec
				    , &codec_node) ) {
	return (status) ;
    }

    KesStatReplyFillStatic (reply
			    , accessor->name
			    , accessor->mode
			    , accessor->mask
			    , (KID) codec_node
			    , size) ;

    UNSET_CURREQ () ;
	
    return (KEDIT_Success) ;
}

static KeStatus KesSlotNotifyHandler (request, generic_reply)
    KesNotifyReq request ;
    KesReply     generic_reply ;

{
    KesStruct         object = (KesStruct) request->closure ;
    KesStructAccessor accessor ;
    KeStatus          status ;

    SET_CURREQ (request) ;
    
    if ( status = KesSlotInfoFromRequest (request, &accessor) ) {
	KesErrorReplyFillStatic (generic_reply, status, ErrSlotInfo) ;
	return (KEDIT_Success) ;
    }

/*
 * Perform only allowed notification :
 */

    if ( (accessor->mask & request->mask) == request->mask ) {
	if ( accessor->mask & KEDIT_NotifyMask )
	    KesNotifyNotify (request->server
			     , request->client->client_tag
			     , request->node
			     , request->mask) ;
	KesNodeNotifySet (request->client, request->node, request->mask) ;
    } else {
	KesErrorReplyFillStatic (generic_reply
				 , KEDIT_InvalidOp
				 , "Node doesn't support this notification.") ;
    }
    UNSET_CURREQ () ;
    return (KEDIT_Success) ;
}

static KeStatus KesSlotSetHandler (request, reply)
    KesSetReq request ;
    KesReply  reply ;

{
    KesStruct         object = (KesStruct) request->closure ;
    KesStructAccessor accessor ;
    KeStatus          status ;
    char 	      *location ;
    char	      *emsg ;
    KeErrorFreeFct    efree ;

    SET_CURREQ (request) ;
    if ( status = KesSlotInfoFromRequest (request, &accessor) ) {
	KesErrorReplyFillStatic (reply, status, ErrSlotInfo) ;
	return (KEDIT_Success) ;
    }
    if ( ! ( accessor->mode & KEDIT_WriteMode ) ) {
	KesErrorReplyFillStatic (reply, KEDIT_InvalidOp, ErrBadOp) ;
	return (KEDIT_Success) ;
    }
    location = ((char *) object->pointer) + accessor->offset ;

/*
 * Check mode, and perform :
 * - Decode
 * - Set
 * - Notify
 */
    
    if ( accessor->set )
    {
	KePointer      value = (KePointer) 0 ;

	if ( status = KeCodecDecode (accessor->codec
				     , request->size
				     , (KePointer) request->value
				     , KesClientSwapped (request->client)
				     , &value
				     , &emsg, &efree) ) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (KEDIT_Success) ;
	}

	if (status = (accessor->set)(object, accessor, value, &emsg, &efree)) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (KEDIT_Success) ;
	}
    }
    else
    {
	if ( status = KeCodecDecode (accessor->codec
				     , request->size
				     , (KePointer) request->value
				     , KesClientSwapped (request->client)
				     , (KePointer *) &location
				     , &emsg, &efree) ) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (KEDIT_Success) ;
	}
    }

/*
 * Ok, slot has been set succesfully, perform notifications:
 */

    if ( accessor->mask & KEDIT_WriteMask )
	KesWriteNotify (request->server
			, request->client->client_tag
			, request->node) ;

    if ( accessor->mask & KEDIT_WriteWithValueMask )
    {
	KePointer value, coded_value ;
	int       size ;

	if ( ! accessor->get ) {
	    value = (KePointer) location ;
	} else {
	    if (status = (accessor->get) (object,accessor,&value,&emsg,&efree))
		return (status) ;
	}

	if ( status = KeCodecEncode (accessor->codec
				     , value
				     , &size
				     , &coded_value
				     , &emsg, &efree) ) {
	    if ( efree && emsg ) 
		(*efree) (emsg) ;
	    return (status) ;
	}

	KesWriteWithValueNotify (request->server
				 , request->client->client_tag
				 , request->node
				 , size
				 , coded_value) ;
	    
	KeCodecEncoderFree (accessor->codec, coded_value) ;
    }
    UNSET_CURREQ () ;
    return (KEDIT_Success) ;
}


static KeStatus KesSlotGetHandler (request, reply) 
    KesGetReq request ;
    KesReply  reply ;

{
    KesStruct           object = (KesStruct) request->closure ;
    KesStructAccessor   accessor ;
    int                 size ;
    KePointer           value ;
    KeStatus            status ;
    char		*emsg ;
    KeErrorFreeFct      efree ;

    SET_CURREQ (request) ;
    if ( status = KesSlotInfoFromRequest (request, &accessor) ) {
	KesErrorReplyFillStatic (reply, status, ErrSlotInfo) ;
	return (KEDIT_Success) ;
    }

    if ( ! ( accessor->mode & KEDIT_ReadMode ) ) {
	KesErrorReplyFillStatic (reply, KEDIT_InvalidOp, ErrBadOp) ;
	return (KEDIT_Success) ;
    }

/*
 * Perform the Get :
 */

    if ( accessor->get )
    {
	int       size ;
	KePointer value, coded_value ;

	if (status = (accessor->get) (object,accessor,&value,&emsg,&efree)) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (KEDIT_Success) ;
	}
	if ( status = KeCodecEncode (accessor->codec
				     , value
				     , &size
				     , &coded_value
				     , &emsg, &efree) ) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (KEDIT_Success) ;
	}

	KesGetReplyFill (reply
			 , size
			 , coded_value
			 , (KesReplyFreeFct) KeCodecEncoderFreeer (accessor->codec)) ;

    }
    else
    {
	char *location = ((char *) object->pointer) + accessor->offset ;

	if ( status = KeCodecEncode (accessor->codec
				     , (KePointer) location
				     , &size
				     , &value
				     , &emsg, &efree) ) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (KEDIT_Success) ;
	}

	KesGetReplyFill (reply
			 , size
			 , value
			 , (KesReplyFreeFct) KeCodecEncoderFreeer (accessor->codec)) ;
    }

/*
 * Ok set is done, notify:
 */

    if ( accessor->mask & KEDIT_ReadMask )
	KesReadNotify (request->server
		       , request->client->client_tag
		       , request->node) ;
    
    if ( accessor->mask & KEDIT_ReadWithValueMask )
	KesReadWithValueNotify (request->server
				, request->client->client_tag
				, request->node
				, size
				, value) ;
    UNSET_CURREQ () ;
    return (KEDIT_Success) ;
}

static KeStatus KesSlotExecHandler (request, reply)
    KesExecReq request ;
    KesReply   reply ;

{
    KesStruct         object = (KesStruct) request->closure ;
    KesStructAccessor accessor ;
    KeStatus          status ;
    char	      *emsg ;
    KeErrorFreeFct    efree ;

    SET_CURREQ (request) ;
    if ( status = KesSlotInfoFromRequest (request, &accessor) ) {
	KesErrorReplyFillStatic (reply, status, ErrSlotInfo) ;
	return (KEDIT_Success) ;
    }
    if ( ( ! ( accessor->mode & KEDIT_ExecMode ) )
	|| ( ! accessor->exec ) ) {
	KesErrorReplyFillStatic (reply, KEDIT_InvalidOp, ErrBadOp) ;
	return (KEDIT_Success) ;
    }

    if ( status = (accessor->exec) (object, accessor, &emsg, &efree) ) {
	KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
    } else {
	KesExecNotify (request->server
		       , request->client->client_tag
		       , request->node) ;
    }
    UNSET_CURREQ () ;
    return (KEDIT_Success) ;
}

KesHandlerSet KesSlotHandlerSetGet ()
    
{
    if ( ( ! KesSlotHandlerSet ) 
	&& ( KesSlotHandlerSet = KesHandlerSetMake () ) )
    {
	KesHandlerSetStatHandler   (KesSlotHandlerSet, KesSlotStatHandler) ;
	KesHandlerSetGetHandler    (KesSlotHandlerSet, KesSlotGetHandler) ;
	KesHandlerSetSetHandler    (KesSlotHandlerSet, KesSlotSetHandler) ;
	KesHandlerSetExecHandler   (KesSlotHandlerSet, KesSlotExecHandler) ;
	KesHandlerSetNotifyHandler (KesSlotHandlerSet, KesSlotNotifyHandler) ;
    }

    return ( KesSlotHandlerSet ) ;
}


/***************************************************************************
 * KesStructAccessorDeclare: declare yet-another struct accessor
 * 
 * Add a field accessor to the given StructClass
 *
 * RETURNS: The augmented class, or NULL
 ***************************************************************************/

KesStructAccessor KesStructClassAccessorFind (kclass, name)
    KesStructClass kclass ;
    char           *name ;

{
    KesStructAccessor p ;
    
    for ( p = kclass->accessors 
	 ; p < kclass->accessors + kclass->accessors_size
	 ; p++ )
    {
	if ( ! strcmp (p->name, name))
	    return (p) ;
    }

    return ((KesStructAccessor) 0) ;
}

KesStructClass KesStructAccessorDeclare (kclass
					 , name
					 , mode, mask
					 , offset
					 , codec
					 , set, get, exec)
    KesStructClass        kclass ;
    char 	          *name ;
    unsigned long         mode ;
    unsigned long         mask ;
    int                   offset ;
    KeCodec               codec ;
    KesStructSlotGet	  get ;
    KesStructSlotSet      set ;
    KesStructSlotExec     exec ;

{
    int               size ;
    KesStructAccessor pnew ;

    size = (kclass->accessors_size + 1) * sizeof (KesStructAccessorRec) ;

/*
 * Realloc the accessors array in class:
 */

    if ( kclass->accessors )
	kclass->accessors = (KesStructAccessor)realloc(kclass->accessors,size);
    else
	kclass->accessors = (KesStructAccessor)malloc(size) ;
    
    if ( ! kclass->accessors ) {
	KesStructClassDelete (kclass) ;
	return ((KesStructClass) 0) ;
    }

    /* Fill in the allocated accessor */

    pnew = kclass->accessors + (kclass->accessors_size)++ ;

    pnew->name   = (char *) strdup (name) ;
    pnew->mode   = mode ;
    pnew->mask   = mask ;
    pnew->offset = offset ;
    pnew->codec  = codec ;
    pnew->set    = set ;
    pnew->get    = get ;
    pnew->exec   = exec ;

    return (kclass) ;
}

/***************************************************************************
 * KesStructClassMake: create a C struct encapsulator.
 * KesStructClassDelete: delete a C struct encapsulator
 *
 * This functions creates a suitable C struct encapsulator to export
 * instances of the ddefined type.
 ***************************************************************************/

void KesStructClassDelete (kclass)
    KesStructClass	kclass ;

{
    int               accessors_size = kclass->accessors_size ;
    KesStructAccessor paccessor = kclass->accessors ;

    /* Free accessors resources */
    while ( accessors_size-- )
    {
	free (paccessor->name) ;
	paccessor++ ;
    }

    /* Free the kclass resources, and the kclass */

    free (kclass->name) ;
    if ( kclass->accessors )
	free (kclass->accessors) ;
    free (kclass) ;
    return ;
}

KesStructClass KesStructClassMake (name)
    char *name ;

{
    KesStructClass kclass = (KesStructClass)malloc(sizeof (KesStructClassRec));

    if ( kclass )
    {
	kclass->name           = (char *) strdup (name) ;
	kclass->accessors_size = 0 ;
	kclass->accessors      = (KesStructAccessor) 0 ;
	kclass->instances      = 0 ;
    }

    return (kclass) ;
}

/***************************************************************************
 * KesStructMake: export the given pointer as an instance of given class
 * KesStructDelete: unexport the given pointer
 *
 * This function creates the adequat K-edit nodes. It is up to the caller
 * to "attach" the returned ID in the K-edit hierarchy.
 *
 * RETURN: a KesStruct.
 ***************************************************************************/

void KesStructDelete (object)
    KesStruct object ;

{
    KesServer      server = object->server ;
    KesStructClass kclass = object->kclass ;

    KesNodeFree (server, object->node) ;
    KesNodeRangeFree (server, object->slot) ;
    (kclass->instances)-- ;

    free (object->name) ;
    free (object) ;

    /* Hook here ? might be really usefull */

    return ;
}
    
KesStruct KesStructMake (server, kclass, name, pointer, mask)
    KesServer      server ;
    KesStructClass kclass ;
    char           *name ;
    KePointer	   pointer ;
    unsigned long  mask ;

{
    KesStruct         object = (KesStruct) malloc (sizeof (KesStructRec)) ;

    if ( object )
    {
	/*
	 * Create the object encapsulator:
	 */

	object->kclass  = kclass ;
	object->server  = server ;
	object->name    = (char *) strdup (name) ;
	object->mask    = mask ;
	object->pointer = pointer ;

	/*
	 * Create the related nodes:
	 * - One for the object itself
	 * - A NodeRange for the slots
	 */

	if ( KesNodeMake (server
			  , KesStructHandlerSetGet()
			  , (KePointer) object
			  , &(object->node)) != KEDIT_Success )
	{
	    free (object->name) ;
	    free (object) ;
	    return ((KesStruct) 0) ;
	}

	if ( KesNodeRangeMake (server
			       , KesSlotHandlerSetGet()
			       , (KePointer) object
			       , kclass->accessors_size
			       , &(object->slot)) != KEDIT_Success )
	{
	    KesNodeFree (server, object->node) ;
	    free (object->name) ;
	    free (object) ;
	    return ((KesStruct) 0) ;
	}

	/*
	 * We are done. 
	 */

	(kclass->instances)++ ;
    }

    return (object) ;
}

/***************************************************************************
 * KesStructCurrentRequest()
 *
 * This can be usefull in handler, if the user handler wants full info
 * about the request
 ***************************************************************************/

KesRequest KesStructCurrentRequest ()
{
    if ( ! theRequest ) 
	fprintf (stderr,"k-Edit: KesStructCurrentRequest() not in handler.\n");
    return (theRequest) ;
}

/***************************************************************************
 * KesStructSetValue
 ***************************************************************************/

KesStructSetValue (obj, acc, value)
    KesStruct obj ;
    KesStructAccessor acc ;
    KePointer value ;

{
    char           *emsg ;
    KeErrorFreeFct efree ;
    KeStatus      status ;
    
    if ( acc->set )
    {
	status = (acc->set)(obj, acc, value, &emsg, &efree) ;
	if ( status && efree )
	    (*efree) (emsg) ;
	return (status) ;
    }
    else
    {
	int size ;
	char *location = ((char *) obj->pointer) + acc->offset ;

	if ( status = KeCodecSize (acc->codec, value, &size) )
	    return (status) ;
	bcopy (value, location, size) ;
    }
    KesStructSlotNotify (obj
			 , (unsigned long) 0
			 , acc
			 , KEDIT_WriteMode | KEDIT_WriteWithValueMask) ;
    return (KEDIT_Success) ;
}

/***************************************************************************
 * Get/Set on malloced strings
 ***************************************************************************/

KeStatus KesStructStringGet (object, accessor, value_ret, emsg, efree)
    KesStruct         object ;
    KesStructAccessor accessor ;
    KePointer	      *value_ret ;
    char	      **emsg ;
    KeErrorFreeFct    *efree ;

{
    char **ploc = (char **) (((char *) object->pointer) + accessor->offset) ;

    *value_ret = (KePointer) *ploc ;
    return (KEDIT_Success) ;
}

KeStatus KesStructStringSet (object, accessor, value, emsg, efree)
    KesStruct         object ;
    KesStructAccessor accessor ;
    KePointer         value ;
    char	      **emsg ;
    KeErrorFreeFct    *efree ;

{
    char **ploc = (char **) (((char *) object->pointer) + accessor->offset) ;

    if ( *ploc )
	KeFree (*ploc) ;
    *ploc = (char *) value ;
    return (KEDIT_Success) ;
}

