/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KeStruct.h
 *
 * Type definitions to use the struct interface on top of K-edit
 ***************************************************************************/

#ifndef _KE_STRUCT_H_
#define _KE_STRUCT_H_

#include <KesCodec.h>

/*
 * Forward definitions:
 */

struct _KesStruct ;
typedef struct _KesStruct *KesStruct ;

struct _KesStructAccessor ;
typedef struct _KesStructAccessor *KesStructAccessor ;

/*
 * The various hook definitions
 */

typedef KeStatus (*KesStructSlotGet)(
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object
    , KesStructAccessor accessor 
    , KePointer *value_ret
    , char	**msg_ret
    , KeErrorFreeFct *free_ret 
#endif
) ;

#define KesStructNullSlotGet ((KesStructSlotGet) 0)

typedef KeStatus (*KesStructSlotSet) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object
    , KesStructAccessor accessor
    , KePointer value
    , char	**msg_ret
    , KeErrorFreeFct *free_ret
#endif
) ;

#define KesStructNullSlotSet ((KesStructSlotSet) 0)

typedef KeStatus (*KesStructSlotExec) (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object 
    , KesStructAccessor accessor
    , char	        **msg_ret
    , KeErrorFreeFct    *free_ret 
#endif
) ;

#define KesStructNullSlotExec ((KesStructSlotExec) 0)

/*
 * Accessors contains all the needed information for k-Edit to:
 * - Get/Encode the value from slot
 * - Set/Decode the value from struct
 * - Present the exported node (name, mode, mask)
 */

typedef struct _KesStructAccessor {
    char		  *name ;
    unsigned long	  mode ;
    unsigned long         mask ;
    int			  offset ;
    KeCodec               codec ;
    KesStructSlotSet	  set ;
    KesStructSlotGet      get ;
    KesStructSlotExec     exec ;
} KesStructAccessorRec ;

/*
 * KesStructClass
 * This structure defines the C struct encapsulator.
 */

typedef struct _KesStructClass {
    char 		*name ;		/* the class name */
    int			accessors_size;	/* size of following array */
    KesStructAccessor	accessors ;	/* class accessors */
    int			instances ; 	/* instance count, to prevent free */
} KesStructClassRec, *KesStructClass ;

/*
 * KesStruct
 * The encapsulator
 */

typedef struct _KesStruct {
    KesStructClass kclass ;		/* class ($%#^#$# C++)*/
    KesServer	   server ;		/* server responsible for the object */
    KePointer	   pointer ;		/* to the underlying C-struct */
    char	   *name ;		/* as an exported object */
    unsigned long  mask ;		/* supported by object (not slots) */
    KID		   node ;		/* object node (directory) */
    KID		   slot ;		/* slot base identifier */
} KesStructRec ;
    
/*
 * Well-known macro :
 */

#define KesOffsetOf(t,f)	((unsigned int) &((t *) 0)->f)

/*
 * Function prototypes :
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

KesStructClass KesStructClassMake (
#ifdef KEDIT_NeedsFunctionPrototype
      char *name
#endif
) ;

void KesStructClassDelete (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStructClass kclass
#endif
) ;

KesStructClass KesStructAccessorDeclare (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStructClass   kclass		/* class to add accessor to */
    , char             *name		/* name of accessor */
    , unsigned long    mode		/* access mode */
    , unsigned long    mask		/* notification supported mask */
    , int              offset		/* offset of field in struct */
    , KeCodec          codec		/* codec to be used for this slot */
    , KesStructSlotSet set		/* setter (if any) */
    , KesStructSlotGet get		/* getter if any */
    , KesStructSlotExec exec		/* exec'er if any */
#endif
) ;

KesStructAccessor KesStructClassAccessorFind (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStructClass kclass 
    , char *name
#endif
) ;

KeStatus KesStructSlotNotify (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object 
    , unsigned long client_tag
    , KesStructAccessor accessor
    , unsigned long mask
#endif
) ;

KesStruct KesStructMake (
#ifdef KEDIT_NeedsFunctionPrototype
      KesServer      server		/* target server */
    , KesStructClass kclass		/* class of given pointer */
    , char           *name		/* name under wich to export */
    , KePointer      pointer		/* c struct to be exported */
    , unsigned long  mask		/* notification mask */
#endif
) ;

void KesStructDelete (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object
#endif
) ;

KesRequest KesStructCurrentRequest (
#ifdef KEDIT_NeedsFunctionPrototype
      void
#endif
) ;
				  
/*
 * Specific accessors for malloced strings:
 */

KeStatus KesStructStringSet (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object
    , KesStructAccessor accessor
    , KePointer value
    , char **emsg
    , KeErrorFreeFct *efree
#endif
) ;

KeStatus KesStructStringGet (
#ifdef KEDIT_NeedsFunctionPrototype
      KesStruct object
    , KesStructAccessor accessor
    , KePointer *value_ret
    , char **emsg
    , KeErrorFreeFct *efree
#endif
) ;

/*
 * Provided Handler Sets :
 */

KesHandlerSet KesStructHandlerSetGet () ;

#ifdef __cplusplus
} /* End of C++ verbosities */
#endif /* __cplusplus */
#endif /* _KE_STRUCT_H_ */
