/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * actions.c
 *
 * This is really in alpha^alpha state...
 ***************************************************************************/

#include <stdio.h>
#include <setjmp.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <X11/CoreP.h>

#include <KeServer.h>
#include <KeCodec.h>

#include "KesXtP.h"

#ifdef KESXT_EXPORT_ACTIONS

static KesHandlerSet KesXtActionDirectoryHandlerSet = (KesHandlerSet) 0 ;
static KesHandlerSet KesXtActionHandlerSet          = (KesHandlerSet) 0 ;

static XtActionsRec * _KesXtGetOneAction (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget 
    , int    position 
#endif
) ;

int _KesXtWidgetActionFind (
#ifdef KEDIT_NeedsFunctionPrototype
      Widget widget 
    , char   *name 
#endif
) ;

/***************************************************************************
 * XtFindActionDescription: the *BAD* Xt hack
 *
 * Xt provides no mean to access to full action description, although
 * it does have it internally. This hack go inside the TM to get the 
 * relevant infos.
 * Well, first solution is really too expensive, let's do it simple, and
 * core when needed (this is a prototype ?)
 * Ok, I could avoid those mem copy, by hacking a little. I'm tired of
 * hacking, and want to look at the result.
 ***************************************************************************/

static XtActionsRec *_KesXtGetOneAction (widget, position)
    Widget   widget ;
    int      position ;

{
    static XtActionsRec the_action ;
    WidgetClass         class = XtClass (widget) ;
    XtActionList 	actions_ret ;
    Cardinal     	num_actions ;

    do
    {
	XtGetActionList (class, &actions_ret, &num_actions) ;
    
	if ( position >= num_actions )
	{
	    position -= num_actions ;
	    XtFree ((char *) actions_ret) ;
	}
	else
	{
	    bcopy (actions_ret + position, &the_action, sizeof (the_action)) ;
	    XtFree ((char *) actions_ret) ;
	    return (&the_action) ;
	}
    }
    while ( class = class->core_class.superclass ) ;

    return ((XtActionsRec *) 0) ;
}


int _KesXtWidgetActionCount (widget)
    Widget widget ;

{
    WidgetClass class = XtClass (widget) ;
    XtActionList actions_ret ;
    Cardinal     num_actions ;
    Cardinal     retvalue = 0 ;

    do
    {
	XtGetActionList (class, &actions_ret, &num_actions) ;
	XtFree ((char *) actions_ret) ;
	retvalue += num_actions ;
    }
    while ( class = class->core_class.superclass ) ;

    return (retvalue) ;
}

int _KesXtWidgetActionFind (widget, name)
    Widget widget ;
    char   *name ;

{
    WidgetClass class = XtClass (widget) ;
    XtActionList paction, actions_ret ;
    Cardinal     num_actions ;
    Cardinal     total = 0 ;
    int          i ;

    do
    {
	XtGetActionList (class, &actions_ret, &num_actions) ;
	for ( i = 0, paction = actions_ret ; i < num_actions ; i++, paction++ )
	{
	    if ( ! strcmp (name, paction->string) )
	    {
		XtFree ((char *) actions_ret) ;
		return (total + i) ;
	    }
	}
	XtFree ((char *) actions_ret) ;
	total += num_actions ;
    }
    while ( class = class->core_class.superclass ) ;

    return (-1) ;
}
    
/***************************************************************************
 * KesActionDirectoryHandlerSetGet: management for the *actions* directory
 ***************************************************************************/


static KeStatus KesXtActionDirectoryLsHandler (request, reply)
    KesLsReq request ;
    KesReply reply ;

{
    Widget widget = (Widget) request->closure ;
    int    i, count = _KesXtWidgetActionCount (widget) ;
    KID	   base, *pnodes ;

    if ( KesNodeRangeMake (request->server
			   , KesActionHandlerSetGet()
			   , (KePointer) widget
			   , count
			   , &base) )
    {
	return (KEDIT_Failure) ;
    }

    pnodes = (KID *) KesMalloc (request->server, sizeof (KID) * (count)) ;

    for ( i = 0 ; i < count ; i++ )
	pnodes[i] = base + i ;
    
    KesLsReplyFill (reply, count, pnodes, KeFree) ;

    return (KEDIT_Success) ;
}

static KeStatus KesXtActionDirectoryStatHandler (request, reply) 
    KesStatReq request ;
    KesReply   reply ;

{
    KesStatReplyFillStatic (reply
			    , "*actions*"
			    , KEDIT_DirectoryMode
			    , (unsigned long) 0
			    , (KID) 0
			    , 0) ;
    return (KEDIT_Success) ;
}

KesHandlerSet KesActionDirectoryHandlerSetGet ()

{
    if ( ( ! KesXtActionDirectoryHandlerSet )
	&& ( KesXtActionDirectoryHandlerSet = KesHandlerSetMake () ) )
    {
	KesHandlerSetLsHandler   (KesXtActionDirectoryHandlerSet
				  , KesXtActionDirectoryLsHandler) ;
	KesHandlerSetStatHandler (KesXtActionDirectoryHandlerSet
				  , KesXtActionDirectoryStatHandler) ;
    }

    return (KesXtActionDirectoryHandlerSet) ;
}

/***************************************************************************
 * KesActionHandlerSetGet: action management handler set
 ***************************************************************************/

static KeStatus KesXtActionStatHandler (request, reply)
    KesStatReq request ;
    KesReply   reply ;

{
    Widget       widget = (Widget) request->closure ;
    int          count  = _KesXtWidgetActionCount (widget) ;
    int          position ;
    KID          base, codec_node ;
    KeStatus     status ;
    XtActionsRec *action ;
    
    if ( status = KesNodeRangeMake (request->server
				    , KesActionHandlerSetGet ()
				    , (KePointer) widget 
				    , count
				    , &base) )
    {
	return (status) ;
    }

    position = KesNodeRangeIndex (request->node, base) ;

    if ( ! ( action = _KesXtGetOneAction (widget, position) ) )
	return (KEDIT_Failure) ;

    if ( status = KesCodecRegister (request->server
				    , KeCodecSTRINGGet()
				    , &codec_node) )
    {
	return (status) ;
    }

/*
 * We could add here the WriteWithValue mask wich would send back with the
 * k-Edit event the parameters of the action.
 * As long as we don't provide the event also, this is of no use ??
 */

    KesStatReplyFillStatic (reply
			    , action->string
			    , KEDIT_WriteMode
			    , (KEDIT_WriteMask
			       | KEDIT_NotifyMask)
			    , codec_node
			    , 0) ;

    return (KEDIT_Success) ;
}

static KeStatus KesXtActionSetHandler (request, reply)
    KesSetReq request ;
    KesReply  reply ;

{
    Widget       widget = (Widget) request->closure ;
    XtActionsRec *action ;
    char         *argv[16] ;
    char         *string ;
    Cardinal     argc = 0 ;
    int          position, count ;
    KID          base ;
    KeStatus     status ;
    XEvent       event ;

/*
 * Get action:
 */

    count = _KesXtWidgetActionCount (widget) ;

    if ( status = KesNodeRangeMake (request->server
				    , KesActionHandlerSetGet ()
				    , (KePointer) widget 
				    , count
				    , &base) )
    {
	return (status) ;
    }

    position = KesNodeRangeIndex (request->node, base) ;

    if ( ! ( action = _KesXtGetOneAction (widget, position)) )
    {
	KesErrorReplyFillStatic (reply
				 , KEDIT_Failure
				 , "Couldn't find back the action.") ;
	return (KEDIT_Failure) ;
    }

/*
 * Get args, parse them, and check arity:
 */
    
    if ( request->size )
    {
	char *p ;
	char **pargv ;
	char *emsg ;
	KeErrorFreeFct efree ;

	if ( status = KeCodecDecode (KeCodecSTRINGGet()
				     , request->size
				     , (KePointer) request->value
				     , KesClientSwapped (request->client)
				     , (KePointer *) &string
				     , &emsg, &efree) ) {
	    KesErrorReplyFill (reply, status, emsg, (KesReplyFreeFct) efree) ;
	    return (status) ;
	}

	for ( p = string, pargv = argv, *pargv++ = p ; *p ;  )
	{
	    if ( *p == ',' )
	    {
		*p++     = '\0' ;
		*pargv++ = p ;
	    }
	    else
	    {
		p++ ;
	    }
	}

	argc = pargv - argv ;
    }

    memset ((char *) &event, sizeof (event), 0) ;

    XtCallActionProc (widget
		      , action->string
		      , &event
		      , argv
		      , argc) ;

    return (KEDIT_Success) ;
}

static void _KesXtActionHook (widget, server_ptr, name, event, argv, pargc)
    Widget    widget ;
    XtPointer server_ptr ;
    String    name ;
    XEvent    *event ;
    String    *argv ;
    Cardinal  *pargc ;

{
    KesServer server = (KesServer) server_ptr ;
    int       idx, count = _KesXtWidgetActionCount (widget) ;
    KID	      base ;

/*
 * Find the correponding node (this is really not optimized yet)
 */

    if ( KesNodeRangeMake (server
			   , KesActionHandlerSetGet()
			   , (KePointer) widget
			   , count
			   , &base) != KEDIT_Success )
    {
	return ;
    }
    
    if ( ( idx = _KesXtWidgetActionFind (widget, name) ) < 0 )
	return ;

    KesWriteNotify (server, (unsigned long) 0, (KID) (base + idx)) ;

    return ;
}

static KeStatus KesXtActionNotifyHandlerSet (request, reply)
    KesNotifyReq request ;
    KesReply     reply ;

{
    static hooked = 0 ;
    Widget widget = (Widget) request->closure ;

/*
 * Check for a correct mask:
 */

    if ( ( request->mask & (KEDIT_WriteMask 
			    | KEDIT_NotifyMask) ) != request->mask )
    {
	KesErrorReplyFillStatic (reply
				 , KEDIT_InvalidOp
				 , "This node doesn't support this mask.") ;
	return (KEDIT_InvalidOp) ;
    }

/*
 * Ok, hook Xt if needed, and set notifications:
 */

    if ( ! hooked )
    {
	hooked = 1 ;
	XtAppAddActionHook (XtWidgetToApplicationContext (widget)
			    , _KesXtActionHook
			    , (XtPointer) request->server) ;
    }
    
    KesNotifyNotify (request->server
		     , request->client->client_tag
		     , request->node
		     , request->mask) ;

    KesNodeNotifySet (request->client
		      , request->node
		      , request->mask) ;

    return (KEDIT_Success) ;
}
    
KesHandlerSet KesActionHandlerSetGet ()

{
    if ( ( ! KesXtActionHandlerSet )
	&& ( KesXtActionHandlerSet = KesHandlerSetMake () ) )
    {
	KesHandlerSetStatHandler    (KesXtActionHandlerSet
				     , KesXtActionStatHandler) ;
	KesHandlerSetSetHandler    (KesXtActionHandlerSet
				    , KesXtActionSetHandler) ;
	KesHandlerSetNotifyHandler (KesXtActionHandlerSet
				    , KesXtActionNotifyHandlerSet) ;
    }

    return (KesXtActionHandlerSet) ;
}

#endif /* KESXT_EXPORT_ACTIONS */
