/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * fake-resource.c
 *
 * Widget fake resources management functions.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>

#include <KeServer.h>
#include <KesCodec.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <X11/ObjectP.h>	/* For XtIs<Classname> macros. */
#include <X11/ShellP.h>		/* for Application Shell Widget class. */

#include "KesXtP.h"

/*
 * Error emiited by these functions:
 */

static char *ErrUnrealizedWidget = "Widget isn't realized." ;

static KeStatus KesXtWidgetWindowGet () ;
static KeStatus KesXtWidgetClassNameGet () ;
static KeStatus KesXtWidgetManagedGet () ;
static KeStatus KesXtWidgetManagedSet () ;

typedef struct _KesXtInitialFakeResources {
    char	  *name ;
    char	  *codec_name ;
    unsigned long mode ;
    unsigned long mask ;
    KesXtGetValue get ;
    KesXtSetValue set ;
    KesXtExec     exec ;
} KesXtInitialFakeResourcesRec ;

static KesXtInitialFakeResourcesRec _InitialFakeResources[] = 
{
    {
	"window" ,
        "CARD32",
	KEDIT_ReadMode ,
	0, 
        KesXtWidgetWindowGet,
	KesXtNullSetValue,
	KesXtNullExec
    } ,
    {
	"managed",
	"BYTE",
	KEDIT_ReadMode | KEDIT_WriteMode,
	0 ,
	KesXtWidgetManagedGet,
	KesXtWidgetManagedSet,
	KesXtNullExec
    } ,
    {
	"class",
	"STRING",
	KEDIT_ReadMode,
	0,
        KesXtWidgetClassNameGet,
	KesXtNullSetValue,
	KesXtNullExec
    }
} ;

static KesXtResourceInfo FakeResources = (KesXtResourceInfo) 0 ;
static int               FakeResourcesCount = 0 ;

/***************************************************************************
 * _KesXtFakeResourceInitialize: initialize fake resource list
 ***************************************************************************/

static void _KesXtFakeResourceInitialize ()

{
    int                          i, count, size ;
    KesXtResourceInfo            pinfo ;
    KesXtInitialFakeResourcesRec *iinfo ;

    count = sizeof(_InitialFakeResources) / sizeof(_InitialFakeResources[0]);
    size  = count * sizeof (KesXtResourceInfoRec) ;

    if ( ! ( FakeResources = (KesXtResourceInfo) malloc (size) ) )
	return ;

    iinfo = _InitialFakeResources ;
    pinfo = FakeResources ;

    for ( i = 0 ; i < count ; i++, pinfo++, iinfo++ )
    {
	KeCodec codec = KeCodecGet (iinfo->codec_name) ;

	pinfo->name  = iinfo->name ;
	pinfo->codec = codec ? codec : KeCodecUNDEFGet () ;
	pinfo->mode  = iinfo->mode ;
	pinfo->mask  = iinfo->mask ;
	pinfo->get   = iinfo->get ;
	pinfo->set   = iinfo->set ;
	pinfo->exec  = iinfo->exec ;
    }

    FakeResourcesCount = count ;

    return ;
}

/***************************************************************************
 * KesXtWidgetFakeResourceCount: # of fake resource for given widget
 ***************************************************************************/

int _KesXtWidgetFakeResourceCount (widget)
    Widget widget ;

{
    if ( ! FakeResources )
	_KesXtFakeResourceInitialize () ;

    return (FakeResourcesCount) ;
}

/***************************************************************************
 * _KesXtGetOneFakeResource: get the indexed fake resource for widget
 ***************************************************************************/

KeStatus _KesXtGetOneFakeResource (widget, index, rinfo)
    Widget            widget ;
    int               index ;
    KesXtResourceInfo rinfo ;

{
    int               count = _KesXtWidgetFakeResourceCount (widget) ;
    KesXtResourceInfo the_info ;

    if ( index >= count )
	return (KEDIT_Failure) ;

/*
 * Initialize if not done:
 */

    if ( ! FakeResources )
	_KesXtFakeResourceInitialize () ;

    the_info = FakeResources + index ;

/*
 * Copy to rinfo:
 */

    rinfo->name  = the_info->name ;
    rinfo->codec = the_info->codec ;
    rinfo->mode  = the_info->mode ;
    rinfo->mask  = the_info->mask ;
    rinfo->get   = the_info->get ;
    rinfo->set   = the_info->set ;
    rinfo->exec  = the_info->exec ;

    return (KEDIT_Success) ;
}

/***************************************************************************
 * KesXtWidgetWindowGet: get the window of a widget
 ***************************************************************************/

static KeStatus KesXtWidgetWindowGet (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer         *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    static Window window ;

    if ( XtIsRealized (widget) )
    {
	window     = XtWindow (widget) ;
	*value_ret = (KePointer) &window ;
	return (KEDIT_Success) ;
    }
    *msg_ret  = ErrUnrealizedWidget ;
    *free_ret = (KeErrorFreeFct) 0 ;
    return (KEDIT_Failure) ;
}

/***************************************************************************
 * KesXtWidgetClassNameGet: get the widget class name
 ***************************************************************************/

static KeStatus KesXtWidgetClassNameGet(widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer         *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    WidgetClass class = XtClass (widget) ;

    *value_ret = (KePointer) class->core_class.class_name ;
    return (KEDIT_Success) ;
}

/***************************************************************************
 * KesXtWidgetManagedGet
 * KesXtWidgetManagedSet
 *
 * Manage the 'managed' fake resource
 ***************************************************************************/

static KeStatus KesXtWidgetManagedGet (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer         *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    static BYTE managed ;
    
    if ( XtIsManaged (widget) )
	managed = ((BYTE) 1) ;
    else
	managed = ((BYTE) 0) ;

    *value_ret = (KePointer) &managed ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtWidgetManagedSet (widget,rinfo,value,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      value ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    BYTE managed = *((BYTE *) value) ;

    if ( managed )
	XtManageChild (widget) ;
    else
	XtUnmanageChild (widget) ;

    return (KEDIT_Success) ;
}

