/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * plug.c
 *
 * Stuff needed to glue server/client connections to Xt main loop.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>

#include <KeServer.h>
#include <KesCodec.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include "KesXtP.h"

static XtAppContext KesXtAppContext = (XtAppContext) 0 ;
static XtInputId    InputArray[32]  = { (XtInputId) 0, } ;	
static jmp_buf      KesXtIceJmpBuf ;

/***************************************************************************
 * KesXtServerPlug: plug the given server to Xt main loop
 *
 * Perform the correct AppAddInput. 
 *
 * RETURNS: KEDIT_Success or KEDIT_Failure
 ***************************************************************************/

static Bool _HostBasedAuthDisabled (hostname)
    char *hostname ;
{
    fprintf (stderr, "Allowing host %s to connect.\n", hostname) ;
    return (True) ;
}

static void _KesXtDoIceProcessMessage (ice_conn_ptr, pfd, input)
    XtPointer ice_conn_ptr ;
    int       *pfd ;
    XtInputId *input ;

{
    IceProcessMessages ((IceConn) ice_conn_ptr, NULL, NULL) ;
    return ;
}

static void _KesXtIceIOErrorHandler (ice_conn)
    IceConn ice_conn ;

{
    int idx = IceConnectionNumber (ice_conn) ;

    if ( InputArray[idx] ) {
	XtRemoveInput (InputArray[idx]) ;
	InputArray[idx] = (XtInputId) 0 ;
    }
    IceCloseConnection (ice_conn) ;
    longjmp (KesXtIceJmpBuf, 1) ;
    
    return ;
}

static void _KesXtKillHandler (client)
    KesClient client ;

{
    IceConn ice_conn = client->ice_connection ;
    int     idx = IceConnectionNumber (ice_conn) ;

    if ( InputArray[idx] )
    {
	XtRemoveInput (InputArray[idx]) ;
	InputArray[idx] = (XtInputId) 0 ;
    }
    return ;
}
    
static void _KesXtDoIceAcceptConnection (listen_obj, pfd, input)
    IceListenObj listen_obj ;
    int          *pfd ;
    XtInputId    *input ;

{
    IceAcceptStatus status ;
    IceConn         ice_conn = IceAcceptConnection (listen_obj, &status) ;
    XtInputId       input_id ;
    
    if ( ice_conn )
    {
	input_id = XtAppAddInput (KesXtAppContext
				  , IceConnectionNumber (ice_conn)
				  , (XtPointer) XtInputReadMask
				  , _KesXtDoIceProcessMessage
				  , (XtPointer) ice_conn) ;

	InputArray[IceConnectionNumber(ice_conn)] = input_id ;

	IceSetIOErrorHandler (_KesXtIceIOErrorHandler) ;
    }
}

KeStatus KesXtServerPlug (xt_app, server, root, error_buf)
    XtAppContext xt_app ;
    KesServer    server ;
    Widget       root ;
    jmp_buf	 error_buf ;

{
    IceListenObj *listens ;
    int          i, listen_count ;
    KeCodec      codec ;
    KID          widget_codec ;

/*
 * Initialize ourself :
 */

    if ( KesXtAppContext )
	return ( (KesXtAppContext == xt_app) ? KEDIT_Success : KEDIT_Failure) ;
    else
	KesXtAppContext = xt_app ;

    bcopy ((char *) error_buf
	   , (char *) KesXtIceJmpBuf
	   , sizeof (KesXtIceJmpBuf)) ;

/*
 * Declare predefined known resource types, and WIGDET type
 */

    KesXtResourceTypeInitialize (server) ;

    codec = KeCodecDefine ("WIDGET"
			   , (KeCodecSizer) 0
			   , (KeCodecEncoder) 0
			   , (KeCodecFreeer) 0
			   , (KeCodecDecoder) 0
			   , (KeCodecFreeer) 0) ;

    KesCodecRegister (server, codec, &widget_codec) ;
    

/*
 * ICE listen for connections, and add to Input sources:
 */

    if ( ! IceListenForConnections (&listen_count
				    , &listens
				    , 0
				    , (char *) 0) )
    {
	return (KEDIT_Failure) ;
    }

    for ( i = 0 ; i < listen_count ; i++ )
    {
	fprintf (stderr, "KeXt: listening on %s\n"
		 , IceGetListenConnectionString (listens[i])) ;

	IceSetHostBasedAuthProc (listens[i], _HostBasedAuthDisabled) ;
	XtAppAddInput (xt_app
		       , IceGetListenConnectionNumber (listens[i])
		       , (XtPointer) XtInputReadMask
		       , _KesXtDoIceAcceptConnection
		       , (XtPointer) listens[i]) ;
    }

    free (listens) ;

    KesSetKillHandler (server, _KesXtKillHandler) ;

/*
 * If a root widget is given export it right now:
 */

    if ( root )
    {
	KID node ;

	if ( KesXtExportWidget (server, root, &node) == KEDIT_Success )
	    KesDirectoryAddChild (KesRootDirectoryOfServer (server), node) ;
    }

    return (KEDIT_Success) ;
}
