/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * real-resource.c
 *
 * Various Xt hacks to get the info we need. Changes in Xt should be
 * located to this file only, if API can be preserved.
 * I can't remember the "inspiration" for this file, but I definetely
 * think it owes a lot to editres sources.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>

#include <KeServer.h>
#include <KesCodec.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>	/* for XtRString. */

#include <X11/ObjectP.h>	/* For XtIs<Classname> macros. */
#include <X11/ShellP.h>		/* for Application Shell Widget class. */

#include <X11/Xatom.h>
#include <X11/Xos.h>		/* for strcpy declaration */

#include "KesXtP.h"

/*
 * This structure is used internally to go from widget, xrm to 
 * a suitable KesXtResourceInfo
 */

typedef struct _KesXtRealResourceInfo {
    XrmQuark		xrm_type ;
    XrmQuark		xrm_name ;
    KesXtResourceInfo   info ;
} KesXtRealResourceInfoRec, *KesXtRealResourceInfo ;

/*
 * From the Xt resource type, retreive the k-Edit needed infos:
 */

static KeHashtable KesXtRealToInfo = (KeHashtable) 0 ;

/***************************************************************************
 * KesXtFillRealResourceInfo: from resource description
 *
 * This function uses a hashtable to maintain per resource type accessors
 * for k-Edit. The FillRealResource looks in this hashtable for the 
 * given type, and returns the approriate infomarions to k-Edit.
 * If nothing is found for this type, no operations at all are permitted
 * on given resource.
 *
 ***************************************************************************/

static unsigned long _real_info_hash_fct (info) 
    KesXtRealResourceInfo info ;

{
    return ( ((unsigned long) info->xrm_name)
	    ^ ((unsigned long) info->xrm_type) ) ;
}

static int _real_info_cmp_fct (info1, info2)
    KesXtRealResourceInfo info1, info2 ;

{
    return ( ! ( ( info1->xrm_type == info2->xrm_type )
		&& ( info1->xrm_name == info2->xrm_name ) ) ) ;
}

static void _KesXtFillRealResourceInfo (real_info, info_ret)
    KesXtRealResourceInfo real_info ;
    KesXtResourceInfo     info_ret ;

{
    KesXtResourceInfo        info ;
    KesXtRealResourceInfoRec *found, key ;

/*
 * Look for a match:
 * - On the type first (this is the genral case)
 * - On the resource name
 */

    key.xrm_type     = real_info->xrm_type ;
    key.xrm_name     = (XrmQuark) 0 ;

    if (found  = (KesXtRealResourceInfo) KeHashtableFind (KesXtRealToInfo, &key))
    {
	info = found->info ;
    }
    else 
    {
	key.xrm_name     = real_info->xrm_name ;
	key.xrm_type     = (XrmQuark) 0 ;
	
	found = (KesXtRealResourceInfo) KeHashtableFind (KesXtRealToInfo, &key) ;
    }
	
/*
 * Fill in the info_ret, with valuable infos, or defaults one:
 */

    if ( found )
    {
	info = found->info ;

	info_ret->name  = XrmQuarkToString (real_info->xrm_name) ;
	info_ret->codec = info->codec ;
	info_ret->get   = info->get ;
	info_ret->set   = info->set ;
	info_ret->exec  = info->exec ;
	info_ret->mode  = info->mode ;
	info_ret->mask  = info->mask ;
	return ;
    }
    else
    {
	info_ret->name  = XrmQuarkToString (real_info->xrm_name) ;
	info_ret->codec = (KeCodec) KeCodecUNDEFGet() ;
	info_ret->get   = (KesXtGetValue) 0 ;
	info_ret->set   = (KesXtSetValue) 0 ;
	info_ret->mode  = (unsigned long) 0 ;
	info_ret->mask  = (unsigned long) 0 ;
    }


    return ;
}

KeStatus KesXtRealResourceDefine (xrm_name, xrm_type, info)
    XrmQuark	      xrm_name ;
    XrmQuark          xrm_type ;
    KesXtResourceInfo info ;

{
    KesXtRealResourceInfo real_info ;

/*
 * Initialize if not done:
 */

    if ( ( ! KesXtRealToInfo )
	&& ( ! ( KesXtRealToInfo = KeHashtableMake (TYPEINFO_HASHTABLE_SIZE
						  , _real_info_hash_fct
						  , _real_info_cmp_fct)) ) )
    {
	return (KEDIT_Failure) ;
    }

/*
 * Create the type description, and add it to the hashtable: 
 */

    real_info=(KesXtRealResourceInfo)malloc(sizeof(KesXtRealResourceInfoRec));
    if ( real_info )
    {
	real_info->xrm_name     = xrm_name ;
	real_info->xrm_type     = xrm_type ;
	real_info->info         = info ;
	
	KeHashtableAdd (KesXtRealToInfo, real_info) ;
	
	return (KEDIT_Success) ;
    }

    return (KEDIT_Failure) ;
}

/***************************************************************************
 * _KesXtWidgetRealResourceCount: find number of resource for this widget
 *
 * RETURNS: number of found resources
 ***************************************************************************/

int _KesXtWidgetRealResourceCount (widget)
    Widget widget ;

{
    WidgetClass class  = XtClass (widget) ;
    Widget      parent = XtParent (widget) ;
    int         count  = 0 ;

    if ( class->core_class.num_resources > 0 )
	count += class->core_class.num_resources ;

    if ( ( ! XtIsShell (widget) )
	&& XtIsConstraint (parent)
	&& widget->core.constraints )
    {
	ConstraintWidgetClass c = (ConstraintWidgetClass) XtClass (parent) ;

	if ( c->constraint_class.num_resources > 0 )
	    count += c->constraint_class.num_resources ;
    }

    return (count) ;
}

/***************************************************************************
 * _KesXtGetOneRealResource: retreieve the resource descr at index
 *
 * This function is still pretty basic, I would like to have some kind
 * of per/class hook, so that I can complete the list with resources that
 * are hidden, or difficult to access (text in TextWidget, for example).
 *
 * RETURNS: an array of resource
 ***************************************************************************/

KeStatus _KesXtGetOneRealResource (widget, position, rinfo) 
    Widget            widget ;
    int               position ;
    KesXtResourceInfo rinfo ;

{
    WidgetClass              class  = XtClass (widget) ;
    int                      count = 0 ;
    KesXtRealResourceInfoRec key ;
    XrmResourceList          *winfo ;

/*
 * Get the right XrmResource, according to position. This can be either 
 * a normal resource, or a constraint resource.
 */

    if ( ( count = class->core_class.num_resources ) > 0 )
	count = class->core_class.num_resources ;
    else 
	return (KEDIT_Failure) ;

    if ( position < count )
    {
        winfo = ((XrmResourceList *) class->core_class.resources) + position ;
    }
    else
    {
	ConstraintWidgetClass c ;

	position -= count ;

	if ( XtIsShell (widget)
	    || ( ! ( XtIsConstraint (XtParent (widget)) ) )
	    || ( ! widget->core.constraints ) )
	{
	    return (KEDIT_Failure) ;
	}

	c = (ConstraintWidgetClass) XtClass(XtParent(widget));
	if ( position >=  c->constraint_class.num_resources )
	    return (KEDIT_Failure) ;

	winfo = ((XrmResourceList *) c->constraint_class.resources) + position;
    }

/*
 * Use infos in XrmResource as a key to fill the KesXtResourceInfo:
 */

    key.xrm_name     = (*winfo)->xrm_name ;
    key.xrm_type     = (*winfo)->xrm_type ;

    _KesXtFillRealResourceInfo (&key, rinfo) ;
    return (KEDIT_Success) ;
}
