/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * resource-type.c
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>

#include <KeServer.h>
#include <KesCodec.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>	/* for XtRString. */

#include <X11/ObjectP.h>	/* For XtIs<Classname> macros. */
#include <X11/ShellP.h>		/* for Application Shell Widget class. */

#include <X11/Xatom.h>
#include <X11/Xos.h>		/* for strcpy declaration */

#include "KesXtP.h"

/*
 * Errors emitted by these functions: 
 */

static char *ErrQueryFont          = "Unable to query given font." ;
static char *ErrFontStructToString = "Unable to convert font to string." ;

/*
 * This special structure is used to initialize the default known
 * types:
 */

static KeStatus KesXtBYTESet()      , KesXtBYTEGet() ;
static KeStatus KesXtCARD16Get()    , KesXtCARD16Set() ;
static KeStatus KesXtCARD32Get()    , KesXtCARD32Set() ;
static KeStatus KesXtSTRINGGet()    , KesXtSTRINGSet() ;
#ifdef KES_XT_FONT_AS_FID
static KeStatus KesXtFontStructGet(), KesXtFontStructSet() ;
#else
static KeStatus KesXtFontStructAsStringGet(), KesXtFontAsStringGet() ;
#endif
static KeStatus KesXtStringAsTypedArgSet () ;
static KeStatus KesXtCallCallbackList () ;

typedef struct {
    char	  *name ;
    char          *type_name ;
    char          *codec_name ;
    KesXtGetValue get ;
    KesXtSetValue set ;
    KesXtExec     exec ;
    unsigned long mode ;
    unsigned long mask ;
} KesXtInitialRealResourceRec ;

static KesXtInitialRealResourceRec _InitialRealResources [] = 
{
    {
	(char *) 0,
	"Bool" ,
	"BYTE" ,
	KesXtBYTEGet,
	KesXtBYTESet,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode ,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Boolean",
	"BYTE" ,
	KesXtBYTEGet,
	KesXtBYTESet,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode ,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Cardinal",
	"CARD32",
	KesXtCARD32Get,
	KesXtCARD32Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode ,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Dimension",
	"CARD16",
	KesXtCARD16Get,
	KesXtCARD16Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Int",
	"CARD32",
	KesXtCARD32Get,
	KesXtCARD32Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Position",
	"CARD16",
	KesXtCARD16Get,
	KesXtCARD16Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Short",
	"CARD16",
	KesXtCARD16Get,
	KesXtCARD16Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"String",
	"STRING",
	KesXtSTRINGGet,
	KesXtSTRINGSet,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Window",
	"CARD32",
	KesXtCARD32Get,
	KesXtCARD32Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Pixel",
	"CARD32",
	KesXtCARD32Get,
	KesXtCARD32Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"Colormap",
	"CARD32",
	KesXtCARD32Get,
	KesXtCARD32Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	XtRCallback,
	"UNDEF",
	KesXtNullGetValue,
	KesXtNullSetValue,
	KesXtCallCallbackList,
	KEDIT_ExecMode,
	((unsigned long) 0)
    } ,
#ifdef KES_XT_FONT_AS_FID
    {
	(char *) 0,
	"Font",
	"CARD32",
	KesXtCARD32Get,
	KesXtCARD32Set,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"FontStruct",
	"CARD32",
	KesXtFontStructGet,
	KesXtFontStructSet,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } 
#else /* Should be KES_XT_FONT_AS_STRING */
    {
	(char *) 0,
	"Font",
	"STRING",
	KesXtFontAsStringGet,
	KesXtStringAsTypedArgSet,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } ,
    {
	(char *) 0,
	"FontStruct",
	"STRING",
	KesXtFontStructAsStringGet,
	KesXtStringAsTypedArgSet,
	KesXtNullExec,
	KEDIT_ReadMode | KEDIT_WriteMode,
	((unsigned long) 0)
    } 
#endif /* KES_XT_FONT_AS_FID */
} ;



/***************************************************************************
 * Various getters and setters:
 * KesXtBYTESet/KesXtBYTEGet
 * KesXtCARD16Set/KesXtCARD16Set
 * KesXtCARD32Get/KesXtCARD32Set
 ***************************************************************************/

static KeStatus KesXtBYTESet (widget, rinfo, value, msg_ret, free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      value ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    BYTE byte = *((BYTE *) value) ;

    XtVaSetValues (widget, rinfo->name, byte, NULL) ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtBYTEGet (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    static BYTE byte ;

    XtVaGetValues (widget, rinfo->name, &byte, NULL) ;
    *value_ret = (KePointer) &byte ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtCARD16Set (widget, rinfo, value,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      value ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    CARD16 card16 = *((CARD16 *) value) ;

    XtVaSetValues (widget, rinfo->name, card16, NULL) ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtCARD16Get (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    static CARD16 card16 ;

    XtVaGetValues (widget, rinfo->name, &card16, NULL) ;
    *value_ret = (KePointer) &card16 ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtCARD32Set (widget,rinfo,value,msg_ret,free_ret) 
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      value ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    CARD32 card32 = *((CARD32 *) value) ;

    XtVaSetValues (widget, rinfo->name, card32, NULL) ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtCARD32Get (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    static CARD32 card32 ;

    XtVaGetValues (widget, rinfo->name, &card32, NULL) ;
    *value_ret = (KePointer) &card32 ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtSTRINGSet (widget, rinfo, value,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      value ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    XtVaSetValues (widget, rinfo->name, value, NULL) ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtSTRINGGet (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    XtVaGetValues (widget, rinfo->name, (char **) value_ret, NULL) ;
    return (KEDIT_Success) ;
}

#ifdef KES_XT_FONT_AS_FID
static KeStatus KesXtFontStructSet (widget,rinfo,value,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      value ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    Font        font = *((Font *) value) ;
    XFontStruct *font_struct ;
    
    if ( ! ( font_struct = XQueryFont (XtDisplay(widget), font)) ) {
	*msg_ret  = ErrQueryFont ;
	*free_ret = (KeErrorFreeFct) 0 ;
	return (KEDIT_Failure) ;
    }

    XtVaSetValues (widget, rinfo->name, font_struct, NULL) ;
    return (KEDIT_Success) ;
}

static KeStatus KesXtFontStructGet (widget, rinfo, value_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer	      *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;

{
    static CARD32 font ;
    XFontStruct   *font_struct ;

    XtVaGetValues (widget, rinfo->name, &font_struct, NULL) ;
    font = font_struct->fid ;
    *value_ret = (KePointer) &font ;
    return (KEDIT_Success) ;
}
#else /* KES_XT_FONT_AS_STRING */
static char *FontStructToString (dpy, fs)
    Display     *dpy ;
    XFontStruct *fs ;

{
    static char theFontName[1024] ;	/* must be some define for this one */
    static theFontAtom = None ;
    int i ;

    if ( theFontAtom == None ) {
	if ( (theFontAtom = XInternAtom (dpy, "FONT", False)) == None )
	    return ((char *) 0) ;
    }
    for ( i = 0 ; i < fs->n_properties ; i++ ) {
	if ( fs->properties[i].name == theFontAtom ) {
	    if ( fs->properties[i].card32 != (Atom) -1 ) {
		char *name = XGetAtomName (dpy, fs->properties[i].card32) ;
		if ( name ) {
		    strcpy (theFontName, name) ;
		    return (theFontName) ;
		}
	    }
	    return ((char *) 0) ;
	}
    }
    return ((char *) 0) ;
}

static KeStatus KesXtFontAsStringGet (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget 	      widget ;
    KesXtResourceInfo rinfo ;
    KePointer         *value_ret ;
    char	      **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    Font fid ;
    XFontStruct *fs ;

    XtVaGetValues (widget, rinfo->name, &fid, NULL) ;
    if ( ! (fs = XQueryFont (XtDisplay(widget), fid)) ) {
	*msg_ret  = ErrQueryFont ;
	*free_ret = (KeErrorFreeFct) 0 ;
	return (KEDIT_Failure) ;
    }
    *((char **) value_ret) = FontStructToString(XtDisplay(widget),fs) ;
    XFreeFontInfo ((char **) 0, fs, 0) ;
    if ( *value_ret ) {
	return (KEDIT_Success) ;
    } else {
	*msg_ret = ErrFontStructToString ;
	*free_ret = (KeErrorFreeFct) 0 ;
	return (KEDIT_Failure) ;
    }
    /* NOT REACHED */
}

static KeStatus KesXtFontStructAsStringGet (widget,rinfo,value_ret,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer         *value_ret ;
    char              **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    XFontStruct *fs ;
    
    XtVaGetValues (widget, rinfo->name, &fs, NULL) ;
    if ( *((char **) value_ret) = FontStructToString (XtDisplay(widget),fs) ) {
	return (KEDIT_Success) ;
    } else {
	*msg_ret = ErrFontStructToString ;
	*free_ret = (KeErrorFreeFct) 0 ;
	return (KEDIT_Failure) ;
    }
    /* NOT REACHED */
}

#endif /* KEST_XT_FONT_AS_STRING */

static KeStatus KesXtStringAsTypedArgSet (widget,rinfo,value,msg_ret,free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;
    KePointer 	      value ;
    char              **msg_ret ;
    KeErrorFreeFct    *free_ret ;
{
    XtVaSetValues (widget, XtVaTypedArg
		   , rinfo->name, XtRString
		   , (char *) value, strlen ((char *) value)
		   , NULL) ;
    return (KEDIT_Success) ;
}


static KeStatus KesXtCallCallbackList (widget, rinfo, msg_ret, free_ret)
    Widget            widget ;
    KesXtResourceInfo rinfo ;

{
    XtCallCallbacks (widget, rinfo->name, (XtPointer) 0) ;
    return (KEDIT_Success) ;
}

/***************************************************************************
 * KesXtResourceTypeInitialize: initialize the predef type
 *
 * RETURN: void.
 ***************************************************************************/

void KesXtResourceTypeInitialize (server)
    KesServer server ;

{
    static int                   done = 0 ;
    int                          i, count ;
    KID                          unused_node ;
    KesXtResourceInfo            info ;
    KesXtInitialRealResourceRec *known ;

    if ( done )
	return ;

    done = 1 ;
    count = sizeof (_InitialRealResources)/sizeof(_InitialRealResources[0]) ;

    for ( i = 0 , known = _InitialRealResources
	 ; i < count
	 ; i++, known++ )
    {
	KeCodec codec = KeCodecGet (known->codec_name) ;
	char    *name = known->name ;
	char    *type = known->type_name ;

	if (!(info=(KesXtResourceInfo) malloc (sizeof(KesXtResourceInfoRec))))
	    return ;

	if ( ! codec )
	    codec = KeCodecUNDEFGet () ;
	
	KesCodecRegister (server, codec, &unused_node) ;

	info->name  = (char *) 0 ;
	info->codec = codec ;
	info->mode  = known->mode ;
	info->mask  = known->mask ;
	info->get   = known->get ;
	info->set   = known->set ;
	info->exec  = known->exec ;

	KesXtRealResourceDefine ((name?XrmStringToQuark(name) : (XrmQuark)0)
				 , (type?XrmStringToQuark(type) : (XrmQuark)0)
				 , info) ;
    }

    return ;
}


