/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * widget.c
 *
 * Exporting/Unexporting widgets, widget callback set.
 ***************************************************************************/

#include <stdio.h>
#include <setjmp.h>

#include <KeServer.h>
#include <KesCodec.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "KesXtP.h"

static KesHandlerSet KesWidgetHandlerSet = (KesHandlerSet) 0 ;

/***************************************************************************
 * KesWidgetHandlerSetGet: initialize widget handler set
 *
 * RETURNS: KesHandlerSet
 ***************************************************************************/

static KeStatus KesWidgetLsHandler (request, reply)
    KesLsReq request ;
    KesReply reply ;

{
    int    n, i, children_count ;
    Widget *children ;
    KID    *pnodes ;
    Widget widget = (Widget) request->closure ;
    int    extra ;

#ifndef KESXT_EXPORT_ACTIONS
    extra = 1 ;	/* the *resources* directory */
#else
    extra = 2 ;	/* and the *actions* directory */
#endif

    children_count = _KesXtFindChildren (widget, &children) ;
    
    pnodes = (KID *) KesMalloc (request->server
				, sizeof (KID) * (children_count + extra)) ;

/*
 * Answer with:
 * - One node per exported widget
 * - An additional "resource directory" node
 * By using here KesNodeMake, we enforce the whole widget hierarchy to be 
 * exported "on-demand". I'm not sure here what is the best solution, 
 * should the use be able to choose wich widgets to export ?
 */

    for ( n = 0, i = 0 ; i < children_count ; i++ )
    {
	if ( KesNodeMake (request->server
			  , KesWidgetHandlerSetGet()
			  , (KePointer) children[i]
			  , &(pnodes[n])) == KEDIT_Success )
	{
	    n++ ;
	}
    }

    if ( KesNodeMake (request->server
		      , KesResourceDirectoryHandlerSetGet()
		      , (KePointer) widget
		      , &(pnodes[n])) == KEDIT_Success )
    {
	n++ ;
    }

#ifdef KESXT_EXPORT_ACTIONS
    
    if ( _KesXtWidgetActionCount (widget) > 0 )
    {
	if ( KesNodeMake (request->server
			  , KesActionDirectoryHandlerSetGet()
			  , (KePointer) widget
			  , &(pnodes[n])) == KEDIT_Success )
	{
	    n++ ;
	}
    }

#endif /* KESXT_EXPORT_ACTIONS */

/*
 * Free the children list, and return gloriously:
 */

    if ( children )
	KeFree (children) ;

    KesLsReplyFill (reply, n, pnodes, KeFree) ;

    return (KEDIT_Success) ;
}

static KeStatus KesWidgetStatHandler (request, reply)
    KesStatReq request ;
    KesReply   reply ;

{
    static KesServer server = (KesServer) 0 ;
    static KID       widget_node  = (KID) 0 ;
    Widget           widget = (Widget) KesClosureOfRequest (request) ;
    
/*
 * Initialize widget_node if not done (or update cache)
 */
    
    if ( ( ! widget_node ) || ( server != request->server ) )
    {
	KeCodec codec = KeCodecDefine ("WIDGET"
				       , (KeCodecSizer) 0
				       , (KeCodecEncoder) 0
				       , (KeCodecFreeer) 0
				       , (KeCodecDecoder) 0
				       , (KeCodecFreeer) 0) ;

	KesCodecRegister (server = request->server
			  , codec
			  , &widget_node) ;
    }

/*
 * Fill in the reply:
 */
	
	
    KesStatReplyFillStatic (reply
			    , XtName (widget)
			    , KEDIT_DirectoryMode
			    , KEDIT_CreateMask | KEDIT_DeleteMask | KEDIT_NotifyMask
			    , widget_node
			    , 0) ;

    return (KEDIT_Success) ;
}

static KeStatus KesWidgetNotifyHandler (request, reply)
    KesNotifyReq request ;
    KesReply     reply ;

{
    unsigned long accepted = KEDIT_CreateMask | KEDIT_DeleteMask | KEDIT_NotifyMask ;

    if ( request->mask & accepted == request->mask )
    {
	KesNodeNotifySet (request->client, request->node, request->mask) ;
	KesNotifyNotify (request->server
			 , request->client->client_tag
			 , request->node
			 , request->mask) ;
    }
    else
    {
	KesErrorReplyFillStatic (reply
				 , KEDIT_InvalidOp
				 , "This node doesn't support given mask") ;
    }
    return (KEDIT_Success) ;
}

KesHandlerSet KesWidgetHandlerSetGet ()

{
    if ( ( ! KesWidgetHandlerSet ) 
	&& ( KesWidgetHandlerSet = KesHandlerSetMake()) )
    {
	KesHandlerSetLsHandler     (KesWidgetHandlerSet, KesWidgetLsHandler) ;
	KesHandlerSetStatHandler   (KesWidgetHandlerSet, KesWidgetStatHandler) ;
	KesHandlerSetNotifyHandler (KesWidgetHandlerSet, KesWidgetNotifyHandler) ;
    }

    return (KesWidgetHandlerSet) ;
}
	
/***************************************************************************
 * KesXtExportWidget  : export this widget
 * KesXtUnexportWidget: unexport this widget
 *
 * RETURNS: widget node 
 ***************************************************************************/

KeStatus KesXtExportWidget (server, widget, node_ret)
    KesServer server ;
    Widget    widget ;
    KID       *node_ret ;

{
    return ( KesNodeMake (server
			  , KesWidgetHandlerSetGet ()
			  , (KePointer) widget
			  , node_ret) ) ;
}

void KesXtUnexportWidget (server, widget)
    KesServer server ;
    Widget    widget ;

{
    KID       node ;

    if ( KesInfoToNode (server
			, KesWidgetHandlerSetGet()
			, (KePointer) widget
			, &node) == KEDIT_Success )
    {
	KesNodeFree (server, node) ;
    }

    fprintf (stderr, "libKeXt[KesXtUnexportWidget]: not fully implemented !\n") ;

    return ;
}

