/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCGet.c
 *
 * Handle the synchronous version of the Get request
 ***************************************************************************/

#include <stdio.h>
#include <string.h>

#include <KeClient.h>
#include <KeClientP.h>

/***************************************************************************
 * _KecDecodeGetReply(): decode the get reply buffer
 *
 * Decode the provide buffer as a Get reply. Allocated data (i.e. returned
 * value) should be freed by caller.
 *
 * RETURN: the KecValue, or NULL
 ***************************************************************************/

static KePointer _KecDecodeGetReply (kec_conn, pbuf, swap)
    KecConnection kec_conn ;
    char          *pbuf ;
    Bool          swap ;

{
    KecValue      value  = (KecValue) 0 ;
    unsigned int  size ;

/*
 * Decode the value :
 */

    EXTRACT_CARD32 (pbuf, swap, size) ;

    value = KecMalloc (kec_conn, size + sizeof (KecValueRec)) ;

    value->size = size ;
    if ( size)
	bcopy (pbuf, value->data, size) ;

    return ((KePointer) value) ;
}

/***************************************************************************
 * _KecDefaultGetHandler(): handle the Get reply by saving result in clos
 *
 * Just set the ret_val to its arg...
 *
 * RETURN: void
 ***************************************************************************/

static void _KecDefaultGetHandler (reply, ret_value)
    KecReply  reply ;
    KePointer ret_value ;

{
    KecValue *retval = ((KecValue *) ret_value) ;

    if ( reply->status == KEDIT_Success )
	*retval = KecValueOfReply (reply) ;
    else
	*retval = (KecValue) 0 ;
    return ;
}

/***************************************************************************
 * KecGet(): retrieve the value associated with a node.
 *
 * RETURNS: a KeStatus indicating opertaion status
 ***************************************************************************/

KeStatus _KecGet (kec, node, ret_value, handler, closure)
    KecConnection kec ;
    KID           node ;
    KecValue      *ret_value ;
    KecHandler    handler ;
    KePointer     closure ;

{
    keditGetRequest *request ;
    KeStatus        retval = KEDIT_Failure ;
    KecPending      pending ;
    char            *pdata ;
    unsigned int    serial = kec->serial ;

    KCLIENT_LOCK (kec) ;

    if ( kec->disconnected )
	return (KEDIT_Failure) ;

    if ( ret_value )
	*ret_value = (KecValue) 0 ;
    retval     = KEDIT_Failure ;

/*
 * Allocate the request in obuf, and set it
 */

    IceGetHeaderExtra (kec->ice_connection
		       , _IceKeditPoProtocolNumber
		       , KEDIT_Get
		       , sizeof (keditRequest)
		       , (sizeof (keditGetRequest) - sizeof (keditRequest)) >> 3
		       , keditGetRequest
		       , request
		       , pdata) ;

    STORE_CARD32 (pdata, node) ;

    (kec->serial)++ ;

/*
 * If async is required, just set the pending block :
 */

    if ( handler )
    {
	if ( (pending = _KecPendingMake (kec
					 , serial
					 , _KecDecodeGetReply
					 , handler
					 , closure)) != ((KecPending) 0) )
	    retval = KEDIT_Success ;
	goto done ;
    }

/*
 * Sync is required:
 * 1. set the pending block ,
 * 2. wait for this particular block to be fired (by _KecWaitForReplies (serial))
 * 3. return the result
 */

    if ( ! (pending = _KecPendingMake (kec
				       , serial
				       , _KecDecodeGetReply
				       , _KecDefaultGetHandler
				       , (KePointer) ret_value)) )
    {
	goto done ;
    }
    
    retval = _KecWaitForReply (kec, serial) ;
    
    if ( (retval == KEDIT_Success) && ( ! *ret_value) )
	retval = (KecErrorOfConnection (kec))->status ;

    /* _KecDefaultGetHandler has been triggerred, and it has set the result in ret_value */

  done:

    KCLIENT_UNLOCK (kec) ;
    KCLIENT_SYNC (kec) ;
    return (retval) ;
}
