/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCIo.c
 *
 * Io functions on the client side. These are quit the same as the one in the
 * server (at least at the basic level). This file is largely inspired 
 * from XlibInt.c, yet I made quit lots of changes (this is why I might
 * still have some bug (;-) ).
 ***************************************************************************/

#include <stdio.h>

#include <KeClient.h>
#include <KeClientP.h>

static void _KecDispatchPacket (
#ifdef KEDIT_NeedsFunctionPrototype
      KecConnection    kec
    , int	       opcode 	
    , keditRequest    *header 
    , char            *body 
    , Bool             swap 
#endif
) ;
    

/***************************************************************************
 * _IceKeditPOProcessMessage: process incomming Ice messages 
 *
 * This is the K-edit way of processing incomming ICE messages. As K-edit
 * already relies aon callback, this is fairly easy (ypus).
 * This function handles the 1.0 protocol sepcification
 *
 * RETURN: According to Ice def
 ***************************************************************************/

void _IceKeditPOProcessMessage_1_0 (ice_conn,ice_ptr,opcode,length,swap,reply_wait,reply_ready)
    IceConn  	     ice_conn ;
    IcePointer       ice_ptr ;
    int      	     opcode ;
    unsigned 	     long length ;
    Bool             swap ;
    IceReplyWaitInfo *reply_wait ;
    Bool             *reply_ready ;

{
    KecConnection kec = (KecConnection) ice_ptr ;
    keditRequest  *msg ;
    char          *pdata ;

    IceReadCompleteMessage (ice_conn
			    , sizeof (keditRequest)
			    , keditRequest
			    , msg
			    , pdata) ;
    /*
     *if ( pdata != ((char *) msg) + sizeof (keditRequest) ) {
     *	fprintf (stderr, "ProcessMessages: invalid pdata pointer.\n") ;
     *}
     */
    if (!pdata) {
      fprintf (stderr, "ProcessMessages: pdata == NULL.\n") ;
      fflush (stderr);
      return;
    }
/*
 * If reply_wait, we know it can only be an OpenConnection request:
 */

    if ( reply_wait ) 
    {
	reply_wait->reply = (IcePointer) pdata ;
	*reply_ready = True ;
	return ;
    } 

/*
 * Else, dispatch the packet:
 */

    _KecDispatchPacket (kec, opcode, msg, pdata, swap) ;

    IceDisposeCompleteMessage (ice_conn, pdata) ;

    return ;
}

/***************************************************************************
 * _KecWaitForReply: wait until the given reply arrives
 *
 * RETURN: KeStatus
 ***************************************************************************/

static void _KecDispatchPacket (kec, opcode, header, body, swap)
    KecConnection    kec ;		/* connection */
    int		     opcode ;		/* opcode of message */
    keditRequest     *header ;		/* beginning of (undecoded) message */
    char	     *body ;		/* var length data of message */
    Bool             swap ;		/* byte order */
    
{
    switch (opcode)
    {
      case KEDIT_Event:
	{
	    KecEvent event ;
	    
	    if ( ! _KecEventDecode (kec, header, body, &event, swap) )
		(kec->event_handler) (kec, event) ;
	}
	break ;
      case KEDIT_Error:
      case KEDIT_Reply:
	{
	    register KecPending kp ;
	    KecReplyRec         reply ;
	    unsigned int 	serial ;
	    char		*pbuf = body;

	    if ( opcode == KEDIT_Reply )
	    {
		EXTRACT_CARD32 (pbuf, swap, serial) ;
	    }
	    else
	    {
		char *pserial = body + sizeof (CARD32) ;
		
		EXTRACT_CARD32 (pserial, swap, serial) ;
	    }

	    /* 
	     * Find the corresponding pending block : as we know the server
	     * replies to request in the right order, we can short cut the 
	     * search before the end of the list.
	     * In most cases, they will be no iterations...
	     */
	    for ( kp = kec->pending ; kp ; kp = kp->next )
	    {
		if ( kp->serial == serial )
		{
		    break ;
		} 
	    } 

	    /*
	     * if it is an error, we might have no pending block, else it is a
	     * Reply we *must* have a pending block, and trigger its handler.
	     * If it is an error, set up the last error, freeing the old one .
	     * before */

	    if ( opcode == KEDIT_Error )
	    {

		if ( kec->last_error.msg )
		    KeFree ( kec->last_error.msg) ;

		_KecErrorDecode (kec, body, &(kec->last_error), swap) ;
		if ( kp )
		{
		    /* Set up the KecReplyRec,for the handler and trigger it:*/
		    reply.connection = kec ;
		    reply.status     = kec->last_error.status ;
		    reply.ru.error   = &(kec->last_error) ;
		    kec->last_reply  = serial ;
		    (kp->handler) (&reply, kp->closure) ;
		    _KecPendingFree (kec, kp) ;
		}
		/* was : "else", but is more practical to call also the error 
		   handler */
		/* Trigger the global error handler : */
		(kec->error_handler) (kec, &(kec->last_error)) ;
	    }
	    else if ( kp && (serial == kp->serial) )
	    {
		kec->last_error.status = KEDIT_Success ;
		reply.connection       = kec ;
		reply.status           = KEDIT_Success ;
		reply.ru.pointer=(KePointer) ((kp->decode) (kec, pbuf, swap)) ;
		kec->last_reply        = serial ;
		(kp->handler)(&reply, kp->closure) ;
		_KecPendingFree (kec, kp) ;
	    }
	    else
	    {
	      fprintf (stderr, "KecLib: Fatal error.\n") ;
	      fprintf (stderr, "_KecDispatchPacket: no pending block for %d\n"
		       , serial) ;
	      exit (1) ;
	    }
	}
	break ;
      default:
	fprintf (stderr, "_KecDispatchPacket: invalid minor %d\n", opcode) ;
	fprintf (stderr, "Fatal error. Exiting\n") ;
	exit (1) ;
    }
 
    return ;

}

KeStatus _KecWaitForReply (kec, serial)
    register KecConnection kec ;
    unsigned int serial ;

{
  Bool reply_ready ;
  IceProcessMessagesStatus ret=IceProcessMessagesSuccess;
    
/*
 * Invoke process message:
 */

    IceFlush (kec->ice_connection) ;
 
    while ( (kec->last_reply < serial) 
	    && (ret == IceProcessMessagesSuccess) ) {
	_IceConnectionSelect (kec->ice_connection) ;
	ret = IceProcessMessages (kec->ice_connection, NULL, &reply_ready) ;
    }

    return (ret == IceProcessMessagesSuccess ? KEDIT_Success : KEDIT_Failure) ;
}

/***************************************************************************
 * KecGobleData: read any data from the connection
 *
 * If no pending blocks are waiting for a reply, the user might be looking
 * for some events, so try to read the connection. As this will block, it
 * is up to the user to be sure that something is ready to be read.
 *
 * RETURN: a status indicating failure or success.
 ***************************************************************************/

KeStatus KecGobbleData (kec)
    KecConnection kec ;

{
    KeStatus status = KEDIT_Success ;

    while ( ( status == KEDIT_Success ) && ( kec->pending_tail ) )
    {
	status = _KecWaitForReply (kec, kec->pending_tail->serial) ;
    }

    return (status) ;
}
