/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCLs.c
 *
 * Handle the synchronous version of the Ls request
 ***************************************************************************/

#include <stdio.h>

#include <KeClient.h>
#include <KeClientP.h>

/***************************************************************************
 * _KecDecodeLsReply(): decode the ls reply buffer
 *
 * Whole reply should be read before invoquing this function.
 *
 * RETURN: The KeRawNodeList and the consumed number of bytes.
 ***************************************************************************/

static KePointer _KecDecodeLsReply (kec_conn, pbuf, swap)
    KecConnection kec_conn ;
    char          *pbuf ;	/* start of LsReply specific data */
    Bool          swap ;	/* byte swapping needed ? */

{
    KeIdArray     ids ;
    unsigned int  num_children ;
    KID           *pid ;

    EXTRACT_CARD32 (pbuf, swap, num_children) ;

    ids = KecMalloc (kec_conn
		     , sizeof(KeIdArrayRec) + num_children * sizeof(KID)) ;

    pid = &(ids->nodes[0]) ;

    ids->count = num_children ;
    while ( num_children ) 
    {
	EXTRACT_CARD32 (pbuf, swap, *pid) ;
	pid++ ; 
	num_children-- ;
    }

    return ((KePointer) ids) ;
}

/***************************************************************************
 * _KecDefaultLsHandler(): handle the Ls reply by saving result in clos
 *
 * RETURN: void
 ***************************************************************************/

static void _KecDefaultLsHandler (reply, ret_value)
    KecReply  reply ;
    KePointer ret_value ;
{
    KeIdArray *retval = ((KeIdArray *) ret_value) ;

    if ( reply->status == KEDIT_Success )
	*retval = KecIdArrayOfReply (reply) ;
    else
	*retval = (KeIdArray) 0 ;
    return ;
}

/***************************************************************************
 * KecLs(): list all the node children
 *
 * RETURN: a KeStatus indicating operation status
 ***************************************************************************/

KeStatus _KecLs (kec, node, ret_value, handler, closure)
    KecConnection kec ;
    KID           node ;
    KeIdArray     *ret_value ;
    KecHandler    handler ;
    KePointer     closure ;

{
    keditLsRequest *request ;
    KeStatus       retval = KEDIT_Failure ;
    KecPending	   pending ;
    char           *pdata ;
    unsigned int   serial = kec->serial ;

    KCLIENT_LOCK (kec) ;

    if ( kec->disconnected )
	return (KEDIT_Failure) ;

    if ( ret_value )
	*ret_value = (KeIdArray) 0 ;
    retval = KEDIT_Failure ;

/*
 * Allocate the request in obuf, and set it
 */

    IceGetHeaderExtra (kec->ice_connection
		       , _IceKeditPoProtocolNumber
		       , KEDIT_Ls
		       , sizeof (keditRequest)
		       , (sizeof (keditLsRequest) - sizeof (keditRequest)) >> 3
		       , keditLsRequest
		       , request
		       , pdata) ;
		       
    STORE_CARD32 (pdata, node) ;

    (kec->serial)++ ;

/*
 * If async is required, just set thepending block
 */
    if ( handler )
    {
	if ( pending = _KecPendingMake (kec
					, serial
					, _KecDecodeLsReply
					, handler
					, closure) )
	    retval = KEDIT_Success ;
	goto done ;
    }

/*
 * Sync is required:
 * 1. set the pending block ,
 * 2. wait for this particular block to be fired (by _KecWaitForReplies (serial))
 * 3. return the result
 */

    if ( ! (pending = _KecPendingMake (kec
				       , serial
				       , _KecDecodeLsReply
				       , _KecDefaultLsHandler
				       , (KePointer) ret_value)) )
    {
	goto done ;
    }

    retval = _KecWaitForReply (kec, serial) ;

    if ( (retval == KEDIT_Success) && ( ! *ret_value) )
	retval = (KecErrorOfConnection (kec))->status ;
    
  done:

    KCLIENT_UNLOCK (kec) ;
    KCLIENT_SYNC (kec) ;
    return (retval) ;
}



