/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCOpenC.c
 *
 * This file defines the KCOpenConnection() function.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#include <sys/types.h>
#include <fcntl.h>

#define _NEEDS_KEDIT_PO_DESCRIPTION_

#include <KeClient.h>
#include <KeClientP.h>

int _IceKeditPoProtocolNumber = -1 ;
static KecConnection _KecConnectionTab[256];

/***************************************************************************
 * KecOpenConnection: open a connection to the given application
 *
 * All the network part of it is done by Ice.
 * 
 * RETURN: a KecConnection
 ***************************************************************************/

static void
IOHandler (conn)
    IceConn conn;
{
    KecConnection kec=_KecConnectionTab[IceConnectionNumber(conn)];

    /* call io handler (actually: a hook) */
    if (*kec->io_error)
	(*kec->io_error)(kec,errno);
}

KecConnection KecOpenConnection (network_id)
    char *network_id ;

{
    KecConnection kec = (KecConnection) malloc (sizeof (KecConnectionRec)) ;
    keditOpenConnectionRequest *request ;
    IceReplyWaitInfo 	       wait_rec ;
    char                       *pbuf, *vendor, *release ;
    int                        major_version, minor_version ;
    int			       numproto ;
    Bool 		       reply_ready ;
    IceProtocolSetupStatus     retVal;
    char                       retErr[BUFSIZ];
    
    if ( ! kec )
	return (kec) ;

/*
 * Declare the K-edit protocol to ICE library :
 */

    numproto = IceRegisterForProtocolSetup (
	KEDIT_ProtocolName, KEDIT_Vendor, KEDIT_Release, KEDIT_PO_NUM_VERSION
	, IceKeditPOVersions, 0, (char **) 0, (IcePoAuthProc *) 0
	, (IceIOErrorProc) IOHandler);
    
    if ( ( _IceKeditPoProtocolNumber = numproto ) < 0 )
    {
	free (kec) ;
	return ((KecConnection) 0) ;
    }

    if ( ! ( kec->ice_connection = IceOpenConnection (network_id
						      , (IcePointer) 0
						      , False		
						      , 0 
						      , 0, (char *) 0))) 
    {
	free (kec) ;
	return ((KecConnection) 0) ;
    }

/*
 * Protocol Setup with server :
 */
    retErr[0]='\0';
    if ( (retVal = IceProtocolSetup (kec->ice_connection
			   , _IceKeditPoProtocolNumber
			   , (IcePointer) kec
			   , False		
			   , &major_version
			   , &minor_version
			   , &vendor
			   , &release
			   , BUFSIZ
			   , retErr)) != IceProtocolSetupSuccess ) 
    {
      fprintf (stderr,"K-edit failed to open client. %s\n",retErr);
      fflush (stderr);
	free (kec) ;
	return ((KecConnection) 0) ;
    }

    _KecConnectionTab[IceConnectionNumber(kec->ice_connection)] = kec;

/*
 * Emit the first packet, and get the reply right now :
 * As long as we know that the only request waiting for a reply is 
 * the KEDIT_OpenConnection request, we have a little nasty hack:
 * On receiving a valid ReplyWaitInfo, the POProcessMessage function
 * sets the reply field of ReplyWaitInfo to the pdata of the message
 * and we read it here.
 * Their should be no trouble because we do not proceed to any IO
 * operation on the IceConn in the mean time.
 */

    IceGetHeader (kec->ice_connection
		  , _IceKeditPoProtocolNumber
		  , KEDIT_OpenConnection
		  , sizeof (keditOpenConnectionRequest)
		  , keditOpenConnectionRequest
		  , request) ;

    wait_rec.sequence_of_request = IceLastSentSequenceNumber(kec->ice_connection) ;
    wait_rec.major_opcode_of_request = _IceKeditPoProtocolNumber ;
    wait_rec.minor_opcode_of_request = KEDIT_OpenConnection ;
    wait_rec.reply 		     = (IcePointer) 0 ;
    
    IceFlush (kec->ice_connection) ;

    for (reply_ready = False ; ! reply_ready ; ) {
	_IceConnectionSelect (kec->ice_connection) ;
	IceProcessMessages (kec->ice_connection, &wait_rec, &reply_ready) ;
    }

/*
 * Decode the reply into the KecConnection structure:
 */

    pbuf = (char *) wait_rec.reply ;

    EXTRACT_CARD32 (pbuf, IceSwapping (kec->ice_connection), kec->tag) ;
    EXTRACT_CARD32 (pbuf, IceSwapping (kec->ice_connection), kec->root) ;
    
    IceDisposeCompleteMessage (kec->ice_connection, wait_rec.reply) ;

/*
 * Initialize the various fields of the Connection :
 */

    kec->name          = (char *) strdup (network_id) ;
    kec->serial        = 1 ;
    kec->last_reply    = 0 ;

    /* handlers */
    kec->io_error      = _KecDefaultIOErrorHandler ;
    kec->sync          = (KecSyncHandler) 0 ;
    kec->lock          = (int (*)()) 0 ;
    kec->unlock        = (int (*)()) 0 ;
    kec->error_handler = _KecDefaultErrorHandler ;
    kec->event_handler = KecToolkitEventHandler ;

    /* memory allocation hooks */

    kec->purge_hook    = (KeMallocPurgeHook) 0 ;
    kec->fail_hook     = (KeMallocFailHook) 0 ;

    /* pending request handlers */
    kec->pending       = (KecPending) 0 ;
    kec->pending_tail  = (KecPending) 0 ;

    /* Last error */
    kec->disconnected       = 0 ;
    kec->last_error.serial  = 0 ;
    kec->last_error.type    = 0 ;
    kec->last_error.subtype = 0 ;
    kec->last_error.node    = (KID) 0 ;
    kec->last_error.status  = (KeStatus) 0 ;
    kec->last_error.msg     = (char *) 0 ;

    /* Toolkit notification management functions */
    kec->notifications = (KeHashtable) 0 ;
    kec->user_data     = (KePointer) 0 ;

    return (kec) ;
}

/***************************************************************************
 * KecCloseConnection: close the connection, and any related resources
 ***************************************************************************/

KeStatus KecCloseConnection (kec)
    KecConnection kec ;

{
  Status ret;
  IceCloseStatus closeRet;
    
/*
 * Free all the stuff :
 */
  assert(kec && (kec->disconnected == 0));

    while ( kec->pending )
	_KecPendingFree (kec, kec->pending) ;
    
    kec->disconnected = 1;

    KecNotifyRemove (kec, ((KecNotifyId) 0)) ;

    /* shutdown a la ICE */
    ret = IceProtocolShutdown (kec->ice_connection, _IceKeditPoProtocolNumber);
    IceSetShutdownNegotiation (kec->ice_connection, False);
    closeRet = IceCloseConnection(kec->ice_connection);

    free (kec->name) ;

    free (kec) ;

    return ( KEDIT_Success ) ;
}
