/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCStat.c
 *
 * Handle sync/async version of stat request.
 ***************************************************************************/

#include <stdio.h>
#include <string.h>

#include <KeClient.h>
#include <KeClientP.h>

/***************************************************************************
 * _KecDecodeStatReply(): decode the stat reply buffer
 *
 * Whole reply should be read before invoquing this function.
 *
 * RETURN: A KeNodeStat, all allocated in same chunk.
 ***************************************************************************/

static KePointer _KecDecodeStatReply (kec_conn, pbuf, swap)
    KecConnection kec_conn ;
    char          *pbuf ;
    Bool          swap ;

{
    KeNodeStat   stat ;
    unsigned int namelen, mode, mask, size ;
    KID          node, info ;

    EXTRACT_CARD16 (pbuf, swap, mode) ;
    EXTRACT_CARD16 (pbuf, swap, mask) ;
    EXTRACT_CARD32 (pbuf, swap, node) ;
    EXTRACT_CARD32 (pbuf, swap, size) ;
    EXTRACT_CARD32 (pbuf, swap, info) ;
    pbuf += sizeof (CARD32) ;			/* skip unused32 */
    EXTRACT_CARD16 (pbuf, swap, namelen) ;

    stat = KecMalloc (kec_conn, sizeof (KeNodeStatRec) + namelen + 1) ;

    stat->mode = mode ;
    stat->mask = mask ;
    stat->size = size ;
    stat->id   = node ;
    stat->info = info ;
    stat->name = ((char *) stat) + sizeof (KeNodeStatRec) ;
    bcopy (pbuf, stat->name, namelen) ;
    *(stat->name + namelen) = '\0' ;

    return ((KePointer) stat) ;
}

/***************************************************************************
 * _KecDefaultLsHandler(): handle the Stat reply by saving result in clos
 *
 * RETURN: void
 ***************************************************************************/

static void _KecDefaultStatHandler (KecReply reply, KePointer ret_value)

{
    KeNodeStat *retval = ((KeNodeStat *) ret_value) ;

    if ( reply->status == KEDIT_Success )
	*retval = KecStatOfReply (reply) ;
    else
	*retval = (KeNodeStat) 0 ;
    return ;
}

/***************************************************************************
 * KecLs(): list all the node children
 *
 * RETURN: a KeStatus indicating operation status
 ***************************************************************************/

KeStatus _KecStat (kec, node, ret_value, handler, closure)
    KecConnection kec ;
    KID           node ;
    KeNodeStat    *ret_value ;
    KecHandler    handler ;
    KePointer     closure ;

{
    keditStatRequest *request ;
    KeStatus         retval = KEDIT_Failure ;
    KecPending	     pending ;
    char             *pdata ;
    unsigned int     serial = kec->serial ;
    
    KCLIENT_LOCK (kec) ;

    if ( kec->disconnected )
	return (KEDIT_Failure) ;

    if ( ret_value )
	*ret_value = (KeNodeStat) 0 ;
    retval = KEDIT_Failure ;

/*
 * Allocate the request in obuf, and set it
 */

    IceGetHeaderExtra (kec->ice_connection
		       , _IceKeditPoProtocolNumber
		       , KEDIT_Stat
		       , sizeof (keditRequest)
		       , (sizeof(keditStatRequest) - sizeof (keditRequest))>>3
		       , keditStatRequest
		       , request
		       , pdata) ;

    STORE_CARD32 (pdata, node) ;

    (kec->serial)++ ;

/*
 * If async is required, just set thepending block
 */
    if ( handler )
    {
	if ( (pending = _KecPendingMake (kec
					 , serial
					 , _KecDecodeStatReply
					 , handler
					 , closure)) != ((KecPending) 0) )
	    retval = KEDIT_Success ;
	goto done ;
    }

/*
 * Sync is required:
 * 1. set the pending block ,
 * 2. wait for this particular block to be fired (by _KecWaitForReplies (serial))
 * 3. return the result
 */

    if ( ! (pending = _KecPendingMake (kec
				       , serial
				       , _KecDecodeStatReply
				       , _KecDefaultStatHandler
				       , (KePointer) ret_value)) )
    {
	goto done ;
    }

    retval = _KecWaitForReply (kec, serial) ;

    if ( (retval == KEDIT_Success) && ( ! *ret_value) )
	retval = (KecErrorOfConnection (kec))->status ;
    
  done:

    KCLIENT_UNLOCK (kec) ;
    KCLIENT_SYNC (kec) ;
    return (retval) ;
}



