/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCSync.c
 *
 * Use the Noop query to synchronize with the server. Defined functions:
 * KecConnectionDoSync (): synchronize the connection
 * KecConnectionSync (): set/unset the connection in sync mode.
 ***************************************************************************/

#include <stdio.h>

#include <KeClient.h>
#include <KeClientP.h>

/***************************************************************************
 * _KecDecodeNoopReply: decode the noop reply 
 *
 * Nothing done, in fact
 *
 * RETURN: a NULL KePointer
 ***************************************************************************/

static KePointer _KecDecodeNoopReply (kec_conn, pbuf, swap)
    KecConnection kec_conn ;
    char          *pbuf ;
    Bool          swap ;

{
    return ((KePointer) 0) ;
}

/***************************************************************************
 * _KecDefaultNoopHandler: nothing done.
 *
 * RETURN: void
 ***************************************************************************/

static void _KecDefaultNoopHandler (reply, closure)
    KecReply  reply ;
    KePointer closure ;

{
    return ;
}

/***************************************************************************
 * KecConnectionSync (); synrchonize with server
 *
 * This is done by round tripping with the server.
 *
 * RETURNS: KeStatus
 ***************************************************************************/


KeStatus KecConnectionDoSync (kec)
    KecConnection kec ;

{
    keditNoopRequest *request ;
    KeStatus         retval = KEDIT_Failure ;
    KecPending       pending ;
    unsigned int     serial = kec->serial ;

    KCLIENT_LOCK (kec) ;

    if ( kec->disconnected )
	return (KEDIT_Failure) ;

    retval = KEDIT_Failure ;

/*
 * Allocate the request, and send it :
 */

    IceGetHeader (kec->ice_connection
		  , _IceKeditPoProtocolNumber
		  , KEDIT_Noop
		  , sizeof (keditNoopRequest)
		  , keditNoopRequest
		  , request) ;

    (kec->serial)++ ;

/*
 * Sync is required, so :
 * 1. set a pending block
 * 2. wait for this particular block to be fired
 * 3. return 
 */

    if ( ! (pending = _KecPendingMake (kec
				       , serial
				       , _KecDecodeNoopReply
				       , _KecDefaultNoopHandler
				       , (KePointer) 0)) )
    {
	goto done ;
    }

    retval = _KecWaitForReply (kec, serial) ;

  done:
    
    KCLIENT_UNLOCK (kec) ;
    /* No KCLIENT_SYNC, as this cuold loop */
    return (retval) ;
}
    
/***************************************************************************
 * KecConnectionSync: set/unset the connection in sync mode
 *
 * By setting the sync connection handler
 *
 * RETURN: old connection state as a Boolean
 ***************************************************************************/

int KecConnectionSync (KecConnection kc, int boolean)

{
    int retval ;

    retval = (kc->sync == KecConnectionDoSync) ;
    if ( boolean )
        kc->sync = KecConnectionDoSync ;
    else
        kc->sync = (KecSyncHandler) 0 ;
    return (retval) ;
}
