/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KCTLsl.c
 *
 * Provide a 'toolkit' ls -l operation. 
 ***************************************************************************/

#include <stdio.h>

#include <KeClient.h>

/***************************************************************************
 * KecLsl: return stat for all children of given id.
 *
 * The main pont is that the user should invoke only one function to free
 * all the recieved data. Let's add a null pointer at end of Stat array
 * to provide the user this facility.
 *
 * keclsl_handler: is the internal handler
 * KecLslFree is to be used to free resulting data
 ***************************************************************************/

void KecLslFree (stats)
    KeNodeStat *stats ;

{
    KeNodeStat *pstat = stats ;

    if ( stats )
    {
	while ( *pstat )
	    KeFree (*pstat++) ;

	KeFree (stats) ;
    }

    return ;
}

static void keclsl_handler (reply, pstat)
    KecReply   reply ;		/* reply */
    KeNodeStat *pstat ;		/* stat to be filled */

{
    *pstat = KecStatOfReply (reply) ;
    return ;
}

int KecLsl (kec, node_id, stat_return)
    KecConnection kec ;
    KID           node_id ;
    KeNodeStat	  **stat_return ;

{
    KeIdArray	  children ;
    KID		  *pid ;
    int           i, do_fail = 0 ;
    KeNodeStat    *pstat ;

    *stat_return = (KeNodeStat *) 0 ;

/*
 * Query children, allocate stat returned array :
 */

    if ( KecLsSync (kec, node_id, &children) != KEDIT_Success )
	return (-1) ;

    if ( children->count )
    {
	int size = children->count + 1 ;

	*stat_return = KecMalloc (kec, size * sizeof (KeNodeStat)) ;
	
	memset (*stat_return, 0, size * sizeof (KeNodeStat)) ;

	for ( pid = children->nodes, pstat = *stat_return, i = 0 
	     ; i < children->count 
	     ; i++ , pid++, pstat++ )
	{
	    if ( KecStat (kec
			  , *pid
			  , (KecHandler) keclsl_handler
			  , (KePointer) pstat) != KEDIT_Success )
	    {
		do_fail = 1 ;
	    }
	}
    }

    KecConnectionDoSync (kec) ;

    if ( do_fail )
    {
	KecLslFree (*stat_return) ;
	*stat_return = (KeNodeStat *) 0 ;
	return (-1) ;
    }

    return (children->count) ;
}

