/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/*****************************************************************************\
   klkeMainUtil.c : connection to a main loop. Here we consider only the 
   XtMainLoop mechanism (it could be worse :-) ) 

   This file is used to embed k-edit into a lisp-like interpreter. All references to klke
   may be discarded (or replaced by your own library). This file is
   not compiled by the k-edit install. It is only a working exemple of
   how to plug nicely k-edit in a main event loop. It is similar to
   the Xt interface, and far better than "IceMainLoop".

   Note that the shutdown protocol of ICE is omitted.

See k-edit manual to figure out the entry points.

See the ICE documentation to figure out how to connect this library to a 
main loop (k-edit is built on top of ICE, ICE is shipped with X11R6. In 
particular, the error handling.

See also Xt documention, and its beautiful callback mechanism.


\*****************************************************************************/



#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <X11/ICE/ICElib.h>
#include <X11/Intrinsic.h>
#include <KeServer.h>
#include <KeClient.h>

#include <assert.h>

/* eurk */
char *ICE_NETWORK_ID=0;

/* KlKeNewClientHook is an external function that 
   adds a new client connection to the main loop */
/* args : KecConnection, KePointer */

extern void (*KlKeNewClientHook) ();
extern KePointer KlKeNewClientHookClosure;

/* same, for closing a client */
extern void (*KlKeCloseClientHook) ();
extern KePointer KlKeCloseClientHookClosure;

/* Ice major opcode (a=accepting, o=originator)*/
extern int _IceKeditPoProtocolNumber,_IceKeditPaProtocolNumber;

static void ProcessMessage();
/***************************************************************************
 * KesMainLoop: simple main server support for main loop
 *
 ***************************************************************************/

static IceConn   KIceConnOfFd[128] = { (void *) 0 , } ;
static int       ProtocolNumber[128];
static XtInputId InputId[128];

static void 
IOErrorHandler (ice_conn)
    IceConn ice_conn ;
{
   /* empty. Yes */
   /* This is because the standard (ICE) IO error handler exits */
    fprintf (stderr, "IOErrorHandler."); fflush stderr;
}

static Bool 
_HostBasedAuthDisabled (hostname)
    char *hostname ;
{
    /* fprintf (stderr, "host %s with no auth accepted.\n", hostname) ; */
    return (True) ;
}

/*
  Here we are defining a function called at each new client connection, to 
  register the new fd in the main loop select. <!> This mechanism should be 
  cleaner...
  */

static void
ClientRegisterSelect (client, app_context)
    KecConnection client;
    KePointer app_context; /* closure */
{
  XtInputId ret;
    IceConn ice_conn=KecIceConnOfConnection(client);

    /* record IceConn */
    KIceConnOfFd[IceConnectionNumber(ice_conn)] = ice_conn;
    /* add to main loop */    
    fprintf(stderr,"opening fd:%d\n",IceConnectionNumber(ice_conn));
    ret = XtAppAddInput((XtAppContext) app_context, IceConnectionNumber(ice_conn),
		  (XtPointer) XtInputReadMask, ProcessMessage,  NULL);
    ProtocolNumber[IceConnectionNumber(ice_conn)]=_IceKeditPoProtocolNumber;
    InputId[IceConnectionNumber(ice_conn)]=ret;
}

static void
ClientRemoveSelect (source, dummy)
    int source;
    KePointer dummy;
{
    /* remove from Xt main loop */
    XtRemoveInput(InputId[source]);
}

/* XtAddInput callback on client socket */
static void
ProcessMessage (setup, source, id)
    XtPointer setup;
    int *source;
    XtInputId *id;
{
    Bool         reply_ready ;
    IceProcessMessagesStatus ret;
    IceCloseStatus closeRet; 
    KlGCMark();

    /* bouncing event on close */
    assert(KIceConnOfFd[*source]);
    /* let ICE process the message */
    ret = IceProcessMessages ( KIceConnOfFd[*source], NULL, &reply_ready) ;
    /* quick return for events, replies pass through KecWaitForReply */
    if ((IceProcessMessagesSuccess == ret) && !setup) {
	KlGC();
	return;
    }
    /* maybe an error ? */
    switch (ret) {
    case IceProcessMessagesIOError:
	fprintf (stderr, "I/O error on %d. Closing\n", *source); fflush stderr;
	XtRemoveInput(*id); 
	if (KIceConnOfFd[*source]) {
	  IceProtocolShutdown(KIceConnOfFd[*source],ProtocolNumber[*source]);
	  IceSetShutdownNegotiation(KIceConnOfFd[*source],False);
	  closeRet=IceCloseConnection(KIceConnOfFd[*source]);
	}
	KIceConnOfFd[*source] = (IceConn) 0 ;
	/* I dont know why I have to do that here */
	close (*source);
	break;
    case IceProcessMessagesConnectionClosed:
	fprintf (stderr, "connection %d closed\n", *source); fflush stderr;
	XtRemoveInput(*id);
	break;
    default:			/* Success, but setup != 0 */
	    /* connection is still coming... */
	    switch (IceConnectionStatus(KIceConnOfFd[*source])) {
	    case IceConnectAccepted:
		/* toggle to stable level */
		XtRemoveInput(*id);
		XtAppAddInput((XtAppContext) setup, *source, 
			(XtPointer) XtInputReadMask, ProcessMessage, NULL);
		break;
	    case IceConnectRejected:
	    case IceConnectIOError:
		fprintf (stderr, "%d rejected\n", *source); fflush stderr;
		XtRemoveInput(*id);
		IceProtocolShutdown(KIceConnOfFd[*source],
				    ProtocolNumber[*source]);
		IceSetShutdownNegotiation(KIceConnOfFd[*source],False);
		IceCloseConnection(KIceConnOfFd[*source]);
		KIceConnOfFd[*source] = (IceConn) 0 ;
		/* I dont know why I have to do that here */
		close (*source);
		break;
	    /* default = still pending state */
	    }
    }
    KlGC();
}

/* XtAddInput callback on listening socket */
static void
NewConnection (app, source, id)
    XtAppContext app;
    int *source;
    XtInputId *id;
{
    IceListenObj    listen_obj = (IceListenObj) KIceConnOfFd[*source];
    IceAcceptStatus status ;
    IceConn         ice_conn = IceAcceptConnection (listen_obj, &status);
    Bool            reply_ready ;
    XtInputId       inputId;

    if (IceAcceptSuccess == status) {
	KIceConnOfFd[IceConnectionNumber(ice_conn)] = ice_conn ;
	inputId = XtAppAddInput(app, IceConnectionNumber(ice_conn), 
		      (XtPointer) XtInputReadMask, 
		      ProcessMessage, (XtPointer) app);
	ProtocolNumber[IceConnectionNumber(ice_conn)] =
	  _IceKeditPaProtocolNumber;
	InputId[IceConnectionNumber(ice_conn)]=inputId;
    }
}

/* klkeInit: record appContext, initialise library and error handler */
static XtAppContext appSaved=0;

void
klkeInit (app)
    XtAppContext app;
{
    static int   done=0;

    if (done) return;		/* all done */
    assert (app);
    appSaved = app;

    /* intialise library */
    KlKeServerInterfaceInitialize ();
    KlKeClientInterfaceInitialize ();

    KlKeNewClientHook = ClientRegisterSelect;
    KlKeNewClientHookClosure = (KePointer) app;

    KlKeCloseClientHook = ClientRemoveSelect;
    KlKeCloseClientHookClosure = (KePointer) NULL;

    IceSetIOErrorHandler (IOErrorHandler) ;
}

/* klkeSInit = server init: create listen socket, add to main loop */

void
klkeSInit ()
{
  int          listen_count ;
  IceListenObj *listens ;
  int          i, fd ;
  static int done=0;
  
  if (done) return;		/* already done */
  assert (appSaved);
  done = 1;
  
  /*
   * Listen for ICE connections:
   */
  
  if ( ! IceListenForConnections (&listen_count, &listens, 0, (char *) 0)) {
    return ;
  } else {
    char *s;
	/* store network ID in a global variable (eurk!) */
    ICE_NETWORK_ID = IceComposeNetworkIdList(listen_count,listens);
    for ( i = 0  ; i < listen_count ; i++ )	{
      IceSetHostBasedAuthProc (listens[i], _HostBasedAuthDisabled) ;
      fd = IceGetListenConnectionNumber (listens[i]) ;
      fprintf (stderr, "Ks: listening on %s\n"
	       , s=IceGetListenConnectionString (listens[i])) ;
      free(s);
      XtAppAddInput(appSaved, fd, (XtPointer) XtInputReadMask,
		    NewConnection, appSaved);
      KIceConnOfFd[fd] = (IceConn) listens[i] ;
    }
  }
}
