/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * alloc.c
 *
 * This file provides some "safe-malloc" function. The principle is quite
 * simple, all malloc functions take two more parameters:
 * - purge_hook: is invoked when raw malloc failed, it should try to free
 *   some memory. Malloc is recalled after this first hook.
 * - failed_hook: is invoked when after (purge_hook)(), malloc still fails.
 *   This hook *should not return*
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <KeLib.h>

extern int errno ;

void KeFree (ptr)
    void *ptr ;

{
    free (ptr) ;
    return ;
}

void *KeMalloc (purge_hook, fail_hook, size)
    KeMallocPurgeHook purge_hook ;
    KeMallocFailHook  fail_hook ;
    int size ;

{
    void *p ;

    if ( p = (void *) malloc (size) )
	return (p) ;

    if ( purge_hook )
	(purge_hook)() ;

    if ( p = (void *) malloc (size) )
	return (p) ;

    if ( fail_hook )
	(fail_hook)() ;
    
    fprintf (stderr, "KeMalloc: malloc failed, with no fail_hook (Fatal).\n");
    exit (1) ; 
}

void *KeRealloc (purge_hook, fail_hook, buffer, size)
    KeMallocPurgeHook purge_hook ;
    KeMallocFailHook  fail_hook ;
    void              *buffer ;
    int               size ;

{
    void *retval ;

    if ( retval = (void *) realloc (buffer, size) )
	return (retval) ;
    
    if ( purge_hook )
	(purge_hook)() ;
    
    if ( retval = (void *) realloc (buffer, size) )
	return (retval) ;
    
    if ( fail_hook )
	(fail_hook)() ;
    
    fprintf (stderr, "KeRealloc: malloc failed, with no fail_hook (Fatal).\n");
    exit (1) ; 
}

	
