/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * kvalue.c
 *
 * Some network shared dependencies.
 ***************************************************************************/

#include <stdio.h>

#include <KeLib.h>

extern int errno ;

/***************************************************************************
 * KeRawValueMake(): create a kEdit value record
 *
 * Make data pointer be 0 if the size is NULL (this can append for an
 * untiliazed node).
 *
 * RETURN: the created value.
 ***************************************************************************/

KeRawValue KeRawValueMake (unsigned int size, void *data, KeRawValueFreeFct f)

{
    KeRawValue kv = KMalloc (sizeof (KeRawValueRec)) ;
    
    kv->size      = size ;
    if ( size )
	kv->data  = data ;
    else
	kv->data  = (unsigned char *) 0 ;
    kv->free_hook = f ;
    return (kv) ;
}

/***************************************************************************
 * KeRawValueFree(): free the KeRawValue record
 *
 * The free hook is invoked on size, data only if data is non null
 *
 * RETURNS: void.
 ***************************************************************************/

void KeRawValueFree (KeRawValue kv)

{
    /* First, if HOOK call it : */
    if ( kv->free_hook && kv->data && kv->size )
	(*(kv->free_hook)) (kv->data) ;
    KFree (kv) ;
    return ;
}

/***************************************************************************
 * KeRawValuePrint (): dump (hexa) of the given value
 ***************************************************************************/

void KeRawValuePrint (FILE *fd, KeRawValue kv, int lmargin)

{
    unsigned char *p ;
    unsigned char *data = (unsigned char *) kv->data ;
    int margin ;

    /* Useful debugger feature */
    if ( ! fd )
	fd = stderr ;
    for (margin = 0 ; margin < lmargin ; margin++)
	fputc (' ', fd) ;
    for (p = data ; p - data < kv->size ; p++ )
    {
	fprintf (fd, "%02x ", *p) ;
	if ( ! ((p + 1 - data) % 16) )
	{
	    fputc ('\n', fd) ;
	    for (margin = 0 ; margin < lmargin ; margin++)
		fputc (' ', fd) ;
	}
    }
    fprintf (fd, "\n") ;
    fflush (fd) ;
    return ;
}
