/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSInitialize.c
 *
 * Initialization functions for the server. This file defines:
 * KesServerInitialize()
 * KesServerClose()
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define _NEEDS_KEDIT_PA_DESCRIPTION_

#include <KeServer.h>
#include <KesServerP.h>

/*
 * I HATE those globals, in particular the Server one, which is due to Ice:
 * - It doesn't allow to accept on multiple sockets
 * - It doesn't provide the accepting socket in protocol setup.
 */

int       _IceKeditPaProtocolNumber = -1 ;
KesClient _KesClientIndex[256]      = { (KesClient) 0, } ;

static KesServer _IceKeditServer    = (KesServer) 0 ;

/***************************************************************************
 * _IceKeditProtocolSetup: setup notify callback for the K-edit protocol
 *
 * This function is invoked when some IceConnection accepts the K-edit 
 * protocol. We build a client record for this connection.
 * If we cannot build the client record, we just shutdown the protocol on
 * this connection (which should be the same as refusing the connection).
 *
 * RETURNS: void
 ***************************************************************************/


static Status _IceKeditProtocolSetup (ice_conn, major, minor, vendor, release, client_data_ret, failure_reason_ret)
    IceConn    ice_conn ;
    int        major, minor ;
    char       *vendor, *release ;
    IcePointer *client_data_ret ;
    char       **failure_reason_ret ;

{
    KesServer server = _IceKeditServer ;
    KesClient client ;

    if ( server )
    {
	client = KesMalloc (server, sizeof (KesClientRec)) ;

	client->ice_connection = ice_conn ;
	client->serial         = 0 ;
	client->client_tag     = (server->connections)++ ;
	client->server         = server ;
	client->user_data      = (KePointer) 0 ;
	client->notification   = 0 ;

	_KesClientIndex[IceConnectionNumber(ice_conn)] = client ;

	if ( server->connection_handler )
	    (server->connection_handler) (client) ;
    }
    else
    {
	IceProtocolShutdown (ice_conn, _IceKeditPaProtocolNumber) ;
    }

    free (vendor) ;
    free (release) ;

    return (1) ;
}

/***************************************************************************
 * _IceKeditIOError: the Kedit-Ice IO error handler
 *
 * Kills the client associated with this connection, if any.
 *
 * RETURNS: void.
 ***************************************************************************/

static void _IceKeditIOError (ice_conn)
    IceConn ice_conn ;

{
    KesClient client = _IceConnToClient (ice_conn) ;

    if ( client ) 
    {
	KesClientKill (client) ;
    }

    return ;
}

/***************************************************************************
 * _SetupIcePAKeditProtocol: set this ice conn to receive K-edit protocol
 ***************************************************************************/

Bool _HostBasedAuthDisabled (hostname)
    char *hostname ;

{
    fprintf (stderr, "host %s not authenticated, but accepted.\n", hostname) ;
    return (True) ;
}

static int _SetupIcePAKeditProtocol ()

{
    int numproto ;

    if ( _IceKeditPaProtocolNumber > 0 )
	return ( _IceKeditPaProtocolNumber ) ;

    numproto = IceRegisterForProtocolReply (KEDIT_ProtocolName
					    , KEDIT_Vendor
					    , KEDIT_Release
					    , KEDIT_PA_NUM_VERSION
					    , IceKeditPAVersions
					    , 0
					    , (char **) 0
					    , (IcePaAuthProc *) 0
					    , _HostBasedAuthDisabled
					    , _IceKeditProtocolSetup
					    , (IceProtocolActivateProc) 0
					    , _IceKeditIOError) ;

    return ( _IceKeditPaProtocolNumber = numproto ) ;
}

/***************************************************************************
 * KsServerInitialize (name): initialize a new server
 *
 * RETURNS: KesServer struct or NULL
 ***************************************************************************/

KesServer KesServerInitialize (name)
    char	*name ;

{
    KesServer    server ;
    KesDirectory rootdir ;

/*
 * If a server has already been initialized, return it:
 */

    if ( _IceKeditServer )
	return ( _IceKeditServer ) ;

/*
 * Setup the K-Edit protocol, and create the server context :
 */

    if ( _SetupIcePAKeditProtocol () < 0 ) 
    {
	return ((KesServer) 0) ;
    }

    if ( server = (KesServer) malloc (sizeof (KesServerRec)) )
    {

	server->name               = (char *) strdup (name) ;
	server->connections        = 1 ;

	server->lock               = (int(*)()) 0 ;
	server->unlock             = (int(*)()) 0 ;
	server->timestamp          = 0 ;

	server->kill_handler       = (KesKillHandler) 0 ;
	server->kill_closure       = (KePointer) 0 ;
	server->connection_handler = (KesConnectionHandler) 0 ;
	server->connection_closure = (KePointer) 0 ;
	server->error_hook         = (KesErrorHook) 0 ;

	server->purge_hook         = (KeMallocPurgeHook) 0 ;
	server->fail_hook          = (KeMallocFailHook) 0 ;
	server->user_data          = (KePointer) 0 ;

	server->node_key      = 0 ;
	server->nodes_count   = 0 ;
	server->nodes_size    = 0 ;
	server->nodes         = (KesNodeInfo *) 0 ;
	server->notifications = (KeHashtable) 0 ;

/*
 * Initialize the fake-root-info
 * This is used in KSTDisp.c in a somehow tricky manner. As we need to construct a 
 * special KesDirectory, we construct it here by hand (see the 'normal' code in
 * KSTDirectory.c)
 */

	rootdir = KesMalloc (server, sizeof (KesDirectoryRec)) ;
	rootdir->server  = server ;
	rootdir->name    = "Root" ;
	rootdir->filter  = (KesDirectoryFilter) 0 ;
	rootdir->size    = 0 ;
	rootdir->used    = 0 ;
	rootdir->node    = (KID) 0 ;
	rootdir->subdirs = (KID *) 0 ;

	server->root_closure     = (KePointer) rootdir ;
	server->root_handler_set = KesDirectoryHandlerSetGet (server) ;

	server->codec_directory  = (KesDirectory) 0 ;

    }
	
    return ( _IceKeditServer = server ) ;
}


/***************************************************************************
 * KesServerKill: closse the server, and destroy any related resources
 * 
 * BUG : never tested yet
 *
 * RETURN: void
 ***************************************************************************/

void KesServerKill (server)
    KesServer server ;

{
/*
 * Remove all clients :
 */


/*
 * Remove all exported IDS :
 */

    
/*
 * Free any remaining data :
 */

    free (server->name) ;
    KeFree (server) ;

    return ;
}

/***************************************************************************
 * KesSetKillHandler      : set a new kill handler for this server.
 * KesSetConnectionHandler: set a connection handler
 * KesSetErrorHook        : override default error behaviour
 *
 * RETURN: old value.
 ***************************************************************************/

KesKillHandler KesSetKillHandler (server,  handler)
    KesServer      server ;
    KesKillHandler handler ;

{
    KesKillHandler retval = server->kill_handler ;

    server->kill_handler = handler ;
    return (retval) ;
}

KesConnectionHandler KesSetConnectionHandler (server, handler)
    KesServer            server ;
    KesConnectionHandler handler ;

{
    KesConnectionHandler retval = server->connection_handler ;
    
    server->connection_handler = handler ;
    return (retval) ;
}

KesErrorHook KesSetErrorHook (server, error_hook)
    KesServer    server ;
    KesErrorHook error_hook ;

{
    KesErrorHook retval = server->error_hook ;

    server->error_hook = error_hook ;
    return (retval) ;
}
/***************************************************************************
 * KesClientKill: kill this client
 *
 * Kill the client:
 * - Invoke the server kill_handler on this client
 * - shutdown the protocol
 * - free client's resources (notifications)
 *
 * RETURN: void.
 ***************************************************************************/

void KesClientKill (client)
    KesClient client ;
{
    KesServer server   = KesServerOfClient (client) ;
    IceConn   ice_conn = client->ice_connection ;

    KesNodeNotifyClientKillHandler (client) ;
    if ( server->kill_handler )
	(server->kill_handler) (client) ;
    _KesClientIndex[IceConnectionNumber(ice_conn)] = (KesClient) 0 ;
/*
 * Use to call IceCloseConnection() here, but this is a bad idea, bevause
 * this handler (KesClientKill) is onvoked potentially from IceIOError proc
 * which needs access to the ICE connection until terminated.
 * I should better use here the ProtocolShutdow () stuff...
 */
    /* IceCloseConnection (ice_conn) */
    KeFree (client) ;
    return ;
}
