/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSTCbSet.c
 *
 * Callback set management functions
 ***************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include <KeServer.h>
#include <KesServerP.h>


/***************************************************************************
 * KesHandlerSetMake: create a Handler set with default handlers
 *
 * This function creates a default handler set, with null handlers
 *
 * RETURN: a handler set or NULL
 ***************************************************************************/

KesHandlerSet KesHandlerSetMake ()
    
{
    KesHandlerSet handler_set ;

    if ( handler_set = (KesHandlerSet) malloc (sizeof (KesHandlerSetRec)) )
    {
	/*
	 * Set all handlers to Null:
	 */
	handler_set->ls_handler     = KesNullLsHandler ;
	handler_set->stat_handler   = KesNullStatHandler ;
	handler_set->get_handler    = KesNullGetHandler ;
	handler_set->set_handler    = KesNullSetHandler ;
	handler_set->exec_handler   = KesNullExecHandler ;
	handler_set->notify_handler = KesNullNotifyHandler ;
    }

    return (handler_set) ;
}

/***************************************************************************
 * KesHandlerSetLsHandler
 * KesHandlerSetStatHandler
 * KesHandlerSetGetHandler
 * KesHandlerSetSetHandler
 * KesHandlerSetExecHandler
 * KesHandlerSetNotifyHandler
 *
 * Change a handler inside a handler set.
 *
 * RETURN: old callback value.
 ***************************************************************************/

KesLsHandler KesHandlerSetLsHandler (handler_set, ls_handler)
    KesHandlerSet handler_set ;
    KesLsHandler  ls_handler ;

{
    KesLsHandler old_value = handler_set->ls_handler ;

    handler_set->ls_handler = ls_handler ;
    return (old_value) ;
}

KesStatHandler KesHandlerSetStatHandler (handler_set, stat_handler)
    KesHandlerSet  handler_set ;
    KesStatHandler stat_handler ;

{
    KesStatHandler old_value = handler_set->stat_handler ;

    handler_set->stat_handler = stat_handler ;
    return (old_value) ;
}

KesGetHandler KesHandlerSetGetHandler (handler_set, get_handler)
    KesHandlerSet handler_set ;
    KesGetHandler get_handler ;

{
    KesGetHandler old_value = handler_set->get_handler ;
    
    handler_set->get_handler = get_handler ;
    return (old_value);
}

KesSetHandler KesHandlerSetSetHandler (handler_set, set_handler)
    KesHandlerSet handler_set ;
    KesSetHandler set_handler ;

{
    KesSetHandler old_value = handler_set->set_handler ;
    
    handler_set->set_handler = set_handler ;
    return (old_value) ;
}

KesExecHandler KesHandlerSetExecHandler (handler_set, exec_handler)
    KesHandlerSet  handler_set ;
    KesExecHandler exec_handler ;

{
    KesExecHandler old_value = handler_set->exec_handler ;
    
    handler_set->exec_handler = exec_handler ;
    return (old_value) ;
}

KesNotifyHandler KesHandlerSetNotifyHandler (handler_set, notify_handler)
    KesHandlerSet    handler_set ;
    KesNotifyHandler notify_handler ;

{
    KesNotifyHandler old_value = handler_set->notify_handler ;
    
    handler_set->notify_handler = notify_handler ;
    return (old_value) ;
}
				  
