/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSTCodec.c
 *
 * Codec management functions.
 ***************************************************************************/

#include <stdio.h>
#include <KeServer.h>
#include <KesCodec.h>

static KesHandlerSet KesCodecHandlerSet = (KesHandlerSet) 0 ;

/***************************************************************************
 * KesCodecHandlerSet: the handler for codec description nodes
 *
 ***************************************************************************/


static KeStatus KesCodecStatHandler (request, reply)
    KesStatReq request ;
    KesReply   reply ;

{
    KeCodec codec = (KeCodec) request->closure ;
    
    KesStatReplyFillStatic (reply
			    , codec->name
			    , (unsigned long) 0		/* mode */
			    , (unsigned long) 0		/* mask */
			    , (KID) 0
			    , -1 ) ;

    return (KEDIT_Success) ;
}

KesHandlerSet KesCodecHandlerSetGet ()

{
    if ( ( ! KesCodecHandlerSet ) 
	&& ( KesCodecHandlerSet = KesHandlerSetMake() ) )
    {
	KesHandlerSetStatHandler (KesCodecHandlerSet, KesCodecStatHandler) ;
    }
    return (KesCodecHandlerSet) ;
}

/***************************************************************************
 * KesCodecRegister: register the given codec to a server
 *
 * This function will always return the same node id for teh same codec.
 *
 * RETURNS: a node identifier to be used for codec
 ***************************************************************************/

KeStatus KesCodecRegister (server, codec, node_ret)
    KesServer server ;
    KeCodec   codec ;
    KID       *node_ret ;

{
    KeStatus status ;

    if ( (status = KesNodeMake (server
				, KesCodecHandlerSetGet()
				, (KePointer) codec
				, node_ret)) == KEDIT_Success )
    {
	int i ;

	if ( ! server->codec_directory )
	{
	    server->codec_directory = KesDirectoryMake (server, "Codecs") ;
	    KesDirectoryAddChild (KesRootDirectoryOfServer (server)
				  , server->codec_directory->node) ;
	}

	for ( i = 0 ; i < server->codec_directory->used ; i++ )
	{
	    if ( server->codec_directory->subdirs[i] == (*node_ret) )
		return (KEDIT_Success) ;
	}

	KesDirectoryAddChild (server->codec_directory, *node_ret) ;
    }

    return (status) ;
}

KeCodec KesCodecFind (server, codec_node)
    KesServer server ;
    KID       codec_node ;

{
    KePointer     codec ;
    KesHandlerSet hset ;

    if ( ! KesNodeToInfo (server, codec_node, &codec, &hset) ) {
	if ( hset == KesCodecHandlerSetGet () )
	    return ((KeCodec) codec) ;
    }
    return ((KeCodec) 0) ;
}
