/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSTIds.c
 *
 * This file is part of the "toolkit". It provides ID maanagement function
 ***************************************************************************/


#include <stdio.h>
#include <string.h>

#include <KeServer.h>
#include <KesServerP.h>

/***************************************************************************
 * bitmask: returns the used bits (as a mask) for given number
 *
 * I'm no good at math, but for sure, there exist a better way for this.
 *
 * RETURNS: used bit as a mask
 ***************************************************************************/

static unsigned long bitmask (n)
    unsigned long n ;

{
    static unsigned long masks[] 
	= { 0x00000001, 0x00000003, 0x00000007, 0x0000000F
	  , 0x0000001F, 0x0000003F, 0x0000007F, 0x000000FF
	  , 0x000001FF, 0x000003FF, 0x000007FF, 0x00000FFF
	  , 0x00001FFF, 0x00003FFF, 0x00007FFF, 0x0000FFFF
	  , 0x0001FFFF, 0x0003FFFF, 0x0007FFFF, 0x000FFFFF
	  , 0x001FFFFF, 0x003FFFFF, 0x007FFFFF, 0x00FFFFFF
	  , 0x01FFFFFF, 0x03FFFFFF, 0x07FFFFFF, 0x0FFFFFFF
	  , 0x1FFFFFFF, 0x3FFFFFFF, 0x7FFFFFFF, 0xFFFFFFFF } ;
    unsigned long nn = ~ n ;
    int           pos ;

    if ( ! n )
	return (0) ;

    for ( pos = 31 ; nn & ( 1 << pos ) ; pos-- )
	;

    return (masks[pos]) ;
}

/***************************************************************************
 * KesNodeMake: create a node inside a server, for a handler set.
 *
 * This function first lookup to see if the node doesn't exist (in which
 * case the found node is returned), otherwise, it creates it.
 * 
 * RETURNS: the node identifier
 ***************************************************************************/

KeStatus KesNodeMake (server, handler_set, closure, node_ret)
    KesServer     server ;
    KesHandlerSet handler_set ;
    KePointer     closure ;
    KID           *node_ret ;

{
    unsigned long hashval  = KesClosureHash (closure) ;
    KesNodeInfo   node_info ;

/*
 * Ok, now look inside the handler set for given closure:
 */

    if ( server->nodes )
    {
	for ( node_info = server->nodes[(hashval & KES_KEY_RANGE) % server->nodes_size]
	     ; node_info 
	     ; node_info = node_info->next )
	{
	    if ( (node_info->closure == closure) 
		&& (node_info->handler_set == handler_set) 
		&& (node_info->count == 1) )
	    {
		*node_ret = node_info->node ;
		return (KEDIT_Success) ;
	    }
	}
    }
    else 
    {
	int default_size = sizeof (KesNodeInfo) * KES_NODESET_STARTSIZE ;

	server->nodes      = (KesNodeInfo *) KesMalloc (server, default_size) ;
	server->nodes_size = KES_NODESET_STARTSIZE ;
	bzero ((char *) server->nodes, default_size) ;
    }

/*
 * Check for a resize of the nodes array :
 */

    if ( server->nodes_count > (server->nodes_size << 2) )
    {
	fprintf (stderr, "KesNodeMake: should resize hashtable (not done)\n") ;
    }

/*
 * Create the node info:
 */

    if (node_info = (KesNodeInfo) KesMalloc (server, sizeof (KesNodeInfoRec)))
    {
	KesNodeInfo *phead ;

	node_info->node        = KesNodeCompose (server->node_key, hashval) ;
	node_info->handler_set = handler_set ;
	node_info->closure     = closure ;
	node_info->count       = 1 ;

	server->node_key = (server->node_key + 1) & KES_KEY_RANGE ;

	phead = &(server->nodes[(hashval &0xFFFF) % server->nodes_size]) ;
	node_info->next = (*phead) ;
	*phead          = node_info ;

	*node_ret       = node_info->node ;

	return (KEDIT_Success) ;
    }
	
    return (KEDIT_BadMalloc) ;
}

/***************************************************************************
 * KesNodeRangeMake: allocate a range of identifiers
 *
 * This function should be used to allocate range of node for the same
 * closure. It is provided so that objects attributes are allocated a 
 * single bunch of nodes (instead of one node per attribute).
 *
 * RETURNS: the base identifier
 ***************************************************************************/

KeStatus KesNodeRangeMake (server, handler_set, closure, count, base_ret)
    KesServer     server ;
    KesHandlerSet handler_set ;
    KePointer     closure ;
    int           count ;
    KID		  *base_ret ;

{
    unsigned long hashval = KesClosureHash (closure) ;
    KesNodeInfo   node_info ;

/*
 * Lookup for an existing node :
 */

    if ( server->nodes )
    {
	for ( node_info = server->nodes[(hashval & KES_KEY_RANGE) % server->nodes_size]
	     ; node_info
	     ; node_info = node_info->next ) 
	{
	    if ( (node_info->closure == closure)
		&& (node_info->handler_set == handler_set) 
		&& (node_info->count == count) )
	    {
		*base_ret = node_info->node ;
		return (KEDIT_Success) ;
	    }
	}
    }
    else 
    {
	int default_size = sizeof (KesNodeInfo) * KES_NODESET_STARTSIZE ;

	if ( server->nodes = (KesNodeInfo *) KesMalloc (server, default_size) )
	{
	    server->nodes_size  = KES_NODESET_STARTSIZE ;
	}
	else
	{
	    return (KEDIT_BadMalloc) ;
	}
    }

/*
 * Check for a resize of the server nodes array :
 */

    if ( server->nodes_count > (server->nodes_size << 2) )
    {
	fprintf (stderr, "KesNodeRangeMake: should resize node_set (not done)\n") ;
    }

/*
 * Add this node-range to the nodes :
 * - Align the key for nodes in order to allocate abit aligned range.
 * - Increment the key properly
 */

    if (node_info = (KesNodeInfo) KesMalloc (server, sizeof (KesNodeInfoRec)))
    {
	KesNodeInfo *phead ;
	int         range_bits = bitmask (count) ;
	
	server->node_key = (server->node_key + range_bits) & (~range_bits) ;

	node_info->handler_set = handler_set ;
	node_info->closure     = closure ;
	node_info->node        = KesNodeCompose (server->node_key, hashval) ;
	node_info->count       = count ;

	server->node_key = (server->node_key + (range_bits + 1)) & KES_KEY_RANGE ;

	phead = &(server->nodes[(hashval & 0xFFFF) % server->nodes_size]) ;
	node_info->next = (*phead) ;
	*phead          = node_info ;

	*base_ret       = node_info->node ;

	return (KEDIT_Success) ;
    }

    return (KEDIT_BadMalloc) ;
}
    
/***************************************************************************
 * KesNodeFree: free the given node.
 *
 * This function doesn't look inside NodeRange's to find the corresponding
 * node. Use KesNodeRangeFree on NodeRange's.
 *
 * RETURN: a KeStatus (success if removed)
 ***************************************************************************/

KeStatus KesNodeFree (server, node)
    KesServer server ;
    KID       node ;
{
    KesNodeInfo   *pnode_info, node_info ;
/*
 * Find back the node in the nodes:
 */
    pnode_info = &(server->nodes[KesNodeKey(node) % server->nodes_size]) ;
    node_info  = *pnode_info ;
	
    while ( node_info )    {
	if ( node_info->node == node )	{
	    *pnode_info = node_info->next ;
	    (server->nodes_count)-- ;
	    KesNodeNotifyRemoveById (server, node) ;
	    KeFree (node_info) ;
	    return (KEDIT_Success) ;
	}
	pnode_info = &(node_info->next) ;
	node_info  = node_info->next ;
    }
    return (KEDIT_InvalidId) ;
}

KeStatus KesNodeRangeFree (server, base)
    KesServer server ;
    KID       base ;
{
    return (KesNodeFree (server, base)) ;
}

/***************************************************************************
 * KesNodeToInfo: retrieve the info associated with the given id
 *
 * The returned valus are unfolded.
 *
 * RETURN: a KeStatus indicating if the return pid_info is valid.
 ***************************************************************************/

KeStatus KesNodeToInfo (server, node, closure_ret, handler_set_ret)
    KesServer     server ;
    KID           node ;
    KePointer     *closure_ret ;
    KesHandlerSet *handler_set_ret ;
{
    KesNodeInfo   node_info ;

    *closure_ret     = (KePointer) 0 ;
    *handler_set_ret = (KesHandlerSet) 0 ;

/*
 * Lookup for given node :
 */

    for ( node_info = server->nodes[KesNodeKey(node) % server->nodes_size]
	 ; node_info
	 ; node_info = node_info->next )     {
	if ( node_info->count > 1 )	{
	    unsigned long nrange_bits = ~ bitmask (node_info->count) ;
	    unsigned long knode       = KesNodeIndex (node) & nrange_bits ;

	    if ( ( knode == (KesNodeIndex (node_info->node) & nrange_bits) ) 
		&&(knode + node_info->count >= KesNodeIndex (node_info->node)))
	      {
		*closure_ret     = node_info->closure ;
		*handler_set_ret = node_info->handler_set ;
		return (KEDIT_Success) ;
	      }
	} else if ( node_info->node == node ) {
	    *closure_ret     = node_info->closure ;
	    *handler_set_ret = node_info->handler_set ;
	    return (KEDIT_Success) ;
	}
    }
    return (KEDIT_InvalidId) ;
}

/***************************************************************************
 * KesInfoToNode: retreive node associated with given infos
 *
 * The returned valus are unfolded.
 *
 * RETURN: a KeStatus indicating if the return pid_info is valid.
 ***************************************************************************/

KeStatus KesInfoToNode (server, handler_set, closure, node_ret)
    KesServer     server ;
    KesHandlerSet handler_set ;
    KePointer     closure ;
    KID           *node_ret ;

{
    KesNodeInfo   node_info ;
    unsigned long hashval  = KesClosureHash (closure) ;

    if ( server->nodes )
    {
	for ( node_info = server->nodes[(hashval & KES_KEY_RANGE) % server->nodes_size]
	     ; node_info 
	     ; node_info = node_info->next )
	{
	    if ( (node_info->closure == closure) 
		&& (node_info->handler_set == handler_set) 
		&& (node_info->count == 1) )
	    {
		*node_ret = node_info->node ;
		return (KEDIT_Success) ;
	    }
	}
    }

    return (KEDIT_InvalidId) ;
}

/***************************************************************************
 * KesInfoToNodeRange: retreive node associated with given infos
 *
 * The returned valus are unfolded.
 *
 * RETURN: a KeStatus indicating if the return pid_info is valid.
 ***************************************************************************/

KeStatus KesInfoToNodeRange (server, handler_set, closure, count, base_ret)
    KesServer     server ;
    KesHandlerSet handler_set ;
    KePointer     closure ;
    int           count ;
    KID           *base_ret ;

{
    KesNodeInfo   node_info ;
    unsigned long hashval  = KesClosureHash (closure) ;

    if ( server->nodes )
    {
	for ( node_info = server->nodes[(hashval & KES_KEY_RANGE) % server->nodes_size]
	     ; node_info 
	     ; node_info = node_info->next )
	{
	    if ( (node_info->closure == closure) 
		&& (node_info->handler_set == handler_set) 
		&& (node_info->count == count) )
	    {
		*base_ret = node_info->node ;
		return (KEDIT_Success) ;
	    }
	}
    }

    return (KEDIT_InvalidId) ;
}

