/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * KSpevent.c
 *
 * Print in a human readable format the event.
 ***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#ifdef NEEDS_SELECT_H
#include <sys/select.h>
#endif /* NEEDS_SELECT_H */
#include <netinet/in.h>
#include <fcntl.h>
#include <netdb.h>
#include <errno.h>

#include <KeServer.h>

/***************************************************************************
 * KesPrintRequest(): print the given request to given stream
 ***************************************************************************/

void KesRequestPrint (FILE *fd, KesRequest request)

{

    /* Usefull debugging feature */
    if ( ! fd )
	fd = stderr ;
    switch ( KesTypeOfRequest (request) )
    {
      case KEDIT_Ls:
	{
	    KesLsReq ls = (KesLsReq) request ;

	    fprintf (fd, "K_Ls\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)) ;
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    fprintf (fd, "\tnode:		x%08x.\n", ls->node) ;
	    return ;
	}
      case KEDIT_Stat:
	{
	    KesStatReq rstat = (KesStatReq) request ;

	    fprintf (fd, "K_Stat\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)) ;
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    fprintf (fd, "\tnode:		x%08x.\n", rstat->node) ;

	    return ;
	}
      case KEDIT_Get:
	{
	    KesGetReq get = (KesGetReq) request ;

	    fprintf (fd, "K_Get\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)) ;
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    fprintf (fd, "\tnode:		x%08x.\n", get->node) ;
	    return ;
	}
      case KEDIT_Set:
	{
	    KesSetReq set = (KesSetReq) request ;

	    fprintf (fd, "K_Set\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)) ;
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    fprintf (fd, "\tnode:		x%08x.\n", set->node) ;
	}
	break ;
      case KEDIT_Exec:
	{
	    KesExecReq exec = (KesExecReq) request ;

	    fprintf (fd, "K_Exec\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)) ;
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    fprintf (fd, "\tnode:		x%08x.\n", exec->node) ;
	    return ;
	}
      case KEDIT_Notify:
	{
	    KesNotifyReq notify = (KesNotifyReq) request ;

	    fprintf (fd, "K_Notify\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)) ;
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    fprintf (fd, "\tnode:		x%08x.\n", notify->node) ;
	    fprintf (fd, "\tmask:                x%08x\n", notify->mask) ;
	    return ;
	}
      case KEDIT_Noop:
	{
	    fprintf (fd, "K_Noop\n") ;
	    fprintf (fd, "\tserial:		%d\n", KesSerialOfRequest (request)); 
	    fprintf (fd, "\tserver:		@x%08x.\n", KesServerOfRequest (request)) ;
	    fprintf (fd, "\tclient:		@x%08x.\n", KesClientOfRequest (request)) ;
	    return ;
	}
      default:
	fprintf (fd, "UNKNOW REQUEST TYPE %d\n", KesTypeOfRequest (request)) ;
	return ;
    }
}

