/***************************************************************************
 * KCTGet.c
 *
 * A - very - naive implementation of a cache of nodes codec.
 ***************************************************************************/


#include <stdio.h>

#include <KeClient.h>
#include <KeCodec.h>

#define KCODEC_CACHE_SIZE	17
#define KCODEC_MX_CACHED 	32

typedef struct _CachedCodec *CachedCodec ;

typedef struct _CachedCodec {
    CachedCodec   lru ;
    CachedCodec   blru ;
    CachedCodec   next ;
    KecConnection kconn ;
    KID		  kid ;
    KeCodec       codec ;
} CachedCodecRec ;

static CachedCodec theCodecCache[KCODEC_CACHE_SIZE] = { (CachedCodec) 0, } ;
static CachedCodec theLRUHead = (CachedCodec) 0, theLRUTail = (CachedCodec) 0 ;
static int         theCachedCount = 0 ;

#define HASH_CACHED(c,i) \
  ((((unsigned long) (i)) ^ (((unsigned long) (c)) >> 2)) % KCODEC_CACHE_SIZE)

/***************************************************************************
 * getConnectionCodecCache: retreives codec from KID
 *
 * This uses the kconn codec cache to speed ops.(NOT IMPLEMENTED)
 ***************************************************************************/

static int           theCodecListS = 0, theCodecListC = 0 ;
static KeCodec       *theCodecList = (KeCodec*) 0 ;
static KID           *theCodecListID = (KID *) 0 ;
static KecConnection theCodecListConn = (KecConnection) 0 ;

static void fillConnectionCodecCache (kconn)
    KecConnection kconn ;

{
    KID        codec_node ;
    char       *codec_path[] = { "Root", "Codecs" } ;
    KeNodeStat *stats ;
    int        count ;

    if ( kconn == theCodecListConn )
	return ;
    if ( (codec_node = KecFindRoot (kconn, codec_path, 2)) == (KID) -1 )
	return ;
    if ( (count = KecLsl (kconn, codec_node, &stats)) == -1 )
	return ;
    if ( theCodecListS < count ) {
	if ( theCodecListS ) {
	    KeFree (theCodecList) ;
	    KeFree (theCodecListID) ;
	}
	theCodecListS  = count ;
	theCodecList   = (KeCodec*) KecMalloc (kconn, count*sizeof (KeCodec));
	theCodecListID = (KID *) KecMalloc (kconn, count * sizeof (KID)) ;
    }
    theCodecListC    = count ;
    theCodecListConn = kconn ;
    while ( count-- ) {
	theCodecList[count]   = KeCodecGet (stats[count]->name) ;
	theCodecListID[count] = stats[count]->id ;
    }
    KecLslFree (stats) ;
    return ;
}

KID KecNamedCodecGet (kconn, name)
    KecConnection kconn ;
    char          *name ;
{
    int i ;
    fillConnectionCodecCache (kconn) ;
    for (i = 0 ; i < theCodecListC ; i++) {
	if ( ! strcmp (theCodecList[i]->name, name))
	    return (theCodecListID[i]) ;
    }
    return ((KID) -1) ;
}

KeCodec KecCodecGet (kconn, kid)
    KecConnection kconn ;
    KID           kid ;
{
    int i ;

    fillConnectionCodecCache (kconn) ;
    for (i = 0 ; i < theCodecListC ; i++ ) {
	if ( theCodecListID[i] == kid ) 
	    return (theCodecList[i]) ;
    }
    return ((KeCodec) 0) ;
}

/***************************************************************************
 * KecGetDecode: get and decode "on the fly" the given node
 * KecSetEncode: encode and set "on the fly" the given node
 *
 * We maintain a small cache - KCODEC_CACHEDSIZE - of lastly examined
 * nodes. This is done with a classical version of LRU (doubly linked list)
 * through wich we obtain the codec associated to the node. This suppose
 * that the association of a codec to a node *DOESN'T* change...
 ***************************************************************************/

static void useCached (cached)
    CachedCodec cached ;

{
    if ( theLRUHead == cached )
	return ;
    if ( theLRUTail == cached ) {
	theLRUTail       = cached->blru ;
	theLRUTail->lru  = (CachedCodec) 0 ;
	cached->lru      = theLRUHead ;
	theLRUHead->blru = cached ;
	theLRUHead       = cached ;
	cached->blru     = (CachedCodec) 0 ;
    } else {
	cached->blru->lru = cached->lru ;
	cached->lru->blru = cached->blru ;
	cached->blru      = (CachedCodec) 0 ;
	theLRUHead->blru  = cached ;
	cached->lru       = theLRUHead ;
	theLRUHead        = cached ;
    }
    return ;
}

static CachedCodec getLRU ()
{
    CachedCodec cached = theLRUTail ;
    CachedCodec ent, *pent ;

    theLRUTail      = cached->blru ;
    theLRUTail->lru = (CachedCodec) 0 ;
    pent = &(theCodecCache[HASH_CACHED(cached->kconn,cached->kid)]) ;
    for ( ent = *pent ; ent != cached ; ) {
	pent = &(ent->next) ;
	ent  = ent->next ;
    }
    *pent        = ent->next ;
    cached->next = (CachedCodec) 0 ;
    return (cached) ;
}

static void addCache (kconn, kid, codec)
    KecConnection kconn ;
    KID           kid ;
    KeCodec       codec ;
    
{
    CachedCodec cached, *pent ;
    
    if ( theCachedCount + 1 >= KCODEC_MX_CACHED ) 
	cached = getLRU () ;
    else
	cached = (CachedCodec) KecMalloc (kconn, sizeof (CachedCodecRec)) ;
    if ( ! cached )
	return ;
    cached->kconn = kconn ;
    cached->kid   = kid ;
    cached->codec = codec ;
    pent          = &(theCodecCache[HASH_CACHED(cached->kconn,cached->kid)]) ;
    cached->next  = *pent ;
    *pent         = cached ;
    if ( ! theLRUHead ) {
	theLRUHead   = cached ;
	cached->lru  = (CachedCodec) 0 ;
	cached->blru = (CachedCodec) 0 ;
    } else if ( ! theLRUTail ) {
	theLRUTail = theLRUHead ;
	theLRUHead = cached ;
	theLRUHead->lru  = theLRUTail ;
	theLRUHead->blru = (CachedCodec) 0 ;
	theLRUTail->blru = theLRUHead ;
	theLRUTail->lru  = (CachedCodec) 0 ;
    } else {
	cached->lru      = theLRUHead ;
	cached->blru     = (CachedCodec) 0 ;
	theLRUHead->blru = cached ;
	theLRUHead       = cached ;
    }
    theCachedCount++ ;
    return ;
}

KeCodec KecNodeCodec (kconn, kid) 
    KecConnection kconn ;
    KID           kid ;

{
    CachedCodec *pent = &(theCodecCache[HASH_CACHED(kconn, kid)]) ;
    CachedCodec cached = *pent ;
    CachedCodec *phead  = pent ;
    KeNodeStat  kstat ;
    KeCodec     codec ;

    if (theLRUHead && (theLRUHead->kconn==kconn) && (theLRUHead->kid==kid))
	return (theLRUHead->codec) ;
    while ( cached ) {
	if ( (cached->kid == kid) && (cached->kconn == kconn) ) {
	    /* bring it back to the front */
	    if ( pent != phead ) {
		*pent        = cached->next ;
		cached->next = *phead ;
		*phead       = cached ;
	    }
	    useCached (cached) ;
	    return (cached->codec) ;
	}
	pent   = &(cached->next) ;
	cached = cached->next ;
    }
    /* We could use here a async call, this would be fun...and would save a
     * round trip (this function is called from KecGetDecode(), but would
     * also involve a CPS style, which is not recommended in C ! */
    if ( KecStatSync (kconn, kid, &kstat) )
	return ((KeCodec) 0) ;
    codec = KecCodecGet (kconn, kstat->info) ;
    addCache (kconn, kid, codec) ;
    KeFree (kstat) ;
    return (codec) ;
}

/***************************************************************************
 * KecGetDecode: get and decode node
 *
 * This is kept asynchronous, so you should better KecConnectionDoSync
 * before getting the result (in presult). The provided result should be
 * freed by calling the KecGetDecodeFree function.
 ***************************************************************************/

typedef struct _getDec {
    KePointer *presult ;
    KeCodec   codec ;
} getDecRec, *getDec ;

#define GETDEC_STACK_SIZE	32

static getDecRec getDecStack[GETDEC_STACK_SIZE] ;
static getDec    getDecPtr = getDecStack ;

#define GETDEC_INC(p) \
    ((++p >= getDecStack + GETDEC_STACK_SIZE) ? (p = getDecStack) : p)

static void getDecodeHandler (reply, item_ptr)
    KecReply  reply ;
    KePointer item_ptr ;
{
    getDec    item = (getDec) item_ptr ;
    
    if ( ! KecStatusOfReply (reply) ) {
	KecValue value = KecValueOfReply (reply) ;
	char *emsg ; KeErrorFreeFct efree ;
	
	KeCodecDecode(item->codec
		      , value->size
		      , (KePointer) value->data
		      , KecConnectionSwapped(KecConnectionOfReply(reply))
		      , item->presult
		      , &emsg, &efree) ;
	KeFree (value) ;
    }
    return ;
}

KeStatus KecGetDecode (kconn, kid, codec, presult)
    KecConnection kconn ;
    KID           kid ;
    KeCodec       codec ;
    KePointer     *presult ;
{
    KeStatus status ;
    getDec   item = getDecPtr ;
    
    item->codec   = codec ;
    item->presult = presult ;
    if (!(status=KecGet (kconn, kid, getDecodeHandler, (KePointer) item))) {
	GETDEC_INC (getDecPtr) ;
    }
    return (status) ;
}

KeStatus KecSetEncode (kconn, kid, codec, value)
    KecConnection kconn ;
    KID           kid ;
    KeCodec       codec ;
    KePointer     value ;

{
    int       size ;
    KePointer evalue ;
    KeStatus  status ;
    
    char *emsg ; KeErrorFreeFct efree ;
    
    if ( status = KeCodecEncode (codec,value,&size,&evalue,&emsg,&efree) ) {
	if (efree) (*efree) (emsg) ;
	return (status) ;
    }
    status = KecSet (kconn, kid, size, value) ;
    KeCodecEncoderFree (codec, evalue) ;
    return (status) ;
}

				 
