/***************************************************************************
 * connection.c
 *
 * Connect to the name server, and setup
 * In lots of places, some kind of XrmQuark should be used...
 ***************************************************************************/

#include <stdio.h>

#include <KeNS.h>

#define KNS_SERVERS_SIZE	17

static void knsServerUpdate (kconn, server)
    KecConnection kconn ;
    KnsServer     server ;

{
    KID propertor ;
    char *path[8] ;
    int  npath = 0 ;

    path[npath++] = "Root" ;
    path[npath++] = "*servers*" ;
    path[npath++] = server->name ;
    path[npath++] = "*propertor*" ;
    path[npath++] = "name" ;
    server->propertor_name = KecFindRoot (kconn, path, npath) ;
    if ( server->propertor_name != (KID) -1 ) {
	path[npath-1] = "type" ;
	server->propertor_type = KecFindRoot (kconn, path, npath) ;
    }
    return ;
}

static void knsServerAdd (kconn, kstat)
    KecConnection kconn ;
    KeNodeStat    kstat ;
{
    KnsData   knsd   = KNS_CONNDATA(kconn) ;
    KnsServer server = (KnsServer) malloc (sizeof (KnsServerRec)) ;
    
    if ( knsd->serversC + 1 >= knsd->serversS ) {
	int nsize = (knsd->serversS + 8) * (sizeof (KnsServerRec *)) ;
	if ( knsd->serversS )
	    knsd->servers = (KnsServer *) realloc (knsd->servers,nsize) ;
	else
	    knsd->servers = (KnsServer *) malloc (nsize) ;
	knsd->serversS += 8;
    }    
    if ( (! server) || (! knsd->servers) ) {
	fprintf (stderr, "knsServerAdd: malloc failed (fatal).\n") ;
	exit (1) ;
    }
    server->name        = (char *) strdup (kstat->name) ;
    server->node        = kstat->id ;
    server->propertiesC = 0 ;
    server->propertiesS = 0 ;
    server->properties  = (KnsProperty *) 0 ;
    server->propertor_name = (KID) -1 ;
    server->propertor_type = (KID) -1 ;
    knsServerUpdate (kconn, server) ;
    knsd->servers[knsd->serversC++] = server ;
    return ;
}

static void knsServerHandler (kconn, event_ptr, unused)
    KecConnection kconn ;
    KecEvent      event_ptr ;
    KePointer     unused ;

{
    if ( event_ptr->any.event_type == KEDIT_CreateMask ) {
	KecCreateEvent ev = (KecCreateEvent) event_ptr ;
	KeNodeStat     kstat ;

	if ( ! KecStatSync (kconn, ev->child, &kstat) ) {
	    knsServerAdd (kconn, kstat) ;
	    KeFree (kstat) ;
	}
    } else if ( event_ptr->any.event_type == KEDIT_DeleteMask ) {
	KecDeleteEvent ev = (KecDeleteEvent) event_ptr ;
	fprintf (stderr, "knsServerHandler: server deleted.\n") ;
    }
    return ;
}
    
KecConnection KnsOpen (name)
    char *name ;

{
    KecConnection kconn ;
    KnsData	  knsd ;
    KeNodeStat    *stats ;
    int           count ;
    char	  *path[8] ;
    KID           kid ;
    
    if ( ( ! name ) && ( ! (name = (char *) getenv ("KNSSERVER")) ) )
	name = KNS_DEFAULT_ADRESS ;
    if ( ! ( kconn = KecOpenConnection (name)) )
	return (kconn) ;
    if ( ! (knsd = (KnsData) malloc (sizeof (KnsDataRec)) ) ) {
	KecCloseConnection (kconn) ;
	return ((KecConnection) 0) ;
    }
    KecSetUserDataOfConnection (kconn, knsd) ;
    knsd->server_dir    = (KID) -1 ;
    knsd->server_notify = (KecNotifyId) 0 ;
    knsd->serversS      = 0 ;
    knsd->serversC      = 0 ;
    knsd->servers       = (KnsServer *) 0 ;
    knsd->servor_name        = (KID) -1 ;
    knsd->servor_adress      = (KID) -1 ;
    knsd->servor_protocol    = (KID) -1 ;
    knsd->servor_description = (KID) -1 ;
    /* Get the *servers* directory, setup notify, and build current servers */
    path[0] = "Root" ;
    path[1] = "*servers*" ;
    if ( (knsd->server_dir = KecFindRoot (kconn, path, 2)) == (KID) -1 ) {
	fprintf (stderr, "KnsOpen: invalid server at %s\n", name) ;
	return ((KecConnection) 0) ;
    }
    if ( ! (knsd->server_notify = KecNotifyAdd (kconn
						, knsd->server_dir
						, (KEDIT_CreateMask
						   | KEDIT_DeleteMask)
						, knsServerHandler
						, (KePointer) 0)) ) {
	KecCloseConnection (kconn) ;
	return ((KecConnection) 0) ;
    }
    if ( (count = KecLsl (kconn, knsd->server_dir, &stats)) < 0 ) {
	KecCloseConnection (kconn) ;
	return ((KecConnection) 0) ;
    }
    while ( count-- )
	knsServerAdd (kconn, stats[count]) ;
    KecLslFree (stats) ;
    /* Get the *servor* stuff */
    path[0] = "Root" ;
    path[1] = "*servers*" ;
    path[2] = "NameServer" ;
    path[3] = "*servor*" ;
    if ( (kid = KecFindRoot (kconn, path, 4)) != (KID) -1 ) {
	if ( (count = KecLsl (kconn, kid, &stats)) > 0 ) {
	    while ( count-- ) {
		if ( ! strcmp ("name", stats[count]->name) ) {
		    knsd->servor_name = stats[count]->id ;
		} else if ( ! strcmp ("adress", stats[count]->name) ) {
		    knsd->servor_adress = stats[count]->id ;
		} else if ( ! strcmp ("protocol", stats[count]->name) ) {
		    knsd->servor_protocol = stats[count]->id ;
		} else if ( ! strcmp ("description", stats[count]->name)) {
		    knsd->servor_description = stats[count]->id ;
		} else {
		    fprintf (stderr, "KnsOpen: skip *servor* %s filed.\n"
			     , stats[count]->name) ;
		}
	    }
	    KecLslFree (stats) ;
	}
    }
    return (kconn) ;
}
