/***************************************************************************
 * kns.c
 ***************************************************************************/

#include <stdio.h>

#define _MAIN_

#include <KeServer.h>
#include <KeStruct.h>

#include "kns.h"

KesServer theKNS ;

static KesDirectory  theServerDir ;
static KID           theStringNode, theCard32Node, theCard16Node, theByteNode ;
static KesHandlerSet thePropertyHandlerSet = (KesHandlerSet) 0 ;
static Server        theServerList = (Server) 0 ;

/***************************************************************************
 * utilities
 ***************************************************************************/

void *SafeMalloc (size)
    unsigned int size ;

{
    void *p = (void *) malloc (size) ;
    if ( p )
	return (p) ;
    fprintf (stderr, "SafeMalloc: malloc failed.\n") ;
    exit (1) ;
}

/***************************************************************************
 * property stuff
 ***************************************************************************/

static KeStatus propertyStat (req, reply)
    KesStatReq req ;
    KesReply   reply ;

{
    Property prop = (Property) req->closure ;
    KID      info ;
    KeStatus status ;
    
    if ( status = KesCodecRegister(req->server, prop->codec, &info) ) {
	KesErrorReplyFillStatic (reply, status, "Couldn't register codec.");
    } else {
	KesStatReplyFillStatic (reply
				, prop->name
				, PropOwner(req,prop) ? PROP_OMODE : PROP_MODE
				, prop->mask
				, info
				, prop->size) ;
    } 
    return (KEDIT_Success) ;
}

static KeStatus propertySet (req,rep)
    KesSetReq  req ;
    KesReply   rep ;
{
    Property prop = (Property) req->closure ;

    if ( ! PropOwner (req, prop) ) {
	KesErrorReplyFillStatic (rep, KEDIT_InvalidOp, "No permission.") ;
	return (KEDIT_Success) ;
    }
    if ( prop->value )
	free (prop->value) ;
    if ( ! (prop->value = (KePointer) SafeMalloc (req->size)) ) 
	return (KEDIT_Failure) ;
    bcopy (req->value, prop->value, req->size) ;
    prop->size  = req->size ;
    KesWriteNotify (KesServerOfRequest (req)
		    , KesClientOfRequest (req)->client_tag
		    , KesNodeOfRequest (req)) ;
    KesWriteWithValueNotify (KesServerOfRequest (req)
			     , KesClientOfRequest(req)->client_tag
			     , KesNodeOfRequest (req)
			     , prop->size
			     , prop->value) ;
    return (KEDIT_Success) ;
}

static KeStatus propertyGet (req,reply)
    KesGetReq req ;
    KesReply  reply ;
{
    Property prop = (Property) req->closure ;
    
    KesGetReplyFillStatic (reply, prop->size, prop->value) ;
    return (KEDIT_Success) ;
}

static KeStatus propertyNotify (req,rep)
    KesNotifyReq req ;
    KesReply     rep ;
{
    return (KEDIT_Failure) ;
}

static KeStatus setProperty (prop, value)
    Property  prop ;
    KePointer value ;

{
    KePointer to = (KePointer) 0 ;
    int       size ;
    KeStatus  status ;
    char *emsg ; KeErrorFreeFct efree ;
       
    if (status = KeCodecEncode (prop->codec,value,&size,&to,&emsg,&efree) ) {
	if ( efree ) (*efree) (emsg) ;
	return (status) ;
    }
    if ( prop->value )
	free (prop->value) ;
    prop->value = (KePointer) SafeMalloc (size) ;
    prop->size  = size ;
    bcopy (to, prop->value, size) ;
    return (KEDIT_Success) ;
}

Property DeclareProperty (server, name, codec, init)
    Server    server ;
    char      *name ;
    KeCodec   codec ;
    KePointer init ;
{
    Property prop = (Property) SafeMalloc (sizeof (PropertyRec)) ;

    prop->server = server ;
    prop->name   = strdup (name) ;
    prop->codec  = codec ;
    prop->value  = (KePointer) 0 ;
    prop->size   = 0 ;
    prop->mask   = PROP_MASK ;
    if ( KesNodeMake(theKNS
		     , thePropertyHandlerSet
		     , (KePointer) prop 
		     , &(prop->node)) ) {
	fprintf (stderr, "DeclareProperty: KesNodeMake failed (fatal).\n");
	exit (1) ;
    }
    if ( init )
	setProperty (prop, init) ;
    KesDirectoryAddChild (server->dir, prop->node) ;
    return (prop) ;
}

void removeProperty (prop)
    Property prop ;
{
    Server server = prop->server ;

    fprintf (stderr, "removeProperty: %s for server %s\n"
	     , prop->name
	     , prop->server->dir->name) ;
    
    KesDirectoryRemoveChild (server->dir, prop->node) ;
    KesDeleteNotify (theKNS, server->owner->client_tag, prop->node) ;
    KesNodeFree (theKNS, prop->node) ;
    if ( prop->name )
	free (prop->name) ;
    if ( prop->value )
	free (prop->value) ;
    free (prop) ;
    return ;
}

/***************************************************************************
 * server stuff
 ***************************************************************************/

static int serverDirFilter (directory, request, kid)
    KesDirectory directory ;
    KesLsReq     request ;
    KID          kid ;

{
    Server    server = (Server) KesClosureOfDirectory (directory) ;
    KesClient client = KesClientOfRequest (request) ;
    
    if (server->owner && (server->owner->client_tag == client->client_tag))
	return (1 /* True */) ;
    else 
	return (kid != server->propertor->node) ;
    /* NOT REACHED */
}

Server DeclareServer (client, name, adress, protocol, description)
    KesClient client ;
    char *name, *adress, *protocol, *description ;

{
    Server   server = (Server) SafeMalloc (sizeof (ServerRec)) ;
    
    fprintf (stderr, "Building new server ofr %d\n"
	     , client ? client->client_tag : 0 ) ;
	     
    server->dir       = KesDirectoryMake (theKNS, name) ;
    KesClosureOfDirectorySet (server->dir, (KePointer) server) ;
    KesDirectoryFilterSet (server->dir, serverDirFilter) ;
    server->owner     = client ;
    server->propertor = (KesStruct) 0 ;
    server->servor    = (KesStruct) 0 ;
    DeclareProperty (server,"adress",KeCodecSTRINGGet(),(KePointer)adress) ;
    DeclareProperty (server,"protocol",KeCodecSTRINGGet(),(KePointer)protocol);
    DeclareProperty (server,"description",KeCodecSTRINGGet(),(KePointer)description) ;
    server->propertor = PropertorMake (server) ;
    KesDirectoryAddChild (server->dir, server->propertor->node) ;
    KesDirectoryAddChild (theServerDir, KesNodeOfDirectory(server->dir)) ;
    server->next  = theServerList ;
    theServerList = server ;
    return (server) ;
}

static void removeServer (server)
    Server server ;
{
    Server *prev, cur ;
    int i ;
    Property prop ;
    KesHandlerSet hset ;
    
/* Remove server from theServerList */
    prev = &theServerList ;
    cur  = theServerList ;
    while ( cur ) {
	if ( cur == server ) {
	    *prev = server->next ;
	    goto found ;
	}
	prev = &(server->next) ;
	cur  = server->next ;
    }
    return ;
    
/* If found, free all associated resources */
  found:
    fprintf (stderr, "removeServer: %s\n", server->dir->name) ;
    
    for (i = 0 ; i < server->dir->used ; i++ ) {
	if ( ! KesNodeToInfo (theKNS
			      , server->dir->subdirs[i]
			      , (KePointer *) &prop
			      , &hset) ) {
	    removeProperty (prop) ;
	}
    }
    KesDirectoryRemoveChild (theServerDir, KesNodeOfDirectory (server->dir));
    if ( server->propertor ) 
	PropertorDelete (server->propertor) ;
    if ( server->servor ) 
	ServorDelete (server->servor) ;
    KesDirectoryDelete (server->dir) ;
    
    return ;
}

/***************************************************************************
 * knsClientKill: clean up on client kill
 ***************************************************************************/

static void knsClientKill (client)
    KesClient client ;
{
    Server s = theServerList, *ps = &theServerList ;

    while ( s ) {
	if ( s->owner == client )
	    removeServer (s) ;
	s  = s->next ;
	ps = &(s->next) ;
    }
    return ;
}

main (argc, argv)
    int  argc ;
    char **argv ;

{
    Server myself ;
    
    PropertorInitialize () ;
    ServorInitialize () ;
    if ( ! (theKNS = KesServerInitialize ("NameServer")) ) {
	fprintf (stderr, "main: couldn't setup (Fatal).\n") ;
	exit (1) ;
    }
    /* Set the client kill handler (no previous!) */
    KesSetKillHandler (theKNS, knsClientKill) ;
    if ( KesCodecRegister (theKNS, KeCodecSTRINGGet(), &theStringNode)
	|| KesCodecRegister (theKNS, KeCodecCARD32Get(), &theCard32Node)
	|| KesCodecRegister (theKNS, KeCodecCARD16Get(), &theCard16Node)
	|| KesCodecRegister (theKNS, KeCodecBYTEGet(), &theByteNode)) {
	fprintf (stderr, "main: unable to register default codecs (Fatal).\n");
	exit (1);
    }
    /* Create the *servers* directory, and attach to roo */
    theServerDir = KesDirectoryMake (theKNS, "*servers*") ;
    KesDirectoryAddChild (KesRootDirectoryOfServer (theKNS)
			  , theServerDir->node) ;
    /* Create the Property handler set */
    thePropertyHandlerSet = KesHandlerSetMake () ;
    KesHandlerSetStatHandler   (thePropertyHandlerSet, propertyStat) ;
    KesHandlerSetGetHandler    (thePropertyHandlerSet, propertyGet) ;
    KesHandlerSetSetHandler    (thePropertyHandlerSet, propertySet) ;
    KesHandlerSetNotifyHandler (thePropertyHandlerSet, propertyNotify) ;
    /* Declare myself: I'm the only one having a  *servor* node */
    myself = DeclareServer ((KesClient) 0
			    , "NameServer"
			    , "tcp/localhost:2000"
			    , "k-Edit"
			    , "Lists running servers") ;
    myself->servor = ServorMake (myself) ;
    KesDirectoryAddChild (myself->dir, myself->servor->node) ;

    MyIceMainLoop () ;
}
