/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * from KSMloop.c
 *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <setjmp.h>
#include <signal.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <X11/ICE/ICElib.h>

extern int errno ;

/***************************************************************************
 * KesMainLoop: simple main server support for main loop
 *
 ***************************************************************************/

static void    *My_IceTranslations[128] = { (void *) 0 , } ;
static jmp_buf My_IceErrorEnv ;

static int My_IceAcceptConnection (listen_obj)
    IceListenObj listen_obj ;

{
    IceAcceptStatus status ;
    IceConn         ice_conn = IceAcceptConnection (listen_obj, &status) ;
    Bool            reply_ready ;
    
    if ( ice_conn )
    {
	My_IceTranslations[IceConnectionNumber(ice_conn)] = (void *) ice_conn ;

	do
	{
	    IceProcessMessages (ice_conn, NULL, &reply_ready) ;
	}
	while ( IceConnectionStatus (ice_conn) == IceConnectPending ) ;

	if ( IceConnectionStatus (ice_conn) != IceConnectAccepted ) 
	{
	    return (-1) ;
	}

	return (IceConnectionNumber (ice_conn)) ;
    }

    return (-1) ;
}

static void My_IceIOErrorHandler (ice_conn)
    IceConn ice_conn ;

{
    My_IceTranslations[IceConnectionNumber(ice_conn)] = (void *) 0 ;
    longjmp (My_IceErrorEnv, 1) ;
}

static Bool My_HostBasedAuthDisabled (hostname)
    char *hostname ;
{
    fprintf (stderr, "host %s with no auth accepted.\n", hostname) ;
    return (1) ;
}

void MyIceMainLoop ()

{
    int          listen_count ;
    IceListenObj *listens ;
    fd_set       rfds, ready_rfds, server_fds ;
    int          nready ;
    int          i, fd, new_fd, max_fd = 0 ;
    Bool         reply_ready ;
    char         *IceRef;
    char         s[BUFSIZ];
    FILE         *F;

/*
 * Listen for ICE connections:
 */

    FD_ZERO (&rfds) ;
    FD_ZERO (&ready_rfds) ;
    FD_ZERO (&server_fds) ;

    if ( ! IceListenForConnections (&listen_count
				    , &listens
				    , 0
				    , (char *) 0) )
    {
	return ;
    }
    else
    {
	sprintf(s,"%s/.kns.ref",getenv("HOME"));
	F=fopen(s,"w");
	for ( i = 0 , max_fd = 0 ; i < listen_count ; i++ )
	{
	    if (i>0) fprintf(F,",");
	    IceSetHostBasedAuthProc (listens[i], My_HostBasedAuthDisabled) ;
	    fd = IceGetListenConnectionNumber (listens[i]) ;
	    fprintf (stderr, "Ks: listening on %s\n"
		     , IceRef=IceGetListenConnectionString (listens[i])) ;
	    /* store ref in a file */
	    fprintf(F,"%s",IceRef);
	    if ( fd >= max_fd )
		max_fd = fd + 1 ;
	    FD_SET (fd, &server_fds) ;
	    My_IceTranslations[fd] = (void *) listens[i] ;
	}
	bcopy ((char *) &server_fds, (char *) &rfds, sizeof (fd_set)) ;
	fclose(F);
    }

/*
 * Enter the main loop :
 */

    IceSetIOErrorHandler (My_IceIOErrorHandler) ;
    signal (SIGPIPE, SIG_IGN) ;
    while ( 1 ) 
    {
       bcopy ((char *) &rfds, (char *) &ready_rfds, sizeof (rfds)) ;
      retry:
	if ( (nready = select (max_fd, &ready_rfds, NULL, NULL, NULL)) < 0 ) 
	{
	    if ( errno == EINTR )
		goto retry ;
	    perror ("libKs[KesMainLoop]: select (fatal)") ;
	    exit (1) ;
	}
	/*
	 * The management of raised fd differs if it is an accepting socket 
	 * or a connection socket
	 */
	for ( fd = 0 ; nready && (fd < max_fd) ; fd++ ) 
	{
	    if ( FD_ISSET (fd, &ready_rfds) ) 
	    {
		nready-- ;
		if ( FD_ISSET (fd, &server_fds ) )
		{
		    new_fd = My_IceAcceptConnection ((IceListenObj)My_IceTranslations[fd]);
		    if ( new_fd >= 0 )
		    {
			if ( new_fd >= max_fd )
			    max_fd = new_fd + 1 ;
			FD_SET (new_fd, &rfds) ;
		    }
		}
		else
		{
		    if ( setjmp (My_IceErrorEnv) != 0 ) 
		    {
			FD_CLR (fd, &rfds) ;
			continue ;
		    }
		    IceProcessMessages ((IceConn) My_IceTranslations[fd]
					, NULL
					, &reply_ready) ;
		}
	    }
	}
    }
    /* NOT REACHED */
}

    
