/***************************************************************************
 * tclInterf.c
 *
 * A sample client interface for the tcl language
 ***************************************************************************/

#include <tcl.h>
#include <tk.h>

#include <KeClient.h>
#include <KeCodec.h>

typedef struct _Tcl_KecWatch {
    Tcl_Interp   *interp ;
    char         *varname ;	/* No better way than this !!!!!!! */
    KeCodec      codec ;
    KecNotifyId  write, read ;
} Tcl_KecWatchRec, *Tcl_KecWatch ;

#define DEF_WATCHES_SIZE	127
#define DEF_CODECS_SIZE		31
    
typedef struct _Tcl_KecUserData {
    KeHashtable	codecs ;
    KeHashtable   watches ;
} Tcl_KecUserDataRec, *Tcl_KecUserData ;

#define TCL_KECUDATA(c) ((Tcl_KecUserData) KecGetUserDataOfConnection(c))
/*
 * The following variable is a special hack that allows applications
 * to be linked using the procedure "main" from the Tk library.  The
 * variable generates a reference to "main", which causes main to
 * be brought in from the library (and all of Tk and Tcl with it).
 */

extern int main();
int *tclDummyMainPtr = (int *) main;

/***************************************************************************
 * Converters
 ***************************************************************************/

static KID KecStringToNode (str)
    char *str ;

{
    KID retval = (KID) 0 ;

    if ( ! sscanf (str, "0x%x", &retval) )
	fprintf (stderr, "KecStringToNode: error on [%s].\n", str) ;
    return (retval) ;
}

static char *KecNodeToString (node)
    KID node ;

{
    static char str[16] ;

    sprintf (str, "0x%08x", node) ;
    return (str) ;
}

static char *KecModeToString (mode)
    unsigned long mode ;

{
    static char retval[5] = "DRWX" ;

    retval[0] = (mode & KEDIT_DirectoryMode) ? 'D' : 'd' ;
    retval[1] = (mode & KEDIT_ReadMode)      ? 'R' : 'r' ;
    retval[2] = (mode & KEDIT_WriteMode)     ? 'W' : 'w' ;
    retval[3] = (mode & KEDIT_ExecMode)      ? 'X' : 'x' ;

    return (retval) ;
}

static char *KecMaskToString (mask)
    unsigned long mask ;

{
    static char retval[] = "CDNRGWG" ;

    retval[0] = (mask & KEDIT_DeleteMask)         ? 'D' : 'd' ;
    retval[1] = (mask & KEDIT_CreateMask)         ? 'C' : 'c' ;
    retval[2] = (mask & KEDIT_NotifyMask)         ? 'N' : 'n' ;
    retval[3] = (mask & KEDIT_ReadMask)           ? 'R' : 'r' ;
    retval[4] = (mask & KEDIT_ReadWithValueMask)  ? 'G' : 'g' ;
    retval[5] = (mask & KEDIT_WriteMask)          ? 'W' : 'w' ;
    retval[6] = (mask & KEDIT_WriteWithValueMask) ? 'G' : 'g' ;

    return (retval) ;
}

/***************************************************************************
 * Tcl_KecDoGet
 * Tcl_KecDoSet
 *
 * Safe, but not effcient (two round trips)
 * Uses a hashtable to go from KID to codec
 ***************************************************************************/

typedef struct _CodecNode {
    KID node ;
    KeCodec codec ;
} CodecNodeRec, *CodecNode ;

static unsigned long codecnode_hash_fct (codecnode)
    CodecNode codecnode ;

{
    return ((unsigned long) codecnode->node) ;
}

static int codecnode_cmp_fct (codecnode1, codecnode2)
    CodecNode codecnode1, codecnode2 ;

{
    return ( codecnode1->node != codecnode2->node ) ;
}

KeCodec GetCodec (kec_conn, node)
    KecConnection kec_conn ;
    KID node ;

{
    Tcl_KecUserData udata  = TCL_KECUDATA (kec_conn) ;
    KeHashtable       codecs = udata->codecs ;     
    CodecNode       codecnode ;
    CodecNodeRec    key ;
    KeNodeStat      stat ;

    if ( ! codecs ) {
	codecs = KeHashtableMake (DEF_CODECS_SIZE
				, codecnode_hash_fct
				, codecnode_cmp_fct) ;
	udata->codecs = codecs ;
    }

    key.node = node ;
    if ( codecnode = (CodecNode) KeHashtableFind (codecs, &key) )
	return (codecnode->codec) ;
    
    codecnode = (CodecNode) malloc (sizeof (CodecNodeRec)) ;
 
    if ( (! codecnode)
	|| KecStatSync (kec_conn, node, &stat)
	|| ( ! (codecnode->codec = KeCodecGet (stat->name))) )
    {
	if ( stat )
	    fprintf (stderr, "GetCodec: cannot find %s\n", stat->name) ;
	else
	    fprintf (stderr, "GetCodec: cannot stat on x%08x\n", node) ;
	fprintf (stderr, "Fatal error: exiting.\n") ;
	exit (1) ;
    }
    
    codecnode->node = node ;

    KeHashtableAdd (codecs, codecnode) ;

    return (codecnode->codec) ;
}

static char *Tcl_KecValueToString (kec_conn, codec, size, value)
    KecConnection kec_conn ;
    KeCodec codec ;
    int     size ;
    KePointer value ;

{
    static char shortvalue[32] ;
    static char *longvalue = (char *) 0 ;
    KePointer pointer = (KePointer) 0 ;
    char *emsg ;
    KeErrorFreeFct efree ;

    if ( KeCodecDecode (codec
			, size, value
			, KecConnectionSwapped (kec_conn)
			, &pointer
			, &emsg, &efree) ) {
	if ( efree )
	    (*efree) (emsg) ;
	return ((char *) 0) ;
    }
    
    if ( codec == KeCodecBYTEGet () ) {
	sprintf (shortvalue, "%d", (int) *((BYTE *) pointer)) ;
	return (shortvalue) ;
    } else if ( codec == KeCodecCARD16Get () ) {
	sprintf (shortvalue, "%d", (int) *((CARD16 *) pointer)) ;
	return (shortvalue) ;
    } else if ( codec == KeCodecCARD32Get () ) {
	sprintf (shortvalue, "%d", (int) *((CARD32 *) pointer)) ;
	return (shortvalue) ;
    } else if ( codec == KeCodecSTRINGGet () ) {
	if ( longvalue )
	    free (longvalue) ;
	longvalue = (char *) strdup ((char *) pointer) ;
	return (longvalue) ;
    } else {
	/* Use raw data */
	return ((char *) 0) ;
    }

    /* NOT REACHED */
}

static char *Tcl_KecDoGet (kec_conn, node)
    KecConnection kec_conn ;
    KID           node ;

{
    KeNodeStat  stat ;
    KeCodec     codec ;
    KecValue    value ;

    if ( KecStatSync (kec_conn, node, &stat) 
	|| KecGetSync (kec_conn, node, &value) )
    {
	return ((char *) 0) ;
    }

    codec = GetCodec (kec_conn, stat->info) ;

    return (Tcl_KecValueToString (kec_conn
				  , codec
				  , value->size
				  , (KePointer) value->data)) ;
}

static int Tcl_KecDoSet (kec_conn, node, str)
    KecConnection kec_conn ;
    KID node ;
    char *str ;

{
    KeStatus status ;
    KeNodeStat stat ;
    KeCodec    codec ;
    BYTE   byte ;
    CARD16 card16 ;
    CARD32 card32 ;
    KePointer pointer, value ;
    int size ;
    char *emsg ;
    KeErrorFreeFct efree ;

    if ( status = KecStatSync (kec_conn, node, &stat) )
	return (status) ;
    
    codec = GetCodec (kec_conn, stat->info) ;

    if ( codec == KeCodecBYTEGet () ) {
	byte = atoi (str) ;
	pointer = (KePointer) &byte ;
    } else if ( codec == KeCodecCARD16Get () ) {
	card16 = atoi (str) ;
	pointer = (KePointer) &card16 ;
    } else if ( codec == KeCodecCARD32Get () ) {
	card32 = atoi (str) ;
	pointer = (KePointer) &card32 ;
    } else if ( codec == KeCodecSTRINGGet () ) {
	pointer = (KePointer) str ;
    } else {
	return (KEDIT_InvalidData) ;
    }

    if (status = KeCodecEncode (codec,pointer,&size,&value,&emsg,&efree) ) {
	if ( efree )
	    (*efree) (emsg) ;
	return (status) ;
    }

    KecSet (kec_conn, node, size, value) ;
    KecConnectionDoSync (kec_conn) ;

    return (KEDIT_Success) ;	/* ie 0 */
}

static unsigned long watch_hash_fct (watch)
    Tcl_KecWatch watch ;

{
    char          *p = watch->varname ;
    unsigned long hashval = 0, carry ;
    
    while ( *p ) {
	if ( carry = (hashval & 0xFF000000) ) 
	    hashval = ((hashval << 8) | (carry >> 24)) ^ *p ;
	else
	    hashval = (hashval << 8) ^ *p ;
	p++ ;
    }
    return (hashval) ;
}

static int watch_cmp_fct (w1, w2) 
    Tcl_KecWatch w1, w2 ;

{
    return (strcmp (w1->varname, w2->varname)) ;
}

static char *Tcl_KecUnsetHandler (kec_conn_ptr, interp, name1, name2, flags)
    ClientData kec_conn_ptr ;
    Tcl_Interp *interp ;
    char       *name1, *name2 ;
    int        flags ;

{
    KecConnection   kec_conn = (KecConnection) kec_conn_ptr ;
    Tcl_KecUserData udata = TCL_KECUDATA (kec_conn) ;
    Tcl_KecWatchRec wkey, *watch ;

    fprintf (stderr, "Unseting <%s>\n", name1) ;
    wkey.varname = name1 ;
    if ( ( ! udata->watches )
	|| ( ! (watch = KeHashtableFind (udata->watches, &wkey))) ) {
	return ((char *) NULL) ;
    }
    if ( watch->read )
	KecNotifyRemove (kec_conn, watch->read) ;
    if ( watch->write )
	KecNotifyRemove (kec_conn, watch->write) ;
    KecConnectionDoSync (kec_conn) ;
    KeHashtableRemove (udata->watches, watch) ;
    free (watch->varname) ; 
    free (watch) ;
    return ((char *) NULL) ;
}

static void Tcl_KecWriteWithValueHandler (kec_conn, event_ptr, watch_ptr)
    KecConnection kec_conn ;
    KecEvent      event_ptr ;
    KePointer     watch_ptr ;

{
    KecWriteWithValueEvent event = (KecWriteWithValueEvent) event_ptr ;
    Tcl_KecWatch watch = (Tcl_KecWatch) watch_ptr ;
    char *varvalue ;

    if ( varvalue = Tcl_KecValueToString (kec_conn
					  , watch->codec
					  , event->size
					  , (KePointer) event->data) )
    {
	Tcl_SetVar (watch->interp, watch->varname, varvalue, TCL_GLOBAL_ONLY);
    }
    
    return ;
}

    
static int Tcl_KecDoWriteTrace (interp, kec_conn, node, varname)
    Tcl_Interp *interp ;
    KecConnection kec_conn ;
    KID node ;
    char *varname ;

{
    Tcl_KecWatchRec wkey, *watch = (Tcl_KecWatch) 0 ;
    Tcl_KecUserData udata = TCL_KECUDATA (kec_conn) ;
    KeNodeStat      stat ;
    KeStatus        status ;

    if ( ! udata->watches ) {
	udata->watches = KeHashtableMake (DEF_WATCHES_SIZE
					, watch_hash_fct
					, watch_cmp_fct) ;
    } else {
	wkey.varname = varname ;
	if ( (watch = KeHashtableFind (udata->watches, &wkey)) && watch->write )
	    return (0) ;
    }

    if ( watch ) {
	if ( ! ( watch->write = KecNotifyAdd (kec_conn
					      , node
					      , KEDIT_WriteWithValueMask
					      , Tcl_KecWriteWithValueHandler
					      , (KePointer) watch)) )
	    return (1) ;
    } else if ( watch = (Tcl_KecWatch) malloc (sizeof (Tcl_KecWatchRec)) ) {
	if ( ! ( watch->write = KecNotifyAdd (kec_conn
					      , node
					      , KEDIT_WriteWithValueMask
					      , Tcl_KecWriteWithValueHandler
					      , (KePointer) watch)) )
	    return (1) ;
	if ( status = KecStatSync (kec_conn, node, &stat) ) {
	    free (watch) ;
	    return (status) ;
	}
	Tcl_TraceVar (interp
		      , varname
		      , TCL_TRACE_UNSETS
		      , Tcl_KecUnsetHandler
		      , (ClientData) kec_conn) ;
	watch->interp  = interp ;
	watch->varname = (char *) strdup (varname) ;
	watch->read    = (KecNotifyId) 0 ;
	watch->codec   = GetCodec (kec_conn, stat->info) ;
	KeHashtableAdd (udata->watches, watch) ;
	KeFree (stat) ;
    }
    KecConnectionDoSync (kec_conn) ;
    return (watch ? 0 : 1 ) ;
}

/***************************************************************************
 * Connection commands:
 * <connection> <node> [command...]
 *     Where command can be:
 *     "ls"   : get children node identifiers
 *     "stat" : get full infos of node
 *     "get"  : get node value
 *     "set"  : set node value
 * <connection> <command>
 *     Where command can be:
 *     "-tag" : get client tag
 ***************************************************************************/

int Tcl_KecConnectionCmd (kec_conn_ptr, interp, argc, argv)
    ClientData kec_conn_ptr ;
    Tcl_Interp *interp ;
    int argc ;
    char **argv ;

{
    KecConnection kec_conn = (KecConnection) kec_conn_ptr ;
    KID           node ;

/*
 * Direct commands have only two parameters:
 */

    if ( argc == 2 )
    {
	if ( ! strcmp ("-tag", argv[1]) ) {
	    char tostring[32] ;

	    sprintf (tostring, "%d", KecTagOfConnection (kec_conn)) ;
	    Tcl_AppendResult (interp, tostring, NULL) ;
	    return (TCL_OK) ;
	}
	else {
	    Tcl_AppendResult (interp, "Invali direct command", NULL) ;
	    return (TCL_ERROR) ;
	}
    }

/*
 * Node command have at least 3 parameters:
 */

    if ( argc < 3 )
    {
	Tcl_AppendResult (interp, "wrong # args", NULL) ;
	return (TCL_ERROR) ;
    }

    node = KecStringToNode (argv[1]) ;

    if ( ! strcmp ("ls", argv[2]) ) {
	KeIdArray  ls ;
	int        i, count ;

	if ( KecLsSync (kec_conn, node, &ls) ) {
	    Tcl_AppendResult (interp, "k-Edit error.", NULL) ;
	    return (TCL_ERROR) ;
	} else {
	    for ( i = 0 ; i < ls->count ; i++ ) 
		Tcl_AppendElement (interp, KecNodeToString (ls->nodes[i])) ;
	    KeFree (ls) ;
	    return (TCL_OK) ;
	}
    }
    else if ( ! strcmp ("stat", argv[2]) ) {
	KeNodeStat stat ;

	if ( KecStatSync (kec_conn, node, &stat) ) {
	    Tcl_AppendResult (interp, "k-Edit error.", NULL) ;
	    return (TCL_ERROR) ;
	} else {
	    Tcl_AppendElement (interp, stat->name) ;
	    Tcl_AppendElement (interp, KecModeToString (stat->mode)) ;
	    Tcl_AppendElement (interp, KecMaskToString (stat->mask)) ;
	    Tcl_AppendElement (interp, KecNodeToString (stat->id)) ;
	    Tcl_AppendElement (interp, KecNodeToString (stat->info)) ;
	    KeFree (stat) ;
	    return (TCL_OK) ;
	}
    }
    else if ( !strcmp ("get", argv[2]) ) {
	char *value ;

	if ( ! ( value = Tcl_KecDoGet (kec_conn, node)) ) {
	    Tcl_AppendResult (interp, "k-Edit error.", NULL) ;
	    return (TCL_ERROR) ;
	}
	else
	{
	    Tcl_AppendElement (interp, value) ;
	    return (TCL_OK) ;
	}
    }
    else if ( (argc == 4) && ( ! strcmp ("set", argv[2])) ) {
	if ( Tcl_KecDoSet (kec_conn, node, argv[3]) )
	{
	    Tcl_AppendResult (interp, "k-Edit error.", NULL) ;
	    return (TCL_ERROR) ;
	}
	else
	{
	    return (TCL_OK) ;
	}
    } else if ( ! strcmp ("exec", argv[2]) ) {
        KecExec (kec_conn, node) ;
        KecConnectionDoSync (kec_conn) ;
        return (TCL_OK) ;
    } else if ( argc == 4) {
	if ( *(argv[2]) == 'w' ) {
	    if ( Tcl_KecDoWriteTrace (interp, kec_conn, node, argv[3]) ) {
		Tcl_AppendResult (interp, "k-Edit error.", NULL) ;
		return (TCL_ERROR) ;
	    } else {
		return (TCL_OK) ;
	    }
	} else if ( *(argv[2]) == 'r' ) {
	    Tcl_AppendResult (interp, "Not implemented yet", NULL) ;
	    return (TCL_OK) ;
	} else if ( *(argv[2]) == 'u' ) {
	    Tcl_AppendResult (interp, "Not implemented yet", NULL) ;
	    return (TCL_OK) ;
	}
    }
    else
    {
	Tcl_AppendResult (interp, "bad arg.", NULL) ;
	return (TCL_ERROR) ;
    }

    /* NOT REACHED */
}

static void Tcl_KecFileHandler (kec_conn_ptr, mask)
    ClientData kec_conn_ptr ;
    int mask ;

{
    IceProcessMessages (((KecConnection) kec_conn_ptr)->ice_connection
                        , NULL
                        , NULL) ;
    return ;
}

int Tcl_KecOpenConnectionCmd (clientData, interp, argc, argv)
    ClientData clientData ;
    Tcl_Interp *interp ;
    int argc ;
    char *argv[] ;

{
    KecConnection   kec_conn ;
    Tcl_KecUserData udata ;

    if ( argc != 3 )
    {
	Tcl_AppendResult (interp, "wrong # args: should be foo.", NULL) ;
	return (TCL_ERROR) ;
    }

    if ( (!(kec_conn = KecOpenConnection (argv[1])) )
	|| (!(udata = (Tcl_KecUserData) malloc (sizeof (Tcl_KecUserDataRec)))))
    {
	Tcl_AppendResult (interp, "Couldn't open connection.", NULL) ;
	return (TCL_ERROR) ;
    }

    Tk_CreateFileHandler (KecFdOfConnection (kec_conn)
			  , TK_READABLE
			  , Tcl_KecFileHandler
			  , (ClientData) kec_conn) ;
    Tcl_CreateCommand (interp
		       , (char *) strdup (argv[2])
		       , Tcl_KecConnectionCmd
		       , (ClientData) kec_conn
		       , (Tcl_CmdDeleteProc *) 0) ;

    udata->codecs  = (KeHashtable) 0 ;
    udata->watches = (KeHashtable) 0 ;

    KecSetUserDataOfConnection (kec_conn, udata) ;

    return (TCL_OK) ;
}

int Tcl_AppInit (interp)
    Tcl_Interp *interp ;
{
    extern Tcl_ServerInit () ;
    Tk_Window main ;

    main = Tk_MainWindow (interp) ;

    if ( Tcl_Init (interp) == TCL_ERROR )
	return (TCL_ERROR) ;

    if ( Tk_Init (interp) == TCL_ERROR )
	return (TCL_ERROR) ;

    Tcl_CreateCommand (interp
		       , "kec:open-connection"
		       , Tcl_KecOpenConnectionCmd
		       , (ClientData) 0
		       , (Tcl_CmdDeleteProc *) 0) ;
    Tcl_ServerInit (interp) ;
    
    return (TCL_OK) ;
}

