/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * ICElib.h
 *
 * This is the one to be included 
 ***************************************************************************/

#ifndef _ICELIB_H_
#define _ICELIB_H_

#include <X11/ICE/iproto.h>

#define D(x) x

#ifndef Bool
#    define Bool int
#endif

#ifndef Status
#    define Status int
#endif

typedef void *IcePointer ;

typedef struct _IceConn *IceConn ;
struct _IceReplyWaitInfo ;

/*
 * Error procs:
 */

typedef void (*IceIOErrorProc)(
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
#endif
) ;
    
typedef Bool (*IceHostBasedAuthProc)(
#ifdef NeedsFunctionPrototype
      char *hostname
#endif
) ;

typedef Status (*IceProtocolSetupProc)(
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
    , int major_version, int minor_version
    , char *vendor, char *release
    , IcePointer *client_data_ret
    , char **failure_reason_ret
#endif				     
) ;

typedef void (*IceProtocolActivateProc)(
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
    , IcePointer client_data
#endif
) ;
    

/*
 * Status
 */

typedef enum {
    IceConnectAccepted ,
    IceConnectPending ,
    IceConnectRejected
} IceStatus ;

typedef enum {
    IceProtocolSetupSuccess ,
    IceProtocolSetupFailure ,
    IceProtocolAlreadyActive
} IceProtocolSetupStatus ;

typedef enum {
    IceAcceptSuccess,
    IceAcceptFailure,
    IceAcceptBadMalloc
} IceAcceptStatus ;

/*
 * Listen objects :
 */

typedef struct _IceListenObj {
    int		fd ;			/* corresponding file descr */
    char	network_id[1] ;		/* network id as parsable */
} *IceListenObj ;

/*
 * Version descriptions:
 */

typedef void (*IcePoProcessMsgProc)(
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
    , IcePointer client_data
    , int opcode
    , unsigned long length
    , Bool swapped 
    , struct _IceReplyWaitInfo *reply_wait
    , Bool *reply_ready
#endif
) ;

#ifndef False
#  define False 0
#endif

#ifndef True
#  define True 1
#endif

typedef struct _IcePoVersionRec {
    int			major_version ;
    int			minor_version ;
    IcePoProcessMsgProc process_msg_proc ;
} *IcePoVersion, IcePoVersionRec ;

typedef void (*IcePaProcessMsgProc)(
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
    , IcePointer client_data
    , int opcode
    , unsigned long length
    , Bool swap
#endif
) ;

typedef struct _IcePaVersionRec {
    int			major_version ;
    int			minor_version ;
    IcePaProcessMsgProc process_msg_proc ;
} *IcePaVersion, IcePaVersionRec ;

/*
 * Authentication stuff (NOT IMPLEMENTED):
 */

typedef enum {
    IcePoAuthHaveReply ,
    IcePoAuthRejected ,
    IcePoAuthFailed ,
    IcePoAuthDoneCleanup 
} IcePoAuthStatus ;
	
typedef IcePoAuthStatus (*IcePoAuthProc)() ;

typedef enum {
    IcePaAuthContinue ,
    IcePaAuthAccepted ,
    IcePaAuthRejected ,
    IcePaAuthFailed 
} IcePaAuthStatus ;

typedef IcePaAuthStatus (*IcePaAuthProc)() ;

/*
 * Reply wait stuff :
 */

typedef struct _IceReplyWaitInfo {
    unsigned long	sequence_of_request ;
    int			major_opcode_of_request ;
    int			minor_opcode_of_request ;
    IcePointer		reply ;
} IceReplyWaitInfo ;

/***************************************************************************
 * Protocol description:
 *
 * This use two data structures: one for the registered protocol, and one
 * for the protocol that are being used on a connection. The second ref
 * one of the first.
 ***************************************************************************/

typedef struct {
    int		    opcode ;		/* opcode assigned to this protocol */
    char	    *name ;
    char	    *vendor ;
    char	    *release ;
    int		    version_count ;
    IcePoVersionRec *versions_rec ;
    int		    auth_count ;
    char	    **auth_names ;
    IcePoAuthProc   auth_proc ;
    IceIOErrorProc  io_error_proc ;
} _IcePoRegistrationRec, *_IcePoRegistration ;

typedef struct {
    int		               opcode ;
    char		       *name ;
    char		       *vendor ;
    char		       *release ;
    int			       version_count ;
    IcePaVersionRec            *versions_rec ;
    int		               auth_count ;
    char	               **auth_names ;
    IcePaAuthProc              auth_proc ;
    IceProtocolSetupProc       setup_proc ;
    IceIOErrorProc             io_error_proc ;
} _IcePaRegistrationRec, *_IcePaRegistration ;

typedef struct {
    int		         orig ;		/* True if Po, false for Pa */
    int		         my_opcode ;	/* my opcode for this protocol */
    int		         his_opcode ;	/* peer opcode for the protocol */
    IcePoProcessMsgProc  oproc ;	/* processing callback */
    IcePaProcessMsgProc  aproc ;	/* processing callback */
    IceProtocolSetupProc setup ;	/* notify setup */
    IcePointer           closure ;
} _IceProcessRec, *_IceProcess ;

/*
 * IceConn object:
 */

typedef struct _IceConn {
    char		*network_id ;
    int			port ;
    int			fd ;
    int			byte_swap ;
    IceStatus		status ;
    unsigned long 	sequence_number ; 	/* cureent ICE sequence number */

    void		*ibufobj ;		/* ptr to buffer obj (see iceio.c) */
    char		*ibuf ;			/* input buffer (NULL if none) */
    char		*ibufptr ;		/* current input pointer */
    char		*reqbase ;		/* request being read base */
    char		*processing ;		/* request being processed */
    int			ibuflen ;		/* total length of input buffer */
    int			ibufcnt ;		/* used bytes of input buffer */
    int			needed ;		/* to get the current request */

    void                *obufobj ; 		/* ptr to buffer obj (see iceio.c) */
    char		*obuf ;			/* output buffer (NULL if none) */
    char		*obufptr ;		/* current output pointer */
    int			obuflen ;		/* total length of output buffer */

    _IceProcess		protocols[256] ; 	/* this could be make better */

} IceConnRec ;

#define IceConnectionStatus(i)   	((i)->status) 
#define IceConnectionNumber(i)	 	((i)->fd)
#define IceLastSentSequenceNumber(i) 	((i)->sequence_number)
#define IceRequestPending(i)	 	((i)->reqbase && ( ! (i)->needed) )
#define IceSwapping(i)		 	((i)->byte_swap)

/***************************************************************************
 * Function prototypes:
 ***************************************************************************/

/*
 * I HATE MY MIPS !!!
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef NEEDS_STRDUP 
char * strdup (
#ifdef NeedsFunctionPrototype
      char *p
#endif
) ;
#endif /* NEEDS_STRDUP */
    
void _IceGetOutputBuffer (
#ifdef NeedsFunctionPrototype
      IceConn ice_conn 
    , int     size 
#endif
) ;

int _IceSecureWrite (
#ifdef NeedsFunctionPrototype
      IceConn ice_conn 
    , char    *buffer 
    , int     size 
#endif
) ;


IceConn IceAcceptConnection (
#ifdef NeedsFunctionPrototype
      IceListenObj    listen_obj	/* listen obj on wich to accept */
    , IceAcceptStatus *status_ret
#endif
) ;

IceConn IceOpenConnection (
#ifdef NeedsFunctionPrototype
      char	*network_id_list	/* to connect to */
    , IcePointer client_data		/* closure */
    , Bool      must_authenticate	/* requires authentication ? */
    , int       major_opcode_check	/* duplicate protocol on conn ? */
    , int       error_length		/* length of returned error */
    , char      *error_ret
#endif
) ;

int IceRegisterForProtocolSetup (
#ifdef NeedsFunctionPrototype
      char 	*name
    , char 	*vendor
    , char	*release
    , int       version_count
    , IcePoVersionRec *versions_rec
    , int             auth_count
    , char            **auth_names
    , IcePoAuthProc   *auth_procs
    , IceIOErrorProc  io_error_proc
#endif
) ;

int IceRegisterForProtocolReply (
#ifdef NeedsFunctionPrototype
      char			 *name
    , char 			 *vendor 
    , char 			 *release 
    , int 			 version_count 
    , IcePaVersionRec 		 *version_recs 
    , int 			 auth_count 
    , char 			 **auth_names 
    , IcePaAuthProc 		 *auth_procs 
    , IceHostBasedAuthProc 	 host_based_auth_proc 
    , IceProtocolSetupProc       protocol_setup_proc
    , IceProtocolActivateProc    protocol_activate_proc
    , IceIOErrorProc 		 io_error_proc 
#endif
) ;

IceProtocolSetupStatus IceProtocolSetup (
#ifdef NeedsFunctionPrototype
      IceConn    ice_conn 
    , int        opcode
    , IcePointer closure
    , Bool       must_authenticate 
    , int        *major_ret 
    , int        *minor_ret 
    , char       **vendor_ret 
    , char       **release_ret 
    , int        error_length 
    , char       *error 
#endif
) ;

IceStatus IceListenForConnections (
#ifdef NeedsFunctionPrototype
      int	   *count_ret 
    , IceListenObj **listen_ret 
    , int          error_length 
    , char	   *error 
#endif 
) ;

char *IceGetListenConnectionString (
#ifdef NeedsFunctionPrototype
      IceListenObj listen_obj 
#endif
) ;

int IceGetListenConnectionNumber (
#ifdef NeedsFunctionPrototype
      IceListenObj listen_obj 
#endif
) ;

void IceSetHostBasedAuthProc (
#ifdef NeedsFunctionPrototype
      IceListenObj 	   listen_obj
    , IceHostBasedAuthProc host_based_auth_proc
#endif
) ;

void IceFreeListenObj (
#ifdef NeedsFunctionPrototype
      IceListenObj listen_obj 
#endif
) ;

void IceProcessMessages (
#ifdef KEDIT_NeedsFunctionPrototype
      IceConn          ice_conn 
    , IceReplyWaitInfo *reply_wait
    , Bool             *reply_ready
#endif
) ;

IceStatus IceProtocolShutdown (
#ifdef NeedsFunctionPrototype
      IceConn ice_conn 
    , int     major 
#endif
) ;

IceIOErrorProc IceSetIOErrorHandler (
#ifdef NeedsFunctionPrototype
      IceIOErrorProc io_error_proc 
#endif
) ;

#ifdef __cplusplus
} /* End of c++ verbosities */
#endif /* __cplusplus */

#endif /* _ICELIB_H_ */

