/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * ICElibint.h
 *
 * For ICE library developpers only
 ***************************************************************************/

#ifndef _ICELIBINT_H_
#define _ICELIBINT_H_

/*
 * ICE starting port number (for TCP/IP)
 */

#define ICE_START_PORT	2000

/***************************************************************************
 * Ice buffer management variables
 ***************************************************************************/

/*
 * ICE_OBUF_MAXSIZE:
 * Allocated output buffers will *never* exceed this water-mark, either by 
 * flushing the connection, either by using a direct write. As buffers are
 * shared amongst connection, we can make this quite big.
 */

#define ICE_OBUF_MAXSIZE	(8 * 1024)

/*
 * ICE_BUF_DEFSIZE:
 * Starting size of buffers.
 */

#define ICE_BUF_DEFSIZE		(1024)

/*
 * ICE_BUF_WATERMARK:
 * Buffers exceeding this size after use will not be put back in 
 * the free buffer list, but will be freed immediatley
 */

#define ICE_BUF_WATERMARK 	(4 * 1024)

/*
 * ICE_BUF_KEEPTHEM:
 * How many buffer is Ice allowed to keep malloced at a given time (buffer
 * free list length).
 */

#define ICE_BUF_KEEPTHEM	10

/***************************************************************************
 * Private prototypes:
 ***************************************************************************/

extern _IcePoRegistration _IcePoRegistrations[] ;
extern int                _IcePoRegistrationCount ;

extern _IcePaRegistration _IcePaRegistrations[] ;
extern int                _IcePaRegistrationCount ;

void _IceGetByteOrder (IceConn i) ;
void _IceSendByteOrder (IceConn i) ;
void IceFlush (IceConn i) ;

void _IceIOError () ;
int _IceWaitForReadable () ;
int _IceWaitForWritable () ;

void _IcePoProcessMessage (
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
    , IcePointer client_data
    , int opcode
    , unsigned long length
    , Bool swapped 
    , struct _IceReplyWaitInfo *reply_wait
    , Bool *reply_ready
#endif
) ;

void _IcePaProcessMessage (
#ifdef NeedsFunctionPrototype
      IceConn ice_conn
    , IcePointer client_data
    , int opcode
    , unsigned long length
    , Bool swap
#endif
) ;
			   

void _IceGetOutpuBuffer (IceConn i, int s) ;
void _IceReleaseBuffer () ;

/***************************************************************************
 * Sending ice messages macros
 *
 * As the spec says "who wants to read/write messages need include this 
 * [*this*] file", I think these macros should be located here.
 ***************************************************************************/

#define _IceGetOutputBytes(_i,_s,_t,_p) \
{ \
    if ( (_i)->obuf ) { \
	if ( (_i)->obuflen - ( (_i)->obufptr - (_i)->obuf ) > (_s) ) { \
	    (_p) = (_t *) (_i)->obufptr ; \
	    (_i)->obufptr += (_s) ; \
	} \
	else { \
	    (void) _IceGetOutputBuffer ((_i), (_s)) ; \
	    (_p) = (_t *) (_i)->obufptr ; \
	    (_i)->obufptr += (_s) ; \
	} \
    } \
    else { \
	(void) _IceGetOutputBuffer ((_i), (_s)) ; \
	(_p) = (_t *) (_i)->obufptr ; \
	(_i)->obufptr += (_s) ; \
    } \
}
    
#define IceGetHeader(_i,_ma,_mi,_hs,_mt,_pm) \
{ \
    _IceGetOutputBytes ((_i), _hs, _mt, _pm) ; \
    _pm->majorOpcode = _ma ; \
    _pm->minorOpcode = _mi ; \
    _pm->length      = ((_hs - sizeof (iceHeader)) >> 3) ; \
}
    
#define IceGetHeaderExtra(_i,_ma,_mi,_hs,_ex,_mt,_pm,_pd) \
{ \
    _IceGetOutputBytes ((_i), _hs + (_ex << 3), _mt, _pm) ; \
    _pm->majorOpcode = _ma ; \
    _pm->minorOpcode = _mi ; \
    _pm->length      = (((_hs) - sizeof (iceHeader)) >> 3) + (_ex) ; \
    _pd              = ((char *) _pm) + _hs ; \
}
      
#define IceSimpleMessage(_i,_ma,_mi) \
{ \
     iceHeader *ice_header ; \
     IceGetHeader ((_i), _ma, _mi, sizeof (iceHeader), iceHeader, ice_header) ; \
}

#define IceWriteData(_i,_n,_b) \
{ \
    if ( _n > ICE_OBUF_MAXSIZE ) { \
	IceFlush ((_i)) ; \
	_IceSecureWrite ((_i), _b, _n) ; \
    } else { \
	char *obuf ; \
	_IceGetOutputBytes ((_i),_n, char, obuf) ; \
	bcopy (_b, obuf, _n) ; \
    } \
}

#define IceWriteData16(_i,_n,_b)	IceWriteData((_i),_n,_b) 
#define IceWriteData32(_i,_n,_b)	IceWriteData((_i),_n,_b)

#define IceSendData(_i,_n,_b) \
{ \
    IceFlush ((_i)) ; \
    _IceSecureWrite ((_i), _b, _n) ; \
}

#define IceSendData16(_i,_n,_b)		IceSendData((_i),_n,_b)
#define IceSendData32(_i,_n,_b)		IceSendData((_i),_n,_b)

#define IceWritePad(_i,_b) \
{ \
    char *obuf ; \
    _IceGetOutputBytes ((_i),_b, char, obuf) ; \
}


/***************************************************************************
 * reading ice messages
 *
 * Same comment as for writing them: this should be the proper place.
 ***************************************************************************/

#define IceReadSimpleMessage(_i,_t,_m) \
    IceReadMessageHeader((_i),sizeof (iceHeader), _t, _m) 

#define IceReadCompleteMessage(_i,_s,_t,_m,_p) \
{ \
    IceReadMessageHeader ((_i), (_s), _t, _m) ; \
    (_p) = (char *) _m + (_s) ; \
}
	
#define IceReadMessageHeader(_i,_s,_t,_m) \
{ \
    if ( ! (_i)->processing ) { \
	fprintf (stderr, "IceReadMessageHeader: implementation error.\n") ; \
	exit (1) ; \
    } \
    _m = (_t *) (_i)->processing ; \
}

#define IceDisposeCompleteMessage(_i,_m)

/***************************************************************************
 * Various counting macros
 ***************************************************************************/

#define WORD32COUNT(n)	(((n) + 3) >> 2)
#define WORD64COUNT(n)	(((n) + 7) >> 3)

#define PAD32(n)	((4 - ((n) & 3)) & 3)
#define PAD64(n)	((8 - ((n) & 7)) & 7)

/***************************************************************************
 * Filling/Extrcationg field macros
 ***************************************************************************/

#define lswaps(s) \
    ((((s) & 0xFF) << 8) | (((s) & 0xFF00) >> 8))

#define lswapl(l) \
    ((((l) & 0xFF) << 24) | (((l) & 0xFF00) << 8) | (((l) & 0xFF0000) >> 8) | (((l) & 0xFF000000) >> 24))

#define ICE_REQUEST_LENGTH(_i,_r,_l) \
{ \
    _l = (*((CARD32 *) (((char *) _r) + sizeof (CARD32)))) << 3 ; \
    if ( (_i)->byte_swap ) \
	_l = lswapl (_l) ; \
}
	   
#define EXTRACT_CARD8(_p,_v) \
{ \
    _v = *((CARD8 *) (_p)) ; \
    (_p)++ ; \
}

#define EXTRACT_CARD16(_p,_s,_v) \
{ \
    _v  = *((CARD16 *) (_p)) ; \
    _p += sizeof (CARD16) ; \
    if ( (_s) ) \
	_v = lswaps (_v) ; \
}

#define EXTRACT_CARD32(_p,_s,_v) \
{ \
    _v  = *((CARD32 *) (_p)) ; \
    (_p) += sizeof (CARD32) ; \
    if ( (_s) ) \
	_v = lswapl (_v) ; \
}

#define EXTRACT_STRING(_p,_s,_v,_l) \
{ \
    EXTRACT_CARD16 ((_p), (_s), _l) ; \
    _v = (char *) (_p) ; \
    (_p) += (_l) + PAD32 (_l) ; \
}

#define COPY_STRING(_p,_s,_d) \
{ \
    int len ; \
    char *s ; \
    EXTRACT_STRING((_p), (_s), s, len) ; \
    if ( _d = (char *) malloc (len + 1) ) \
    { \
	bcopy (s, _d, len) ; \
	*(_d + len) = '\0' ; \
    } \
}

#define STORE_CARD8(_p,_v) \
{ \
    *((CARD8 *) (_p)) = _v ; \
    (_p) += sizeof (CARD8) ; \
}

#define STORE_CARD16(_p,_v) \
{ \
    *((CARD16 *) (_p)) = _v ; \
    (_p) += sizeof (CARD16) ; \
}

#define STORE_CARD32(_p,_v) \
{ \
    *((CARD32 *) (_p)) = _v ; \
    (_p) += sizeof (CARD32) ; \
}

#define STORE_STRING(_p,_v) \
{ \
    int len = strlen (_v) ; \
    STORE_CARD16((_p),len) ; \
    bcopy (_v,(_p),len) ; \
    (_p) += (len + PAD32(len)) ; \
}

#endif /* _ICELIBINT_H_ */
