/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iproto.h
 *
 * ICE protocol description file
 ***************************************************************************/

#ifndef _ICEPROTO_H_
#define _ICEPROTO_H_

#include <X11/Xmd.h>

/*
 * Ice opcodes :
 */

#define ICE_MajorOpcode			0

#define ICE_Error			0
#define ICE_ByteOrder			1
#define ICE_ConnectionSetup		2
#define ICE_AuthenticationRequired	3
#define ICE_AuthenticationReply		4
#define ICE_AuthenticationNextPhase	5
#define ICE_ConnectionReply		6
#define ICE_ProtocolSetup		7
#define ICE_ProtocolReply		8
#define ICE_Ping			9
#define ICE_PingReply			10
#define ICE_WantToClose			11
#define ICE_NoClose			12

/*
 * Ice values for specific fields of requests:
 */

#define ICE_LSBFirst			0
#define ICE_MSBFirst 			1

/*
 * Generic Error Class :
 */

#define ICE_GenericBadMinor	0x8000
#define ICE_GenericBadState	0x8001 
#define ICE_GenericBadLength	0x8002
#define ICE_GenericBadValue	0x8003

/*
 * ICE specific error class:
 */

#define ICE_SpecificBadMajor			0
#define ICE_SpecificNoAuthentication 		1
#define ICE_SpecificNoVersion			2
#define ICE_SpecificAuthenticationRejected	3
#define ICE_SpecificAuthenticationFailed	4
#define ICE_SpecificProtocolDuplicate		5
#define ICE_SpecificMajorOpcodeDuplicate	6
#define ICE_SpecificUnknownProtocol		7

/*
 * ICE-specific error class :
 */

#define IceCanContinue				0
#define IceFatalToProtocol			1
#define IceFatalToConnection			2

/*
 * Ice protocol packets:
 */

typedef struct _iceError {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
    CARD8	offendingMinorOpcode ;
    CARD8	severity ;
    CARD16	unused ;
    CARD32	sequence ;			/* of erroneous message */
    /* values + pad */
} iceError ;

typedef struct _iceHeader {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	_unused[2] ;
    CARD32	length ;
} iceHeader ;

typedef struct _iceByteOrder {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	byteOrder ;
    CARD8	_unused ;
    CARD32	length ;
} iceByteOrder ;

typedef struct _iceConnectionSetup {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	numberOfVersions ;
    CARD8	numberOfAuthentications ;
    CARD32	length ;
    CARD8	mustAuthenticate ;
    CARD8	_unused8[3] ;
    CARD32	_unused32 ;
    /* STRING 		vendor*/
    /* STRING		release */
    /* LISTofSTRING	authentication protocol name */
    /* LISTofVERSION 	version-list */
} iceConnectionSetup ;

typedef struct _iceAuthenticationRequired {	/* NOT IMPLEMENTED */
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
} iceAuthenticationRequired ;

typedef struct _iceAuthenticationReply {	/* NOT IMPLEMENTED */
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
} iceAuthenticationReply ;

typedef struct _iceConnectionReply {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	versionIndex ;
    CARD8	_unused ;
    CARD32	length ;
    /* STRING	vendor */
    /* STRING	release */
} iceConnectionReply ;

typedef struct _iceProtocolSetup {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	major_opcode ;
    CARD8 	mustAuthenticate ;
    CARD32	length ;
    CARD8	numberOfVersions ;
    CARD8	numberOfAuthentications ;
    CARD8	_unused8[2] ;
    CARD32	_unused32 ;
    /* STRING		protocol-name */
    /* STRING		vendor */
    /* STRING		release */
    /* LISTofSTRING 	authentication-protocl-name */
    /* LISTofVERSION 	version-list */
} iceProtocolSetup ;

typedef struct _iceProtocolReply {
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD8	versionIndex ;
    CARD8	major_opcode ;
    CARD32	length ;
    /* STRING	vendor */
    /* STRING	release */
} iceProtocolReply ;

typedef struct _icePing {			/* NOT IMPLEMENTED */
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
} icePing ;

typedef struct _icePingReply {			/* NOT IMPLEMENTED */
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
} icePingReply ;
   
typedef struct _iceWantToClose {		/* NOT IMPLEMENTED */
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
} iceWantToClose ;

typedef struct _iceNoClose {			/* NOT IMPLEMENTED */
    CARD8	majorOpcode ;
    CARD8	minorOpcode ;
    CARD16	iclass ;
    CARD32	length ;
} iceNoClose ;
    
#endif /* _ICEPROTO_H_ */
