/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iceconn.c
 *
 * Connection setup/reply. 
 * This file deals only with TCP/IP connections.
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <netdb.h>
#include <errno.h>

extern int errno ;

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

_IceProcessRec _IcePaStartupProcessProtocol =
{
    0,					/* orig */
    0, 0,				/* my/his opcode */
    (IcePoProcessMsgProc) 0,		/* no Po process */
    _IcePaProcessMessage,		/* got it ? */
    (IceProtocolSetupProc) 0,
    (IcePointer) 0
} ;

_IceProcessRec _IcePoStartupProcessProtocol =
{
    1,
    0, 0,
    _IcePoProcessMessage,
    (IcePaProcessMsgProc) 0,
    (IceProtocolSetupProc) 0,
    (IcePointer) 0
} ;

/***************************************************************************
 * _IceParseNetworkId: parse the string as a net id
 *
 * We are only dealing with TCP here ...
 * Static results = no reentrency !
 *
 * RETURNS: 0 on success, error otherwise
 ***************************************************************************/

static int _IceParseNetworkId (str, phost, pport)
    char	*str ;
    char	**phost ;
    int		*pport ;

{
    static char current[128] ;
    char	*p ;

    if ( strncmp (str, "tcp/", 4) )
	return (1) ;

    strcpy (current, str) ;

    for ( p = current + 4 ; *p && ( *p != ':' ) ; p++ )
	;

    if ( ! *p )
	return (1) ;

    *p++ = '\0' ;
    *phost = current + 4 ;
    *pport = atoi (p) ;

    return (0) ;
}
	
/***************************************************************************
 * IceOpenConnection: open a connection to some other ice (server)
 *
 * RETURNS: the connection
 ***************************************************************************/

IceConn IceOpenConnection (id, client_data, must_authenticate, major, error_length, error)
    char *id ;
    IcePointer client_data ;
    Bool must_authenticate ;
    int  major ;
    int  error_length ;
    char *error ;

{
    IceConn		ice_conn ;
    char		*hostname ;
    int			fd, port ;
    int 		flags ;
    struct sockaddr_in	inaddr ;
    struct sockaddr     *addr ;
    struct hostent	*h ;
    unsigned long       hostinetaddr ;
    int                 addrlen ;

/*
 * Initialize :
 */

    _IceRegisterIce () ;

/*
 * Lookup in our database to see if some connection with this client is ready
 */

    if ( major && ( ! client_data) ) 
    {
	fprintf (stderr, "IceOpenConnection: no check against existing conn\n") ;
    }

/*
 * No connection was found, create a fresh one:
 */

    if ( _IceParseNetworkId (id, &hostname, &port) )
    {
	D ( fprintf (stderr, "IceOpenConnection: bad id syntax\n") ; ) ;
	return ((IceConn) 0) ;
    }


    if ( isascii (*hostname) && isdigit (*hostname) )
    {
	hostinetaddr             = inet_addr (hostname) ;
	inaddr.sin_family        = AF_INET ;
	inaddr.sin_addr.s_addr   = hostinetaddr ;
    }
    else
    {
	if ( (! (h = gethostbyname (hostname))) 
	    || (! h->h_addr_list) 
	    || (h->h_addrtype != AF_INET) )
	{
	    extern int h_errno ;

	    D ( fprintf (stderr, "Invalid host %s (h_errno = %d).\n"
			 , hostname
			 , h_errno) ; ) ;
	    return ((IceConn) 0) ;
	}

	inaddr.sin_family = h->h_addrtype ;
	bcopy ((char *) h->h_addr, (char *) &(inaddr.sin_addr), sizeof (inaddr.sin_addr));
    }

    addr            = (struct sockaddr *) &inaddr ;
    addrlen         = sizeof (struct sockaddr_in) ;
    inaddr.sin_port = htons (port) ;

    if ( (fd = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 )
	return ((IceConn) 0) ;

    if ( connect (fd, addr, addrlen) < 0 )
    {
	close (fd) ;
	return ((IceConn) 0) ;
    }

    fcntl (fd, F_GETFL, &flags) ;
    flags |= O_NDELAY ;
    fcntl (fd, F_SETFL, flags) ;

/*
 * port and fd are valid, setup the connection object:
 */

    if ( ! (ice_conn = (IceConn) malloc (sizeof (IceConnRec))) ) 
	return (ice_conn) ;

    ice_conn->network_id      = (char *) strdup (id) ;
    ice_conn->port            = port ;
    ice_conn->fd              = fd ;
    ice_conn->byte_swap       = -1 ;
    ice_conn->status          = IceConnectPending ;
    ice_conn->sequence_number = 0 ;

    ice_conn->ibufobj         = (void *) 0 ;
    ice_conn->ibuf	      = (char *) 0 ;
    ice_conn->ibufptr         = (char *) 0 ;
    ice_conn->reqbase         = (char *) 0 ;
    ice_conn->processing      = (char *) 0 ;
    ice_conn->ibuflen         = 0 ;
    ice_conn->ibufcnt         = 0 ;
    ice_conn->needed          = 0 ;

    ice_conn->obufobj         = (void *) 0 ;
    ice_conn->obuf            = (char *) 0 ;
    ice_conn->obufptr         = (char *) 0 ;
    ice_conn->obuflen         = 0 ;

    /*
     * We need a startup process handler for ICE, it will be overriden
     * by the Setup requests/replies
     */

    memset (ice_conn->protocols, 0, sizeof (ice_conn->protocols)) ;
    ice_conn->protocols[0] = &_IcePoStartupProcessProtocol ;

/*
 * Now, initiate communications:
 */

    _IceDoByteOrder (ice_conn) ;
    _IceDoConnectionSetup (ice_conn) ;

    return (ice_conn) ;
}

/***************************************************************************
 * IceCloseConnection: close this connection
 *
 * RETURNS: status
 ***************************************************************************/

IceStatus IceCloseConnection (ice_conn)
    IceConn	ice_conn ;

{
    int i ;

    for ( i = 1 ; i < 256 ; i++ )
    {
	if ( ice_conn->protocols[i] ) {
	    ice_conn->protocols[i] = (_IceProcess) 0 ;
	    free (ice_conn->protocols[i]) ;
	}
    }
    close (ice_conn->fd) ;
    free (ice_conn->network_id) ;
    if ( ice_conn->ibuf )
	_IceReleaseBuffer (ice_conn->ibuf) ;
    if ( ice_conn->obuf )
	_IceReleaseBuffer (ice_conn->obuf) ;

    free (ice_conn) ;
    return ;
}

/***************************************************************************
 * IceAcceptConnection: accept a new connection
 *
 * This accepts only TCPIP connections.
 *
 * RETURNS: a new ic_conn
 ***************************************************************************/

IceConn IceAcceptConnection (listen_obj, status_ret)
    IceListenObj    listen_obj ;
    IceAcceptStatus *status_ret ;

{
    IceConn 	    ice_conn ;
    struct sockaddr incomming_addr ;
    int 	    incomming_addrlen = sizeof (struct sockaddr) ;
    int             fd, flags = 0  ;

/*
 * Accept the incomming connection from this client :
 */    

    if ( (fd = accept (listen_obj->fd
		       , &incomming_addr
		       , &incomming_addrlen)) < 0 ) {
	*status_ret = IceAcceptFailure ;
	return ((IceConn) 0) ;
    }

    fcntl (fd, F_GETFL, &flags) ;
    flags |= O_NDELAY ;
    fcntl (fd, F_SETFL, flags) ;

    if ( ! (ice_conn = (IceConn) malloc (sizeof (IceConnRec))) )
    {
	close (fd) ;
	*status_ret = IceAcceptBadMalloc ;
	return ((IceConn) 0) ;
    }

    ice_conn->network_id      = (char *) 0 ;
    ice_conn->port	      = -1 ;
    ice_conn->fd              = fd ;
    ice_conn->byte_swap       = -1 ;
    ice_conn->status          = IceConnectPending ;
    ice_conn->sequence_number = 0 ;

    ice_conn->ibufobj         = (void *) 0 ;
    ice_conn->ibuf            = (char *) 0 ;
    ice_conn->ibufptr         = (char *) 0 ;
    ice_conn->reqbase         = (char *) 0 ;
    ice_conn->processing      = (char *) 0 ;
    ice_conn->ibuflen         = 0 ;
    ice_conn->ibufcnt         = 0 ;
    ice_conn->needed          = 0 ;

    ice_conn->obufobj         = (void *) 0 ;
    ice_conn->obuf            = (char *) 0 ;
    ice_conn->obufptr         = (char *) 0 ;
    ice_conn->obuflen         = 0 ;

    /*
     * We need a "default" proc handler for the connection setup.
     * It will be overrided after setup has take place.
     */

    memset (ice_conn->protocols, 0, sizeof (ice_conn->protocols)) ;
    ice_conn->protocols[0] = &_IcePaStartupProcessProtocol ;

/*
 * Initiate connection:
 */

    _IceDoByteOrder (ice_conn) ;

    *status_ret = IceAcceptSuccess ;
    return (ice_conn) ;
}

/***************************************************************************
 * IceListenForConnections: create a listen "socket"
 *
 * RETURNS: status
 ***************************************************************************/

void IceFreeListenObj (listen_obj)
    IceListenObj listen_obj ;
    
{
    free (listen_obj) ;
    return ;
}

int IceGetListenConnectionNumber (listen_obj)
    IceListenObj listen_obj ;

{
    return (listen_obj->fd) ;
}

char *IceGetListenConnectionString (listen_obj)
    IceListenObj listen_obj ;

{
    return (listen_obj->network_id) ;
}

void IceSetHostBasedAuthProc (listen_obj, host_based_auth_proc)
    IceListenObj 	 listen_obj ;
    IceHostBasedAuthProc host_based_auth_proc ;

{
    /* NOT IMPLEMENETED */
    return ;
}

IceStatus IceListenForConnections (count_ret, listen_ret, error_length, error)
    int			*count_ret ;
    IceListenObj	**listen_ret ;
    int			error_length ;
    char		*error ;

{
    static char		hostname[128] = "" ;
    IceListenObj	listen_obj ;
    int			flags, fd, port = ICE_START_PORT ;
    struct sockaddr_in	saddr ;
    char		network_id[64] ;
    int			size ;

/*
 * Initialize (if first call only) :
 */

    _IceRegisterIce () ;

    if ( ! hostname[0] ) 
    {
	gethostname (hostname, 128) ;
    }

/*
 * Open the socket    
 */

    if ( (fd = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ) 
    {
	int error = errno ;

	D ( fprintf (stderr, "IceListenForConnections: invalid socket return.\n") ; )
	return (0) ;
    }

/*
 * Bind it to some port :
 */

    memset ( &saddr, 0, sizeof (saddr) ) ;
    saddr.sin_port        = htons (port) ;
    saddr.sin_family      = AF_INET;
    saddr.sin_addr.s_addr = htonl (INADDR_ANY) ;

    while (bind (fd, (struct sockaddr *) &saddr, sizeof(saddr)))
	saddr.sin_port = htons (++port) ;

    fcntl (fd, F_GETFL, &flags) ;
    flags |= O_NDELAY ;
    fcntl (fd, F_SETFL, flags) ;

    listen (fd, 5) ;

/*
 * Pack the stuff in listen_obj, and set returned values:
 */

    sprintf (network_id, "tcp/%s:%d", ((hostname) ? (hostname) : "") , port) ;
   
    size = sizeof (struct _IceListenObj) + strlen (network_id) + 1 ;

    if ( ( ! ( listen_obj = (IceListenObj) malloc (size) ) )
	|| ( ! ( *listen_ret = (IceListenObj *) malloc (sizeof (IceListenObj *)))) )
    {
	int error = errno ;

	D ( fprintf (stderr, "IceListenForConnections: bad malloc\n") ; ) ;
	close (fd) ;
	return (0) ;
    }
    
    strcpy (listen_obj->network_id, network_id) ;
    listen_obj->fd = fd ;

    **listen_ret = listen_obj ;
    *count_ret  = 1 ;

    return (1) ;
}
