/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iceerror.c
 *
 * Ice error management functions.
 ***************************************************************************/

#include <stdio.h>

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

static IceIOErrorProc IceIOErrorHandler = (IceIOErrorProc) 0 ;

/***************************************************************************
 * IceSetIOErrorHandler: set our static IO error handler
 *
 * RETURN: the previous handler
 ***************************************************************************/

IceIOErrorProc IceSetIOErrorHandler (io_error_proc)
    IceIOErrorProc io_error_proc ;

{
    IceIOErrorProc old = IceIOErrorHandler ;
    
    IceIOErrorHandler = io_error_proc ;
    return (old) ;
}

/***************************************************************************
 * _IceIOError: trigger IO errors for this connection
 *
 * RETURN: never
 ***************************************************************************/

void _IceIOError (ice_conn)
    IceConn ice_conn ;

{
    _IceProcess process ;
    int         i ;

    /*
     * Invoke the protocols IO Error handlers:
     */

    for ( i = 0 ; i < 256 ; i++ )
    {
	if ( process = ice_conn->protocols[i] ) {
	    if ( process->orig ) {
		_IcePoRegistration r = _IcePoRegistrations[process->my_opcode];

		if ( r->io_error_proc )
		    (*(r->io_error_proc))(ice_conn) ;
	    } else {
		_IcePaRegistration r = _IcePaRegistrations[process->my_opcode];
		
		if ( r->io_error_proc )
		    (*(r->io_error_proc)) (ice_conn) ;
	    }
	    /* Ice protocol process is statically allocated */
	    if ( i > 0 )
		free (process) ;
	}
    }

    /*
     * Time to free connection resources:
     */

    if ( ice_conn->ibufobj )
    {
	_IceReleaseBuffer (ice_conn->ibufobj) ;
	ice_conn->ibufobj    = (void *) 0 ;
	ice_conn->ibuf       = (char *) 0 ;
	ice_conn->ibufptr    = (char *) 0 ;
	ice_conn->ibuflen    = 0 ;
	ice_conn->ibufcnt    = 0 ;
	ice_conn->reqbase    = (char *) 0 ;
	ice_conn->processing = (char *) 0 ;
    }

    if ( ice_conn->obufobj )
    {
	_IceReleaseBuffer (ice_conn->obufobj) ;
	ice_conn->obuf    = (char *) 0 ;
	ice_conn->obufptr = (char *) 0 ;
	ice_conn->obuflen = 0 ;
    }
	    
    /*
     * Invoke the global IO error handler (should not return)
     */
    
    if ( IceIOErrorHandler )
    {
	IceIOErrorHandler (ice_conn) ;
    }

    fprintf (stderr, "_IceIOError: fatal IO error on ice connection.\n") ;
    exit (1) ;
}


