/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iceprocess.c
 *
 * Process messages.
 ***************************************************************************/

#include <stdio.h>

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

/***************************************************************************
 * IceProcessMessages: porcessincomming messages
 *
 * It is a nightmare to have to fill in the ICE spec round here, because 
 * there are in fact two different functionnnalities here:
 * - Reading the connection (after select)
 * - Waitin/reading a specific reply
 * I do think there should be two API for this and that.
 ***************************************************************************/

void IceProcessMessages (ice_conn, reply_wait, reply_ready)
    IceConn          ice_conn ;
    IceReplyWaitInfo *reply_wait ;
    Bool             *reply_ready ;
{
    iceHeader   *ice_header ;
    int         opcode, length, status ;
    _IceProcess process ;
    int         freable_input = 0 ;

/*
 * Fill in the input buffer with the data: if a reply is waited for, read until
 * a full message is ready, otherwise, return now.
 */

    if ( ! IceRequestPending (ice_conn) )
    {
	while ( 1 )
	{
	    if ( _IceWaitForReadable (ice_conn) <= 0 )
		_IceIOError (ice_conn) ;

	    if ( IceReadConnection (ice_conn) <= 0 )
	    {
		if ( (! reply_wait) && reply_ready ) {
		    *reply_ready = False ;
		    return ;
		}		    
	    }
	    else
	    {
		break ;
	    }
	}
    }

/*
 * Process all (readed) messages :
 */

    while ( ice_header = (iceHeader *) _IceGetMessage (ice_conn, &freable_input) )
    {
	/*
	 * Retrieve from connection, the right handler:
	 */
	opcode = ice_header->majorOpcode ;
	ICE_REQUEST_LENGTH (ice_conn, ice_header, length) ;

	if ( process = ice_conn->protocols[opcode] )
	{
	    if ( process->orig )
	    {
                if ( reply_ready )
		    *reply_ready = False ;
		(process->oproc) (ice_conn
				  , process->closure
				  , ice_header->minorOpcode
				  , length
				  , ice_conn->byte_swap
				  , reply_wait
				  , reply_ready) ;
		if ( reply_ready && *reply_ready )
		    return ;
	    }
	    else
	    {
		(process->aproc) (ice_conn
				  , process->closure
				  , ice_header->minorOpcode
				  , length
				  , ice_conn->byte_swap) ;
	    }
	}
	else
	{
	    fprintf (stderr, "IceProcessMessage: invalid protocol %d\n", opcode) ;
	    exit (1) ;
	}

	/*
	 * If the input buffer is freable, free it. OOPS ! forgot the break.
	 */

	if ( freable_input && ice_conn->ibufobj )
	{
	    freable_input = 0 ;
	    _IceReleaseBuffer (ice_conn->ibufobj) ;
	    ice_conn->ibufobj    = (void *) 0 ;
	    ice_conn->ibuf       = (char *) 0 ;
	    ice_conn->ibufptr    = (char *) 0 ;
	    ice_conn->reqbase    = (char *) 0 ;
	    ice_conn->processing = (char *) 0 ;
	    ice_conn->needed     = 0 ;
	    ice_conn->ibuflen    = 0 ;
	    ice_conn->ibufcnt    = 0 ;
	    break ;
	}
    }

    if ( freable_input && ice_conn->ibufobj )
    {
	freable_input = 0 ;
	_IceReleaseBuffer (ice_conn->ibufobj) ;
	ice_conn->ibufobj    = (void *) 0 ;
	ice_conn->ibuf       = (char *) 0 ;
	ice_conn->ibufptr    = (char *) 0 ;
	ice_conn->reqbase    = (char *) 0 ;
	ice_conn->processing = (char *) 0 ;
	ice_conn->needed     = 0 ;
	ice_conn->ibuflen    = 0 ;
	ice_conn->ibufcnt    = 0 ;
    }

    return ;
}
