/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iceprotosetup.c
 * 
 * Protocol setup on a connection
 ***************************************************************************/

#include <stdio.h>

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

/*
 * This strust is used for communication between handler and orig
 * functions to pass the reply of ProtocolSetup:
 */

typedef struct _IceProtocolReply {
    int		major_opcode ;
    int         version_index ;
    char	*vendor ;
    char	*release ;
} _IceProtocolReplyRec, *_IceProtocolReply ;

/***************************************************************************
 * IceProtocolSetup: setup a protocol on the given connection
 *
 * RETURNS: IceProtocolSetupStatus.
 ***************************************************************************/

IceProtocolSetupStatus IceProtocolSetup (ice_conn
					 , opcode
					 , closure
					 , must_authenticate
					 , major_ret, minor_ret
					 , vendor_ret, release_ret
					 , error_length, error)
    IceConn    ice_conn ;
    int        opcode ;
    IcePointer closure ;
    Bool       must_authenticate ;
    int        *major_ret ;
    int        *minor_ret ;
    char       **vendor_ret ;
    char       **release_ret ;
    int        error_length ;
    char       *error ;

{
    iceProtocolSetup     *request ;
    _IceProtocolReplyRec reply ;
    char	         *preq, *pdata ;
    int                  i, l, extra ;
    _IcePoRegistration   registration ;
    IceReplyWaitInfo     reply_wait ;
    _IceProcess          process ;
    Bool                 ready ;
    
/*
 * Check that opcode is valid :
 */

    if ( ( opcode > _IcePoRegistrationCount) || ( opcode < 1 ) )
    {
	return (IceProtocolSetupFailure) ;
    }

/*
 * Is this protocol already active here ?
 */

    registration = _IcePoRegistrations[opcode] ;


/*
 * Compute extra needed bytes, and get header:
 * I could use the X11R6 ICElinint STRING_BYTES macro, but I hate computing
 * two times the same thing (strlen)
 */


    extra = 3 * sizeof (CARD16)
	+ ( l = strlen (registration->name), l + PAD32(l))
	+ ( l = strlen (registration->vendor), l + PAD32 (l))
	+ ( l = strlen (registration->release), l + PAD32(l)) ;

    for ( i = 0 ; i < registration->auth_count ; i++ )
    {
	extra += sizeof (CARD16) ;
	extra += ( l = strlen (registration->auth_names[i]), l + PAD32(l)) ;
    }

    extra += (2 * sizeof (CARD16) * registration->version_count) ;

    IceGetHeaderExtra (ice_conn
		       , ICE_MajorOpcode
		       , ICE_ProtocolSetup
		       , sizeof (iceProtocolSetup)
		       , WORD64COUNT(extra)
		       , iceProtocolSetup
		       , request
		       , pdata) ;

/*
 * Fill in the request:
 */

    request->major_opcode            = opcode ;
    request->mustAuthenticate        = must_authenticate ;
    request->numberOfVersions        = registration->version_count ;
    request->numberOfAuthentications = registration->auth_count ;

    STORE_STRING (pdata, registration->name) ;
    STORE_STRING (pdata, registration->vendor) ;
    STORE_STRING (pdata, registration->release) ;
    
    for ( i = 0 ; i < registration->auth_count ; i++ )
    {
	STORE_STRING (pdata, registration->auth_names[i]) ;
    }

    for ( i = 0 ; i < registration->version_count ; i++ )
    {
	STORE_CARD16 (pdata, registration->versions_rec[i].major_version) ;
	STORE_CARD16 (pdata, registration->versions_rec[i].minor_version) ;
    }

    IceFlush (ice_conn) ;

/*
 * Sit for reply :
 */

    reply_wait.sequence_of_request     = (ice_conn->sequence_number)++ ;
    reply_wait.major_opcode_of_request = ICE_MajorOpcode ;
    reply_wait.minor_opcode_of_request = ICE_ProtocolSetup ;
    reply_wait.reply                   = (IcePointer) &reply ;

    for (ready = False ; ! ready ; )
	IceProcessMessages (ice_conn, &reply_wait, &ready) ;

/*
 * Install this protocol procs:
 */

    if ( process = (_IceProcess) malloc (sizeof (_IceProcessRec)) )
    {
	IcePoVersionRec *v = &(registration->versions_rec[reply.version_index]) ;

	*vendor_ret  = reply.vendor ;
	*release_ret = reply.release ;

	process->oproc      = v->process_msg_proc ;
	process->my_opcode  = opcode ;
	process->his_opcode = reply.major_opcode ;
	process->orig       = 1 ;
	process->closure    = closure ;
	ice_conn->protocols[process->his_opcode] = process ;

	return (IceProtocolSetupSuccess) ;
    }

    return (IceProtocolSetupFailure) ;
}
    
/***************************************************************************
 * _IceProcessProtocolSetup: process proto setup request
 *
 * RETURNS: void.
 ***************************************************************************/

void _IceProcessProtocolSetup (ice_conn, swap, length)
    IceConn ice_conn ;
    Bool    swap ;
    int     length ;

{
    iceProtocolSetup   *request ;
    iceProtocolReply   *reply ;
    int		       i, extra, maj, min, slen ;
    char	       *name, *vendor, *release, *auth_name, *pdata ;
    _IcePaRegistration registration ;
    IcePaVersionRec    *version ;
    _IceProcess        process ;

    IceReadCompleteMessage (ice_conn
			    , sizeof (iceProtocolSetup)
			    , iceProtocolSetup
			    , request
			    , pdata) ;

    if ( request->numberOfAuthentications )
	fprintf (stderr, "_IceProcessProtocolSetup: got auth !\n") ;

    if ( request->numberOfVersions > 1 )
	fprintf (stderr, "_IceProcessProtocolSetup: got multiple vers !\n") ;

    COPY_STRING (pdata, swap, name) ; 
    COPY_STRING (pdata, swap, vendor) ;
    COPY_STRING (pdata, swap, release) ;

    for ( i = 0 ; i < request->numberOfAuthentications ; i++ )
    {
	EXTRACT_STRING (pdata, swap, auth_name, slen) ;
    }

    for ( i = 0 ; i < request->numberOfVersions ; i++ )
    {
	EXTRACT_CARD16 (pdata, swap, maj) ;
	EXTRACT_CARD16 (pdata, swap, min) ;
    }

/*
 * Look in our database for a matching protocol:
 */

    for ( i = 0 ; i < _IcePaRegistrationCount ; i++ )
    {
	registration = _IcePaRegistrations[i] ;
	if ( ! strcmp (registration->name, name) )
	{
	    free (name) ;
	    goto found ;
	}
    }
	 
    free (name) ;
    registration = (_IcePaRegistration) 0 ;

  found:

    if ( registration )
    {
	/*
	 * Reply with our protocol description:
	 */

	extra = 2 * sizeof (CARD16)
	    + ( i = strlen (registration->name), i + PAD32(i))
	    + ( i = strlen (registration->vendor), i + PAD32(i)) ;

	IceGetHeaderExtra (ice_conn
			   , ICE_MajorOpcode
			   , ICE_ProtocolReply
			   , sizeof (iceProtocolReply)
			   , WORD64COUNT(extra)
			   , iceProtocolReply
			   , reply
			   , pdata) ;
	
	reply->versionIndex = 0 ;
	reply->major_opcode = registration->opcode ;
	
	STORE_STRING (pdata, registration->vendor) ;
	STORE_STRING (pdata, registration->release) ;

	/*
	 * Setup this protocol for this connection, invoking the SetupNotifyProc, 
	 * if any.
	 */

	if ( process = (_IceProcess) malloc (sizeof (_IceProcessRec)) )
	{
	    version = &(registration->versions_rec[0]) ;

	    process->orig       = 0 ;
	    process->my_opcode  = registration->opcode ;
	    process->his_opcode = request->major_opcode ;
	    process->aproc      = version->process_msg_proc ;
	    process->closure    = (IcePointer) 0 ;
	    
	    ice_conn->protocols[process->his_opcode] = process ;

	    if ( registration->setup_proc )
	    {
		char *unused = (char *) 0 ;
		
		(*(registration->setup_proc)) (ice_conn
					       , version->major_version
					       , version->minor_version
					       , vendor
					       , release
					       , &(process->closure)
					       , &unused) ;
	    }
	}

	IceFlush (ice_conn) ;
    }
    else
    {
	fprintf (stderr, "_IceProcessProtocolSetup: not found\n") ;
	exit (1) ;
    }
    
    return ;
}

/***************************************************************************
 * _IceProcessProtocolReply: pack info into the reply_wait
 *
 * RETURNS: True (always ?)
 ***************************************************************************/

Bool _IceProcessProtocolReply (ice_conn, length, swap, reply_wait)
    IceConn          ice_conn ;
    unsigned long    length ;
    Bool             swap ;
    IceReplyWaitInfo *reply_wait ;
    
{
    iceProtocolReply *reply ;
    char	     *pdata ;

    IceReadCompleteMessage (ice_conn
			    , sizeof (iceProtocolReply)
			    , iceProtocolReply
			    , reply
			    , pdata) ;

    if ( reply_wait )
    {
	_IceProtocolReply waited = (_IceProtocolReply) reply_wait->reply ;

	COPY_STRING (pdata, swap, waited->vendor) ;
	COPY_STRING (pdata, swap, waited->release) ;
	waited->version_index = reply->versionIndex ;
	waited->major_opcode  = reply->major_opcode ;

	return (True) ;
    }

    fprintf (stderr, "_IceProcessProtocolReply: no reply waited for\n") ;
    exit (1) ;
}

/***************************************************************************
 * IceProtocolShutdown: shutdown the given protocol on given connection
 *
 * NOT IMPLEMENTED YET !
 *
 * RETURNS: X status
 ***************************************************************************/

IceStatus IceProtocolShutdown (ice_conn, major)
    IceConn ice_conn ;
    int     major ;

{
    fprintf (stderr, "IceProtocolShutdown: not implemented\n") ;

    return (1) ;
}
