/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * iceregister.c
 *
 * This file deals with (static & local) protocol registration
 ***************************************************************************/

#include <stdio.h>

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

/*
 * Comments !
 * - I cannot understand why the specs doesn't use some kind of context
 *   instead of these globals.
 */

_IcePoRegistration _IcePoRegistrations[256] ;
int	  	   _IcePoRegistrationCount = 0 ;

_IcePaRegistration _IcePaRegistrations[256] ;
int                _IcePaRegistrationCount = 0 ;

static IcePoVersionRec _IcePoProtocolVersion = { 1, 0, _IcePoProcessMessage } ;
static IcePaVersionRec _IcePaProtocolVersion = { 1, 0, _IcePaProcessMessage } ;

/***************************************************************************
 * _IceRegisterIce: register ICE itself in our database.
 *
 * This does the Po/Pa parts of it. This should be called once at init
 *
 * RETURNS: void (exit on error)
 ***************************************************************************/

void _IceRegisterIce ()

{
    static int done = 0 ;

    if ( done )
	return ;
    
    IceRegisterForProtocolSetup ("ICE"
				 , "random-hacker"
				 , "release"
				 , 1, &_IcePoProtocolVersion
				 , 0, (char **) 0, (IcePoAuthProc *) 0
				 , (IceIOErrorProc) 0) ;

    IceRegisterForProtocolReply ("ICE"
				 , "random-hacker"
				 , "release"
				 , 1, &_IcePaProtocolVersion
				 , 0, (char **) 0, (IcePaAuthProc *) 0
				 , (IceHostBasedAuthProc) 0
				 , (IceProtocolSetupProc) 0
				 , (IceProtocolActivateProc) 0
				 , (IceIOErrorProc) 0) ;

    done = 1 ;

    return ;
}

/***************************************************************************
 * IceRegisterForProtocolSetup: register this protocol as know (outgoing).
 *
 * RETURNS: the protocol number
 ***************************************************************************/

int IceRegisterForProtocolSetup (name, vendor, release
				 , version_count, version_recs
				 , auth_count, auth_names, auth_procs
				 , io_error_proc)
    char		*name ;
    char		*vendor ;
    char		*release ;
    int 		version_count ;
    IcePoVersionRec	*version_recs ;
    int			auth_count ;
    char		**auth_names ;
    IcePoAuthProc	*auth_procs ;
    IceIOErrorProc	io_error_proc ;

{
    static int           ice_done = 0 ;
    _IcePoRegistration   registration ;
    int			 i, size ;

/*
 * Initialize ICE protocol if not done (caution loop !)
 */

    if ( ! ice_done )
    {
	ice_done = 1 ;
	_IceRegisterIce () ;
    }

/*
 * Look in our database to see if already declared :
 */

    for ( i = 0 ; i < _IcePoRegistrationCount ; i++ )
    {
	registration = _IcePoRegistrations[i] ;
	if ( ! strcmp (name, registration->name) )
	    return (registration->opcode) ;
    }

/*
 * Not in data base, add it :
 */

    if ( _IcePoRegistrationCount + 1 >= 256 )
	return (-1) ;

    registration = (_IcePoRegistration) malloc (sizeof (_IcePoRegistrationRec)) ;

    if ( ! registration )
	return (-1) ;

    registration->opcode        = _IcePoRegistrationCount++ ;

    registration->name          = (char *) strdup (name) ;
    registration->vendor        = (char *) strdup (vendor) ;
    registration->release       = (char *) strdup (release) ;

    registration->version_count = version_count ;
    size = version_count * sizeof (IcePoVersionRec) ;
    if ( ! ( registration->versions_rec = (IcePoVersionRec *) malloc (size) ) )
    {
	free ( registration ) ;
	return (-1) ;
    }
    bcopy ((char *) version_recs, (char *) registration->versions_rec, size) ;

    /* Auth not implemented */
    registration->auth_count    = 0 ;
    registration->auth_names    = (char **) 0 ;
    registration->auth_proc     = (IcePoAuthProc) 0 ;

    registration->io_error_proc = io_error_proc ;

/*
 * Now et the registration in our data base:
 */
    
    _IcePoRegistrations[registration->opcode] = registration ;

    return (registration->opcode) ;
}

/***************************************************************************
 * IceRegisterForProtocolReply: register this protocol as know (incomming)
 *
 * RETURNS: the protocol number
 ***************************************************************************/

int IceRegisterForProtocolReply (name, vendor, release
				 , version_count, version_recs
				 , auth_count, auth_names, auth_procs
				 , host_based_auth_proc
				 , protocol_setup_notify_proc
				 , protocol_activate_proc
				 , io_error_proc)
    char 			*name ;
    char 			*vendor ;
    char 			*release ;
    int 			version_count ;
    IcePaVersionRec 		*version_recs ;
    int 			auth_count ;
    char 			**auth_names ;
    IcePaAuthProc 		*auth_procs ;
    IceHostBasedAuthProc 	host_based_auth_proc ;
    IceProtocolSetupProc 	protocol_setup_notify_proc ;
    IceProtocolActivateProc     protocol_activate_proc ;
    IceIOErrorProc 		io_error_proc ;

{
    static int           ice_done = 0 ;
    _IcePaRegistration   registration ;
    int			 i, size ;

/*
 * Initialize ICE for setup if not done (caution loop)
 */

    if ( ! ice_done )
    {
	ice_done = 1 ;
	_IceRegisterIce () ;
    }

/*
 * Look in our database to see if already declared :
 */

    for ( i = 0 ; i < _IcePaRegistrationCount ; i++ )
    {
	registration = _IcePaRegistrations[i] ;
	if ( ! strcmp (name, registration->name) )
	    return (registration->opcode) ;
    }

/*
 * Not in data base, add it :
 */

    if ( _IcePaRegistrationCount + 1 >= 256 )
	return (-1) ;

    registration = (_IcePaRegistration) malloc (sizeof (_IcePaRegistrationRec)) ;

    if ( ! registration )
	return (-1) ;

    registration->opcode        = _IcePaRegistrationCount++ ;

    registration->name          = (char *) strdup (name) ;
    registration->vendor        = (char *) strdup (vendor) ;
    registration->release       = (char *) strdup (release) ;

    registration->version_count = version_count ;
    size = version_count * sizeof (IcePaVersionRec) ;
    if ( ! ( registration->versions_rec = (IcePaVersionRec *) malloc (size) ) )
    {
	free ( registration ) ;
	return (-1) ;
    }
    bcopy ((char *) version_recs, (char *) registration->versions_rec, size) ;

    /* Auth not implemented */
    registration->auth_count    = 0 ;
    registration->auth_names    = (char **) 0 ;
    registration->auth_proc     = (IcePaAuthProc) 0 ;

    registration->setup_proc    = protocol_setup_notify_proc ;
    registration->io_error_proc = io_error_proc ;

/*
 * Now et the registration in our data base:
 */
    
    _IcePaRegistrations[registration->opcode] = registration ;

    return (registration->opcode) ;
}

