/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * icesetup.c
 *
 * ConnectionSetup request management
 ***************************************************************************/

#include <stdio.h>
#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICElibint.h>

/*
 * This struct is used to pass info from callback to initiating function:
 */

typedef struct _IceConnectionReply {
    int		version_index ;
    char	*vendor ;
    char	*release ;
} IceConnectionReplyRec, *IceConnectionReply ;

/***************************************************************************
 * _IceDoConnectionSetup: perform ConnectionSetup request.
 *
 * RETURNS: void
 ***************************************************************************/

void _IceDoConnectionSetup (ice_conn)
    IceConn ice_conn ;

{
    iceConnectionSetup    *request ;
    IceConnectionReplyRec reply ;
    _IceProcess           process ;
    char	          *pdata ;
    IceReplyWaitInfo      reply_wait ;
    int		          i, extra ;
    _IcePoRegistration    iceproto = _IcePoRegistrations[0] ;
    Bool 		  ready ;
    
/*
 * Count how many extra bytes are needed:
 */

    extra = 2 * sizeof (CARD16)
	+ ( i = strlen (iceproto->vendor), i + PAD32 (i))
	+ ( i = strlen (iceproto->release), i + PAD32(i)) ;

    extra += (iceproto->version_count * sizeof (CARD16)) ; 

    IceGetHeaderExtra (ice_conn
		       , ICE_MajorOpcode
		       , ICE_ConnectionSetup
		       , sizeof (iceConnectionSetup)
		       , WORD64COUNT(extra)
		       , iceConnectionSetup
		       , request
		       , pdata) ;

    STORE_STRING (pdata, iceproto->vendor) ;
    STORE_STRING (pdata, iceproto->release) ;
    /* No authentication protocol name list */

    for ( i = 0 ; i < iceproto->version_count ; i++ )
    {
	STORE_CARD16 (pdata, iceproto->versions_rec[i].major_version) ;
	STORE_CARD16 (pdata, iceproto->versions_rec[i].minor_version) ;
    }    
	
    request->numberOfVersions        = iceproto->version_count ;
    request->numberOfAuthentications = 0 ;
    request->mustAuthenticate        = 0 ;

    (ice_conn->sequence_number)++ ;

/*
 * Sit for the reply.
 * I suspect authentication take place during this sitting. I'm not 
 * implementing it but still implement this function as properly as I can.
 */
    
    IceFlush (ice_conn) ;

    reply_wait.sequence_of_request     = ice_conn->sequence_number ;
    reply_wait.major_opcode_of_request = ICE_MajorOpcode ;
    reply_wait.minor_opcode_of_request = ICE_ConnectionSetup ;
    reply_wait.reply                   = (IcePointer) &reply ;

    for ( ready = False ; ! ready ; )
	IceProcessMessages (ice_conn, &reply_wait, &ready) ;


    if ( process = (_IceProcess) malloc (sizeof (_IceProcessRec)) )
    {
	IcePoVersionRec *version = &(iceproto->versions_rec[reply.version_index]);

	free (reply.release) ;
	free (reply.vendor) ;

	/*
	 * I should use versionIndex for this part, although I'm in a hurry !
	 */

	process->orig          = True ;
	process->my_opcode     = 0 ;
	process->his_opcode    = 0 ;
	process->oproc         = version->process_msg_proc ;
	process->closure       = (IcePointer) 0 ;
	ice_conn->protocols[0] = process ;

	ice_conn->status       = IceConnectAccepted ;
    }
    else
    {
	fprintf (stderr, "_IceDoConnectionSetup: bad malloc\n") ;
	exit (1) ;
    }

    return ;
}

/***************************************************************************
 * _IceProcessConnectionSetup: process connection setup request
 *
 * RETURNS: void.
 ***************************************************************************/

void _IceProcessConnectionSetup (ice_conn, swap, length)
    IceConn ice_conn ;
    Bool    swap ;
    int     length ;

{
    iceConnectionSetup *request ;
    iceConnectionReply *reply ;
    char	       *preq ;
    char	       *vendor, *release, *pdata, *auth_name ;
    int		       i, maj, min, extra, slen ;
    _IcePaRegistration iceproto = _IcePaRegistrations[0] ;
    IcePaVersionRec    *version ;

    IceReadCompleteMessage (ice_conn
			    , sizeof (iceConnectionSetup)
			    , iceConnectionSetup
			    , request
			    , pdata) ;

    if ( request->numberOfAuthentications )
	fprintf (stderr, "_IceProcessConnectionSetup: got auth !\n") ;

    EXTRACT_STRING (pdata, swap, vendor, slen) ;
    EXTRACT_STRING (pdata, swap, release, slen) ;
    
    for ( i = 0 ; i < request->numberOfAuthentications ; i++ )
    {
	COPY_STRING (pdata, swap, auth_name) ;
	free (auth_name) ;
    }
    
    for ( i = 0 ; i < request->numberOfVersions ; i++ )
    {
	EXTRACT_CARD16 (pdata, swap, maj) ;
	EXTRACT_CARD16 (pdata, swap, min) ;
    }

/*
 * Ok, look in our own description of ice to find the match
 */

    version = iceproto->versions_rec ;
    for ( i = 0 ; i < iceproto->version_count ; i++, version++ )
    {
	if ( version->major_version == maj )
	    break ;
    }

    if ( version->major_version != maj )
    {
	fprintf (stderr, "_IceProcessConnectionSetup: no matching ICE\n") ;
	exit (1) ;
    }

    extra = 2 * sizeof (CARD16)
	+ ( i = strlen (iceproto->vendor), i + PAD32(i))
	+ ( i = strlen (iceproto->release), i + PAD32(i)) ;

    IceGetHeaderExtra (ice_conn
		       , ICE_MajorOpcode
		       , ICE_ConnectionReply
		       , sizeof (iceConnectionReply)
		       , WORD64COUNT(extra)
		       , iceConnectionReply
		       , reply
		       , pdata) ;

    reply->versionIndex = 0 ;

    STORE_STRING (pdata, iceproto->vendor) ;
    STORE_STRING (pdata, iceproto->release) ;

    ice_conn->status = IceConnectAccepted ;

    IceFlush (ice_conn) ;
}

/***************************************************************************
 * _IceProcessConnectionReply: process connection reply message
 *
 * RETURNS: according to reply_wait
 ***************************************************************************/

Bool _IceProcessConnectionReply (ice_conn, length, swap, reply_wait)
    IceConn          ice_conn ;
    int              length ;
    Bool             swap ;
    IceReplyWaitInfo *reply_wait ;

{
    iceConnectionReply *reply ;
    char	       *pdata ;

    IceReadCompleteMessage (ice_conn
			    , sizeof (iceConnectionReply)
			    , iceConnectionReply
			    , reply
			    , pdata) ;

    if ( reply_wait 
	&& ( reply_wait->minor_opcode_of_request == ICE_ConnectionSetup)
	&& reply_wait->reply )
    {
	IceConnectionReply waited = (IceConnectionReply) reply_wait->reply ;

	/* 
	 * Copy the reply into the provided reply:
	 */
	waited->version_index = reply->versionIndex ;
	COPY_STRING (pdata, swap, waited->vendor) ;
	COPY_STRING (pdata, swap, waited->release) ;

	return (True) ;
    }
    
    /* This is implementation bug */
    
    fprintf (stderr, "_IceProcessConnectionReply: no reply_wait provided (fatal)\n");
    exit (1) ;
}


    

			    



